/*
  EdiNoS - Editeur de NoS
  Copyright (C) 2018-2019 - Luc Poittevin — Tous droits réservés.
  Contact : edinos@free.fr  -  https://www.edinos.org
  
  Ce programme est un logiciel libre ; vous pouvez le redistribuer ou le
  modifier suivant les termes de la “GNU General Public License” telle que
  publiée par la Free Software Foundation : soit la version 3 de cette
  licence, soit (à votre gré) toute version ultérieure.
  
  Ce programme est distribué dans l’espoir qu’il vous sera utile, mais SANS
  AUCUNE GARANTIE : sans même la garantie implicite de COMMERCIALISABILITÉ
  ni d’ADÉQUATION À UN OBJECTIF PARTICULIER. Consultez la Licence Générale
  Publique GNU pour plus de détails.
  
  Vous devriez avoir reçu une copie de la Licence Générale Publique GNU avec
  ce programme ; si ce n’est pas le cas, consultez :
  <http://www.gnu.org/licenses/>. 
 */

package nos.edit;

import java.util.AbstractMap;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;

import javax.swing.table.AbstractTableModel;

import nos.edit.html.FenMessage;
import nos.edit.html.FenQuestion;
import nos.obj.Action;
import nos.obj.Edi;
import nos.obj.FaitAttendu;
import nos.obj.Message;
import nos.obj.Procedure;
import nos.obj.Question;

@SuppressWarnings("serial")
public class Actions extends AbstractTableModel {

	public static enum TypeAction {
		Procédure, Question, Message,
	}

	private String[] colonnes = new String[] { "Nom", "Type" };

	private List<Map.Entry<String, Action>> listeActions;

	private List<TypeAction> listeTypes;

	public Actions() {
		super();
		listeActions = new ArrayList<Map.Entry<String, Action>>();
		listeTypes = new ArrayList<TypeAction>();
	}

	public int getColumnCount() {
		return colonnes.length;
	}

	public int getRowCount() {
		return listeActions.size();
	}

	public Object getValueAt(int i, int j) {
		switch (j) {
		case 0: // colonne Nom
			return listeActions.get(i).getKey();
		case 1: // colonne Type
			switch (listeTypes.get(i)) {
			case Procédure:
				return "Procédure";
			case Question:
				return "Question";
			case Message:
				return "Message";
			default:
				return "";
			}
		default:
			return null;
		}
	}

	public boolean isCellEditable(int row, int col) {
		return false;
	}

	public void setValueAt(Object value, int row, int col) {
		
	}

	public String getColumnName(int c) {
		return (colonnes[c]);
	}

	public void ouvre(int i, FenActions fa) {
		Action act = listeActions.get(i).getValue();

		if (act.getClass().getName() == "nos.obj.Question") {
				FenQuestion f = new FenQuestion((Question) act, fa);
				act.getBase().ajouteFen(f);
				((FenEdiNoS) Edi.nos().fen()).ouvreInternalFrame(f);
				f.init();
		}
		if (act.getClass().getName() == "nos.obj.Message") {
				FenMessage f = new FenMessage((Message) act);
				act.getBase().ajouteFen(f);
				((FenEdiNoS) Edi.nos().fen()).ouvreInternalFrame(f);
				f.init();
			}
		if (act.getClass().getName() == "nos.obj.Procedure") {
				FenProcedure f = new FenProcedure((Procedure) act, fa);
				act.getBase().ajouteFen(f);
				((FenEdiNoS) Edi.nos().fen()).ouvreInternalFrame(f);
				f.init();
		}
	}

	public void ajouteAction(Action a) {
		listeActions.add(new AbstractMap.SimpleEntry<String, Action>(
				a.getNom(), a));
		if (a.getClass().getName() == "nos.obj.Question") {
			listeTypes.add(TypeAction.Question);
		} else if (a.getClass().getName() == "nos.obj.Message") {
			listeTypes.add(TypeAction.Message);
		} else if (a.getClass().getName() == "nos.obj.Procedure") {
			listeTypes.add(TypeAction.Procédure);
		}
		 this.fireTableDataChanged();
	}

	public String getDescription(int i) {
		return listeActions.get(i).getValue().getDescription();
	}

	public void vide() {
		listeActions.clear();
		listeTypes.clear();
	}

	public String nomAction(int i) {
		return listeActions.get(i).getValue().getNom();
	}
	
	public int numLigne(Action act) {
		Map.Entry<String, Action> m = new AbstractMap.SimpleEntry<String, Action>(
				act.getNom(), act);
		return listeActions.indexOf(m);
	}
	
	
	public List<FaitAttendu> getFaitsAttendus(int i) {
		return listeActions.get(i).getValue().getFaitsAttendus();
	}
}
