/*
  EdiNoS - Editeur de NoS
  Copyright (C) 2018-2021 - Luc Poittevin — Tous droits réservés.
  Contact : edinos@free.fr  -  https://www.edinos.org
  
  Ce programme est un logiciel libre ; vous pouvez le redistribuer ou le
  modifier suivant les termes de la “GNU General Public License” telle que
  publiée par la Free Software Foundation : soit la version 3 de cette
  licence, soit (à votre gré) toute version ultérieure.
  
  Ce programme est distribué dans l’espoir qu’il vous sera utile, mais SANS
  AUCUNE GARANTIE : sans même la garantie implicite de COMMERCIALISABILITÉ
  ni d’ADÉQUATION À UN OBJECTIF PARTICULIER. Consultez la Licence Générale
  Publique GNU pour plus de détails.
  
  Vous devriez avoir reçu une copie de la Licence Générale Publique GNU avec
  ce programme ; si ce n’est pas le cas, consultez :
  <http://www.gnu.org/licenses/>. 
 */

package nos.edit;

import java.awt.Color;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.KeyEvent;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.util.ArrayList;
import java.util.Collections;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

import javax.swing.ButtonGroup;
import javax.swing.JComponent;
import javax.swing.JDialog;
import javax.swing.JFrame;
import javax.swing.JInternalFrame;
import javax.swing.JMenu;
import javax.swing.JMenuBar;
import javax.swing.JMenuItem;
import javax.swing.JOptionPane;
import javax.swing.JTable;
import javax.swing.KeyStroke;
import javax.swing.event.InternalFrameEvent;
import javax.swing.event.InternalFrameListener;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;

import nos.edit.html.TableFaits;
import nos.obj.Action;
import nos.obj.Base;
import nos.obj.Edi;
import nos.obj.FaitAttendu;
import nos.obj.Message;
import nos.obj.Question;
import nos.run.Moteur;
import nos.utils.*;

@SuppressWarnings("serial")
public class FenActions extends JInternalFrame implements InternalFrameListener {

	private String htmlContent = "<html dir=\"ltr\"><head></head><body contenteditable=\"true\"></body></html>";

	private ButtonGroup groupeBoutons = new ButtonGroup();

	private Map<String, Action> collection;

	private Base base;

	private Actions actions = new Actions();

	private JMenuBar menuBar;

	private Moteur revinos;

	public FenActions(Base b) {
		super("Actions de la base <" + b.getNom()+">", true, // resizable
				true, // closable
				true, // maximizable
				true);// iconifiable


		addInternalFrameListener(this);
		base = b;
		collection = base.getAssocActions();
		initComponents();
		initComponents2();
		installeMenus();
	}

	public FenActions(Base b, Moteur c) {
		super("Actions : " + b.getNom(), true, // resizable
				true, // closable
				true, // maximizable
				true);// iconifiable
		addInternalFrameListener(this);
		base = b;
		collection = base.getAssocActions();
		revinos = c;
		initComponents();
		initComponents2();
		installeMenus();
	}

	private void initComponents2() {

		effaceCriteres();
		
		jButton3.setName("NouvelleAction"); // pour FEST
		jButton5.setName("FermerActions"); // pour FEST

		jRadioButtonTous.setActionCommand("Tous");
		jRadioButtonProc.setActionCommand("Procédure");
		jRadioButtonQuestion.setActionCommand("Question");
		jRadioButtonMessage.setActionCommand("Message");

		groupeBoutons.add(jRadioButtonTous);
		groupeBoutons.add(jRadioButtonProc);
		groupeBoutons.add(jRadioButtonQuestion);
		groupeBoutons.add(jRadioButtonMessage);
		
		jTable1.setModel(actions);
		jTable1.getColumnModel().getColumn(1).setPreferredWidth(80);
		jTable1.getColumnModel().getColumn(1).setMaxWidth(80);
		
		jTextArea1.setLineWrap(true);
		jTextArea1.setMargin(new Insets(0, 3, 0, 3));
		jTextArea1.setEditable(false);
		
		construitListe();
		
		List<FaitAttendu> l = new ArrayList<FaitAttendu>();
		jTable2.setModel(new TableFaits(l));

		setTailleColonnesFaits();
		
		jLabel1.setForeground(Color.BLUE);
		jLabel2.setForeground(Color.BLUE);

		// Register the Listeners
		registerEventListeners();
	}

	private void setTailleColonnesFaits() {
		jTable2.getColumnModel().getColumn(1).setPreferredWidth(80);
		jTable2.getColumnModel().getColumn(1).setMaxWidth(80);
	}

	private void installeMenus() {
		FenEdiNoS ed = (FenEdiNoS) Edi.nos().fen();
		menuBar = ed.menuBarEdinos();
		menuBar.add(ed.menuBase());
		menuBar.add(menuInterface());
		Edi.nos().fen().setJMenuBar(menuBar);
		Edi.nos().fen().setVisible(true);
	}

	public JMenu menuInterface() {

		JMenuItem menuItem;
		JMenu menu = new JMenu("Actions");

		menuItem = new JMenuItem("Nouvelle Action");
		menuItem.setAccelerator(KeyStroke.getKeyStroke(
				java.awt.event.KeyEvent.VK_N, ToolNos.renvoieModifiers()));
		menuItem.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				saisieNouvelleAction();
			}
		});
		menu.add(menuItem);
		menuItem = new JMenuItem("Renommer une Action");
		menuItem.setAccelerator(KeyStroke.getKeyStroke(
				java.awt.event.KeyEvent.VK_R, ToolNos.renvoieModifiers()));
		menuItem.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				saisieNouveauNomAction();
			}
		});
		menu.add(menuItem);
		menu.addSeparator();
		menuItem = new JMenuItem("Supprimer une Action");
		menuItem.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				supprimeAction();
			}
		});
		menu.add(menuItem);

		menu.addSeparator();
		menuItem = new JMenuItem("Dupliquer une Action");
		menuItem.setEnabled(false);
		menu.add(menuItem);
		menuItem = new JMenuItem("Déplacer une Action");
		menuItem.setEnabled(false);
		menu.add(menuItem);

		menu.addSeparator();
		menuItem = new JMenuItem("Fermer la fenêtre");
		menuItem.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				fermeFen();
			}
		});
		menuItem.setAccelerator(KeyStroke.getKeyStroke(
				java.awt.event.KeyEvent.VK_W, ToolNos.renvoieModifiers()));
		menu.add(menuItem);

		return (menu);
	}

	private void registerEventListeners() {
		
		jTable1.getSelectionModel().addListSelectionListener(
				new ListSelectionListener() {
					public void valueChanged(ListSelectionEvent evt) {
						if (jTable1.getSelectedRow() != -1) {
							montreAction(jTable1.getSelectedRow());
						}
					}
				});

		jTable1.addMouseListener(new MouseAdapter() {
			public void mousePressed(MouseEvent mouseEvent) {
				JTable table = (JTable) mouseEvent.getSource();
				if (mouseEvent.getClickCount() == 2
						&& table.getSelectedRow() != -1) {
					int row = jTable1.convertRowIndexToModel(table
							.getSelectedRow());
					ouvre(((Actions) jTable1.getModel()), row);
				}
			}
		});

		ActionListener actionListenerEsc = new ActionListener() {
			public void actionPerformed(ActionEvent actionEvent) {
				fermeFen();
			}
		};

		KeyStroke stroke = KeyStroke.getKeyStroke(KeyEvent.VK_ESCAPE, 0);
		getRootPane().registerKeyboardAction(actionListenerEsc, stroke,
				JComponent.WHEN_IN_FOCUSED_WINDOW);
	}

	protected void fermeFen() {
		if (ToolNos.nomSysteme().equals("Mac")) {
			ToolNos.fermeFen((JInternalFrame) this);
		} else {
			dispose();
		} 
		((FenEdiNoS) Edi.nos().fen()).checkDerniereFen(); 
	}

	private void ouvre(Actions actions, int row) {
		actions.ouvre(row, this);
	}
	
	private void saisieNouvelleAction() {
		JFrame.setDefaultLookAndFeelDecorated(true);
		JDialog.setDefaultLookAndFeelDecorated(true);

		FenCreeAction customDialog = new FenCreeAction(this, null == revinos);
		// on ne propose pas le wizard
		// lorsque REVINOS est lancé
		customDialog.pack();

		customDialog.setLocationRelativeTo(this);
		customDialog.setVisible(true);
	}

	private void supprimeAction() {
		if (jTable1.getSelectedRow() == -1) {
			// aucune action sélectionnée
			Edi.nos().message("Veuillez d'abord sélectionner une action...");
			return;
		}

		String nomAction = ((Actions) jTable1.getModel()).nomAction(jTable1
				.getSelectedRow());

		int rep = JOptionPane.showConfirmDialog(Edi.nos().fen(),
				"Confirmez-vous la suppression de l'action <" + nomAction
						+ "> ?", "Suppression d'une action",
				JOptionPane.YES_NO_OPTION);
		if (rep == 0) {
			// Suppression confirmée
			base.supprimeAction(nomAction);
			collection = base.getAssocActions();
			construitListe();
		}
	}

	private void saisieNouveauNomAction() {
		if (jTable1.getSelectedRow() == -1) {
			// aucune action sélectionnée
			JOptionPane.showMessageDialog(Edi.nos().fen(),
					"Veuillez d'abord sélectionner une action...");
			return;
		}

		JFrame.setDefaultLookAndFeelDecorated(true);
		JDialog.setDefaultLookAndFeelDecorated(true);

		String nomAction = ((Actions) jTable1.getModel()).nomAction(jTable1
				.getSelectedRow());
		FenRenommeAction customDialog = new FenRenommeAction(this, nomAction);

		customDialog.pack();
		customDialog.setLocationRelativeTo(this);
		customDialog.setVisible(true);
	}

	public void creeAction(String choix, String nom, boolean wizard) {
		// retour de FenCreeAction :
		// <choix> vaut "Question" , "Procédure" ou "Message"

		if (choix.equals("Question")) {
			base.creeQuestion(nom);			
			Question q = (Question) base.getAction(nom);
			q.setTexte(htmlContent);
		} else if (choix.equals("Procédure")) {
			base.creeProcedure(nom);
		} else if (choix.equals("Message")) {
			base.creeMessage(nom);
			Message m = (Message) base.getAction(nom);
			m.setTexte(htmlContent);
		}

		collection = base.getAssocActions();

		if (revinos != null) {
			revinos.retourne("actionAutiliser.nom " + nom);
			if (choix.equals("Question")) {
				revinos.retourne("actionAutiliser.type demander:");
			} else if (choix.equals("Procédure")) {
				revinos.retourne("actionAutiliser.type evaluer:");
			} else if (choix.equals("Message")) {
				revinos.retourne("actionAutiliser.type afficher:");
			}
		}

		effaceCriteres();
		construitListe();
		
		positionneSur(nom);

//		}
	}

	private void montreAction(int i) {
		int row = jTable1.convertRowIndexToModel(i);
		Actions a = ((Actions) jTable1.getModel());
		String desc = a.getDescription(row);

		jTextArea1.setText(desc);
		
		jTable2.setModel(new TableFaits(a.getFaitsAttendus(row)));
		setTailleColonnesFaits();
	}

	private void construitListe() {

		List<String> l = new ArrayList<String>();

		String filtreType = groupeBoutons.getSelection().getActionCommand();

		if (filtreType.equals("Tous")) {
			l.addAll(collection.keySet());
		} else {
			for (Action act : collection.values()) {
				if (act.getClass().getName() == "nos.obj.Question") {
					if (filtreType.equals("Question")) {
						l.add(act.getNom());
					}
				} else if (act.getClass().getName() == "nos.obj.Procedure") {
					if (filtreType.equals("Procédure")) {
						l.add(act.getNom());
					}
				} else if (act.getClass().getName() == "nos.obj.Message") {
					if (filtreType.equals("Message")) {
						l.add(act.getNom());
					}
				}
			}
		}

		List<String> listeFiltre = l
				.stream()
				.filter(nom -> nom.contains(jTextField1.getText()))
				.map(String::new).collect(Collectors.toList());

		Collections.sort(listeFiltre);

		actions.vide();
		for (String nomAction : listeFiltre) {
			actions.ajouteAction(collection.get(nomAction));
		}
		jTable1.repaint();
	}

	public void renommeAction(String nouveauNom) {
		// retour de FenRenommeAction

		String ancienNom = ((Actions) jTable1.getModel()).nomAction(jTable1
				.getSelectedRow());
		Action act = base.getAssocActions().get(ancienNom);
		act.setNom(nouveauNom);
		base.getAssocActions().remove(ancienNom);
		base.getAssocActions().put(nouveauNom, act);

		collection = base.getAssocActions();
		construitListe();
		positionneSur(nouveauNom);
		// TODO : modifier les références
	}

	 public void positionneSur(String text) {
		Action a = Edi.nos().getBaseCourante().getAction(text);
		 int i = ((Actions) jTable1.getModel()).numLigne(a);
		 jTable1.setRowSelectionInterval(i, i);
	 }

	private void effaceCriteres() {
		jTextField1.setText(""); // Texte à rechercher
		jRadioButtonTous.setSelected(true);
	}
	
	public void rafraichitLigneCourante() {
		int i = jTable1.getSelectedRow();
		 ((Actions) jTable1.getModel()).fireTableDataChanged();
		 jTable1.setRowSelectionInterval(i, i);
	}	

	public void internalFrameActivated(InternalFrameEvent e) {
		Edi.nos().fen().setJMenuBar(menuBar);
		Edi.nos().fen().setVisible(true);
	}

	public void internalFrameDeactivated(InternalFrameEvent e) {
		((FenEdiNoS) Edi.nos().fen()).setMenuBar();
	}

	public void internalFrameClosed(InternalFrameEvent e) {
	}

	public void internalFrameClosing(InternalFrameEvent e) {
		fermeFen();
	}

	public void internalFrameDeiconified(InternalFrameEvent e) {
	}

	public void internalFrameIconified(InternalFrameEvent e) {
	}

	public void internalFrameOpened(InternalFrameEvent e) {
	}

	private void jButton1ActionPerformed(java.awt.event.ActionEvent evt) {
		// Bouton "Rechercher"
		construitListe();
	}

	private void jButton3ActionPerformed(java.awt.event.ActionEvent evt) {
		// Bouton "Nouvelle Action"
		saisieNouvelleAction();
	}

	private void jButton2ActionPerformed(java.awt.event.ActionEvent evt) {
		// Bouton "Effacer"
		effaceCriteres();
	}

	private void jButton5ActionPerformed(java.awt.event.ActionEvent evt) {
		// Bouton "Fermer"
		fermeFen();
	}

	private void jRadioButtonTousActionPerformed(java.awt.event.ActionEvent evt) {
	}

	private void jRadioButtonMessageActionPerformed(
			java.awt.event.ActionEvent evt) {
	}

	private void jRadioButtonQuestionActionPerformed(
			java.awt.event.ActionEvent evt) {
	}

	private void jTextField1ActionPerformed(java.awt.event.ActionEvent evt) {		
	}

	  @SuppressWarnings("unchecked")
	    // <editor-fold defaultstate="collapsed" desc="Generated Code">                          
	    private void initComponents() {

	        jPanel1 = new javax.swing.JPanel();
	        jButton3 = new javax.swing.JButton();
	        jButton5 = new javax.swing.JButton();
	        jScrollPane2 = new javax.swing.JScrollPane();
	        jTable1 = new javax.swing.JTable();
	        jScrollPane1 = new javax.swing.JScrollPane();
	        jTextArea1 = new javax.swing.JTextArea();
	        jScrollPane3 = new javax.swing.JScrollPane();
	        jTable2 = new javax.swing.JTable();
	        jLabel1 = new javax.swing.JLabel();
	        jLabel2 = new javax.swing.JLabel();
	        jPanel2 = new javax.swing.JPanel();
	        jButton2 = new javax.swing.JButton();
	        jButton1 = new javax.swing.JButton();
	        jRadioButtonQuestion = new javax.swing.JRadioButton();
	        jRadioButtonMessage = new javax.swing.JRadioButton();
	        jRadioButtonTous = new javax.swing.JRadioButton();
	        jRadioButtonProc = new javax.swing.JRadioButton();
	        jLabel4 = new javax.swing.JLabel();
	        jTextField1 = new javax.swing.JTextField();
	        jLabel3 = new javax.swing.JLabel();

//	        setDefaultCloseOperation(javax.swing.WindowConstants.DISPOSE_ON_CLOSE);

	        jPanel1.setBorder(javax.swing.BorderFactory.createEtchedBorder());

	        jButton3.setText("Nouvelle action");
	        jButton3.setFocusable(false);
	        jButton3.setIgnoreRepaint(true);
	        jButton3.setMaximumSize(new java.awt.Dimension(60, 30));
	        jButton3.addActionListener(new java.awt.event.ActionListener() {
	            public void actionPerformed(java.awt.event.ActionEvent evt) {
	                jButton3ActionPerformed(evt);
	            }
	        });

	        jButton5.setText("Fermer");
	        jButton5.setFocusable(false);
	        jButton5.setIgnoreRepaint(true);
	        jButton5.setMaximumSize(new java.awt.Dimension(60, 30));
	        jButton5.addActionListener(new java.awt.event.ActionListener() {
	            public void actionPerformed(java.awt.event.ActionEvent evt) {
	                jButton5ActionPerformed(evt);
	            }
	        });

	        org.jdesktop.layout.GroupLayout jPanel1Layout = new org.jdesktop.layout.GroupLayout(jPanel1);
	        jPanel1.setLayout(jPanel1Layout);
	        jPanel1Layout.setHorizontalGroup(
	            jPanel1Layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
	            .add(jPanel1Layout.createSequentialGroup()
	                .add(jButton3, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
	                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
	                .add(jButton5, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
	                .addContainerGap(org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE))
	        );
	        jPanel1Layout.setVerticalGroup(
	            jPanel1Layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
	            .add(jPanel1Layout.createSequentialGroup()
	                .add(jPanel1Layout.createParallelGroup(org.jdesktop.layout.GroupLayout.BASELINE)
	                    .add(jButton3, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
	                    .add(jButton5, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE))
	                .addContainerGap())
	        );

	        jTable1.setModel(new javax.swing.table.DefaultTableModel(
	            new Object [][] {
	                {null, null},
	                {null, null},
	                {null, null},
	                {null, null}
	            },
	            new String [] {
	                "Nom", "Type"
	            }
	        ) {
	            Class[] types = new Class [] {
	                java.lang.String.class, java.lang.String.class
	            };
	            boolean[] canEdit = new boolean [] {
	                true, false
	            };

	            public Class getColumnClass(int columnIndex) {
	                return types [columnIndex];
	            }

	            public boolean isCellEditable(int rowIndex, int columnIndex) {
	                return canEdit [columnIndex];
	            }
	        });
	        jScrollPane2.setViewportView(jTable1);
	        if (jTable1.getColumnModel().getColumnCount() > 0) {
	            jTable1.getColumnModel().getColumn(1).setPreferredWidth(100);
	            jTable1.getColumnModel().getColumn(1).setMaxWidth(100);
	        }

	        jTextArea1.setColumns(20);
	        jTextArea1.setRows(5);
	        jScrollPane1.setViewportView(jTextArea1);

	        jTable2.setModel(new javax.swing.table.DefaultTableModel(
	            new Object [][] {
	                {null, null, null},
	                {null, null, null},
	                {null, null, null},
	                {null, null, null}
	            },
	            new String [] {
	                "Nom", "Type", "Description"
	            }
	        ));
	        jScrollPane3.setViewportView(jTable2);

	        jLabel1.setText("Description de l'action");

	        jLabel2.setText("Faits renvoyés");

	        jPanel2.setBorder(javax.swing.BorderFactory.createLineBorder(new java.awt.Color(0, 0, 0)));

	        jButton2.setText("Effacer");
	        jButton2.setToolTipText("");
	        jButton2.addActionListener(new java.awt.event.ActionListener() {
	            public void actionPerformed(java.awt.event.ActionEvent evt) {
	                jButton2ActionPerformed(evt);
	            }
	        });

	        jButton1.setText("Rechercher");
	        jButton1.setToolTipText("");
	        jButton1.addActionListener(new java.awt.event.ActionListener() {
	            public void actionPerformed(java.awt.event.ActionEvent evt) {
	                jButton1ActionPerformed(evt);
	            }
	        });

	        jRadioButtonQuestion.setText("Question");
	        jRadioButtonQuestion.addActionListener(new java.awt.event.ActionListener() {
	            public void actionPerformed(java.awt.event.ActionEvent evt) {
	                jRadioButtonQuestionActionPerformed(evt);
	            }
	        });

	        jRadioButtonMessage.setText("Message");
	        jRadioButtonMessage.addActionListener(new java.awt.event.ActionListener() {
	            public void actionPerformed(java.awt.event.ActionEvent evt) {
	                jRadioButtonMessageActionPerformed(evt);
	            }
	        });

	        jRadioButtonTous.setText("Tous");
	        jRadioButtonTous.addActionListener(new java.awt.event.ActionListener() {
	            public void actionPerformed(java.awt.event.ActionEvent evt) {
	                jRadioButtonTousActionPerformed(evt);
	            }
	        });

	        jRadioButtonProc.setText("Procédure");

	        jLabel4.setHorizontalAlignment(javax.swing.SwingConstants.CENTER);
	        jLabel4.setText("Type :");

	        jTextField1.setText("jTextField1");
	        jTextField1.addActionListener(new java.awt.event.ActionListener() {
	            public void actionPerformed(java.awt.event.ActionEvent evt) {
	                jTextField1ActionPerformed(evt);
	            }
	        });

	        jLabel3.setHorizontalAlignment(javax.swing.SwingConstants.CENTER);
	        jLabel3.setText("Nom :");

	        org.jdesktop.layout.GroupLayout jPanel2Layout = new org.jdesktop.layout.GroupLayout(jPanel2);
	        jPanel2.setLayout(jPanel2Layout);
	        jPanel2Layout.setHorizontalGroup(
	            jPanel2Layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
	            .add(jPanel2Layout.createSequentialGroup()
	                .add(jPanel2Layout.createParallelGroup(org.jdesktop.layout.GroupLayout.TRAILING)
	                    .add(jPanel2Layout.createSequentialGroup()
	                        .add(12, 12, 12)
	                        .add(jLabel3)
	                        .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
	                        .add(jTextField1, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, 84, Short.MAX_VALUE)
	                        .addPreferredGap(org.jdesktop.layout.LayoutStyle.UNRELATED)
	                        .add(jLabel4, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, 72, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
	                        .addPreferredGap(org.jdesktop.layout.LayoutStyle.UNRELATED)
	                        .add(jRadioButtonTous, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, 63, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
	                        .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
	                        .add(jRadioButtonProc)
	                        .add(5, 5, 5)
	                        .add(jRadioButtonQuestion, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, 89, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE))
	                    .add(jButton2, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, 95, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE))
	                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
	                .add(jPanel2Layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
	                    .add(jRadioButtonMessage)
	                    .add(org.jdesktop.layout.GroupLayout.TRAILING, jButton1, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, 98, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE))
	                .addContainerGap())
	        );
	        jPanel2Layout.setVerticalGroup(
	            jPanel2Layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
	            .add(org.jdesktop.layout.GroupLayout.TRAILING, jPanel2Layout.createSequentialGroup()
	                .addContainerGap()
	                .add(jPanel2Layout.createParallelGroup(org.jdesktop.layout.GroupLayout.BASELINE)
	                    .add(jRadioButtonQuestion)
	                    .add(jRadioButtonMessage)
	                    .add(jRadioButtonTous)
	                    .add(jRadioButtonProc)
	                    .add(jLabel4, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, 20, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
	                    .add(jTextField1, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
	                    .add(jLabel3, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, 16, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE))
	                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
	                .add(jPanel2Layout.createParallelGroup(org.jdesktop.layout.GroupLayout.BASELINE)
	                    .add(jButton2)
	                    .add(jButton1))
	                .addContainerGap(org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE))
	        );

	        org.jdesktop.layout.GroupLayout layout = new org.jdesktop.layout.GroupLayout(getContentPane());
	        getContentPane().setLayout(layout);
	        layout.setHorizontalGroup(
	            layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
	            .add(org.jdesktop.layout.GroupLayout.TRAILING, jPanel1, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
	            .add(org.jdesktop.layout.GroupLayout.TRAILING, layout.createSequentialGroup()
	                .addContainerGap()
	                .add(layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
	                    .add(jPanel2, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
	                    .add(layout.createSequentialGroup()
	                        .add(jScrollPane2, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, 286, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
	                        .addPreferredGap(org.jdesktop.layout.LayoutStyle.UNRELATED)
	                        .add(layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
	                            .add(jLabel2, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
	                            .add(org.jdesktop.layout.GroupLayout.TRAILING, jScrollPane1, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, 307, Short.MAX_VALUE)
	                            .add(org.jdesktop.layout.GroupLayout.TRAILING, jLabel1, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
	                            .add(jScrollPane3, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, 0, Short.MAX_VALUE))))
	                .addContainerGap())
	        );
	        layout.setVerticalGroup(
	            layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
	            .add(layout.createSequentialGroup()
	                .add(jPanel1, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, 32, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
	                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
	                .add(jPanel2, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
	                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
	                .add(layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
	                    .add(layout.createSequentialGroup()
	                        .add(jLabel1, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, 28, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
	                        .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
	                        .add(jScrollPane1, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, 107, Short.MAX_VALUE)
	                        .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
	                        .add(jLabel2, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, 26, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
	                        .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
	                        .add(jScrollPane3, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, 109, Short.MAX_VALUE))
	                    .add(jScrollPane2, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, 0, Short.MAX_VALUE))
	                .addContainerGap())
	        );

	        pack();
	    }// </editor-fold>                        


	// Variables declaration - do not modify
	private javax.swing.JButton jButton1;
	private javax.swing.JButton jButton2;
	private javax.swing.JButton jButton3;
	private javax.swing.JButton jButton5;
	private javax.swing.JLabel jLabel1;
	private javax.swing.JLabel jLabel2;
	private javax.swing.JLabel jLabel3;
	private javax.swing.JLabel jLabel4;
	private javax.swing.JPanel jPanel1;
	private javax.swing.JPanel jPanel2;
	private javax.swing.JRadioButton jRadioButtonMessage;
	private javax.swing.JRadioButton jRadioButtonProc;
	private javax.swing.JRadioButton jRadioButtonQuestion;
	private javax.swing.JRadioButton jRadioButtonTous;
	private javax.swing.JScrollPane jScrollPane1;
	private javax.swing.JScrollPane jScrollPane2;
	private javax.swing.JScrollPane jScrollPane3;
	private javax.swing.JTable jTable1;
	private javax.swing.JTable jTable2;
	private javax.swing.JTextArea jTextArea1;
	private javax.swing.JTextField jTextField1;
	// End of variables declaration

}
