/*
  EdiNoS - Editeur de NoS
  Copyright (C) 2013-2019 - Luc Poittevin — Tous droits réservés.
  Contact : edinos@free.fr  -  https://www.edinos.org
  
  Ce programme est un logiciel libre ; vous pouvez le redistribuer ou le
  modifier suivant les termes de la “GNU General Public License” telle que
  publiée par la Free Software Foundation : soit la version 3 de cette
  licence, soit (à votre gré) toute version ultérieure.
  
  Ce programme est distribué dans l’espoir qu’il vous sera utile, mais SANS
  AUCUNE GARANTIE : sans même la garantie implicite de COMMERCIALISABILITÉ
  ni d’ADÉQUATION À UN OBJECTIF PARTICULIER. Consultez la Licence Générale
  Publique GNU pour plus de détails.
  
  Vous devriez avoir reçu une copie de la Licence Générale Publique GNU avec
  ce programme ; si ce n’est pas le cas, consultez :
  <http://www.gnu.org/licenses/>. 
 */

package nos.edit;

import java.awt.BorderLayout;
import java.awt.Container;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;

import javax.swing.BoxLayout;
import javax.swing.ButtonGroup;
import javax.swing.JCheckBox;
import javax.swing.JDialog;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JRadioButton;
import javax.swing.JTextField;

import nos.obj.Edi;

@SuppressWarnings("serial")
public class FenCreeAction extends JDialog implements PropertyChangeListener,
		ActionListener {

	private String choix = null;

	private JOptionPane optionPane;

	private String btnOK = "OK";

	private String btnAnnuler = "Annuler";

	private JTextField nomAction = new JTextField(20);

	private ButtonGroup groupeBoutons;

	private JRadioButton boutonProc = new JRadioButton("Procédure", false);

	private JRadioButton boutonQuest = new JRadioButton("Question", false);

	private JRadioButton boutonMess = new JRadioButton("Message", false);

	private FenActions fen = null;

	private boolean proposeWizard = true;

	private JCheckBox checkBox = new JCheckBox("Utiliser le wizard");

	public FenCreeAction(FenActions f, boolean p) {
		super();
		fen = f;
		proposeWizard = p;
		setModalityType(DEFAULT_MODALITY_TYPE);
		Object[] options = { btnOK, btnAnnuler };
		this.setTitle("Création d'une action");

		JPanel panel1 = new JPanel();
		JLabel lab = new JLabel("Entrez le nom de l'action :");
		panel1.add(lab);
		panel1.add(nomAction);

		JPanel panel2 = new JPanel();
		groupeBoutons = new ButtonGroup();
		boutonProc.setActionCommand("Procédure");
		boutonProc.addActionListener(this);
		groupeBoutons.add(boutonProc);
		panel2.add(boutonProc);
		boutonQuest.setActionCommand("Question");
		boutonQuest.addActionListener(this);
		groupeBoutons.add(boutonQuest);
		panel2.add(boutonQuest);
		boutonMess.setActionCommand("Message");
		boutonMess.addActionListener(this);
		groupeBoutons.add(boutonMess);
		panel2.add(boutonMess);

		checkBox.setActionCommand("cocheCheckBox");
		checkBox.addActionListener(this);
		checkBox.setSelected(false);

		if (!proposeWizard) {
			checkBox.setEnabled(false);
		}


		JPanel panel4 = new JPanel();
		panel4.setLayout(new BoxLayout(panel4, BoxLayout.Y_AXIS));
		panel4.add(panel1);
		panel4.add(panel2);

		getContentPane().add(panel4, BorderLayout.NORTH);

		JPanel panel5 = new JPanel();
		panel5.setLayout(new BorderLayout());
		// Create the JOptionPane.
		optionPane = new JOptionPane(null,
		// JOptionPane.QUESTION_MESSAGE,
				JOptionPane.PLAIN_MESSAGE,
				JOptionPane.OK_CANCEL_OPTION, null, options, options[0]);

		optionPane.addPropertyChangeListener("value", this);

		panel5.add(optionPane, BorderLayout.CENTER);
		getContentPane().add(panel5, BorderLayout.SOUTH);
		
		if (((Container) optionPane.getComponent(1)).getComponentCount() > 0) {
			// Sous Windows, la structure de l'optionPane est différente,
			// mais on fait tourner FEST  sous Mac
			((Container) optionPane.getComponent(1)).getComponent(0).setName(
					"Annuler"); // pour FEST
			((Container) optionPane.getComponent(1)).getComponent(0).setName(
					"OK"); // pour FEST
		}

		lab.setName("label"); // pour FEST
		nomAction.setName("nomAction"); // pour FEST
		boutonProc.setName("Procédure"); // pour FEST
		boutonQuest.setName("Question"); // pour FEST
		boutonMess.setName("Message"); // pour FEST
	}

	/** This method reacts to state changes in the option pane. */
	public void propertyChange(PropertyChangeEvent e) {
		String prop = e.getPropertyName();
		if ((e.getSource() == optionPane)
				&& (JOptionPane.VALUE_PROPERTY.equals(prop) || JOptionPane.INPUT_VALUE_PROPERTY
						.equals(prop))) {
			Object value = optionPane.getValue();

			if (value == JOptionPane.UNINITIALIZED_VALUE) {
				// ignore reset
				return;
			}

			if (btnOK.equals(value)) {
				if (groupeBoutons.getSelection() == null) {
					JOptionPane
							.showMessageDialog(
									this,
									"Vous devez indiquer si l'action est une procédure, une question ou un message...");

					// Reset the JOptionPane's value.
					// If you don't do this, then if the user
					// presses the same button next time, no
					// property change event will be fired :
					optionPane.setValue(JOptionPane.UNINITIALIZED_VALUE);

				} else if (nomAction.getText().isEmpty()) {
					// Ne pas utiliser Edi.nos().message() car FenCreeAction
					// est déjà une JOptionPane de Edi.nos()
					JOptionPane.showMessageDialog(this,
							"Vous devez saisir un nom...");
					optionPane.setValue(JOptionPane.UNINITIALIZED_VALUE);
				} else {
					choix = groupeBoutons.getSelection().getActionCommand();
					if ((Edi.nos().getBaseCourante().contientProc(nomAction
							.getText()))
							|| (Edi.nos().getBaseCourante()
									.contientQuestion(nomAction.getText()))
							|| (Edi.nos().getBaseCourante()
									.contientMessage(nomAction.getText()))) {
						// Ne pas utiliser Edi.nos().message() car FenCreeAction
						// est déjà une JOptionPane de Edi.nos()
						JOptionPane.showMessageDialog(this, "L'action <"
								+ nomAction.getText() + "> existe déjà...");
						optionPane.setValue(JOptionPane.UNINITIALIZED_VALUE);
					} else {
						dispose();
						fen.creeAction(choix, nomAction.getText(), checkBox
								.getModel().isSelected());
					}
				}
			} else { // user closed dialog or clicked cancel
				choix = null;
				dispose();
			}
		}
	}

	public void actionPerformed(ActionEvent e) {
		String s = e.getActionCommand().toString();

		if (s.equals("Procédure")) {
			if (proposeWizard) {
				checkBox.setSelected(true);
			}
		} else if (s.equals("Question")) {
			checkBox.setSelected(false);
		} else if (s.equals("Message")) {
			checkBox.setSelected(false);
		}
	}
}
