/*
  EdiNoS - Editeur de NoS
  
  Copyright (C) 2013-2021 - Luc Poittevin — Tous droits réservés.
  Contact : edinos@free.fr  -  https://www.edinos.org
  
  Ce programme est un logiciel libre ; vous pouvez le redistribuer ou le
  modifier suivant les termes de la “GNU General Public License” telle que
  publiée par la Free Software Foundation : soit la version 3 de cette
  licence, soit (à votre gré) toute version ultérieure.
  
  Ce programme est distribué dans l’espoir qu’il vous sera utile, mais SANS
  AUCUNE GARANTIE : sans même la garantie implicite de COMMERCIALISABILITÉ
  ni d’ADÉQUATION À UN OBJECTIF PARTICULIER. Consultez la Licence Générale
  Publique GNU pour plus de détails.
  
  Vous devriez avoir reçu une copie de la Licence Générale Publique GNU avec
  ce programme ; si ce n’est pas le cas, consultez :
  <http://www.gnu.org/licenses/>. 
 */

package nos.edit;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Cursor;
import java.awt.Dimension;
import java.awt.Point;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.beans.PropertyVetoException;
import java.io.BufferedReader;
import java.io.File;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.IOException;
import java.io.PrintWriter;

import javax.swing.BorderFactory;
import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.DefaultComboBoxModel;
import javax.swing.DefaultListModel;
import javax.swing.ImageIcon;
import javax.swing.JButton;
import javax.swing.JComboBox;
import javax.swing.JDesktopPane;
import javax.swing.JDialog;
import javax.swing.JFileChooser;
import javax.swing.JFrame;
import javax.swing.JInternalFrame;
import javax.swing.JLabel;
import javax.swing.JList;
import javax.swing.JMenu;
import javax.swing.JMenuBar;
import javax.swing.JMenuItem;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JSplitPane;
import javax.swing.JTextArea;
import javax.swing.JOptionPane;
import javax.swing.KeyStroke;
//import javax.swing.SwingUtilities;
import javax.swing.UIManager;
import javax.swing.filechooser.FileNameExtensionFilter;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import nos.obj.*;
import nos.revinos.Revi;
import nos.revinos.Sessions;
import nos.run.FenVoirSituation;
import nos.run.Memoire;
import nos.run.Moteur;
import nos.run.Moteur.Contexte;
import nos.run.Moteur.ModeExecutionProc;
import nos.run.edit.FenLancement;
import nos.utils.*;

@SuppressWarnings("serial")
public class FenEdiNoS extends JFrame implements ActionListener {

	static private final String newline = "\n";


	private JButton boutonOuvrir = new JButton("Ouvrir");

	private JButton boutonLancer = new JButton("Lancer");

	private JButton boutonInterface = new JButton("Actions");

	private JButton boutonTrace = new JButton("Test");



	private JComboBox<String> combo = new JComboBox<String>();

	private JDesktopPane desktop = null;

	private JList<String> listeGraphes = new JList<String>();

	private JList<String> listeNoS = new JList<String>();

	private DefaultListModel<String> listeModelGraphes = new DefaultListModel<String>();

	private DefaultListModel<String> listeModelNoS = new DefaultListModel<String>();

	private String typeConversion = "";

	private JTextArea zoneTexte = new JTextArea(5, 45);

	private Point pos;

	private final Logger log;

	private File file;

	private String chaine = "";

	public FenEdiNoS() {
		super("EdiNoS");

		initPos();

		log = LoggerFactory.getLogger(this.getClass());

		log.info("{} sous {}", UIManager.getSystemLookAndFeelClassName(), UIManager.getLookAndFeel().getName());
		log.info("texte : {}", zoneTexte.getFont().toString());
		log.info("liste : {}", listeNoS.getFont().toString());

		desktop = new JDesktopPane();


		getContentPane().setLayout(new BorderLayout()); 
		JPanel panelHaut = new JPanel();
		JPanel panelListes = new JPanel();
		JPanel panelGraphes = new JPanel();
		JPanel panelNos = new JPanel();
		JPanel panelImage = new JPanel();

		// Put the JComboBox in a JPanel to get a nicer look.
		JPanel comboBoxPane = new JPanel(); 
		String comboBoxItems[] = {};
		DefaultComboBoxModel<String> cbm = new DefaultComboBoxModel<String>(comboBoxItems);
		combo.setModel(cbm);
		combo.setEditable(false);
		combo.addActionListener(this);
		combo.setName("Bases"); // pour FEST
		comboBoxPane.add(combo);

		JLabel text = new JLabel(" Base courante :");
		text.setOpaque(false);

		boutonOuvrir.setName("Ouvrir"); // pour FEST
		boutonLancer.setName("Lancer"); // pour FEST
		boutonInterface.setName("Interface"); // pour FEST
		boutonTrace.setName("Trace"); // pour FEST
		boutonOuvrir.setToolTipText("Ouvrir une base");
		boutonLancer.setToolTipText("Lancer le graphe sélectionné");
		boutonInterface.setToolTipText("Voir les actions de la base courante");
		boutonTrace.setToolTipText("Ouvrir une trace");
		panelHaut.setLayout(new BoxLayout(panelHaut, BoxLayout.LINE_AXIS));
		panelHaut.add(text);
		panelHaut.add(combo);
		panelHaut.add(boutonOuvrir);
		panelHaut.add(boutonLancer);
		panelHaut.add(boutonInterface);
		panelHaut.add(Box.createHorizontalGlue());
		panelHaut.setBorder(BorderFactory.createLineBorder(Color.white));
		getContentPane().add(panelHaut, BorderLayout.NORTH);



		listeGraphes.setName("Graphes"); // pour FEST
		listeNoS.setName("NoS"); // pour FEST
		JScrollPane listPaneGraphes = new JScrollPane(listeGraphes);
		JScrollPane listPaneNoS = new JScrollPane(listeNoS);

		listPaneGraphes.setPreferredSize(new Dimension(200, 270));
		listPaneNoS.setPreferredSize(new Dimension(200, 270));

		JLabel text1 = new JLabel("Graphes :", JLabel.CENTER);
		text1.setOpaque(false);
		text1.setPreferredSize(new Dimension(200, 30));
		panelGraphes.setLayout(new BorderLayout());
		panelGraphes.add(text1, BorderLayout.NORTH);
		panelGraphes.add(listPaneGraphes, BorderLayout.CENTER);

		JLabel text2 = new JLabel("NoS :", JLabel.CENTER);
		text2.setOpaque(false);
		text2.setPreferredSize(new Dimension(200, 30));
		panelNos.setLayout(new BorderLayout());
		panelNos.add(text2, BorderLayout.NORTH);
		panelNos.add(listPaneNoS, BorderLayout.CENTER);

		panelListes.setLayout(new BoxLayout(panelListes, BoxLayout.Y_AXIS));

		panelListes.setBorder(BorderFactory.createLineBorder(Color.black));

		log.info("Desktop UI = {}", desktop.getUI().toString());
		log.info("couleur JdeskTopPane RGB: {}", desktop.getBackground().toString());
		if (desktop.getBackground().equals(Color.BLACK)) {
			desktop.setBackground(Color.getHSBColor((float) 0.6031746, (float) 0.61764705, (float) 0.6666667));
		}

		JScrollPane scrollPane = new JScrollPane(zoneTexte);
		scrollPane.setPreferredSize(new Dimension(1130, 150)); 

		java.net.URL imgURL = getClass().getResource("/images/newLogoChien190.png");
		ImageIcon image = null;
		if (imgURL != null) {
			image = new ImageIcon(imgURL);
		} else {
			montre("Logo non trouvé");
		}

		JLabel label3 = new JLabel(image);
		panelImage.add(label3);
		panelImage.setMaximumSize(new Dimension(150, 150));
		panelImage.addMouseListener(new MouseAdapter() {
			public void mouseClicked(MouseEvent e) {
				FenInfos f = new FenInfos();
				f.setSize(500, 540);
				ouvreInternalFrame(f);
			}
		});

		JSplitPane splitPaneH = new JSplitPane(JSplitPane.VERTICAL_SPLIT, desktop, scrollPane);
		splitPaneH.setResizeWeight(1);

		panelListes.add(panelGraphes);
		panelListes.add(panelNos);
		panelListes.add(panelImage);

		JSplitPane splitPaneV = new JSplitPane(JSplitPane.HORIZONTAL_SPLIT, panelListes, splitPaneH);

		getContentPane().add(splitPaneV, BorderLayout.CENTER);

		// installation des menus
		installeMenus();

		// Register the Listeners
		registerEventListeners();

		// Resize the container to the prefered size
		pack();
		
	}

	public void actionPerformed(ActionEvent e) {

		String s = e.getActionCommand().toString();


		if (s.equals("comboBoxChanged")) {
			JComboBox<?> cb = (JComboBox<?>) e.getSource();
			String c = (String) cb.getSelectedItem();
			choisitBase(c);
			return;
		}
	}

	public JInternalFrame activeFen(String titre) {
		// active et renvoie la fenêtre si celle-ci est déjà ouverte
		// renvoie null si aucune fenêtre de ce titre n'est déjà ouverte
		
		JInternalFrame[] iframes = desktop.getAllFrames();
		for (JInternalFrame iframe : iframes) {
			if (iframe.getTitle().equals(titre)) {
				// iframe.toFront();
				try {
					iframe.setSelected(true);
				} catch (PropertyVetoException e) {
				}
				return iframe;
			}
		}
		return null;
	}

	public void afficheBase() {
		listeModelGraphes.clear();
		listeModelNoS.clear();

		if (Edi.nos().getBaseCourante() != null) {
			for (String n : Edi.nos().getBaseCourante().getListeGraphes()) {
				listeModelGraphes.addElement(n);
			}
		}

		if (Edi.nos().getBaseCourante() != null) {
			for (String n : Edi.nos().getBaseCourante().getListeNoS()) {
				listeModelNoS.addElement(n);
			}
		}

		listeGraphes.setModel(listeModelGraphes);
		listeNoS.setModel(listeModelNoS);

		// le premier graphe est le graphe courant
		listeGraphes.setSelectedIndex(0);

		if (Edi.nos().getBaseCourante() != null) {
			log.info("Affiche Base {}", Edi.nos().getBaseCourante().getNom());
		} else {
			log.info("Affiche Base vide");
		}
	}

	public void ajouteBase(String nom) {
		// on désactive la fenêtre courante car c'est le menu de la nouvelle
		// base qui va être installé
		deSelectFenCourante();

		DefaultComboBoxModel<String> model = (DefaultComboBoxModel<String>) combo.getModel();
		model.addElement(nom);
		model.setSelectedItem(nom);

		// réinitialisation de la position des fenêtres
		initPos();

		setMenuBar();
	}

	private void chargeBase(String repTravail, String nom) {
		String uri = repTravail + "/" + nom + "/" + nom + ".int.xml";
		File file = new File(uri).getAbsoluteFile();
		if (!file.exists()) {
			montre("Base <" + nom + "> non trouvée");
			return;
		}
		Edi.nos().creeBase(file);
	}

	public void chargeBases(String repTravail) {
		setCursor(Cursor.getPredefinedCursor(Cursor.WAIT_CURSOR));
		chargeBase(repTravail, "REVINOS");
		chargeBase(repTravail, "COMMUN");
		setCursor(Cursor.getPredefinedCursor(Cursor.DEFAULT_CURSOR));
	}

	public void chargeTrace(File file) {
		Moteur m = ToolNos.chargeMoteur(file);
		if (m != null) {
			Edi.nos().montreInfo("Ouverture de : " + file.getName() + "." + "\n");

			FenVoirSituation f = new FenVoirSituation(m);
			f.setLocation(2, 2);
			m.getNoS().getBase().ajouteFen(f);
			((FenEdiNoS) Edi.nos().fen()).ouvreInternalFrameSansPositionner(f);
			f.setVisible(true);

			Revi.nos().ouvreFenSessions(m, Sessions.Statut.NonDémarrée);
		}
	}

	public void checkXML(String fic) {
		try {
			zoneTexte.append(fic + newline);
			ConstruitDom h = new ConstruitDom(fic);
			h.document();

		} catch (Exception t) {
			t.printStackTrace();
			zoneTexte.append(t.toString() + newline);
			return;
		}
		zoneTexte.append("fichier vérifi" + Accent.eAigu() + newline);
	}

	private void choisitBase(String ch) {
		Edi.nos().setBaseCourante(Edi.nos().getBase(ch));
		deSelectFenCourante();
		afficheBase();
	}

	@SuppressWarnings("unused")
	private void convertit() {
		zoneTexte.setText("");
		boolean fichierOuvert = ouvreFichier();
		if (fichierOuvert == true) {
			suiteConversion();
		}
	}

	private void deSelectFenCourante() {
		JInternalFrame f = desktop.getSelectedFrame();
		if (f != null) {
			try {
				f.setSelected(false);
			} catch (PropertyVetoException e) {
			}
		}
	}

	private void fermeBase() {
		Edi.nos().getBaseCourante().fermeFens();
		String n = Edi.nos().getBaseCourante().getNom();
		Edi.nos().retireBaseCourante();
		retireBase(n);
		montre("Base <" + n + "> fermée");

	}

	public void checkDerniereFen() {
		if (desktop.getAllFrames().length == 0) {
			((FenEdiNoS) Edi.nos().fen()).setMenuBar();
		}
	}
	
	public void fermeFen(String titre) {
		JInternalFrame[] iframes = desktop.getAllFrames();
		for (JInternalFrame iframe : iframes) {
			if (iframe.getTitle().contains(titre)) {
				if (ToolNos.nomSysteme().equals("Mac")) {
					ToolNos.fermeFen(iframe);
				} else {
					iframe.dispose();
				}
			}
		}
	}

	private void finConversion() {
		String nomFic = "";
		String type = "";

		ParserNod parser = null;
		try {

			type = file.getName();
			type = type.substring(type.length() - 8, type.length() - 5).toLowerCase();

			if (type.equals("nod")) {
				parser = new ParserNod(file.getAbsolutePath());
			} else if (type.equals("int")) {
				parser = new ParserInt(file.getAbsolutePath());
			}

			PrintWriter p = null;
			try {
				nomFic = file.getAbsolutePath();
				nomFic = nomFic.replaceAll(".rev2", ".xml");
				FileWriter fos = new FileWriter(nomFic);

				p = new PrintWriter(fos, true);
				p.print(parser.resultat());
			} catch (IOException e) {
				zoneTexte.append("Erreur sauve fichier" + newline);
			} finally {
				p.close();

			}
		} catch (Exception t) {
			zoneTexte.append(t.toString());
			t.printStackTrace();
			return;
		}

		zoneTexte.append("Fichier <.xml> généré" + newline);
		checkXML(nomFic);
	}

	public void initPos() {
		pos = new Point(10, 20);
	}

	private void installeMenus() {

		JMenuBar menuBar = menuBarEdinos();
		if (Edi.nos().getBaseCourante() != null) {
			menuBar.add(menuBase());
		}
		this.setJMenuBar(menuBar);
	}

	private void lanceGraphe(boolean avecFenLancement) {


		String nomGraphe = (String) listeGraphes.getSelectedValue();
		if (nomGraphe == null) {
			if (listeGraphes.getModel().getSize() == 0) {
				JOptionPane.showMessageDialog(this, "Il n'y a aucun graphe dans cette base...");
				return;
			} else if (listeGraphes.getModel().getSize() > 1) {
				JOptionPane.showMessageDialog(this, "Veuillez d'abord sélectionner un graphe...");
				return;
			} else {
				nomGraphe = (String) listeGraphes.getModel().getElementAt(0);
			}
		}

		NoS n = (NoS) Edi.nos().getBaseCourante().getAssocNos().get(nomGraphe);
		if (n == null) {
			JOptionPane.showMessageDialog(this, "nos <" + nomGraphe + "> introuvable ...");
		} else {
			if (avecFenLancement) {
				FenLancement f = new FenLancement(n);
				f.setSize(550, 350);
				// f.setLocationRelativeTo(this);
				Edi.nos().getBaseCourante().ajouteFen(f);
				((FenEdiNoS) Edi.nos().fen()).ouvreInternalFrameSansPositionner(f);
				f.setVisible(true);
			} else {
				Moteur mot = new Moteur(n, Contexte.Lancement, ModeExecutionProc.Mixte);
				Memoire mem = new Memoire();
				mot.declencheAvec(mem);
			}
		}
	}

	private void lectureFichier() {
		try {
			BufferedReader in = new BufferedReader(new FileReader(file));

			String s = new String();
			chaine = "";

			while ((s = in.readLine()) != null)
				chaine += s + "\n";
			 
			if (chaine.startsWith("Graphe:")) {
				typeConversion = "nod";
			} else if (chaine.startsWith("Interface:")) {
				typeConversion = "int";
			}
			in.close();
		} catch (IOException e) {
			montre("Erreur lors de l'ouverture du fichier");
		}
	}

	public JMenuBar menuBarEdinos() {
		JMenu menu;
		JMenuItem menuItem;
		JMenuBar menuBar;

		menuBar = new JMenuBar();
		menu = new JMenu("Fichier");

		menuItem = new JMenuItem("Nouvelle Base");
		menuItem.setAccelerator(KeyStroke.getKeyStroke(java.awt.event.KeyEvent.VK_B, ToolNos.renvoieModifiers()));
		menuItem.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				saisieNouvelleBase();
			}
		});
		menu.add(menuItem);
		menuItem = new JMenuItem("Ouvrir une base");
		menuItem.setAccelerator(KeyStroke.getKeyStroke(java.awt.event.KeyEvent.VK_O, ToolNos.renvoieModifiers()));
		menuItem.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				ouvreBase();
			}
		});
		menuItem.setName("OuvrirBase"); // pour FEST
		menu.add(menuItem);
		menuItem = new JMenuItem("Fermer la base courante");
		menuItem.setAccelerator(KeyStroke.getKeyStroke(java.awt.event.KeyEvent.VK_F, ToolNos.renvoieModifiers()));
		menuItem.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				fermeBase();
			}
		});
		menuItem.setEnabled((Edi.nos().getBaseCourante() != null));
		menuItem.setName("FermerBase"); // pour FEST
		menu.add(menuItem);

		menu.addSeparator();
		menuItem = new JMenuItem("Sauvegarder la base courante");
		menuItem.setAccelerator(KeyStroke.getKeyStroke(java.awt.event.KeyEvent.VK_S, ToolNos.renvoieModifiers()));
		menuItem.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				sauveBase();
			}
		});
		menuItem.setEnabled((Edi.nos().getBaseCourante() != null));
		menu.add(menuItem);
		menuItem = new JMenuItem("Sauvegarder sous ...");
		menuItem.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				sauveBaseSous();
			}
		});
		menu.add(menuItem);


		menu.addSeparator();
		menuItem = new JMenuItem("Préférences");
		menuItem.setAccelerator(KeyStroke.getKeyStroke(java.awt.event.KeyEvent.VK_P, ToolNos.renvoieModifiers()));
		menuItem.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				ouvrePreferences();
			}
		});
		menu.add(menuItem);

		if (!ToolNos.nomSysteme().equals("Mac")) {
			menu.addSeparator();
			menuItem = new JMenuItem("Quitter");
			menuItem.setName("Quitter"); // pour FEST
			menuItem.setAccelerator(KeyStroke.getKeyStroke(java.awt.event.KeyEvent.VK_Q, ToolNos.renvoieModifiers()));
			menuItem.addActionListener(new ActionListener() {
				public void actionPerformed(ActionEvent e) {
					quitter();
				}
			});
			menu.add(menuItem);
		}
		;

		menuBar.add(menu);
		return (menuBar);
	}

	public JMenu menuBase() {

		JMenuItem menuItem;
		JMenu menu = new JMenu("Base");

		menuItem = new JMenuItem("Nouveau Graphe");
		menuItem.setAccelerator(KeyStroke.getKeyStroke(java.awt.event.KeyEvent.VK_G, ToolNos.renvoieModifiers()));
		menuItem.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				saisieNouveauGraphe();
			}
		});
		menuItem.setEnabled((Edi.nos().getBaseCourante() != null));
		menu.add(menuItem);
		menuItem = new JMenuItem("Renommer un graphe");
		menuItem.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				saisieNouveauNomGraphe();
			}
		});
		menuItem.setEnabled((Edi.nos().getBaseCourante() != null));
		menu.add(menuItem);
		menuItem = new JMenuItem("Supprimer un graphe");
		menuItem.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				supprimeGraphe();
			}
		});
		menuItem.setEnabled((Edi.nos().getBaseCourante() != null));
		menu.add(menuItem);

		menu.addSeparator();
		menuItem = new JMenuItem("Voir un graphe");
		menuItem.setName("VoirGraphe"); // pour FEST
		menuItem.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				ouvreGraphe();
			}
		});
		menuItem.setEnabled((Edi.nos().getBaseCourante() != null));
		menu.add(menuItem);
		menuItem = new JMenuItem("Lancer un graphe");
		menuItem.setAccelerator(KeyStroke.getKeyStroke(java.awt.event.KeyEvent.VK_L, ToolNos.renvoieModifiers()));
		menuItem.setName("LancerGraphe"); // pour FEST
		menuItem.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				lanceGraphe(false);
			}
		});
		menuItem.setEnabled((Edi.nos().getBaseCourante() != null));
		menu.add(menuItem);
		menuItem = new JMenuItem("ParaMètres du lancement");
		menuItem.setAccelerator(KeyStroke.getKeyStroke(java.awt.event.KeyEvent.VK_M, ToolNos.renvoieModifiers()));
		menuItem.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				lanceGraphe(true);
			}
		});
		menuItem.setEnabled((Edi.nos().getBaseCourante() != null));
		menu.add(menuItem);
		menuItem = new JMenuItem("Ouvrir une trace");
		menuItem.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				ouvreTrace();
			}
		});
		menuItem.setEnabled((Edi.nos().getBaseCourante() != null));
		menuItem.setName("OuvrirTrace"); // pour FEST
		menu.add(menuItem);

		menu.addSeparator();
		menuItem = new JMenuItem("ActIons");
		menuItem.setAccelerator(KeyStroke.getKeyStroke(java.awt.event.KeyEvent.VK_A, ToolNos.renvoieModifiers()));
		menuItem.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				ouvreInterface();
				return;
			}
		});
		menuItem.setEnabled(Edi.nos().getBaseCourante() != null);
		menuItem.setName("Actions"); // pour FEST
		menu.add(menuItem);

		return (menu);
	}

	public void montre(String s) {
		if (zoneTexte.getDocument().getLength() == 0) {
			zoneTexte.append(s);
		} else {
			zoneTexte.append(newline + s);
		}
		zoneTexte.setCaretPosition(zoneTexte.getDocument().getLength());
	}

	private void ouvreBase() {
		int returnVal;
		File dir = null;

		file = null;

		System.out.println("Rép travail = " + Edi.nos().getRepertoireTravail());

		dir = new File(Edi.nos().getRepertoireTravail());

		JFileChooser fc = new JFileChooser(dir);

		FiltreNoS filter = new FiltreNoS();
		fc.setFileFilter(filter);

		returnVal = fc.showOpenDialog(this);

		if (returnVal == JFileChooser.APPROVE_OPTION) {
			file = fc.getSelectedFile();
		} else {
			montre("Aucun fichier choisi...");
			return;
		}

		Edi.nos().creeBase(file);
		// ajout 10/03/21 :   //LPO  
		// On ouvre la fenêtre Actions après l'ouverture d'une base :
		FenActions f = new FenActions(Edi.nos().getBaseCourante());
		f.setSize(690, 450); 
		Edi.nos().getBaseCourante().ajouteFen(f);
		((FenEdiNoS) Edi.nos().fen()).ouvreInternalFrame(f);
	}

	private boolean ouvreFichier() {
		File dir = null;
		file = null;
		dir = new File(Edi.nos().getRepertoireTravail());

		JFileChooser fc = new JFileChooser(dir);

		FiltrePourConversion filter = new FiltrePourConversion();

		fc.setFileFilter(filter);

		int returnVal = fc.showOpenDialog(this);

		if (returnVal == JFileChooser.APPROVE_OPTION) {
			file = fc.getSelectedFile();
		} else {
			montre("Aucun fichier choisi...");
			return false;
		}

		lectureFichier();
		montre("Ouverture de : " + file.getName() + ".");
		return true;
	}

	private void ouvreInterface() {
		FenActions f = new FenActions(Edi.nos().getBaseCourante());
		f.setSize(690, 450); 
		Edi.nos().getBaseCourante().ajouteFen(f);
		ouvreInternalFrame(f);
	}

	public void ouvreInternalFrame(JInternalFrame f) {
		f.setVisible(true);

		reinitPos();
		f.setLocation(pos);
		desktop.add(f);
		try {
			f.setSelected(true);
		} catch (java.beans.PropertyVetoException ex) {
		}
	}

	public void ouvreInternalFrameSansPositionner(JInternalFrame f) {
		desktop.add(f);
		try {
			f.setSelected(true);
		} catch (java.beans.PropertyVetoException ex) {
		}
	}

	private void ouvrePreferences() {
		FenPrefs f = new FenPrefs(this);
		f.setSize(540, 350);
		ouvreInternalFrame(f);
	}

	public void ouvreTrace() {
		File file = null;
		JFileChooser fc = new JFileChooser();

		FileNameExtensionFilter xmlfilter = new FileNameExtensionFilter("xml files (*.xml)", "xml");
		fc.setFileFilter(xmlfilter);
		fc.setDialogTitle("Choisissez le fichier trace à ouvrir");

		int returnVal = fc.showOpenDialog(Edi.nos().fen());

		if (returnVal == JFileChooser.APPROVE_OPTION) {
			file = fc.getSelectedFile();
		} else {
			Edi.nos().montreInfo("Aucun fichier choisi..." + "\n");
			return;
		}

		chargeTrace(file);
	}

	public void quitter() {
		int rep = JOptionPane.showConfirmDialog(Edi.nos().fen(), "Voulez-vous quitter EdiNoS ?", "Quitter EdiNoS",
				JOptionPane.YES_NO_OPTION);
		if (rep == 0) {
			// confirmation
			System.exit(0);
		}
	}

	public void refreshListeNoS() {
		listeModelNoS.clear();
		for (String n : Edi.nos().getBaseCourante().getListeNoS()) {
			listeModelNoS.addElement(n);
		}
	}

	private void registerEventListeners() {
		boutonOuvrir.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent evt) {
				ouvreBase();
			}
		});

		boutonLancer.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent evt) {
				lanceGraphe(false);
			}
		});

		boutonInterface.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent evt) {
				ouvreInterface();
			}
		});

		// bouton invisible :
		boutonTrace.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent evt) {

				FenActions f = new FenActions(Edi.nos().getBaseCourante());
				Edi.nos().getBaseCourante().ajouteFen(f);
				ouvreInternalFrame(f);
			}
		});

		listeGraphes.addMouseListener(new MouseAdapter() {
			public void mouseClicked(MouseEvent e) {
				if (e.getClickCount() == 2) {
					ouvreGraphe();
				}
			}
		});

		listeNoS.addMouseListener(new MouseAdapter() {
			public void mouseClicked(MouseEvent e) {
				if (e.getClickCount() == 2) {
					FenNos f = new FenNos(
							(NoS) Edi.nos().getBaseCourante().getAssocNos().get(listeNoS.getSelectedValue()));
					Edi.nos().getBaseCourante().ajouteFen(f);
					ouvreInternalFrame(f);
				}
			}
		});
	}

	private void reinitPos() {
		Point locationMax = new Point(0, 0);
		JInternalFrame[] iframes = desktop.getAllFrames();
		for (JInternalFrame iframe : iframes) {
			Point p = iframe.getLocation();
			System.out.println();
			if ((locationMax.getX() < iframe.getX()) && (locationMax.getY() < iframe.getY())) {
				if ((!iframe.getTitle().startsWith("Palette")) && (!iframe.getTitle().equals("ReviNoS"))
						&& (!iframe.getTitle().equals("Sessions de révision"))) {
					locationMax = p;
				}
			}
		}
		;
		pos = locationMax;
		pos.translate(20, 20);
	}

	public void renommeGraphe(String nouveauNom) {
		String ancienNom = (String) listeGraphes.getSelectedValue();
		NoS n = (NoS) Edi.nos().getBaseCourante().getAssocNos().get(ancienNom);
		Edi.nos().getBaseCourante().renommeGraphe(n, nouveauNom);
	}

	public void replace() {

		chaine = chaine.replace('<', '"');
		chaine = chaine.replace('>', '"');
		chaine = chaine.replace('\\', '/');
		if (typeConversion.equals("nod")) {

			chaine = chaine.replaceAll("Graphe: ", "<?xml version='1.0' encoding='iso-8859-1' standalone='yes'?>"
					+ System.getProperty("line.separator") + "<graphe>");
			chaine = chaine.replaceAll("/finGraphe",
					"</suivantsVue>" + System.getProperty("line.separator") + "</graphe>");
			chaine = chaine.replaceAll("/nod: ", "<nos>");
			chaine = chaine.replaceAll("/finNod", "</choix>" + "</nos>");
			chaine = chaine.replaceAll("/actions: ", "<actions>");
			chaine = chaine.replaceAll("/choix: ", "</actions>" + System.getProperty("line.separator") + "<choix>");
			chaine = chaine.replaceAll("/commentaires: ",
					"</choix>" + System.getProperty("line.separator") + "<commentaires>");
			chaine = chaine.replaceAll("/suivantsVue: ", "</suivantsVue>\n<suivantsVue>");

		} else if (typeConversion.equals("int")) {

			chaine = chaine.replaceAll("Interface: ", "<?xml version='1.0' encoding='iso-8859-1' standalone='yes'?>"
					+ System.getProperty("line.separator") + "<interface>");
			chaine = chaine.replaceAll("/finInterface", "</description>" + System.getProperty("line.separator")
					+ "</procedure>" + System.getProperty("line.separator") + "</interface>");
			chaine = chaine.replaceAll("/procedure: ", "</description>" + System.getProperty("line.separator")
					+ "</procedure>" + System.getProperty("line.separator") + "<procedure>");
			chaine = chaine.replaceAll("/commentaires: ", "<description>");

			chaine = chaine.replaceAll("/ecran: ", "</nomBasEcran>" + System.getProperty("line.separator") + "</ecran>"
					+ System.getProperty("line.separator") + "<ecran>");
			chaine = chaine.replaceAll("/texte: ", "</description>" + System.getProperty("line.separator") + "<texte>");
			chaine = chaine.replaceAll("/regles: ", "</texte>" + System.getProperty("line.separator") + "<regles>");
			chaine = chaine.replaceAll("/titre: ", "</regles>" + System.getProperty("line.separator") + "<titre>");
			chaine = chaine.replaceAll("/nomBasEcran: ",
					"</titre>" + System.getProperty("line.separator") + "<nomBasEcran>");

			chaine = chaine.replaceAll("/basEcran: ", "</zone>" + System.getProperty("line.separator") + "</basEcran>"
					+ System.getProperty("line.separator") + "<basEcran>");
			chaine = chaine.replaceAll("/zone: ", "<zone>");
		}
		
	}

	public void retireBase(String nom) {

		DefaultComboBoxModel<String> model = (DefaultComboBoxModel<String>) combo.getModel();
		model.removeElement(nom);

		setMenuBar();
	}

	private void saisieNouveauGraphe() {
		JFrame.setDefaultLookAndFeelDecorated(true);
		JDialog.setDefaultLookAndFeelDecorated(true);

		FenCreeGraphe customDialog = new FenCreeGraphe(Edi.nos().getBaseCourante());
		customDialog.pack();

		customDialog.setLocationRelativeTo(this);
		customDialog.setVisible(true);
	}

	protected void saisieNouveauNomGraphe() {
		String nomGraphe = (String) listeGraphes.getSelectedValue();
		if (nomGraphe == null) {
			JOptionPane.showMessageDialog(this, "Veuillez d'abord sélectionner un graphe...");
			return;
		}

		NoS n = (NoS) Edi.nos().getBaseCourante().getAssocNos().get(nomGraphe);
		if (n == null) {
			JOptionPane.showMessageDialog(this, "nos <" + nomGraphe + "> introuvable ...");
		} else {
			JFrame.setDefaultLookAndFeelDecorated(true);
			JDialog.setDefaultLookAndFeelDecorated(true);

			FenRenommeGraphe customDialog = new FenRenommeGraphe(n.getNom());
			customDialog.pack();

			customDialog.setLocationRelativeTo(this);
			customDialog.setVisible(true);
		}
	}

	protected void saisieNouvelleBase() {
		JFrame.setDefaultLookAndFeelDecorated(true);
		JDialog.setDefaultLookAndFeelDecorated(true);

		FenCreeBase customDialog = new FenCreeBase();
		customDialog.pack();

		customDialog.setLocationRelativeTo(this);
		customDialog.setVisible(true);
	}

	private void sauveBase() {
		Edi.nos().getBaseCourante().sauve();
	}

	private void sauveBaseSous() {
		JFrame.setDefaultLookAndFeelDecorated(true);
		JDialog.setDefaultLookAndFeelDecorated(true);

		FenSauveBaseSous customDialog = new FenSauveBaseSous();
		customDialog.pack();

		customDialog.setLocationRelativeTo(this);
		customDialog.setVisible(true);
	}

	private void sauveFichier() {
		PrintWriter p = null;
		try {
			FileWriter fos = new FileWriter(file.getAbsolutePath() + ".rev");
			p = new PrintWriter(fos, true);

			p.print(chaine);

		} catch (IOException e) {
			montre("Erreur lors de la sauvegarde du fichier");
		} finally {
			p.close();
		}
	}

	public void setMenuBar() {
		installeMenus();
		setVisible(true);
	}

	private void suiteConversion() {
		File fileRev2 = new File(file.getAbsolutePath() + ".rev2").getAbsoluteFile();
		if (fileRev2.exists()) {
			file = fileRev2;
			finConversion();
			return;
		}
		;
		// sinon on crÈe le .rev et on propose de crÈer le .rev2
		replace();
		sauveFichier();

		/*
		 * JOptionPane.showMessageDialog(this, "Vérifiez/corrigez le fichier
		 * <.rev>,\n" + "enregistrez le nouveau fichier en <.rev2>,\n" + "puis relancez
		 * la conversion...");
		 */
		if (ToolNos.nomSysteme().equals("Windows")) {
			try {
				Runtime.getRuntime().exec(
						"\"C:/Program Files/Windows NT/Accessoires/wordpad.exe\" " + file.getAbsolutePath() + ".rev");
			} catch (IOException e) {
				e.printStackTrace();
			}

		} else if (ToolNos.nomSysteme().equals("Mac")) {
			try {
				Runtime.getRuntime()
						.exec("/Applications/TextEdit.app/Contents/MacOS/TextEdit " + file.getAbsolutePath() + ".rev");
			} catch (IOException e) {
				e.printStackTrace();
			}

		}

		Object[] options = { "Annuler", "Continuer" };
		int n = JOptionPane.showOptionDialog(this,
				"Vérifiez/corrigez le fichier <.rev>,\n" + "enregistrez le nouveau fichier en <.rev2>,\n"
						+ "puis cliquez sur <Continuer>...",
				"Veuillez créer le fichier <.rev2>...", JOptionPane.YES_NO_OPTION, JOptionPane.QUESTION_MESSAGE, null, // no
				// custom
				// Icon
				options, // the titles of buttons
				options[1]); // default button title

		if (n == 1) {// Continuer
			suiteConversion();
		}
	}

	protected void supprimeGraphe() {
		String nomGraphe = (String) listeGraphes.getSelectedValue();
		if (nomGraphe == null) {
			JOptionPane.showMessageDialog(this, "Veuillez d'abord sélectionner un graphe...");
			return;
		}

		NoS n = (NoS) Edi.nos().getBaseCourante().getAssocNos().get(nomGraphe);
		if (n == null) {
			JOptionPane.showMessageDialog(this, "nos <" + nomGraphe + "> introuvable ...");
		} else {

			int rep = JOptionPane.showConfirmDialog(Edi.nos().fen(),
					"Confirmez-vous la suppression du graphe <" + nomGraphe + "> ?", "Suppression d'un graphe",
					JOptionPane.YES_NO_OPTION);
			if (rep == 0) {
				// Suppression confirmée
				Edi.nos().getBaseCourante().supprimeGraphe(nomGraphe);
			}
		}
	}

	public void testeOrdreIF() {
		JInternalFrame[] iframes = desktop.getAllFrames();
		for (JInternalFrame iframe : iframes) {
			System.out.println(iframe.getTitle() + "\t" + desktop.getComponentZOrder(iframe));
		}
	}

	private void ouvreGraphe() {
		Graphe gr = (Graphe) Edi.nos().getBaseCourante().getAssocGraphes().get(listeGraphes.getSelectedValue());
		FenGraphe f = new FenGraphe(gr.getNom(), Edi.nos().getBaseCourante());
		Edi.nos().getBaseCourante().ajouteFen(f);
		f.setSize(gr.getTaille());
		ouvreInternalFrame(f);
	}
}
