/*
  EdiNoS - Editeur de NoS
  Copyright (C) 2013 - 2021 - Luc Poittevin — Tous droits réservés.
  Contact : edinos@free.fr  -  https://www.edinos.org
  
  Ce programme est un logiciel libre ; vous pouvez le redistribuer ou le
  modifier suivant les termes de la “GNU General Public License” telle que
  publiée par la Free Software Foundation : soit la version 3 de cette
  licence, soit (à votre gré) toute version ultérieure.
  
  Ce programme est distribué dans l’espoir qu’il vous sera utile, mais SANS
  AUCUNE GARANTIE : sans même la garantie implicite de COMMERCIALISABILITÉ
  ni d’ADÉQUATION À UN OBJECTIF PARTICULIER. Consultez la Licence Générale
  Publique GNU pour plus de détails.
  
  Vous devriez avoir reçu une copie de la Licence Générale Publique GNU avec
  ce programme ; si ce n’est pas le cas, consultez :
  <http://www.gnu.org/licenses/>. 
*/

package nos.edit;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Cursor;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.KeyEvent;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.awt.font.TextLayout;
import java.awt.geom.Line2D;
import java.util.ArrayList;
import java.util.HashSet;
import java.util.List;
import java.util.Set;
import java.lang.Math;

import javax.swing.Action;
import javax.swing.JButton;
import javax.swing.JComponent;
import javax.swing.JDialog;
import javax.swing.JFrame;
import javax.swing.JInternalFrame;
import javax.swing.JMenu;
import javax.swing.JMenuBar;
import javax.swing.JMenuItem;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.KeyStroke;
import javax.swing.event.InternalFrameEvent;
import javax.swing.event.InternalFrameListener;

import nos.obj.Base;
import nos.obj.Edi;
import nos.obj.NoS;
import nos.run.Bilan;
import nos.run.ErreursVerif;
import nos.run.Memoire;
import nos.run.MoteurVerifie;
import nos.run.edit.FenTexte;
import nos.utils.ToolNos;

@SuppressWarnings("serial")
public class FenGraphe extends JInternalFrame implements InternalFrameListener {

	public enum Mode {
		normal, creeLien, creeFils, supprimeLien
	}

	public class PanelGraphe extends JPanel implements MouseListener {

		private Dimension taille;

		public PanelGraphe(int x, int y) {

			mode = Mode.normal;
			reinitTaille();
			setLayout(null);
			setBackground(Color.white);
			setOpaque(true);
			registerEventListeners();
		}

		public void actionCreeFils() {
			mode = Mode.creeFils;
			saisieNouveauNoS();
		}

		public void actionCreeLien() {
			changeCurseur(Cursor.CROSSHAIR_CURSOR);
			mode = Mode.creeLien;
		}

		public void actionRenommeNoS() {
			saisieNouveauNomNoS();
		}

		public void ajouteNoS(PanelNoS n) {
			// TODO : ne pas passer par getComponents mais une liste de nos ou
			// panelNos
			Object[] o = this.getComponents();
			for (int i = 0; i < o.length; i++) {
				if (o[i].getClass().getName() == "nos.edit.PanelNoS") {
					PanelNoS p = (PanelNoS) o[i];
					if (p.getNoS().equals(n.getNoS())) {
						// le nos est déjà affiché dans le graphe, on se
						// contente de le raffraichir et on quitte la méthode
						n.repaint();
						return;
					}
				}
			}
			this.add(n);
			if (n.getX() > taille.getWidth()) {
				taille.setSize(n.getX(), taille.getHeight());
			}

			if (n.getY() > taille.getHeight()) {
				taille.setSize(taille.getWidth(), n.getY());
			}
		}

		public int checkNosOrphelin() {
			int option = 1, count = 0;
			// TODO : ne pas passer par getComponents mais une liste de nos ou
			// panelNos
			Object[] o = this.getComponents();
			for (int i = 0; i < o.length; i++) {
				if (o[i].getClass().getName() == "nos.edit.PanelNoS") {
					PanelNoS p = (PanelNoS) o[i];
					if (!nosInit.tousLesNoSSuivants().contains(p.getNoS())) {
						count++;
					}
				}
			}
			if (count == 1) { // il y a 1 nos affiché non rattaché
				option = JOptionPane.showConfirmDialog(this,
						"Il y a un NoS non rattaché aux autres NoS qui sera perdu\n"
								+ "si vous fermez cette fenêtre. Annuler la fermeture ?",
						"Attention...", JOptionPane.YES_NO_OPTION);
			}
			if (count > 1) { // il y a <count> nos affichés non rattachés
				option = JOptionPane.showConfirmDialog(this,
						"Il y a " + count + " NoS non rattachés aux autres NoS qui seront perdus\n"
								+ "si vous fermez cette fenêtre. Annuler la fermeture ?",
						"Attention...", JOptionPane.YES_NO_OPTION);
			}
			return option;
		}

		public boolean contientNoS(NoS n) {
			// TODO : ne pas passer par getComponents mais une liste de nos ou
			// panelNos
			Object[] o = this.getComponents();
			for (int i = 0; i < o.length; i++) {
				if (o[i].getClass().getName() == "nos.edit.PanelNoS") {
					PanelNoS p = (PanelNoS) o[i];
					if (p.getNoS().equals(n)) {
						return true;
					}
				}
			}
			return false;
		}

		private void dessineUneFleche(Graphics2D g2, int x1, int y1, int x2, int y2) {
			int x = (x1 + x2) / 2;
			int y = (y1 + y2) / 2;
			double angle = 30;
			int longFleche = 10; // 20

			g2.draw(new Line2D.Double(x1, y1, x2, y2));

			double b = (double) (x2 - x1) / (y2 - y1);


			double alpha = Math.PI / 2 - Math.toRadians(angle) - Math.atan(b);
			double beta = Math.atan(b) - Math.toRadians(angle);


			if (y1 > y2) {
				alpha = alpha + Math.PI;
				beta = beta + Math.PI;
			}


			double cx = x - longFleche * Math.cos(alpha);
			double cy = y - longFleche * Math.sin(alpha);
			g2.draw(new Line2D.Double(x, y, cx, cy));

			double bx = x - longFleche * Math.sin(beta);
			double by = y - longFleche * Math.cos(beta);
			g2.draw(new Line2D.Double(x, y, bx, by));
		}

		private void dessineLibelleFleche(NoS nos1, NoS nos2, Graphics2D g2, int x1, int y1, int x2, int y2) {

			int x = (x1 + x2) / 2;
			int y = (y1 + y2) / 2;

			String s = nos1.getLibelleFlecheVers(nos2);
			if (!s.isEmpty()) {
				TextLayout thisTl = new TextLayout(s, new Font("Verdana", 0, 10), g2.getFontRenderContext());
				thisTl.draw(g2, x + 15, y);
			}
		}

		private void dessineUnLien(NoS nos1, NoS nos2, Graphics g) {

			Graphics2D g2 = (Graphics2D) g;
			g2.setColor(Color.BLACK);
			int x1 = nos1.getPosX();
			int y1 = nos1.getPosY();
			int x2 = nos2.getPosX();
			int y2 = nos2.getPosY();

			int l = 140;
			int h = 40;

			dessineUneFleche(g2, x1 + l / 2, y1 + h / 2, x2 + l / 2, y2 + h / 2);
			dessineLibelleFleche(nos1, nos2, g2, x1 + l / 2, y1 + h / 2, x2 + l / 2, y2 + h / 2);
		}

		public void effaceLienSous(NoS n) {
			// INUTILE
			/****
			 * efface tous les liens de <vueCas> à ses <suivantsVue> ***"
			 * 
			 * (vueCas suivantsVue) do: [:s| self effaceLienDe: vueCas a: s. ].
			 */
			for (String s : n.getSuivants()) {
				effaceUnLien(n, base.getNoS(s));
			}
		}

		public void effaceNoS(Set<NoS> s) {
			// TODO : ne pas passer par getComponents mais une liste de nos en
			// var. d'instance
			reinitTaille();
			Object[] o = this.getComponents();
			for (int i = 0; i < o.length; i++) {
				if (o[i].getClass().getName() == "nos.edit.PanelNoS") {
					PanelNoS p = (PanelNoS) o[i];
					if (s.contains(p.getNoS())) {
						// System.out.println(p.getNoS().getNom());
						this.remove(p);
						// this.effaceLienSous(p.getNoS());
					}
				}
			}
		}

		private void effaceUnLien(NoS nos1, NoS nos2) {

			Graphics2D g2 = (Graphics2D) this.getGraphics();
			g2.setColor(Color.WHITE);

			int x1 = nos1.getPosX();
			int y1 = nos1.getPosY();
			int x2 = nos2.getPosX();
			int y2 = nos2.getPosY();

			int l = 140;
			int h = 40;

			dessineUneFleche(g2, x1 + l / 2, y1 + h / 2, x2 + l / 2, y2 + h / 2);
		}

		private Action escape() {
			if (mode == Mode.creeLien) {
				changeCurseur(Cursor.DEFAULT_CURSOR);
				mode = Mode.normal;
			} else {
				fermeFen();
			}
			return null;
		}

		public Set<NoS> getListeNosAffiches() {
			// TODO : la liste de nos doit être maintenue et renvoyée
			List<NoS> l = new ArrayList<NoS>();
			l = nosInit.tousLesNoSSuivants();
			List<NoS> l2 = new ArrayList<NoS>();
			for (NoS n1 : l) {
				if (!contientNoS(n1)) {
					l2.add(n1);
				}
			}
			l.removeAll(l2);
			Set<NoS> e = new HashSet<NoS>(l);
			return e;
		}

		public Dimension getTaille() {
			return new Dimension(taille.width + 140, taille.height + 40);
		}

		public void mouseClicked(MouseEvent arg0) {
		}

		public void mouseEntered(MouseEvent arg0) {
		}

		public void mouseExited(MouseEvent arg0) {
		}

		public void mousePressed(MouseEvent arg0) {
			if (mode == Mode.creeFils) {
				creeFils();
			}
		}

		public void mouseReleased(MouseEvent arg0) {
		}

		public void noduleClique(PanelNoS vueNod) {
			if (mode == Mode.creeLien) {
				vueNodCourant.getNoS().addSuivant(vueNod.getNoS().getNom());
				changeCurseur(Cursor.DEFAULT_CURSOR);
				mode = Mode.normal;
			}

			if (mode == Mode.supprimeLien) {
				if (vueNodCourant.getNoS().getSuivants().contains(vueNod.getNoS().getNom())) {

					vueNodCourant.getNoS().removeSuivant(vueNod.getNoS().getNom());
				}
				if (vueNod.getNoS().getSuivants().contains(vueNodCourant.getNoS().getNom())) {
					// inversement : cas où c'est le père qui est cliqué
					vueNod.getNoS().removeSuivant(vueNodCourant.getNoS().getNom());
				}
				changeCurseur(Cursor.DEFAULT_CURSOR);
				mode = Mode.normal;
			}

			if (vueNodCourant != null) {
				vueNodCourant.dessineNormal();
			}

			vueNodCourant = vueNod;
		}

		public void paintComponent(Graphics g) {
			super.paintComponent(g);


			dessineLiens(g);
		}

		private void registerEventListeners() {
			addMouseListener(this);

			boutonEnregistrer.addActionListener(new ActionListener() {
				public void actionPerformed(ActionEvent evt) {
					// sauvegarde...
				}
			});

			boutonFermer.addActionListener(new ActionListener() {
				public void actionPerformed(ActionEvent evt) {
					fermeFen();
				}
			});

			ActionListener actionListenerEsc = new ActionListener() {
				public void actionPerformed(ActionEvent actionEvent) {
					escape();
				}
			};

			KeyStroke stroke = KeyStroke.getKeyStroke(KeyEvent.VK_ESCAPE, 0);
			this.registerKeyboardAction(actionListenerEsc, stroke, JComponent.WHEN_IN_FOCUSED_WINDOW);
		}

		private void reinitTaille() {
			taille = new Dimension(300, 300);
		}
	}

	private Base base;

	private NoS nosInit;

	private PanelNoS vueNodCourant;

	private JButton boutonEnregistrer = new JButton("Enregistrer");

	private JButton boutonFermer = new JButton("Fermer");

	private PanelGraphe panelGraphe;;

	private String nomFils;

	private JMenuBar menuBar;

	public Mode mode;

	public FenGraphe(String nomNodInit, Base b) {
		super("Graphe <" + nomNodInit + ">", true, // resizable
				true, // closable
				true, // maximizable
				true);// iconifiable
		addInternalFrameListener(this);
		base = b;
		PanelNoS n;

		// pour intercepter la fermeture dans internalFrameClosing
		setDefaultCloseOperation(JInternalFrame.DO_NOTHING_ON_CLOSE);


		panelGraphe = new PanelGraphe(550, 440);
		JScrollPane scrollPane = new JScrollPane(panelGraphe);

		nosInit = (NoS) base.getAssocNos().get(nomNodInit);


		for (NoS nos : nosInit.tousLesNoSSuivantsAafficher(new HashSet<NoS>())) {
			n = new PanelNoS(nos, panelGraphe);
			panelGraphe.ajouteNoS(n);
		}

		getContentPane().add(scrollPane, BorderLayout.CENTER);
		// System.out.println(_panelGraphe.getTaille());
		panelGraphe.setPreferredSize(panelGraphe.getTaille());


		installeMenus();
	}

	public void cancelCreeNos() {
		mode = Mode.normal;
	}

	public void changeCurseur(int valeurCurseur) {
		setCursor(Cursor.getPredefinedCursor(valeurCurseur));
	}

	public void creeFils() {
		// création du nos : 
		NoS nos = base.creeNoS(nomFils);
		nos.setPosition(panelGraphe.getMousePosition().x + "@" + panelGraphe.getMousePosition().y);
		PanelNoS n = new PanelNoS(nos, panelGraphe);
		panelGraphe.add(n);
		repaint();
		Edi.nos().refreshListeNoS();
		// création du lien :
		vueNodCourant.getNoS().addSuivant(nomFils);

		setCursor(Cursor.getPredefinedCursor(Cursor.DEFAULT_CURSOR));
		mode = Mode.normal;
		nomFils = null;
	}

	public void creeNoS(String nom) {
		if (mode == Mode.creeFils) {
			setCursor(Cursor.getPredefinedCursor(Cursor.CROSSHAIR_CURSOR));
			nomFils = nom;
		} else {// création depuis menu Graphe
			NoS nos = base.creeNoS(nom);
			PanelNoS n = new PanelNoS(nos, panelGraphe);
			panelGraphe.add(n);
			repaint();
			Edi.nos().refreshListeNoS();
		}
	}

	private void dessineLiens(Graphics g) {

		for (NoS nos1 : nosInit.tousLesNoSSuivantsAafficher(new HashSet<NoS>())) {
			if (nos1.getRaccourci() == false) {
				// on ne dessine les liens suivants
				// seulement si le nos n'est pas raccourci
				for (String nom : nos1.getSuivants()) {
					NoS nos2 = (NoS) base.getAssocNos().get(nom);
					// il faut aussi que le panelNos dest soit dans le graphe
					if (panelGraphe.contientNoS(nos2) == true) {
						panelGraphe.dessineUnLien(nos1, nos2, g);
					}
				}
			}
		}
	}

	private void installeMenus() {
		FenEdiNoS ed = (FenEdiNoS) Edi.nos().fen();
		menuBar = ed.menuBarEdinos();
		menuBar.add(ed.menuBase());
		menuBar.add(menuGraphe());
		Edi.nos().fen().setJMenuBar(menuBar);
		Edi.nos().fen().setVisible(true);
	}

	public void internalFrameActivated(InternalFrameEvent e) {
		Edi.nos().fen().setJMenuBar(menuBar);
		Edi.nos().fen().setVisible(true);
	}

	public void internalFrameClosed(InternalFrameEvent e) {
		base.getGraphe(nosInit.getNom()).setTaille(this.getSize());
	}

	public void internalFrameClosing(InternalFrameEvent e) {
		fermeFen();
	}

	private void fermeFen() {
		int option = panelGraphe.checkNosOrphelin();
		if (option == JOptionPane.NO_OPTION) { // fermeture confirmée
			if (ToolNos.nomSysteme().equals("Mac")) {
				ToolNos.fermeFen((JInternalFrame) this);
			} else {
				dispose();
			}
			((FenEdiNoS) Edi.nos().fen()).checkDerniereFen(); 
		}
	}

	public void internalFrameDeactivated(InternalFrameEvent e) {
		((FenEdiNoS) Edi.nos().fen()).setMenuBar();
	}

	public void internalFrameDeiconified(InternalFrameEvent e) {
	}

	public void internalFrameIconified(InternalFrameEvent e) {
	}

	public void internalFrameOpened(InternalFrameEvent e) {
	}

	public JMenu menuGraphe() {

		JMenuItem menuItem;
		JMenu menu = new JMenu("Graphe");

		menuItem = new JMenuItem("Vérifier le graphe");
		menuItem.setName("VérifierGraphe"); // pour FEST
		menuItem.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				verifie();
			}
		});
		menu.add(menuItem);
		menuItem = new JMenuItem("Fabriquer un bilan");
		menuItem.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				fabriqueBilan();
			}
		});
		menu.add(menuItem);

		menu.addSeparator();
		menuItem = new JMenuItem("Nouveau NoS");
		menuItem.setAccelerator(KeyStroke.getKeyStroke(java.awt.event.KeyEvent.VK_N, ToolNos.renvoieModifiers()));
		menuItem.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				saisieNouveauNoS();
			}
		});
		menu.add(menuItem);
		menuItem = new JMenuItem("Renommer NoS");
		menuItem.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				saisieNouveauNomNoS();
			}
		});
		menu.add(menuItem);
		menuItem = new JMenuItem("Nouveau Lien");
		menuItem.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				nouveauLien();
			}
		});
		menu.add(menuItem);
		

		menu.addSeparator();
		menuItem = new JMenuItem("Supprimer un NoS");
		menuItem.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				supprimeNoS();
			}
		});
		menu.add(menuItem);
		menuItem = new JMenuItem("Supprimer un Lien");
		menuItem.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				supprimeLien();
			}
		});
		menu.add(menuItem);
		

		menu.addSeparator();
		menuItem = new JMenuItem("Tout Rallonger");
		menuItem.setAccelerator(KeyStroke.getKeyStroke(java.awt.event.KeyEvent.VK_R, ToolNos.renvoieModifiers()));
		menuItem.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				rallongeTout();
			}
		});
		menu.add(menuItem);
		menuItem = new JMenuItem("Tout raCcourcir");
		menuItem.setAccelerator(KeyStroke.getKeyStroke(java.awt.event.KeyEvent.VK_C, ToolNos.renvoieModifiers()));
		menuItem.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				raccourcitTout();
			}
		});
		menu.add(menuItem);

		menu.addSeparator();
		menuItem = new JMenuItem("Fermer le graphe");
		menuItem.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				fermeFen();
			}
		});
		menuItem.setAccelerator(KeyStroke.getKeyStroke(java.awt.event.KeyEvent.VK_W, ToolNos.renvoieModifiers()));
		menu.add(menuItem);

		return (menu);
	}

	protected void supprimeNoS() {
		if (vueNodCourant == null) {
			JOptionPane.showMessageDialog(Edi.nos().fen(), "Veuillez d'abord sélectionner un nos...");
			return;
		}

		String nomNoS = vueNodCourant.getNoS().getNom();
		int rep = JOptionPane.showConfirmDialog(Edi.nos().fen(),
				"Confirmez-vous la suppression du nos <" + nomNoS + "> ?", "Suppression d'un nos",
				JOptionPane.YES_NO_OPTION);
		if (rep == 0) {
			// Suppression confirmée
			base.getAssocNos().remove(nomNoS);
			panelGraphe.remove(vueNodCourant);
			repaint();
			Edi.nos().refreshListeNoS();
			base.supprimeSuivants(nomNoS);
		}
	}

	protected void nouveauLien() {
		if (vueNodCourant == null) {
			JOptionPane.showMessageDialog(Edi.nos().fen(), "Veuillez d'abord sélectionner un nos...");
			return;
		}
		panelGraphe.actionCreeLien();
	}

	protected void raccourcitTout() {
		PanelNoS n = null;
		nosInit.raccourcitTousLesSuivants();

		// on supprime tous les nos
		Set<NoS> set = new HashSet<NoS>();
		set.addAll(nosInit.tousLesNoSSuivants());
		panelGraphe.effaceNoS(set);

		// il faut recréer les suivantsVue de nosInit ...
		for (String s : nosInit.getSuivants()) {
			n = new PanelNoS(base.getNoS(s), panelGraphe);
			panelGraphe.ajouteNoS(n);
		}

		// ... ainsi que nosInit :
		n = new PanelNoS(nosInit, panelGraphe);
		panelGraphe.add(n);
		panelGraphe.setPreferredSize(panelGraphe.getTaille());
		this.repaint();
	}

	protected void rallongeTout() {
		PanelNoS n = null;
		nosInit.rallongeTousLesSuivants();

		// on supprime tous les nos
		Set<NoS> set = new HashSet<NoS>();
		set.addAll(nosInit.tousLesNoSSuivants());
		panelGraphe.effaceNoS(set);

		// et on les recrée en conformité avec leur statut ...
		for (NoS nos : nosInit.tousLesNoSSuivants()) {
			n = new PanelNoS(nos, panelGraphe);
			panelGraphe.ajouteNoS(n);
		}

		panelGraphe.setPreferredSize(panelGraphe.getTaille());
		this.repaint();
	}

	public void renommeNoS(String nouveauNom) {
		String ancienNom = vueNodCourant.getNoS().getNom();
		vueNodCourant.getNoS().setNom(nouveauNom);
		base.getAssocNos().remove(ancienNom);
		base.getAssocNos().put(nouveauNom, vueNodCourant.getNoS());
		vueNodCourant.repaint();
		Edi.nos().refreshListeNoS();
		base.majSuivants(ancienNom, nouveauNom);

	}

	protected void saisieNouveauNomNoS() {
		if (vueNodCourant == null) {
			JOptionPane.showMessageDialog(Edi.nos().fen(), "Veuillez d'abord sélectionner un nos...");
			return;
		}
		if (vueNodCourant.getNoS().equals(nosInit)) {
			JOptionPane.showMessageDialog(Edi.nos().fen(),
					"Veuillez choisir <Renommer un graphe> dans le menu Base...");
			return;
		}

		JFrame.setDefaultLookAndFeelDecorated(true);
		JDialog.setDefaultLookAndFeelDecorated(true);

		FenRenommeNoS customDialog = new FenRenommeNoS(this, vueNodCourant.getNoS().getNom());
		customDialog.pack();

		customDialog.setLocationRelativeTo(this);
		customDialog.setVisible(true);
	}

	protected void saisieNouveauNoS() {
		JFrame.setDefaultLookAndFeelDecorated(true);
		JDialog.setDefaultLookAndFeelDecorated(true);

		FenCreeNoS customDialog = new FenCreeNoS(this);
		customDialog.pack();

		customDialog.setLocationRelativeTo(this);
		customDialog.setVisible(true);
	}

	public void setVueNodCourant(PanelNoS vueNod) {
		vueNodCourant = vueNod;
	}

	protected void supprimeLien() {
		if (vueNodCourant == null) {
			JOptionPane.showMessageDialog(Edi.nos().fen(), "Veuillez d'abord sélectionner un nos...");
			return;
		}
		changeCurseur(Cursor.CROSSHAIR_CURSOR);
		mode = Mode.supprimeLien;
	}

	protected void verifie() {
		MoteurVerifie moteur = new MoteurVerifie(nosInit);
		moteur.declencheAvec(new Memoire());

		ErreursVerif erreurs = moteur.getErreursVerif();
		if (erreurs.getRowCount() == 0) {
			Edi.nos().message("Vérification terminée : aucune erreur détectée...");
		} else {
			FenVerif fenVerif = new FenVerif(erreurs, nosInit.getNom());
			fenVerif.setSize(600, 120); // 400, 160);
			Edi.nos().getBaseCourante().ajouteFen(fenVerif);
			((FenEdiNoS) Edi.nos().fen()).ouvreInternalFrame(fenVerif);
			fenVerif.setVisible(true);
		}
	}

	protected void fabriqueBilan() {

		Bilan bilan = base.getGraphe(nosInit.getNom()).bilan();

		FenTexte f = new FenTexte(bilan.resultat(), "Bilan de <" + bilan.getNomNosInitial() + ">", "Bilan",
				bilan.getNomNosInitial(), "cou");
		f.setSize(500, 350);
		Edi.nos().getBaseCourante().ajouteFen(f);
		((FenEdiNoS) Edi.nos().fen()).ouvreInternalFrame(f);

	}
}
