/*
  EdiNoS - Editeur de NoS
  Copyright (C) 2013-2021 - Luc Poittevin — Tous droits réservés.
  Contact : edinos@free.fr  -  https://www.edinos.org
  
  Ce programme est un logiciel libre ; vous pouvez le redistribuer ou le
  modifier suivant les termes de la “GNU General Public License” telle que
  publiée par la Free Software Foundation : soit la version 3 de cette
  licence, soit (à votre gré) toute version ultérieure.
  
  Ce programme est distribué dans l’espoir qu’il vous sera utile, mais SANS
  AUCUNE GARANTIE : sans même la garantie implicite de COMMERCIALISABILITÉ
  ni d’ADÉQUATION À UN OBJECTIF PARTICULIER. Consultez la Licence Générale
  Publique GNU pour plus de détails.
  
  Vous devriez avoir reçu une copie de la Licence Générale Publique GNU avec
  ce programme ; si ce n’est pas le cas, consultez :
  <http://www.gnu.org/licenses/>. 
 */

package nos.edit;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Dimension;
import java.awt.Insets;
import java.awt.Point;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.KeyEvent;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.beans.PropertyVetoException;
import java.util.ArrayList;
import java.util.Collections;
import java.util.StringTokenizer;

import javax.swing.BorderFactory;
import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.JButton;
import javax.swing.JComponent;
import javax.swing.JInternalFrame;
import javax.swing.JLabel;
import javax.swing.JMenu;
import javax.swing.JMenuBar;
import javax.swing.JMenuItem;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTextArea;
import javax.swing.KeyStroke;
import javax.swing.SwingUtilities;
import javax.swing.event.InternalFrameEvent;
import javax.swing.event.InternalFrameListener;
import javax.swing.text.BadLocationException;
import javax.swing.text.Utilities;

import nos.edit.html.FenMessage;
import nos.edit.html.FenQuestion;
import nos.obj.Edi;
import nos.obj.Message;
import nos.obj.NoS;
import nos.obj.Question;
import nos.run.Bilan;
import nos.run.ErreursVerif;
import nos.run.Memoire;
import nos.run.Moteur.Contexte;
import nos.run.MoteurVerifie;
import nos.run.edit.FenTexte;
import nos.utils.ToolNos;

@SuppressWarnings("serial")
public class FenNos extends JInternalFrame implements InternalFrameListener {

	private JButton boutonEnregistrer = new JButton("Enregistrer");

	private JButton boutonModifier = new JButton("Palette");
	
	private JButton boutonFermer = new JButton("Fermer");


	private JTextArea zoneTexteActions = new JTextArea(6, 22);

	private JTextArea zoneTexteComm = new JTextArea(6, 25);

	private JTextArea zoneTexteRegles = new JTextArea(6, 48);

	private NoS nos;

	private JInternalFrame palette;

	private JMenuBar menuBar;

	private Color couleurRegles = Color.LIGHT_GRAY;

	public FenNos(NoS n) {
		super("NoS <" + n.getNom() + ">", true, // resizable
				true, // closable
				true, // maximizable
				true);// iconifiable
		addInternalFrameListener(this);
		nos = n;

		Dimension dimFen = new Dimension(580, 310);
		Dimension dimTitre = new Dimension(280, 30);
		this.setTitle("NoS <" + nos.getNom() + ">");
		getContentPane().setLayout(new BorderLayout());
		getContentPane().setPreferredSize(dimFen);

		boutonModifier.setToolTipText("Ouvre la palette d'édition du NoS");
		JPanel panel = new JPanel();
		panel.setLayout(new BoxLayout(panel, BoxLayout.X_AXIS));
		panel.add(boutonEnregistrer);
		panel.add(boutonModifier);
		panel.add(boutonFermer);
		panel.add(Box.createHorizontalGlue());
		panel.setBorder(BorderFactory.createLineBorder(Color.white));
		getContentPane().add(panel, BorderLayout.NORTH);

		JPanel panelMilieu = new JPanel();
		panelMilieu.setLayout(new BoxLayout(panelMilieu, BoxLayout.X_AXIS));

		JPanel panel2 = new JPanel();
		panel2.setLayout(new BorderLayout());
		panel2.setBorder(BorderFactory.createLineBorder(Color.white));
		JLabel l2 = new JLabel("Actions", JLabel.CENTER);
		l2.setForeground(Color.BLUE);
		l2.setPreferredSize(dimTitre);
		panel2.add(l2, BorderLayout.NORTH);
		zoneTexteActions.setName("Actions");
		zoneTexteActions.setLineWrap(true);
		zoneTexteActions.setMargin(new Insets(0, 3, 0, 3));
		zoneTexteActions.addMouseListener(new MouseAdapter() {
			public void mouseClicked(MouseEvent me) {
				// if (me.getButton() == MouseEvent.BUTTON3) {
				if ((SwingUtilities.isRightMouseButton(me)) || (me.isControlDown())) {
					// Edi.nos().message("Clic droit");
					detectWord(me, zoneTexteActions);
				}
			}
		});
		JScrollPane scrollPaneActions = new JScrollPane(zoneTexteActions);
		panel2.add(scrollPaneActions, BorderLayout.CENTER);
		panelMilieu.add(panel2);

		JPanel panel4 = new JPanel();
		panel4.setLayout(new BorderLayout());
		panel4.setBorder(BorderFactory.createLineBorder(Color.white));
		JLabel l4 = new JLabel("Description", JLabel.CENTER);
		l4.setForeground(Color.BLUE);
		l4.setPreferredSize(dimTitre);
		panel4.add(l4, BorderLayout.NORTH);
		zoneTexteComm.setLineWrap(true);
		zoneTexteComm.setMargin(new Insets(0, 3, 0, 3));
		JScrollPane scrollPaneComm = new JScrollPane(zoneTexteComm);
		panel4.add(scrollPaneComm, BorderLayout.CENTER);
		panelMilieu.add(panel4);

		getContentPane().add(panelMilieu);

		JPanel panel3 = new JPanel();
		panel3.setLayout(new BoxLayout(panel3, BoxLayout.Y_AXIS));
		panel3.setBorder(BorderFactory.createLineBorder(Color.white));
		JPanel panel3b = new JPanel();
		JLabel l3 = new JLabel("Règles", JLabel.CENTER);
		l3.setForeground(Color.BLUE);
		panel3b.add(l3, BorderLayout.CENTER);
		panel3b.setBackground(couleurRegles);
		panel3b.setOpaque(true);
		panel3.add(panel3b);
		zoneTexteRegles.setName("Règles");
		zoneTexteRegles.setLineWrap(true);
		zoneTexteRegles.setMargin(new Insets(0, 3, 0, 3));
		zoneTexteRegles.addMouseListener(new MouseAdapter() {
			public void mouseClicked(MouseEvent me) {
				if ((SwingUtilities.isRightMouseButton(me)) || (me.isControlDown())) {
					detectWord(me, zoneTexteRegles);
				}
			}
		});

		JScrollPane scrollPaneRegles = new JScrollPane(zoneTexteRegles);
		panel3.add(scrollPaneRegles);

		getContentPane().add(panel3, BorderLayout.SOUTH);

		// installation des menus
		installeMenus();

		// Register the Listeners
		registerEventListeners();

		remplitZones();

		// Resize the container to the preferred size
		pack();
	}

	private void registerEventListeners() {
		boutonEnregistrer.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent evt) {
				boolean verifOK = verifieNoS(false);
				if (verifOK) {
					enregistrer();
				}
			}
		});

		boutonModifier.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent evt) {
				ouvrePalette();
			}
		});

		boutonFermer.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent evt) {
				fermetureDemandée();
			}
		});

		ActionListener actionListenerEsc = new ActionListener() {
			public void actionPerformed(ActionEvent actionEvent) {
				fermetureDemandée();
			}
		};

		KeyStroke stroke = KeyStroke.getKeyStroke(KeyEvent.VK_ESCAPE, 0);
		getRootPane().registerKeyboardAction(actionListenerEsc, stroke, JComponent.WHEN_IN_FOCUSED_WINDOW);

	}

	protected void fermetureDemandée() {
		boolean verifOK = verifieNoS(false);
		checkeModif();
		fermeNoS();
	}

	private void remplitZones() {
		zoneTexteActions.append(nos.getActions());
		zoneTexteRegles.append(nos.getRegles());
		zoneTexteComm.append(nos.getCommentaires());
	}

	private void installeMenus() {
		FenEdiNoS ed = (FenEdiNoS) Edi.nos().fen();
		menuBar = ed.menuBarEdinos();
		menuBar.add(ed.menuBase());
		menuBar.add(menuNoS());
		Edi.nos().fen().setJMenuBar(menuBar);
		Edi.nos().fen().setVisible(true);
	}

	public NoS getNoS() {
		return nos;
	}

	public JMenu menuNoS() {

		JMenuItem menuItem;
		JMenu menu = new JMenu("NoS");

		menuItem = new JMenuItem("Vérifier le NoS");
		menuItem.setName("VérifierNoS"); // pour FEST
		menuItem.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				verifieNoS(true);
			}
		});
		menu.add(menuItem);

		menuItem = new JMenuItem("Voir les chemins");
		menuItem.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				chemins();
			}
		});
		menu.add(menuItem);

		menu.addSeparator();
		menuItem = new JMenuItem("Couper");
		menuItem.setAccelerator(KeyStroke.getKeyStroke(java.awt.event.KeyEvent.VK_X, ToolNos.renvoieModifiers()));
		menuItem.setEnabled(false);
		menu.add(menuItem);
		menuItem = new JMenuItem("Copier");
		menuItem.setAccelerator(KeyStroke.getKeyStroke(java.awt.event.KeyEvent.VK_C, ToolNos.renvoieModifiers()));
		menuItem.setEnabled(false);
		menu.add(menuItem);
		menuItem = new JMenuItem("Coller");
		menuItem.setAccelerator(KeyStroke.getKeyStroke(java.awt.event.KeyEvent.VK_V, ToolNos.renvoieModifiers()));
		menuItem.setEnabled(false);
		menu.add(menuItem);

		menu.addSeparator();
		menuItem = new JMenuItem("Voir l'Objet");
		menuItem.setAccelerator(KeyStroke.getKeyStroke(java.awt.event.KeyEvent.VK_O, ToolNos.renvoieModifiers()));
		menuItem.setEnabled(false);
		menu.add(menuItem);
		menuItem = new JMenuItem("Voir les Références");
		menuItem.setAccelerator(KeyStroke.getKeyStroke(java.awt.event.KeyEvent.VK_R, ToolNos.renvoieModifiers()));
		menuItem.setEnabled(false);
		menu.add(menuItem);

		menu.addSeparator();
		menuItem = new JMenuItem("Fermer la fenêtre");
		menuItem.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				fermetureDemandée();
			}
		});
		menuItem.setAccelerator(KeyStroke.getKeyStroke(java.awt.event.KeyEvent.VK_W, ToolNos.renvoieModifiers()));
		menu.add(menuItem);

		return (menu);
	}

	protected boolean verifieNoS(boolean verifDemandéeParLeUser) {

		// Pour lancer la vérification, on doit temporairement
		// modifier le nos pour lui donner les valeurs saisies :

		String actions = nos.getActions();
		String regles = nos.getRegles();
		nos.setActions(zoneTexteActions.getText());
		nos.setRegles(zoneTexteRegles.getText());

		MoteurVerifie moteur = new MoteurVerifie(nos, Contexte.ControleNoS);
		moteur.declencheAvec(new Memoire());

		ErreursVerif erreurs = moteur.getErreursVerif();
		String s = "";
		if (erreurs.getRowCount() == 0) {
			s = "Vérification terminée : aucune erreur détectée...";
			if (verifDemandéeParLeUser) {
				Edi.nos().message(s);
			}
			nos.setActions(actions);
			nos.setRegles(regles);
			return true;
		} else if (erreurs.getRowCount() == 1) {
			s = "Vérification du nos <" + nos.getNom() + "> :\n";
			s = s + erreurs.getValueAt(0, 2).toString();
		} else {
			s = "Vérification du nos <" + nos.getNom() + "> :\n";
			for (int i = 0; i < erreurs.getRowCount(); i++) {
				int j = i + 1;
				s = s + j + ") " + erreurs.getValueAt(i, 2).toString() + "\n";
			}
		}
		Edi.nos().message(s);
		nos.setActions(actions);
		nos.setRegles(regles);
		return false;
	}

	protected void chemins() {
		Bilan bilan = new Bilan();
		NoS premierNos = nos.getBase().getGrapheQuiContientNos(nos.getNom()).getPremierNoS();
		bilan.lance(premierNos, nos);

		FenTexte f = new FenTexte(bilan.resultat(), "Chemins vers <" + nos.getNom() + ">", "Chemins", nos.getNom(),
				"che");
		f.setSize(500, 350);
		Edi.nos().getBaseCourante().ajouteFen(f);
		((FenEdiNoS) Edi.nos().fen()).ouvreInternalFrame(f);
	}

	public void ouvrePalette() {
		if (palette == null) {
			palette = new PaletteNoS(this);
			Point p = getLocation();
			Dimension size = getContentPane().getSize();
			p.setLocation(p.x + size.width + 15, p.y); // +5
			palette.setLocation(p);
			((FenEdiNoS) Edi.nos().fen()).ouvreInternalFrameSansPositionner(palette);
		}
		palette.setVisible(true);
	}

	public JTextArea getZoneTexteActions() {
		return zoneTexteActions;
	}

	public JTextArea getZoneTexteRegles() {
		return zoneTexteRegles;
	}

	public void internalFrameActivated(InternalFrameEvent e) {
		Edi.nos().fen().setJMenuBar(menuBar);
		Edi.nos().fen().setVisible(true);
	}

	public void internalFrameDeactivated(InternalFrameEvent e) {
		((FenEdiNoS) Edi.nos().fen()).setMenuBar();
	}

	public void internalFrameClosed(InternalFrameEvent e) {
	}

	public void internalFrameClosing(InternalFrameEvent e) {
		fermetureDemandée();
	}

	public void internalFrameDeiconified(InternalFrameEvent e) {
	}

	public void internalFrameIconified(InternalFrameEvent e) {
	}

	public void internalFrameOpened(InternalFrameEvent e) {
	}

	private void checkeModif() {
		if ((!zoneTexteActions.getText().equals(nos.getActions()))
				|| (!zoneTexteRegles.getText().equals(nos.getRegles()))
				|| (!zoneTexteComm.getText().equals(nos.getCommentaires()))) {
			int rep = JOptionPane.showConfirmDialog(Edi.nos().fen(),
					"Vous avez effectué des modifications : voulez-vous les enregistrer ?", "Enregistrer ?",
					JOptionPane.YES_NO_OPTION);
			if (rep == JOptionPane.YES_OPTION) {
				// Enregistrement confirmé
				enregistrer();
			}
		}
	}

	private void enregistrer() {

		nos.setActions(zoneTexteActions.getText());
		nos.setRegles(zoneTexteRegles.getText());
		nos.setCommentaires(zoneTexteComm.getText());
	}

	private void fermeNoS() {
		if (palette != null) {
			palette.dispose();
		}
		if (ToolNos.nomSysteme().equals("Mac")) {
			ToolNos.fermeFen((JInternalFrame) this);
		} else {
			dispose();
		}
		((FenEdiNoS) Edi.nos().fen()).checkDerniereFen(); 
	}

	public void fermePalette() {
		palette = null;
		try {
			this.setSelected(true);
		} catch (PropertyVetoException e) {
		}
	}

	private void detectWord(MouseEvent me, JTextArea zoneTexte) {

		ArrayList<Object> MOT_CLES = Collections.list(new StringTokenizer(
				"evaluer: demander: afficher: declencher: priorite: aller: alorsAller: si: NON ET OU"));

		int x = me.getX();
		int y = me.getY();
		int start = 0;
		int startOffset = zoneTexte.viewToModel(new Point(x, y));

		try {
			start = Utilities.getWordStart(zoneTexte, startOffset);

		} catch (BadLocationException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		String text = zoneTexte.getText().substring(start).split(";")[0];
		text = text.split(" ")[0];
		if (!MOT_CLES.contains(text)) {
			 
			if (Edi.nos().getBaseCourante().contientNoS(text)) {
				FenNos f = new FenNos((NoS) Edi.nos().getBaseCourante().getAssocNos().get(text));
				Edi.nos().getBaseCourante().ajouteFen(f);
				((FenEdiNoS) Edi.nos().fen()).ouvreInternalFrame(f);
			} else if (Edi.nos().getBaseCourante().contientQuestion(text)) {
				Question q = Edi.nos().getBaseCourante().getQuestion(text);
				if (ToolNos.getJavaVersion() >= 1.8) {
					FenQuestion f = new FenQuestion(q);
					Edi.nos().getBaseCourante().ajouteFen(f);
					((FenEdiNoS) Edi.nos().fen()).ouvreInternalFrame(f);
					f.init();
				} else {
					// Java 6 ou 7 : Java FX absent...
					// FenQuestionHTML f = new FenQuestionHTML(q);
					// Edi.nos().getBaseCourante().ajouteFen(f);
					// ((FenEdiNoS) Edi.nos().fen()).ouvreInternalFrame(f);
					// f.init();
				}
			} else if (Edi.nos().getBaseCourante().contientMessage(text)) {
				Message m = Edi.nos().getBaseCourante().getMessage(text);
				if (ToolNos.getJavaVersion() >= 1.8) {
					FenMessage f = new FenMessage(m);
					Edi.nos().getBaseCourante().ajouteFen(f);
					((FenEdiNoS) Edi.nos().fen()).ouvreInternalFrame(f);
					f.init();
				} else {
					// Java 6 ou 7 : Java FX absent...
					// FenMessage f = new FenMessage(m);
					// Edi.nos().getBaseCourante().ajouteFen(f);
					// ((FenEdiNoS) Edi.nos().fen()).ouvreInternalFrame(f);
				}
			} else if (Edi.nos().getBaseCourante().contientProc(text)) {
				FenActions f = new FenActions(Edi.nos().getBaseCourante());
				f.setSize(690, 450); 
				f.positionneSur(text);
				Edi.nos().getBaseCourante().ajouteFen(f);
				((FenEdiNoS) Edi.nos().fen()).ouvreInternalFrame(f);
			} else {
				String act = Edi.nos().getBaseCourante().getActionQuiRenvoieFait(text);
				FenActions f = new FenActions(Edi.nos().getBaseCourante());
				f.setSize(690, 450); 
				f.positionneSur(act);
				Edi.nos().getBaseCourante().ajouteFen(f);
				((FenEdiNoS) Edi.nos().fen()).ouvreInternalFrame(f);
			}
		}
	}
}
