/*
  EdiNoS - Editeur de NoS
  Copyright (C) 2013-2021 - Luc Poittevin — Tous droits réservés.
  Contact : edinos@free.fr  -  https://www.edinos.org
  
  Ce programme est un logiciel libre ; vous pouvez le redistribuer ou le
  modifier suivant les termes de la “GNU General Public License” telle que
  publiée par la Free Software Foundation : soit la version 3 de cette
  licence, soit (à votre gré) toute version ultérieure.
  
  Ce programme est distribué dans l’espoir qu’il vous sera utile, mais SANS
  AUCUNE GARANTIE : sans même la garantie implicite de COMMERCIALISABILITÉ
  ni d’ADÉQUATION À UN OBJECTIF PARTICULIER. Consultez la Licence Générale
  Publique GNU pour plus de détails.
  
  Vous devriez avoir reçu une copie de la Licence Générale Publique GNU avec
  ce programme ; si ce n’est pas le cas, consultez :
  <http://www.gnu.org/licenses/>. 
*/

package nos.edit;

import java.awt.Color;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.File;

import javax.swing.BorderFactory;
import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.JButton;
import javax.swing.JFileChooser;
import javax.swing.JInternalFrame;
import javax.swing.JMenu;
import javax.swing.JMenuBar;
import javax.swing.JMenuItem;
import javax.swing.KeyStroke;
import javax.swing.event.InternalFrameEvent;
import javax.swing.event.InternalFrameListener;

import nos.obj.Edi;
import nos.utils.ToolNos;
import nos.utils.PrefsManager;

@SuppressWarnings("serial")
public class FenPrefs extends JInternalFrame implements InternalFrameListener  {
	
	private JMenuBar menuBar;

	private JButton boutonEnregistrer = new JButton("Enregistrer");

	private JButton boutonFermer = new JButton("Fermer");

	public FenPrefs(FenEdiNoS e) {
		
        super("Préférences EdiNoS", 
                true, //resizable
                true, //closable
                true, //maximizable
                true);//iconifiable
		addInternalFrameListener(this);

		initComponents();
		initComponents2();
		registerEventListeners();
		installeMenus();
	}

	private void initComponents2() {
		jPanel6.setLayout(new BoxLayout(jPanel6, BoxLayout.LINE_AXIS));
		jPanel6.add(boutonEnregistrer);
		jPanel6.add(boutonFermer);
		jPanel6.add(Box.createHorizontalGlue());
		jPanel6.setBorder(BorderFactory.createLineBorder(Color.white));
 
		buttonGroup3.setSelected(jRadioButton1.getModel(), ((PrefsManager) Edi.nos().getPreferences()).getSimuleProcBoutons().equals("faits"));
		buttonGroup3.setSelected(jRadioButton2.getModel(), ((PrefsManager) Edi.nos().getPreferences()).getSimuleProcBoutons().equals("numéros"));
		
		buttonGroup4.setSelected(jRadioButton3.getModel(), ((PrefsManager) Edi.nos().getPreferences()).getQuestionsBoutonRadio().equals("sans"));
		buttonGroup4.setSelected(jRadioButton4.getModel(), ((PrefsManager) Edi.nos().getPreferences()).getQuestionsBoutonRadio().equals("avec"));

		jTextField1.setText(Edi.nos().getRepertoireTravail());
		
		jTextField2.setText(String.valueOf(((PrefsManager) Edi.nos().getPreferences()).getFenDialogueHauteur()));
		jTextField3.setText(String.valueOf(((PrefsManager) Edi.nos().getPreferences()).getFenDialogueLargeur()));
		
		jRadioButton1.setActionCommand("faits");
		jRadioButton2.setActionCommand("numéros");
		
		jRadioButton3.setActionCommand("sans");
		jRadioButton4.setActionCommand("avec");
	}
	
	private void installeMenus() {
		menuBar = new JMenuBar();
		menuBar.add(menuPrefs());
		Edi.nos().fen().setJMenuBar(menuBar);
		Edi.nos().fen().setVisible(true);
	}
	
	private JMenu menuPrefs() {

		JMenuItem menuItem;
		JMenu menu = new JMenu("Préférences");

		menuItem = new JMenuItem("Enregistrer");
		menuItem.setAccelerator(KeyStroke.getKeyStroke(
				java.awt.event.KeyEvent.VK_S, ToolNos.renvoieModifiers()));
		menuItem.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				enregistre();
			}
		});
		menu.add(menuItem);

		menu.addSeparator();
		menuItem = new JMenuItem("Fermer la fenêtre");
		menuItem.setAccelerator(KeyStroke.getKeyStroke(
				java.awt.event.KeyEvent.VK_W, ToolNos.renvoieModifiers()));
		menuItem.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				fermePrefs();
			}
		});
		menu.add(menuItem);
		return (menu);
	}
	
	private void enregistre() {
		((PrefsManager) Edi.nos().getPreferences()).setFenDialogueHauteur(Integer.valueOf(jTextField2.getText()));
		((PrefsManager) Edi.nos().getPreferences()).setFenDialogueLargeur(Integer.valueOf(jTextField3.getText()));

		((PrefsManager) Edi.nos().getPreferences()).setSimuleProcBoutons(buttonGroup3.getSelection().getActionCommand());
		((PrefsManager) Edi.nos().getPreferences()).setQuestionsBoutonRadio(buttonGroup4.getSelection().getActionCommand());
		((PrefsManager) Edi.nos().getPreferences()).setRepertoireTravail(jTextField1.getText());
	}

    private void choisitRepertoire() {
		File dir = new File(jTextField1.getText());
		JFileChooser fc = new JFileChooser(dir);
		fc.setFileSelectionMode(JFileChooser.DIRECTORIES_ONLY);
		int returnVal = fc.showOpenDialog(this);
		if (returnVal == JFileChooser.APPROVE_OPTION) {
	        jTextField1.setText(fc.getSelectedFile().getAbsolutePath());
		 }
	}
    
    private void fermePrefs() {
		if (ToolNos.nomSysteme().equals("Mac")) {
			ToolNos.fermeFen((JInternalFrame) this);
		} else {
			dispose();
		} 
		((FenEdiNoS) Edi.nos().fen()).checkDerniereFen();
	}
    
	private void registerEventListeners() {
		boutonEnregistrer.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent evt) {
				enregistre();
				((PrefsManager) Edi.nos().getPreferences()).sauve();
			}
		});

		boutonFermer.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent evt) {
				fermePrefs();
			}
		});
	}
    
	public void internalFrameActivated(InternalFrameEvent e) {
		Edi.nos().fen().setJMenuBar(menuBar);
		Edi.nos().fen().setVisible(true);
	}

	public void internalFrameClosed(InternalFrameEvent e) {
	}

	public void internalFrameClosing(InternalFrameEvent e) {
		fermePrefs();
	}

	public void internalFrameDeactivated(InternalFrameEvent e) {
		((FenEdiNoS) Edi.nos().fen()).setMenuBar();
	}

	public void internalFrameDeiconified(InternalFrameEvent e) {
	}

	public void internalFrameIconified(InternalFrameEvent e) {
	}

	public void internalFrameOpened(InternalFrameEvent e) {
	}

    private void jButton1ActionPerformed(java.awt.event.ActionEvent evt) {                                         
    	choisitRepertoire();
    }   
    
    private void jTextField1ActionPerformed(java.awt.event.ActionEvent evt) {                                            
        // Si on modifie manuellement le champ texte "Répertoire de travail", on ne
    	// fait rien de particulier
    }

    private void jButton2ActionPerformed(java.awt.event.ActionEvent evt) {  
    	// valeurs par défaut :
		jTextField2.setText(String.valueOf(((PrefsManager) Edi.nos().getPreferences()).getFenDialogueHauteurDefaut()));
		jTextField3.setText(String.valueOf(((PrefsManager) Edi.nos().getPreferences()).getFenDialogueLargeurDefaut()));

    }   
    
    // <editor-fold defaultstate="collapsed" desc="Generated Code">                          
    private void initComponents() {

        buttonGroup3 = new javax.swing.ButtonGroup();
        buttonGroup4 = new javax.swing.ButtonGroup();
        jTabbedPane1 = new javax.swing.JTabbedPane();
        jPanel1 = new javax.swing.JPanel();
        jTextField1 = new javax.swing.JTextField();
        jLabel1 = new javax.swing.JLabel();
        jButton1 = new javax.swing.JButton();
        jPanel2 = new javax.swing.JPanel();
        jLabel4 = new javax.swing.JLabel();
        jLabel5 = new javax.swing.JLabel();
        jTextField2 = new javax.swing.JTextField();
        jLabel6 = new javax.swing.JLabel();
        jTextField3 = new javax.swing.JTextField();
        jButton2 = new javax.swing.JButton();
        jPanel3 = new javax.swing.JPanel();
        jRadioButton1 = new javax.swing.JRadioButton();
        jRadioButton2 = new javax.swing.JRadioButton();
        jLabel2 = new javax.swing.JLabel();
        jLabel3 = new javax.swing.JLabel();
        jRadioButton3 = new javax.swing.JRadioButton();
        jRadioButton4 = new javax.swing.JRadioButton();
        jPanel4 = new javax.swing.JPanel();
        jPanel5 = new javax.swing.JPanel();
        jPanel6 = new javax.swing.JPanel();

//        setDefaultCloseOperation(javax.swing.WindowConstants.DISPOSE_ON_CLOSE);

        jTextField1.setText("jTextField1");
        jTextField1.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jTextField1ActionPerformed(evt);
            }
        });

        jLabel1.setText("Répertoire de travail :");

        jButton1.setText("...");
        jButton1.setToolTipText("Choisir le répertoire");
        jButton1.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jButton1ActionPerformed(evt);
            }
        });

        org.jdesktop.layout.GroupLayout jPanel1Layout = new org.jdesktop.layout.GroupLayout(jPanel1);
        jPanel1.setLayout(jPanel1Layout);
        jPanel1Layout.setHorizontalGroup(
            jPanel1Layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
            .add(jPanel1Layout.createSequentialGroup()
                .add(20, 20, 20)
                .add(jPanel1Layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
                    .add(jLabel1)
                    .add(jPanel1Layout.createSequentialGroup()
                        .add(jTextField1, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, 419, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                        .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                        .add(jButton1, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, 23, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)))
                .addContainerGap())
        );
        jPanel1Layout.setVerticalGroup(
            jPanel1Layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
            .add(jPanel1Layout.createSequentialGroup()
                .add(10, 10, 10)
                .add(jLabel1)
                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                .add(jPanel1Layout.createParallelGroup(org.jdesktop.layout.GroupLayout.BASELINE)
                    .add(jTextField1, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                    .add(jButton1, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, 15, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE))
                .addContainerGap(153, Short.MAX_VALUE))
        );

        jTabbedPane1.addTab("Général", jPanel1);

        jLabel4.setText("Taille des fenêtres Questions et Messages :");

        jLabel5.setText("Hauteur");

        jTextField2.setText("jTextField2");

        jLabel6.setText("Largeur ");

        jTextField3.setText("jTextField2");

        jButton2.setText("Valeurs par défaut");
        jButton2.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jButton2ActionPerformed(evt);
            }
        });

        org.jdesktop.layout.GroupLayout jPanel2Layout = new org.jdesktop.layout.GroupLayout(jPanel2);
        jPanel2.setLayout(jPanel2Layout);
        jPanel2Layout.setHorizontalGroup(
            jPanel2Layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
            .add(org.jdesktop.layout.GroupLayout.TRAILING, jPanel2Layout.createSequentialGroup()
                .add(0, 0, Short.MAX_VALUE)
                .add(jButton2)
                .add(69, 69, 69))
            .add(jPanel2Layout.createSequentialGroup()
                .add(jPanel2Layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
                    .add(jPanel2Layout.createSequentialGroup()
                        .add(66, 66, 66)
                        .add(jLabel4))
                    .add(jPanel2Layout.createSequentialGroup()
                        .add(131, 131, 131)
                        .add(jPanel2Layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
                            .add(jPanel2Layout.createSequentialGroup()
                                .add(jLabel6)
                                .addPreferredGap(org.jdesktop.layout.LayoutStyle.UNRELATED)
                                .add(jTextField3, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE))
                            .add(jPanel2Layout.createSequentialGroup()
                                .add(jLabel5)
                                .addPreferredGap(org.jdesktop.layout.LayoutStyle.UNRELATED)
                                .add(jTextField2, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)))))
                .addContainerGap(150, Short.MAX_VALUE))
        );
        jPanel2Layout.setVerticalGroup(
            jPanel2Layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
            .add(jPanel2Layout.createSequentialGroup()
                .add(24, 24, 24)
                .add(jLabel4, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, 28, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                .add(18, 18, 18)
                .add(jPanel2Layout.createParallelGroup(org.jdesktop.layout.GroupLayout.BASELINE)
                    .add(jLabel5)
                    .add(jTextField2, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE))
                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                .add(jPanel2Layout.createParallelGroup(org.jdesktop.layout.GroupLayout.BASELINE)
                    .add(jLabel6)
                    .add(jTextField3, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE))
                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED, 39, Short.MAX_VALUE)
                .add(jButton2)
                .add(19, 19, 19))
        );

        jTabbedPane1.addTab("Présentation", jPanel2);

        buttonGroup3.add(jRadioButton1);
        jRadioButton1.setText("Boutons = faits");

        buttonGroup3.add(jRadioButton2);
        jRadioButton2.setText("Boutons = numéros");

        jLabel2.setText("Simulation de procédures :");

        jLabel3.setText("Questions :");

        buttonGroup4.add(jRadioButton3);
        jRadioButton3.setText("Sans bouton radio");
//        jRadioButton3.addActionListener(new java.awt.event.ActionListener() {
//            public void actionPerformed(java.awt.event.ActionEvent evt) {
//                jRadioButton3ActionPerformed(evt);
//            }
//        });

        buttonGroup4.add(jRadioButton4);
        jRadioButton4.setText("Avec bouton radio");

        org.jdesktop.layout.GroupLayout jPanel3Layout = new org.jdesktop.layout.GroupLayout(jPanel3);
        jPanel3.setLayout(jPanel3Layout);
        jPanel3Layout.setHorizontalGroup(
            jPanel3Layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
            .add(jPanel3Layout.createSequentialGroup()
                .add(41, 41, 41)
                .add(jPanel3Layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
                    .add(jLabel3)
                    .add(jPanel3Layout.createSequentialGroup()
                        .add(jPanel3Layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
                            .add(jLabel2)
                            .add(jPanel3Layout.createSequentialGroup()
                                .add(57, 57, 57)
                                .add(jPanel3Layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
                                    .add(jRadioButton3)
                                    .add(jRadioButton1))))
                        .add(28, 28, 28)
                        .add(jPanel3Layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
                            .add(jRadioButton2)
                            .add(jRadioButton4))))
                .addContainerGap(60, Short.MAX_VALUE))
        );
        jPanel3Layout.setVerticalGroup(
            jPanel3Layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
            .add(org.jdesktop.layout.GroupLayout.TRAILING, jPanel3Layout.createSequentialGroup()
                .add(25, 25, 25)
                .add(jLabel3)
                .add(11, 11, 11)
                .add(jPanel3Layout.createParallelGroup(org.jdesktop.layout.GroupLayout.BASELINE)
                    .add(jRadioButton3)
                    .add(jRadioButton4))
                .add(44, 44, 44)
                .add(jLabel2)
                .add(11, 11, 11)
                .add(jPanel3Layout.createParallelGroup(org.jdesktop.layout.GroupLayout.BASELINE)
                    .add(jRadioButton1)
                    .add(jRadioButton2))
                .addContainerGap(46, Short.MAX_VALUE))
        );

        jTabbedPane1.addTab("Run", jPanel3);

        org.jdesktop.layout.GroupLayout jPanel4Layout = new org.jdesktop.layout.GroupLayout(jPanel4);
        jPanel4.setLayout(jPanel4Layout);
        jPanel4Layout.setHorizontalGroup(
            jPanel4Layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
            .add(0, 488, Short.MAX_VALUE)
        );
        jPanel4Layout.setVerticalGroup(
            jPanel4Layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
            .add(0, 215, Short.MAX_VALUE)
        );

        jTabbedPane1.addTab("ReviNoS", jPanel4);

        org.jdesktop.layout.GroupLayout jPanel5Layout = new org.jdesktop.layout.GroupLayout(jPanel5);
        jPanel5.setLayout(jPanel5Layout);
        jPanel5Layout.setHorizontalGroup(
            jPanel5Layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
            .add(0, 488, Short.MAX_VALUE)
        );
        jPanel5Layout.setVerticalGroup(
            jPanel5Layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
            .add(0, 215, Short.MAX_VALUE)
        );

        jTabbedPane1.addTab("Divers", jPanel5);

        org.jdesktop.layout.GroupLayout jPanel6Layout = new org.jdesktop.layout.GroupLayout(jPanel6);
        jPanel6.setLayout(jPanel6Layout);
        jPanel6Layout.setHorizontalGroup(
            jPanel6Layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
            .add(0, 0, Short.MAX_VALUE)
        );
        jPanel6Layout.setVerticalGroup(
            jPanel6Layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
            .add(0, 33, Short.MAX_VALUE)
        );

        org.jdesktop.layout.GroupLayout layout = new org.jdesktop.layout.GroupLayout(getContentPane());
        getContentPane().setLayout(layout);
        layout.setHorizontalGroup(
            layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
            .add(jPanel6, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
            .add(jTabbedPane1)
        );
        layout.setVerticalGroup(
            layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
            .add(layout.createSequentialGroup()
                .add(jPanel6, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                .add(jTabbedPane1))
        );

        pack();
    }// </editor-fold>                        
                                      
    // Variables declaration - do not modify                     
    private javax.swing.ButtonGroup buttonGroup3;
    private javax.swing.ButtonGroup buttonGroup4;
    private javax.swing.JButton jButton1;
    private javax.swing.JButton jButton2;
    private javax.swing.JLabel jLabel1;
    private javax.swing.JLabel jLabel2;
    private javax.swing.JLabel jLabel3;
    private javax.swing.JLabel jLabel4;
    private javax.swing.JLabel jLabel5;
    private javax.swing.JLabel jLabel6;
    private javax.swing.JPanel jPanel1;
    private javax.swing.JPanel jPanel2;
    private javax.swing.JPanel jPanel3;
    private javax.swing.JPanel jPanel4;
    private javax.swing.JPanel jPanel5;
    private javax.swing.JPanel jPanel6;
    private javax.swing.JRadioButton jRadioButton1;
    private javax.swing.JRadioButton jRadioButton2;
    private javax.swing.JRadioButton jRadioButton3;
    private javax.swing.JRadioButton jRadioButton4;
    private javax.swing.JTabbedPane jTabbedPane1;
    private javax.swing.JTextField jTextField1;
    private javax.swing.JTextField jTextField2;
    private javax.swing.JTextField jTextField3;
    // End of variables declaration                   
}
