/*
  EdiNoS - Editeur de NoS
  Copyright (C) 2020 - Luc Poittevin — Tous droits réservés.
  Contact : edinos@free.fr  -  https://www.edinos.org
  
  Ce programme est un logiciel libre ; vous pouvez le redistribuer ou le
  modifier suivant les termes de la “GNU General Public License” telle que
  publiée par la Free Software Foundation : soit la version 3 de cette
  licence, soit (à votre gré) toute version ultérieure.
  
  Ce programme est distribué dans l’espoir qu’il vous sera utile, mais SANS
  AUCUNE GARANTIE : sans même la garantie implicite de COMMERCIALISABILITÉ
  ni d’ADÉQUATION À UN OBJECTIF PARTICULIER. Consultez la Licence Générale
  Publique GNU pour plus de détails.
  
  Vous devriez avoir reçu une copie de la Licence Générale Publique GNU avec
  ce programme ; si ce n’est pas le cas, consultez :
  <http://www.gnu.org/licenses/>. 
 */

package nos.edit;

import java.awt.Component;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.KeyEvent;

import javax.swing.AbstractCellEditor;
import javax.swing.Icon;
import javax.swing.JButton;
import javax.swing.JComponent;
import javax.swing.JInternalFrame;
import javax.swing.JMenu;
import javax.swing.JMenuBar;
import javax.swing.JMenuItem;
import javax.swing.JTable;
import javax.swing.KeyStroke;
import javax.swing.event.InternalFrameEvent;
import javax.swing.event.InternalFrameListener;
import javax.swing.table.TableCellEditor;
import javax.swing.table.TableCellRenderer;

import nos.obj.Edi;
import nos.obj.NoS;
import nos.run.ErreursVerif;
import nos.utils.ToolNos;

@SuppressWarnings("serial")
public class FenVerif extends JInternalFrame implements InternalFrameListener {

	private JMenuBar menuBar;

	public FenVerif(ErreursVerif t, String nomGraphe) {
		super("Vérification du graphe <" + nomGraphe + ">", true, // resizable
				true, // closable
				true, // maximizable
				true);// iconifiable
		// Pour retirer la bordure basse sous MacOS :
		this.putClientProperty("JInternalFrame.frameType", "normal");
		addInternalFrameListener(this);

		initComponents();
		jTable1.setModel(t);
		initComponents2();
		installeMenus();
		registerEventListeners();
	}

	private void initComponents2() {

		jTable1.getColumnModel().getColumn(0).setMaxWidth(30);
		jTable1.getColumnModel().getColumn(3).setMaxWidth(30);
		 
		jTable1.getColumnModel().getColumn(2).setMinWidth(120);
		jTable1.getColumnModel().getColumn(2).setPreferredWidth(300);
		
		jTable1.getColumnModel().getColumn(0).setCellRenderer(new ButtonRenderer());
		jTable1.getColumnModel().getColumn(0).setCellEditor(new ButtonEditor(jTable1));
		jTable1.getColumnModel().getColumn(3).setCellRenderer(new ButtonRenderer());
		jTable1.getColumnModel().getColumn(3).setCellEditor(new ButtonEditor(jTable1));
		jTable1.setName("jTable1"); // pour FEST
	}

	private void installeMenus() {
		FenEdiNoS ed = (FenEdiNoS) Edi.nos().fen();
		menuBar = ed.menuBarEdinos();
		menuBar.add(ed.menuBase());
		menuBar.add(menuVerif());
		Edi.nos().fen().setJMenuBar(menuBar);
		Edi.nos().fen().setVisible(true);
	}

	private JMenu menuVerif() {
		JMenu menu = new JMenu("Vérification");
		JMenuItem menuItem = new JMenuItem("Fermer la fenêtre");
		menuItem.setName("FermerFenêtre"); // pour FEST
		menuItem.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				// Modif 18/03/2020 :
				// dispose();
				fermeFen();
			}


		});
		menuItem.setAccelerator(KeyStroke.getKeyStroke(
				java.awt.event.KeyEvent.VK_W, ToolNos.renvoieModifiers()));
		menuItem.setName("FermerFenêtre"); // pour FEST
		menu.add(menuItem);

		return menu;
	}
	
	private void registerEventListeners() {
		ActionListener actionListenerEsc = new ActionListener() {
			public void actionPerformed(ActionEvent actionEvent) {
				fermeFen();
			}
		};

		KeyStroke stroke = KeyStroke.getKeyStroke(KeyEvent.VK_ESCAPE, 0);
		getRootPane().registerKeyboardAction(actionListenerEsc, stroke,
				JComponent.WHEN_IN_FOCUSED_WINDOW);
	}
	
	private void fermeFen() {
		if (ToolNos.nomSysteme().equals("Mac")) {
			ToolNos.fermeFen((JInternalFrame) this);
		} else {
			dispose();
		} 
		((FenEdiNoS) Edi.nos().fen()).checkDerniereFen(); 
	}
	
	public void internalFrameActivated(InternalFrameEvent e) {
		Edi.nos().fen().setJMenuBar(menuBar);
		Edi.nos().fen().setVisible(true);
	}

	public void internalFrameClosed(InternalFrameEvent e) {
	}

	public void internalFrameClosing(InternalFrameEvent e) {
		fermeFen();
	}

	public void internalFrameDeactivated(InternalFrameEvent e) {
		((FenEdiNoS) Edi.nos().fen()).setMenuBar();
	}

	public void internalFrameDeiconified(InternalFrameEvent e) {
	}

	public void internalFrameIconified(InternalFrameEvent e) {
	}

	public void internalFrameOpened(InternalFrameEvent e) {
	}

	// <editor-fold defaultstate="collapsed" desc="Generated Code">
	private void initComponents() {

		jScrollPane1 = new javax.swing.JScrollPane();
		jTable1 = new javax.swing.JTable();

//		setDefaultCloseOperation(javax.swing.WindowConstants.DISPOSE_ON_CLOSE);

		jTable1.setModel(new javax.swing.table.DefaultTableModel(
				new Object[][] { { null, null, null, null, null, null }, { null, null, null, null, null, null },
						{ null, null, null, null, null, null }, { null, null, null, null, null, null },
						{ null, null, null, null, null, null } },
				new String[] { "", "N°", "Nom du cas", "Statut", "", "" }));

		jTable1.setRowHeight(19);
		jScrollPane1.setViewportView(jTable1);
		jTable1.getColumnModel().getColumn(0).setPreferredWidth(20);
		jTable1.getColumnModel().getColumn(0).setMaxWidth(20);
		jTable1.getColumnModel().getColumn(1).setPreferredWidth(20);
		jTable1.getColumnModel().getColumn(1).setMaxWidth(20);
		jTable1.getColumnModel().getColumn(2).setPreferredWidth(140);
		jTable1.getColumnModel().getColumn(3).setPreferredWidth(60);
		jTable1.getColumnModel().getColumn(4).setPreferredWidth(20);
		jTable1.getColumnModel().getColumn(4).setMaxWidth(20);
		jTable1.getColumnModel().getColumn(5).setPreferredWidth(20);
		jTable1.getColumnModel().getColumn(5).setMaxWidth(20);

		org.jdesktop.layout.GroupLayout layout = new org.jdesktop.layout.GroupLayout(getContentPane());
		getContentPane().setLayout(layout);
		layout.setHorizontalGroup(layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING).add(jScrollPane1,
				org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, 334, Short.MAX_VALUE));
		layout.setVerticalGroup(layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING).add(jScrollPane1,
				org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, 164, Short.MAX_VALUE));
		pack();
	}// </editor-fold>

	// Variables declaration - do not modify
	private javax.swing.JScrollPane jScrollPane1;
	private javax.swing.JTable jTable1;
	// End of variables declaration
}

@SuppressWarnings("serial")
class ButtonRenderer extends JButton implements TableCellRenderer {

	public ButtonRenderer() {
		setOpaque(true);
	}

	public Component getTableCellRendererComponent(JTable table, Object value, boolean isSelected, boolean hasFocus,
			int row, int column) {
		setIcon((Icon) value);
		setContentAreaFilled(false);
		switch (column) {
		case 0: // colonne Bouton Supprimer
			setToolTipText("Supprimer la ligne");
			return this;
		case 4: // colonne Bouton Voir
			setToolTipText("Corriger l'erreur");
			return this;
		}
		return this;
	}
}

@SuppressWarnings("serial")
class ButtonEditor extends AbstractCellEditor implements TableCellEditor {
	protected JButton button;
	protected Object editorValue;
	protected JTable table;

	public ButtonEditor(JTable t) {
		super();
		table = t;
		button = new JButton();
		button.setOpaque(true); // semble inutile
		button.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent arg0) {
				int row = table.convertRowIndexToModel(table.getEditingRow());
				int col = table.convertColumnIndexToModel(table.getEditingColumn());
				fireEditingStopped();
				if (col == 0) {
					((ErreursVerif) table.getModel()).supprime(row);
				} else if (col == 3) {
					
					String nomNoS = ((ErreursVerif) table.getModel()).getValueAt(row,1).toString();
					Edi.nos().montreAvertissement("Ouverture de " + nomNoS);
					
					FenNos f = new FenNos((NoS) Edi.nos().getBaseCourante()
							.getAssocNos().get(nomNoS));
					Edi.nos().getBaseCourante().ajouteFen(f);
					((FenEdiNoS) Edi.nos().fen()).ouvreInternalFrame(f);
					f.setVisible(true);
				}
			}
		});
	}

	public Object getCellEditorValue() {
		return editorValue;
	}

	public Component getTableCellEditorComponent(JTable table, Object value, boolean isSelected, int row, int column) {
		if (value == null) {
			button.setText("");
			button.setIcon(null);
		} else if (value instanceof Icon) {
			button.setText("");
			button.setIcon((Icon) value);
//			button.setContentAreaFilled(false); // le fait de laisser cette ligne en commentaire
//			permet de montrer un changement en cas de clic sur le bouton sous Windows et Linux
		} else {
			button.setText(value.toString());
			button.setIcon(null);
		}
		this.editorValue = value;
		return button;
	}
}
