/*
  EdiNoS - Editeur de NoS
  Copyright (C) 2017-2021 - Luc Poittevin — Tous droits réservés.
  Contact : edinos@free.fr  -  https://www.edinos.org
  
  Ce programme est un logiciel libre ; vous pouvez le redistribuer ou le
  modifier suivant les termes de la “GNU General Public License” telle que
  publiée par la Free Software Foundation : soit la version 3 de cette
  licence, soit (à votre gré) toute version ultérieure.
  
  Ce programme est distribué dans l’espoir qu’il vous sera utile, mais SANS
  AUCUNE GARANTIE : sans même la garantie implicite de COMMERCIALISABILITÉ
  ni d’ADÉQUATION À UN OBJECTIF PARTICULIER. Consultez la Licence Générale
  Publique GNU pour plus de détails.
  
  Vous devriez avoir reçu une copie de la Licence Générale Publique GNU avec
  ce programme ; si ce n’est pas le cas, consultez :
  <http://www.gnu.org/licenses/>. 
 */

package nos.edit.html;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Dimension;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.KeyEvent;

import javafx.application.Platform;
import javafx.scene.Node;
import javafx.scene.web.HTMLEditor;
import javafx.scene.web.WebEngine;
import javafx.scene.web.WebView;

import javax.swing.BorderFactory;
import javax.swing.BoxLayout;
import javax.swing.DefaultComboBoxModel;
import javax.swing.JButton;
import javax.swing.JComboBox;
import javax.swing.JComponent;
import javax.swing.JInternalFrame;
import javax.swing.JLabel;
import javax.swing.JMenuBar;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTextArea;
import javax.swing.JTextField;
import javax.swing.KeyStroke;
import javax.swing.event.InternalFrameEvent;
import javax.swing.event.InternalFrameListener;

import netscape.javascript.JSException;
import nos.edit.FenEdiNoS;
import nos.obj.Edi;
import nos.utils.ToolNos;

@SuppressWarnings("serial")
public class FenChoix extends JInternalFrame implements InternalFrameListener {

	private JButton boutonOK = new JButton("   OK    ");

	private JButton boutonAnnuler = new JButton("Annuler");

	private FenQuestion fenQuestion;

	private JTextArea zoneTexteLibelle = new JTextArea(2, 6);

	private JComboBox<String> combo = null;

	private JTextField zoneTexteFait = new JTextField();

	private HTMLEditor htmlEditor;

	private String sourceHTML;

	private String fin = "";

	private JMenuBar menuBar;

	public FenChoix(String s, HTMLEditor h, String libelle, String fait,
			FenQuestion fen) {
		super("Choix d'une réponse", true, // resizable
				true, // closable
				true, // maximizable
				true);// iconifiable

		fenQuestion = fen;
		sourceHTML = s;
		htmlEditor = h;
		if (!sourceHTML.isEmpty()) {
			detecteFin(libelle);
		}

		addInternalFrameListener(this);

		Dimension dimFen = new Dimension(300, 200);
		Dimension dimTitre = new Dimension(150, 30);
		getContentPane().setLayout(new BorderLayout());
		getContentPane().setPreferredSize(dimFen);

		JPanel panel2 = new JPanel();
		panel2.setLayout(new BorderLayout());
		JLabel l2 = new JLabel("Libellé de la réponse", JLabel.CENTER);
		l2.setForeground(Color.BLUE);
		l2.setPreferredSize(dimTitre);
		panel2.add(l2, BorderLayout.NORTH);
		zoneTexteLibelle.setLineWrap(true);
		zoneTexteLibelle.setMargin(new Insets(0, 3, 0, 3));
		JScrollPane scrollPaneActions = new JScrollPane(zoneTexteLibelle);
		panel2.add(scrollPaneActions, BorderLayout.CENTER);

		getContentPane().add(panel2, BorderLayout.CENTER);

		JPanel panel3 = new JPanel();
		panel3.setLayout(new BoxLayout(panel3, BoxLayout.Y_AXIS));
		JPanel panel3b = new JPanel();
		JLabel l3 = new JLabel("Fait associé à la réponse", JLabel.CENTER);
		l3.setForeground(Color.BLUE);
		panel3b.add(l3, BorderLayout.CENTER);
		panel3b.setOpaque(true);
		panel3.add(panel3b, BorderLayout.NORTH);

		JPanel comboBoxPane = new JPanel(); // use FlowLayout
		String comboBoxItems[] = {};
		DefaultComboBoxModel<String> cbm = new DefaultComboBoxModel<String>(
				comboBoxItems);
		combo = new JComboBox<String>();
		combo.setModel(cbm);
		combo.setEditable(false);
		comboBoxPane.add(combo);
		panel3.add(comboBoxPane, BorderLayout.CENTER);

		zoneTexteFait.setMargin(new Insets(0, 3, 0, 3));
		panel3.add(zoneTexteFait, BorderLayout.CENTER);

		JPanel panel = new JPanel();
		panel.add(boutonAnnuler, BorderLayout.WEST);
		panel.add(boutonOK, BorderLayout.EAST);
		panel.setBorder(BorderFactory.createLineBorder(Color.white));
		panel3.add(panel, BorderLayout.SOUTH);

		getContentPane().add(panel3, BorderLayout.SOUTH);

		zoneTexteLibelle.append(libelle);
		zoneTexteFait.setText(fait);
		loadCombo();

		installeMenus();

		// Register the Listeners
		registerEventListeners();

		// Resize the container to the preferred size
		pack();
	}

	private void registerEventListeners() {
		boutonOK.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent evt) {

				if (!sourceHTML.isEmpty()) {
					// Appel depuis menu droit contextuel : on remplace
					// sourceHTML par la nouvelle chaine;
					Platform.runLater(new Runnable() {
						public void run() {
							String htmlText = htmlEditor.getHtmlText();
							String targethtml = getChaine();
							htmlText = htmlText.replace(sourceHTML, targethtml);
							htmlEditor.setHtmlText(htmlText);
							fenQuestion.reload();
						}
					});
				} else {
					// Appel depuis bouton Choix :
					// on insère la nouvelle chaine à l'endroit du curseur :
					Platform.runLater(new Runnable() {
						public void run() {
							// This method is invoked on the JavaFX thread
							// http://stackoverflow.com/questions/2213376/how-to-find-cursor-position-in-a-contenteditable-div
							String jsCodeInsertHtml = "function insertHtmlAtCursor(html) {\n"
									+ "    var range, node;\n"
									+ "    document.body.focus();\n"
									+ "    if (window.getSelection && window.getSelection().getRangeAt) {\n"
									+ "        range = window.getSelection().getRangeAt(0);\n"
									+ "        node = range.createContextualFragment(html);\n"
									+ "        range.insertNode(node);\n"
									+ "    } else if (document.selection && document.selection.createRange) {\n"
									+ "        document.selection.createRange().pasteHTML(html);\n"
									+ "    }\n"
									+ "}insertHtmlAtCursor('####html####')";

							String html = "<p>" + getChaine();

							Node webNode = htmlEditor.lookup(".web-view");
							if (webNode instanceof WebView) {
								WebEngine engine = ((WebView) webNode)
										.getEngine();
								try {
									engine.executeScript(jsCodeInsertHtml
											.replace("####html####", html));
								} catch (JSException e) {
									// A JavaScript Exception Occured
									e.printStackTrace();
								}
								engine.getLocation();
								fenQuestion.reload();
							}
						}
					});
				}
				dispose();
			}
		});

		boutonAnnuler.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent evt) {
				dispose();
			}
		});

		combo.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent evt) {
				if (evt.getActionCommand().equals("comboBoxChanged")) {
					cliqueFait(combo.getSelectedItem());
				}
			}
		});

		ActionListener actionListenerEsc = new ActionListener() {
			public void actionPerformed(ActionEvent actionEvent) {
				dispose();
			}
		};

		KeyStroke stroke = KeyStroke.getKeyStroke(KeyEvent.VK_ESCAPE, 0);
		getRootPane().registerKeyboardAction(actionListenerEsc, stroke,
				JComponent.WHEN_IN_FOCUSED_WINDOW);
	}

	private String getChaine() {
		String html = "<input type=\"radio\" name=\"choix\" value=\"";
		html += getFait() + "\" id=\"" + getFait() + "\">";
		html += getLibelle();
		if (!sourceHTML.isEmpty()) {
			html = html + fin;
		}
		html = html + "</p>";
		return html;
	}

	private void detecteFin(String l) {
		String texte = sourceHTML;
		String a = ">" + l;
		String b = "</p>";
		int i, j;
		i = texte.indexOf(a);
		texte = texte.substring(i + a.length());
		j = texte.indexOf(b);
		fin = texte.substring(0, j);
	}

	private void loadCombo() {

		DefaultComboBoxModel<String> model = (DefaultComboBoxModel<String>) combo
				.getModel();

		if (!fenQuestion.getQuestion().formateFaitsAttendus().isEmpty()) {
			for (String s : fenQuestion.getQuestion().formateFaitsAttendus()
					.get(0)) {
				model.addElement(s);
			}
		}

		model.setSelectedItem(zoneTexteFait.getText());
	}

	protected void cliqueFait(Object object) {
		if (object != null) {
			zoneTexteFait.setText(object.toString());
		}
	}

	public String getLibelle() {
		return zoneTexteLibelle.getText();
	}

	public String getFait() {
		return zoneTexteFait.getText();
	}

	private void installeMenus() {
		FenEdiNoS ed = (FenEdiNoS) Edi.nos().fen();
		menuBar = ed.menuBarEdinos();
		menuBar.add(ed.menuBase());
		menuBar.add(fenQuestion.menuQuestion());
		Edi.nos().fen().setJMenuBar(menuBar);
		Edi.nos().fen().setVisible(true);
	}

	public void internalFrameActivated(InternalFrameEvent e) {
		Edi.nos().fen().setJMenuBar(menuBar);
		Edi.nos().fen().setVisible(true);
	}

	public void internalFrameDeactivated(InternalFrameEvent e) {
	}

	public void internalFrameClosed(InternalFrameEvent e) {
		fenQuestion.fermeChoix();
	}

	public void internalFrameClosing(InternalFrameEvent e) {
		ToolNos.fermeFen((JInternalFrame) this);
	}

	public void internalFrameDeiconified(InternalFrameEvent e) {
	}

	public void internalFrameIconified(InternalFrameEvent e) {
	}

	public void internalFrameOpened(InternalFrameEvent e) {
	}
}
