/*
  EdiNoS - Editeur de NoS
  Copyright (C) 2017-2020 - Luc Poittevin — Tous droits réservés.
  Contact : edinos@free.fr  -  https://www.edinos.org
  
  Ce programme est un logiciel libre ; vous pouvez le redistribuer ou le
  modifier suivant les termes de la “GNU General Public License” telle que
  publiée par la Free Software Foundation : soit la version 3 de cette
  licence, soit (à votre gré) toute version ultérieure.
  
  Ce programme est distribué dans l’espoir qu’il vous sera utile, mais SANS
  AUCUNE GARANTIE : sans même la garantie implicite de COMMERCIALISABILITÉ
  ni d’ADÉQUATION À UN OBJECTIF PARTICULIER. Consultez la Licence Générale
  Publique GNU pour plus de détails.
  
  Vous devriez avoir reçu une copie de la Licence Générale Publique GNU avec
  ce programme ; si ce n’est pas le cas, consultez :
  <http://www.gnu.org/licenses/>. 
 */

package nos.edit.html;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Cursor;
import java.awt.Dimension;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.KeyEvent;

import javafx.application.Platform;
import javafx.scene.Scene;
import javafx.scene.control.Control;

import javax.swing.BorderFactory;
import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.JButton;
import javax.swing.JComponent;
import javax.swing.JInternalFrame;
import javax.swing.JLabel;
import javax.swing.JMenu;
import javax.swing.JMenuBar;
import javax.swing.JMenuItem;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JTextArea;
import javax.swing.KeyStroke;
import javax.swing.event.InternalFrameEvent;
import javax.swing.event.InternalFrameListener;

import nos.edit.FenEdiNoS;
import nos.obj.Action;
import nos.obj.Edi;
import nos.obj.Message;
import nos.run.edit.FenAffiche;
import nos.utils.PrefsManager;
import nos.utils.ToolNos;

@SuppressWarnings("serial")
public class FenMessage extends JInternalFrame implements
		InternalFrameListener {
	private int largeurMin = 655; // nécessaire pour la toolbar de l'HTMLEditor

	private JButton boutonEnregistrer = new JButton("Enregistrer");

	private JButton boutonTester = new JButton("Tester");

	private JButton boutonFermer = new JButton("Fermer");

	private Message message;

	private JTextArea zoneTexteComm = new JTextArea();

	private HTMLEditorTabPane editorTapPane = null;

	private JMenuBar menuBar;

	public FenMessage(Message m) {
		super("Message <" + m.getNom() + ">", true, // resizable
				true, // closable
				true, // maximizable
				true);// iconifiable
		addInternalFrameListener(this);
		message = m;

		editorTapPane = new HTMLEditorTabPane();

		getContentPane().setLayout(new BorderLayout());

		JPanel panel = new JPanel();
		panel.setLayout(new BoxLayout(panel, BoxLayout.LINE_AXIS));
		panel.add(boutonEnregistrer);
		panel.add(boutonTester);
		panel.add(boutonFermer);
		panel.add(Box.createHorizontalGlue());
		panel.setBorder(BorderFactory.createLineBorder(Color.white));
		getContentPane().add(panel, BorderLayout.NORTH);

		JPanel panel2 = new JPanel();
		panel2.setLayout(new BoxLayout(panel2, BoxLayout.Y_AXIS));
		JPanel panel2b = new JPanel();
		JLabel l2 = new JLabel("Texte", JLabel.CENTER);
		l2.setForeground(Color.BLUE);
		panel2b.add(l2, BorderLayout.CENTER);
		panel2.add(panel2b);

		Edi.nos().fen().setCursor(Cursor.getPredefinedCursor(Cursor.WAIT_CURSOR));

		Platform.runLater(new Runnable() {
			public void run() {
				// This method is invoked on the JavaFX thread
				Control c = editorTapPane.init(false);
				editorTapPane.setText(message.getTexte());
				Scene scene = new Scene(c);
				editorTapPane.setScene(scene);
			}
		});

		panel2.add(editorTapPane);
		panel2.setPreferredSize(new Dimension(getLargeur(), getHauteur()));
		getContentPane().add(panel2, BorderLayout.CENTER);

		// installation des menus
		installeMenus();

		// Register the Listeners
		registerEventListeners();

		// Resize the container to the preferred size
		pack();
		
		Edi.nos().fen().setCursor(Cursor.getDefaultCursor());
	}

	private void installeMenus() {
		FenEdiNoS ed = (FenEdiNoS) Edi.nos().fen();
		menuBar = ed.menuBarEdinos();
		menuBar.add(ed.menuBase());
		menuBar.add(menuQuestion());
		Edi.nos().fen().setJMenuBar(menuBar);
		Edi.nos().fen().setVisible(true);
	}

	private void registerEventListeners() {
		boutonEnregistrer.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent evt) {
				enregistrer();
			}
		});

		boutonFermer.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent evt) {
				checkeModif();
				fermeMessage();
			}
		});

		boutonTester.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent evt) {
				checkeModif();
				FenAffiche f = new FenAffiche(message);
				message.getBase().ajouteFen(f);
				((FenEdiNoS) Edi.nos().fen())
						.ouvreInternalFrameSansPositionner(f);
				f.setVisible(true);
			}
		});

		ActionListener actionListenerEsc = new ActionListener() {
			public void actionPerformed(ActionEvent actionEvent) {
				checkeModif();
				fermeMessage();
			}
		};

		KeyStroke stroke = KeyStroke.getKeyStroke(KeyEvent.VK_ESCAPE, 0);
		getRootPane().registerKeyboardAction(actionListenerEsc, stroke,
				JComponent.WHEN_IN_FOCUSED_WINDOW);
	}

	protected String formateTexte() {
		String s = editorTapPane.getText();

		return s;
	}

	private void fermeMessage() {
		if (ToolNos.nomSysteme().equals("Mac")) {
			ToolNos.fermeFen((JInternalFrame) this);
		} else {
			dispose();
		} 
		((FenEdiNoS) Edi.nos().fen()).checkDerniereFen(); 
	}

	public void internalFrameActivated(InternalFrameEvent e) {
		Edi.nos().fen().setJMenuBar(menuBar);
		Edi.nos().fen().setVisible(true);
	}

	public void internalFrameDeactivated(InternalFrameEvent e) {
		((FenEdiNoS) Edi.nos().fen()).setMenuBar();
	}

	public void internalFrameClosed(InternalFrameEvent e) {
	}

	public void internalFrameClosing(InternalFrameEvent e) {
//		ToolNos.fermeFen((JInternalFrame) this);
		fermeMessage();
	}

	public void internalFrameDeiconified(InternalFrameEvent e) {
	}

	public void internalFrameIconified(InternalFrameEvent e) {
	}

	public void internalFrameOpened(InternalFrameEvent e) {
	}

	public JMenu menuQuestion() {

		JMenuItem menuItem;
		JMenu menu = new JMenu("Question");

		menuItem = new JMenuItem("Couper");
		menuItem.setAccelerator(KeyStroke.getKeyStroke(
				java.awt.event.KeyEvent.VK_X, ToolNos.renvoieModifiers()));
		menuItem.setEnabled(false);
		menu.add(menuItem);
		menuItem = new JMenuItem("Copier");
		menuItem.setAccelerator(KeyStroke.getKeyStroke(
				java.awt.event.KeyEvent.VK_C, ToolNos.renvoieModifiers()));
		menuItem.setEnabled(false);
		menu.add(menuItem);
		menuItem = new JMenuItem("Coller");
		menuItem.setAccelerator(KeyStroke.getKeyStroke(
				java.awt.event.KeyEvent.VK_V, ToolNos.renvoieModifiers()));
		menuItem.setEnabled(false);
		menu.add(menuItem);

		menu.addSeparator();
		menuItem = new JMenuItem("Voir les Références");
		menuItem.setAccelerator(KeyStroke.getKeyStroke(
				java.awt.event.KeyEvent.VK_R, ToolNos.renvoieModifiers()));
		menuItem.setEnabled(false);
		menu.add(menuItem);

		menu.addSeparator();
		menuItem = new JMenuItem("Fermer la fenêtre");
		menuItem.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				checkeModif();
				fermeMessage();
			}
		});
		menuItem.setAccelerator(KeyStroke.getKeyStroke(
				java.awt.event.KeyEvent.VK_W, ToolNos.renvoieModifiers()));
		menu.add(menuItem);

		return (menu);
	}

	public String getDescription() {
		return zoneTexteComm.getText();
	}

	public void init() {
	}

	public Action getMessage() {
		return message;
	}

	private int getLargeur() {
		int i = ((PrefsManager) Edi.nos().getPreferences())
				.getFenDialogueLargeur();
		if (i > largeurMin) {
			return i;
		} else {
			return largeurMin;
		}
	}

	private int getHauteur() {
		int i = ((PrefsManager) Edi.nos().getPreferences())
				.getFenDialogueHauteur();
		return (i + 110);
	}

	private void enregistrer() {
		message.setTexte(formateTexte());
	}

	private void checkeModif() {
		if (!message.getTexte().equals(formateTexte())) {
			int rep = JOptionPane
					.showConfirmDialog(
							Edi.nos().fen(),
							"Vous avez effectué des modifications : voulez-vous les enregistrer ?",
							"Enregistrer ?", JOptionPane.YES_NO_OPTION);
			if (rep == JOptionPane.YES_OPTION) {
				// Enregistrement confirmé
				enregistrer();
			}
		}
	}
}
