/*
  EdiNoS - Editeur de NoS
  Copyright (C) 2017-2021 - Luc Poittevin — Tous droits réservés.
  Contact : edinos@free.fr  -  https://www.edinos.org
  
  Ce programme est un logiciel libre ; vous pouvez le redistribuer ou le
  modifier suivant les termes de la “GNU General Public License” telle que
  publiée par la Free Software Foundation : soit la version 3 de cette
  licence, soit (à votre gré) toute version ultérieure.
  
  Ce programme est distribué dans l’espoir qu’il vous sera utile, mais SANS
  AUCUNE GARANTIE : sans même la garantie implicite de COMMERCIALISABILITÉ
  ni d’ADÉQUATION À UN OBJECTIF PARTICULIER. Consultez la Licence Générale
  Publique GNU pour plus de détails.
  
  Vous devriez avoir reçu une copie de la Licence Générale Publique GNU avec
  ce programme ; si ce n’est pas le cas, consultez :
  <http://www.gnu.org/licenses/>. 
 */

package nos.edit.html;

import java.awt.Cursor;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.KeyEvent;
import java.beans.PropertyVetoException;
import java.util.List;

import javafx.application.Platform;
import javafx.scene.Scene;
import javafx.scene.control.Control;

import javax.swing.BoxLayout;
import javax.swing.JComponent;
import javax.swing.JInternalFrame;
import javax.swing.JMenu;
import javax.swing.JMenuBar;
import javax.swing.JMenuItem;
import javax.swing.JOptionPane;
import javax.swing.KeyStroke;
import javax.swing.SwingUtilities;
import javax.swing.event.InternalFrameEvent;
import javax.swing.event.InternalFrameListener;

import nos.edit.FenActions;
import nos.edit.FenEdiNoS;
import nos.obj.Action;
import nos.obj.Edi;
import nos.obj.FaitAttendu;
import nos.obj.Question;
import nos.run.edit.FenDialogueQuest;
import nos.utils.ToolNos;

@SuppressWarnings("serial")
public class FenQuestion extends JInternalFrame implements
		InternalFrameListener {

	private Question question;
	
	private FenActions fenActions;

	private HTMLEditorTabPane editorTapPane = null;

	private PanelFaitsAction panelFaits = null;

	private JMenuBar menuBar;

	public FenQuestion(Question q) {
		super("Question <" + q.getNom() + ">", true, // resizable
				true, // closable
				true, // maximizable
				true);// iconifiable
		addInternalFrameListener(this);
		question = q;
		
		Edi.nos().fen().setCursor(Cursor.getPredefinedCursor(Cursor.WAIT_CURSOR));


		
		

		initComponents();
		initComponents2();

		// installation des menus
		installeMenus();

		// Resize the container to the preferred size
		pack();
		Edi.nos().fen().setCursor(Cursor.getDefaultCursor());
	}

	public FenQuestion(Question act, FenActions fa) {
		super("Question <" + act.getNom() + ">", true, // resizable
				true, // closable
				true, // maximizable
				true);// iconifiable
		addInternalFrameListener(this);
		question = act;
		fenActions = fa;
		
		Edi.nos().fen().setCursor(Cursor.getPredefinedCursor(Cursor.WAIT_CURSOR));

		initComponents();
		initComponents2();

		// installation des menus
		installeMenus();
		
		// Resize the container to the preferred size
		pack();
		Edi.nos().fen().setCursor(Cursor.getDefaultCursor());
		

	}

	private void initComponents2() {

		editorTapPane = new HTMLEditorTabPane();
		
		Platform.runLater(new Runnable() {
			public void run() {
				// This method is invoked on the JavaFX thread
				Control c = editorTapPane.init(true);
				editorTapPane.setText(question.getTexte());
				Scene scene = new Scene(c);
				editorTapPane.setScene(scene);
			}
		});

		jPanel3.add(editorTapPane);

		jTextArea1.setLineWrap(true);
		jTextArea1.setMargin(new Insets(0, 3, 0, 3));
		jTextArea1.append(question.getDescription());

		panelFaits = new PanelFaitsAction((Action) question);
		jPanel2.setLayout(new BoxLayout(jPanel2, BoxLayout.Y_AXIS));
		jPanel2.add(panelFaits);

		// Register the Listeners
		registerEventListeners();
	}
	
	public void reload() {
		editorTapPane.reload();
	}

	private void installeMenus() {
		FenEdiNoS ed = (FenEdiNoS) Edi.nos().fen();
		menuBar = ed.menuBarEdinos();
		menuBar.add(ed.menuBase());
		menuBar.add(menuQuestion());
		Edi.nos().fen().setJMenuBar(menuBar);
		Edi.nos().fen().setVisible(true);
	}

	private void registerEventListeners() {



		ActionListener actionListenerEsc = new ActionListener() {
			public void actionPerformed(ActionEvent actionEvent) {
				checkeModif();
				fermeQuestion();
			}
		};

		KeyStroke stroke = KeyStroke.getKeyStroke(KeyEvent.VK_ESCAPE, 0);
		getRootPane().registerKeyboardAction(actionListenerEsc, stroke,
				JComponent.WHEN_IN_FOCUSED_WINDOW);
	}

	protected void convertit() {
		
		question.convertitEn20();
		jTextArea1.setText(question.getDescription());
		panelFaits.updateModel();
	}

	protected String formateTexte() {

		String s = editorTapPane.getText();
		
//
		return s;
	}

	private void fermeQuestion() {
		if (editorTapPane != null) {
			editorTapPane.fermeChoix();
			editorTapPane.fermeSaisie();
		}
		
		if (fenActions != null) {
			fenActions.rafraichitLigneCourante();
		}
		if (ToolNos.nomSysteme().equals("Mac")) {
			ToolNos.fermeFen((JInternalFrame) this);
		} else {
			dispose();
		}
		((FenEdiNoS) Edi.nos().fen()).checkDerniereFen(); 
	}

	public void internalFrameActivated(InternalFrameEvent e) {
		Edi.nos().fen().setJMenuBar(menuBar);
		Edi.nos().fen().setVisible(true);
	}

	public void internalFrameDeactivated(InternalFrameEvent e) {
		((FenEdiNoS) Edi.nos().fen()).setMenuBar();
	}

	public void internalFrameClosed(InternalFrameEvent e) {
	}

	public void internalFrameClosing(InternalFrameEvent e) {
		fermeQuestion();
	}

	public void internalFrameDeiconified(InternalFrameEvent e) {
	}

	public void internalFrameIconified(InternalFrameEvent e) {
	}

	public void internalFrameOpened(InternalFrameEvent e) {
	}

	public JMenu menuQuestion() {

		JMenuItem menuItem;
		JMenu menu = new JMenu("Question");

		menuItem = new JMenuItem("Couper");
		menuItem.setAccelerator(KeyStroke.getKeyStroke(
				java.awt.event.KeyEvent.VK_X, ToolNos.renvoieModifiers()));
		menuItem.setEnabled(false);
		menu.add(menuItem);
		menuItem = new JMenuItem("Copier");
		menuItem.setAccelerator(KeyStroke.getKeyStroke(
				java.awt.event.KeyEvent.VK_C, ToolNos.renvoieModifiers()));
		menuItem.setEnabled(false);
		menu.add(menuItem);
		menuItem = new JMenuItem("Coller");
		menuItem.setAccelerator(KeyStroke.getKeyStroke(
				java.awt.event.KeyEvent.VK_V, ToolNos.renvoieModifiers()));
		menuItem.setEnabled(false);
		menu.add(menuItem);

		menu.addSeparator();
		menuItem = new JMenuItem("Voir les Références");
		menuItem.setAccelerator(KeyStroke.getKeyStroke(
				java.awt.event.KeyEvent.VK_R, ToolNos.renvoieModifiers()));
		menuItem.setEnabled(false);
		menu.add(menuItem);

		menu.addSeparator();
		menuItem = new JMenuItem("Fermer la fenêtre");
		menuItem.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				checkeModif();
				fermeQuestion();
			}
		});
		menuItem.setAccelerator(KeyStroke.getKeyStroke(
				java.awt.event.KeyEvent.VK_W, ToolNos.renvoieModifiers()));
		menu.add(menuItem);

		return (menu);
	}

	public String getDescription() {
		return jTextArea1.getText();
	}

	public void init() {
		// pour forcer editor à s'initialiser et prendre le focus
		// editor.init();
	}

	public Action getQuestion() {
		return question;
	}


	public void fermeChoix() {
		editorTapPane.reinitFenChoix();
		// on remet FenQuestion Au Premier Plan :
		try {
			this.setSelected(true);
		} catch (PropertyVetoException e) {
		}
	}

	public void fermeSaisie() {
		editorTapPane.reinitFenSaisie();
		// on remet FenQuestion Au Premier Plan :
		try {
			this.setSelected(true);
		} catch (PropertyVetoException e) {
		}
	}

	private void enregistrer() {
		question.setTexte(formateTexte());
		question.setDescription(jTextArea1.getText());
		question.setFaitsAttendus(panelFaits.listeFaitsAttendus());
	}

	private void checkeModif() {
		boolean enregistrementConfirmé = false;
		
		editorTapPane.getText();
		if (!question.getTexte().equals(editorTapPane.getText())) {
			int rep = JOptionPane
					.showConfirmDialog(
							Edi.nos().fen(),
							"Vous avez effectué des modifications sur le texte : voulez-vous les enregistrer ?",
							"Enregistrer ?", JOptionPane.YES_NO_OPTION);
			if (rep == JOptionPane.YES_OPTION) {
				// Enregistrement confirmé
				enregistrementConfirmé = true;
			}
		}
		if (!question.getDescription().equals(jTextArea1.getText())) {
			int rep = JOptionPane
					.showConfirmDialog(
							Edi.nos().fen(),
							"Vous avez effectué des modifications sur la description : voulez-vous les enregistrer ?",
							"Enregistrer ?", JOptionPane.YES_NO_OPTION);
			if (rep == JOptionPane.YES_OPTION) {
				// Enregistrement confirmé
				enregistrementConfirmé = true;
			}
		}
		if (faitsAttendusModifies()) {
			int rep = JOptionPane
					.showConfirmDialog(
							Edi.nos().fen(),
							"Vous avez effectué des modifications sur les faits attendus : voulez-vous les enregistrer ?",
							"Enregistrer ?", JOptionPane.YES_NO_OPTION);
			if (rep == JOptionPane.YES_OPTION) {
				// Enregistrement confirmé
				enregistrementConfirmé = true;
			}
		}
		if (enregistrementConfirmé) {
			// Enregistrement confirmé
			enregistrer();
		}
	}
	
	private boolean faitsAttendusModifies() {
		List<FaitAttendu> l = panelFaits.listeFaitsAttendus();
		for (FaitAttendu fa : l) {
			List<FaitAttendu> listeActuelle = question.getFaitsAttendus();
			if (listeActuelle.size() != l.size()) {
				return true;
			}
			boolean faitTrouve = false;
			for (FaitAttendu faitActuel : listeActuelle) {
				if (faitActuel.getNom().equals(fa.getNom())) {
					faitTrouve = true;
					if ((!faitActuel.getType().equals(fa.getType())) || (!faitActuel.getDescription().equals(fa.getDescription()))) {
						// différence..
						return true;
					}
				}
			}
			if (!faitTrouve) {
				return true;
			}
		}
		return false;
	}

	public boolean cliqueChoixPermis() {
		if ((question.getTexte().contains("input type=\"text"))
				|| (editorTapPane.getText().contains("input type=\"text"))) {
			SwingUtilities.invokeLater(new Runnable() {
				@Override
				public void run() {
					Edi.nos()
							.message(
									"Il y a déjà un champ de saisie dans cette question. Vous ne pouvez pas ajouter de champ de choix de réponse...");
				}
			});
			return false;
		}
		return true;
	}

	public boolean cliqueChoixSaisie() {
		if ((question.getTexte().contains("input type=\"radio"))
				|| (editorTapPane.getText().contains("input type=\"radio"))) {
			SwingUtilities.invokeLater(new Runnable() {
				@Override
				public void run() {
					Edi.nos()
							.message(
									"Il y a déjà un champ pour choisir une réponse dans cette question. Vous ne pouvez pas ajouter de champ de saisie...");
				}
			});
			return false;
		}
		if ((question.getTexte().contains("input type=\"text"))
				|| (editorTapPane.getText().contains("input type=\"text"))) {
			SwingUtilities.invokeLater(new Runnable() {
				@Override
				public void run() {
					Edi.nos()
							.message(
									"Un seul champ de saisie par question n'est possible...");
				}
			});
			return false;
		}
		return true;
	}

	private void jButtonEnregistrerActionPerformed(
			java.awt.event.ActionEvent evt) {
		enregistrer();
	}

	private void jButtonTesterActionPerformed(java.awt.event.ActionEvent evt) {
		checkeModif();
		FenDialogueQuest f = new FenDialogueQuest(question);
		question.getBase().ajouteFen(f);
		((FenEdiNoS) Edi.nos().fen()).ouvreInternalFrameSansPositionner(f);
		f.setVisible(true);
	}

	private void jButtonFermerActionPerformed(java.awt.event.ActionEvent evt) {
		checkeModif();
		fermeQuestion();
	}

	// <editor-fold defaultstate="collapsed" desc="Generated Code">
	private void initComponents() {

		jPanel1 = new javax.swing.JPanel();
		jButtonEnregistrer = new javax.swing.JButton();
		jButtonFermer = new javax.swing.JButton();
		jButtonTester = new javax.swing.JButton();
		jPanel3 = new javax.swing.JPanel();
		jPanel2 = new javax.swing.JPanel();
		jPanel4 = new javax.swing.JPanel();
		jScrollPane1 = new javax.swing.JScrollPane();
		jTextArea1 = new javax.swing.JTextArea();
		jLabel1 = new javax.swing.JLabel();

//		setDefaultCloseOperation(javax.swing.WindowConstants.DISPOSE_ON_CLOSE);

		jPanel1.setBorder(javax.swing.BorderFactory.createEtchedBorder());

		jButtonEnregistrer.setText("Enregistrer");
		jButtonEnregistrer.setFocusable(false);
		jButtonEnregistrer.setIgnoreRepaint(true);
		jButtonEnregistrer.setMaximumSize(new java.awt.Dimension(60, 30));
		jButtonEnregistrer
				.addActionListener(new java.awt.event.ActionListener() {
					public void actionPerformed(java.awt.event.ActionEvent evt) {
						jButtonEnregistrerActionPerformed(evt);
					}
				});

		jButtonFermer.setText("Fermer");
		jButtonFermer.setFocusable(false);
		jButtonFermer.setIgnoreRepaint(true);
		jButtonFermer.setMaximumSize(new java.awt.Dimension(60, 30));
		jButtonFermer.addActionListener(new java.awt.event.ActionListener() {
			public void actionPerformed(java.awt.event.ActionEvent evt) {
				jButtonFermerActionPerformed(evt);
			}
		});

		jButtonTester.setText("Tester");
		jButtonTester.addActionListener(new java.awt.event.ActionListener() {
			public void actionPerformed(java.awt.event.ActionEvent evt) {
				jButtonTesterActionPerformed(evt);
			}
		});

		org.jdesktop.layout.GroupLayout jPanel1Layout = new org.jdesktop.layout.GroupLayout(
				jPanel1);
		jPanel1.setLayout(jPanel1Layout);
		jPanel1Layout.setHorizontalGroup(jPanel1Layout.createParallelGroup(
				org.jdesktop.layout.GroupLayout.LEADING).add(
				jPanel1Layout
						.createSequentialGroup()
						.add(jButtonEnregistrer,
								org.jdesktop.layout.GroupLayout.PREFERRED_SIZE,
								100,
								org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
						.addPreferredGap(
								org.jdesktop.layout.LayoutStyle.RELATED)
						.add(jButtonTester)
						.addPreferredGap(
								org.jdesktop.layout.LayoutStyle.RELATED)
						.add(jButtonFermer,
								org.jdesktop.layout.GroupLayout.PREFERRED_SIZE,
								org.jdesktop.layout.GroupLayout.DEFAULT_SIZE,
								org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
						.addContainerGap(
								org.jdesktop.layout.GroupLayout.DEFAULT_SIZE,
								Short.MAX_VALUE)));
		jPanel1Layout
				.setVerticalGroup(jPanel1Layout
						.createParallelGroup(
								org.jdesktop.layout.GroupLayout.LEADING)
						.add(jPanel1Layout
								.createSequentialGroup()
								.add(jPanel1Layout
										.createParallelGroup(
												org.jdesktop.layout.GroupLayout.BASELINE)
										.add(jButtonEnregistrer,
												org.jdesktop.layout.GroupLayout.DEFAULT_SIZE,
												org.jdesktop.layout.GroupLayout.DEFAULT_SIZE,
												Short.MAX_VALUE)
										.add(jButtonFermer,
												org.jdesktop.layout.GroupLayout.DEFAULT_SIZE,
												org.jdesktop.layout.GroupLayout.DEFAULT_SIZE,
												Short.MAX_VALUE)
										.add(jButtonTester)).addContainerGap()));

		jPanel3.setLayout(new javax.swing.BoxLayout(jPanel3,
				javax.swing.BoxLayout.LINE_AXIS));

		org.jdesktop.layout.GroupLayout jPanel2Layout = new org.jdesktop.layout.GroupLayout(
				jPanel2);
		jPanel2.setLayout(jPanel2Layout);
		jPanel2Layout.setHorizontalGroup(jPanel2Layout.createParallelGroup(
				org.jdesktop.layout.GroupLayout.LEADING).add(0, 0,
				Short.MAX_VALUE));
		jPanel2Layout.setVerticalGroup(jPanel2Layout.createParallelGroup(
				org.jdesktop.layout.GroupLayout.LEADING).add(0, 214,
				Short.MAX_VALUE));

		jTextArea1.setColumns(20);
		jTextArea1.setRows(5);
		jScrollPane1.setViewportView(jTextArea1);

		jLabel1.setForeground(new java.awt.Color(51, 51, 255));
		jLabel1.setText("Description de l'action");

		org.jdesktop.layout.GroupLayout jPanel4Layout = new org.jdesktop.layout.GroupLayout(
				jPanel4);
		jPanel4.setLayout(jPanel4Layout);
		jPanel4Layout
				.setHorizontalGroup(jPanel4Layout
						.createParallelGroup(
								org.jdesktop.layout.GroupLayout.LEADING)
						.add(jPanel4Layout
								.createSequentialGroup()
								.addContainerGap()
								.add(jPanel4Layout
										.createParallelGroup(
												org.jdesktop.layout.GroupLayout.LEADING)
										.add(org.jdesktop.layout.GroupLayout.TRAILING,
												jScrollPane1,
												org.jdesktop.layout.GroupLayout.PREFERRED_SIZE,
												0, Short.MAX_VALUE)
										.add(org.jdesktop.layout.GroupLayout.TRAILING,
												jLabel1,
												org.jdesktop.layout.GroupLayout.DEFAULT_SIZE,
												315, Short.MAX_VALUE))
								.addContainerGap()));
		jPanel4Layout.setVerticalGroup(jPanel4Layout.createParallelGroup(
				org.jdesktop.layout.GroupLayout.LEADING).add(
				jPanel4Layout
						.createSequentialGroup()
						.addContainerGap()
						.add(jLabel1,
								org.jdesktop.layout.GroupLayout.PREFERRED_SIZE,
								28,
								org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
						.addPreferredGap(
								org.jdesktop.layout.LayoutStyle.RELATED)
						.add(jScrollPane1,
								org.jdesktop.layout.GroupLayout.DEFAULT_SIZE,
								160, Short.MAX_VALUE).addContainerGap()));

		org.jdesktop.layout.GroupLayout layout = new org.jdesktop.layout.GroupLayout(
				getContentPane());
		getContentPane().setLayout(layout);
		layout.setHorizontalGroup(layout
				.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
				.add(org.jdesktop.layout.GroupLayout.TRAILING, jPanel1,
						org.jdesktop.layout.GroupLayout.DEFAULT_SIZE,
						org.jdesktop.layout.GroupLayout.DEFAULT_SIZE,
						Short.MAX_VALUE)
				.add(org.jdesktop.layout.GroupLayout.TRAILING,
						layout.createSequentialGroup()
								.addContainerGap()
								.add(jPanel3,
										org.jdesktop.layout.GroupLayout.PREFERRED_SIZE,
										655,
										org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
								.addPreferredGap(
										org.jdesktop.layout.LayoutStyle.RELATED)
								.add(layout
										.createParallelGroup(
												org.jdesktop.layout.GroupLayout.LEADING)
										.add(jPanel4,
												org.jdesktop.layout.GroupLayout.DEFAULT_SIZE,
												org.jdesktop.layout.GroupLayout.DEFAULT_SIZE,
												Short.MAX_VALUE)
										.add(jPanel2,
												org.jdesktop.layout.GroupLayout.DEFAULT_SIZE,
												org.jdesktop.layout.GroupLayout.DEFAULT_SIZE,
												Short.MAX_VALUE))
								.addContainerGap()));
		layout.setVerticalGroup(layout
				.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
				.add(layout
						.createSequentialGroup()
						.add(jPanel1,
								org.jdesktop.layout.GroupLayout.PREFERRED_SIZE,
								32,
								org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
						.addPreferredGap(
								org.jdesktop.layout.LayoutStyle.RELATED)
						.add(layout
								.createParallelGroup(
										org.jdesktop.layout.GroupLayout.LEADING)
								.add(layout
										.createSequentialGroup()
										.add(jPanel4,
												org.jdesktop.layout.GroupLayout.DEFAULT_SIZE,
												org.jdesktop.layout.GroupLayout.DEFAULT_SIZE,
												Short.MAX_VALUE)
										.addPreferredGap(
												org.jdesktop.layout.LayoutStyle.RELATED)
										.add(jPanel2,
												org.jdesktop.layout.GroupLayout.PREFERRED_SIZE,
												org.jdesktop.layout.GroupLayout.DEFAULT_SIZE,
												org.jdesktop.layout.GroupLayout.PREFERRED_SIZE))
								.add(jPanel3,
										org.jdesktop.layout.GroupLayout.DEFAULT_SIZE,
										org.jdesktop.layout.GroupLayout.DEFAULT_SIZE,
										Short.MAX_VALUE)).addContainerGap()));

		pack();
	}// </editor-fold>

	// Variables declaration - do not modify
	private javax.swing.JButton jButtonEnregistrer;
	private javax.swing.JButton jButtonFermer;
	private javax.swing.JButton jButtonTester;
	private javax.swing.JLabel jLabel1;
	private javax.swing.JPanel jPanel1;
	private javax.swing.JPanel jPanel2;
	private javax.swing.JPanel jPanel3;
	private javax.swing.JPanel jPanel4;
	private javax.swing.JScrollPane jScrollPane1;
	private javax.swing.JTextArea jTextArea1;
	// End of variables declaration

}
