/*
  EdiNoS - Editeur de NoS
  Copyright (C) 2017-2021 - Luc Poittevin — Tous droits réservés.
  Contact : edinos@free.fr  -  https://www.edinos.org
  
  Ce programme est un logiciel libre ; vous pouvez le redistribuer ou le
  modifier suivant les termes de la “GNU General Public License” telle que
  publiée par la Free Software Foundation : soit la version 3 de cette
  licence, soit (à votre gré) toute version ultérieure.
  
  Ce programme est distribué dans l’espoir qu’il vous sera utile, mais SANS
  AUCUNE GARANTIE : sans même la garantie implicite de COMMERCIALISABILITÉ
  ni d’ADÉQUATION À UN OBJECTIF PARTICULIER. Consultez la Licence Générale
  Publique GNU pour plus de détails.
  
  Vous devriez avoir reçu une copie de la Licence Générale Publique GNU avec
  ce programme ; si ce n’est pas le cas, consultez :
  <http://www.gnu.org/licenses/>. 
 */

package nos.edit.html;

import javax.swing.JInternalFrame;
import javax.swing.SwingUtilities;

import org.w3c.dom.NodeList;
import org.w3c.dom.html.HTMLElement;

import com.sun.webkit.dom.HTMLElementImpl;
import com.sun.webkit.dom.HTMLFontElementImpl;
import com.sun.webkit.dom.HTMLParagraphElementImpl;

import netscape.javascript.JSException;
import netscape.javascript.JSObject;
import nos.edit.FenEdiNoS;
import nos.obj.Edi;
import javafx.embed.swing.JFXPanel;
import javafx.event.ActionEvent;
import javafx.event.Event;
import javafx.event.EventHandler;
import javafx.scene.Node;
import javafx.scene.control.Button;
import javafx.scene.control.ContextMenu;
import javafx.scene.control.Control;
import javafx.scene.control.MenuItem;
import javafx.scene.control.Tab;
import javafx.scene.control.TabPane;
import javafx.scene.control.TextArea;
import javafx.scene.control.ToolBar;
import javafx.scene.control.Tooltip;
import javafx.scene.input.MouseButton;
import javafx.scene.input.MouseEvent;
import javafx.scene.web.HTMLEditor;
import javafx.scene.web.WebEngine;
import javafx.scene.web.WebView;

@SuppressWarnings("serial")
public class HTMLEditorTabPane extends JFXPanel {

	private HTMLEditor htmlEditor = null;
	private Tab tabHTML, tabTexte;
	private JInternalFrame fenChoix;
	private JInternalFrame fenSaisie;
	private TextArea htmlCode;
	private WebEngine engine;
	private String libelle;
	private String faitCourant;
	private String source;
	private ContextMenu menuChoix, menuSaisie;
	private double posX;
	private double posY;

	public Control init(boolean avecChoix) {
		htmlEditor = new HTMLEditor();
		if (avecChoix) {
			addButton();
			contextMenu();
		}
		htmlCode = new TextArea();

		TabPane tabPane = new TabPane();
		tabTexte = new Tab();
		tabTexte.setText("Texte");
		tabTexte.setClosable(false);
		tabTexte.setContent(htmlEditor);
		tabTexte.setOnSelectionChanged(new EventHandler<Event>() {
			@Override
			public void handle(Event t) {
				if (tabTexte.isSelected()) {
					htmlEditor.setHtmlText(htmlCode.getText());
				}
			}
		});
		tabPane.getTabs().add(tabTexte);
		tabHTML = new Tab();
		tabHTML.setText("HTML");
		tabHTML.setClosable(false);
		htmlCode.setWrapText(true);
		tabHTML.setContent(htmlCode);
		tabHTML.setOnSelectionChanged(new EventHandler<Event>() {
			@Override
			public void handle(Event t) {
				if (tabHTML.isSelected()) {
					htmlCode.setText(htmlEditor.getHtmlText());
				}
			}
		});

		tabPane.getTabs().add(tabHTML);
		return tabPane;
	}

	public void addButton() {
		// add custom buttons to the top toolbar.
		Node node = htmlEditor.lookup(".top-toolbar");
		if (node instanceof ToolBar) {
			ToolBar bar = (ToolBar) node;
			Button boutonChoix = new Button("Choix");
			boutonChoix.setTooltip(new Tooltip("Insère un choix"));
			bar.getItems().add(boutonChoix);
			Button boutonSaisie = new Button("Saisie");
			boutonSaisie.setTooltip(new Tooltip("Insère un champ de saisie"));
			bar.getItems().add(boutonSaisie);
			boutonChoix.setOnAction(new EventHandler<ActionEvent>() {
				@Override
				public void handle(ActionEvent arg0) {
					source = "";
					libelle = "";
					faitCourant = "";
					if (getFenetre().cliqueChoixPermis()) {
						ouvreChoix();
					}
				}
			});
			boutonSaisie.setOnAction(new EventHandler<ActionEvent>() {
				@Override
				public void handle(ActionEvent arg0) {
					source = "";
					libelle = "";
					faitCourant = "";
					if (getFenetre().cliqueChoixSaisie()) {
						ouvreSaisie();
					}
				}
			});
		}
	}

	protected void ouvreChoix() {
		SwingUtilities.invokeLater(new Runnable() {
			@Override
			public void run() {
				if (fenChoix == null) {
					fenChoix = new FenChoix(source, htmlEditor, libelle,
							faitCourant, getFenetre());
					Edi.nos().getBaseCourante().ajouteFen(fenChoix);
					((FenEdiNoS) Edi.nos().fen()).ouvreInternalFrame(fenChoix);
				}
				fenChoix.setVisible(true);
			}
		});
	}

	protected void ouvreSaisie() {
		SwingUtilities.invokeLater(new Runnable() {
			@Override
			public void run() {
				if (fenSaisie == null) {
					fenSaisie = new FenSaisie(source, htmlEditor, "20",
							faitCourant, getFenetre());
					Edi.nos().getBaseCourante().ajouteFen(fenSaisie);
					((FenEdiNoS) Edi.nos().fen()).ouvreInternalFrame(fenSaisie);
				}
				fenSaisie.setVisible(true);
			}
		});
	}

	protected FenQuestion getFenetre() {
		return (FenQuestion) this.getRootPane().getParent();
	}

	public void contextMenu() {
		
		Node webNode = htmlEditor.lookup(".web-view");
		if (webNode instanceof WebView) {
			((WebView) webNode).setContextMenuEnabled(false);
			engine = ((WebView) webNode).getEngine();
		}

		menuChoix = new ContextMenu();
		MenuItem menuItem1 = new MenuItem("Modifier le choix");
		menuItem1.setOnAction(new EventHandler<javafx.event.ActionEvent>() {
			@Override
			public void handle(javafx.event.ActionEvent actionEvent) {
				ouvreChoix();
			}
		});
		menuChoix.getItems().add(menuItem1);

		menuSaisie = new ContextMenu();
		MenuItem menuItem2 = new MenuItem("Modifier le champ de saisie");
		menuItem2.setOnAction(new EventHandler<javafx.event.ActionEvent>() {
			@Override
			public void handle(javafx.event.ActionEvent actionEvent) {
				ouvreSaisie();
			}
		});

		menuSaisie.getItems().add(menuItem2);

		htmlEditor.setOnMouseClicked(new EventHandler<MouseEvent>() {
			@Override
			public void handle(MouseEvent mouse) {
				if ((mouse.getButton() == MouseButton.SECONDARY)
				// pour trackpads Mac :
						|| mouse.isControlDown()) {
					posX = mouse.getScreenX();
					posY = mouse.getScreenY();
					engine.getLocation();
					detecteNode();
				} else {
					if (menuChoix != null) {
						menuChoix.hide();
					}
					if (menuSaisie != null) {
						menuSaisie.hide();
					}
				}
			}
		});
	}

	private void analyse(JSObject jso) {
		if (jso instanceof HTMLElement) {
			HTMLElement el = (HTMLElement) jso;

			org.w3c.dom.Node node = (org.w3c.dom.Node) el;


			while ((node != null) && !(node instanceof HTMLElementImpl)
					&& !(node instanceof HTMLParagraphElementImpl)
					&& !(node instanceof HTMLFontElementImpl)) {
				node = node.getParentNode();
			}
			if (node == null) {
			} else { // Cas où choix formatté : HTMLParagraphElementImpl...
				NodeList l = node.getChildNodes();
				String f = "vide";
				boolean trouveInput = false;
				for (int i = 0, size = l.getLength(); i < size; i++) {
					org.w3c.dom.Node nl = l.item(i);
					if (nl.getNodeName().equals("INPUT")) {
						trouveInput = true;
					} else if (nl.getNodeName().equals("#text")) {
						f = nl.getTextContent();
					} else if (nl.getNodeName().equals("FONT")) {
						// detection de input text ...
						NodeList l2 = nl.getChildNodes();
						for (int j = 0, size2 = l2.getLength(); j < size2; j++) {
							org.w3c.dom.Node nl2 = l2.item(j);
							if (nl2.getNodeName().equals("INPUT")) {
								identifieSaisie(f);
							}
						}
					}
				}

				if (trouveInput) {
					// System.out.println(f);
					identifieChoix(f);
				}
			}
		}
	}

	private void identifieChoix(String f) {
		String texte = htmlEditor.getHtmlText();

		String a = "<input";
		while (texte.indexOf(a) >= 0) {
			texte = positionneTexte(texte, a);
			texte = traiteTagInputRadio(texte, f);
		}
	}

	private void identifieSaisie(String f) {
		String texte = htmlEditor.getHtmlText();

		String a = "<input";
		while (texte.indexOf(a) >= 0) {
			texte = positionneTexte(texte, a);
			texte = traiteTagInputText(texte, f);
		}
	}

	private String positionneTexte(String texte, String a) {
		int i;
		i = texte.indexOf(a);
		if (i == -1) {
			return texte;
		} else {
			texte = texte.substring(i);
		}
		return texte;
	}

	public String traiteTagInputRadio(String texte, String f) {
		String b = "</p>";
		int i;


		String fait = detecteFait(texte);
		String contenu = detecteContenu(texte, fait);

		i = texte.indexOf(b);
		String t = texte.substring(0, i + b.length());

		if (f.equals(contenu)) {
			source = t;
			libelle = f;
			faitCourant = fait;
			menuChoix.show(htmlEditor, posX, posY);
		}
		return texte.substring(i + b.length());
	}

	public String traiteTagInputText(String texte, String f) {
		String b = "</p>";
		int i;


		String fait = detecteFait(texte);

		i = texte.indexOf(b);
		String t = texte.substring(0, i + b.length());

		source = t;
		faitCourant = fait;
		menuSaisie.show(htmlEditor, posX, posY);
		return texte.substring(i + b.length());
	}

	public void setText(String s) {
		htmlEditor.setHtmlText(s);
	}
	
	public void reload() {
		htmlCode.setText(htmlEditor.getHtmlText());
		htmlEditor.setHtmlText(htmlCode.getText());
	}

	public String getText() {
		if (tabHTML.isSelected()) {
			return htmlCode.getText();
		} else {
			return htmlEditor.getHtmlText();
		}
	}

	public String detecteFait(String texte) {
		String a = "id=\"";
		String b = "\"";
		int i, j;
		i = texte.indexOf(a);
		texte = texte.substring(i + a.length());
		j = texte.indexOf(b);
		return texte.substring(0, j);
	}

	public String detecteSize(String texte) {
		String a = "size=\"";
		String b = "\"";
		int i, j;
		i = texte.indexOf(a);
		texte = texte.substring(i + a.length());
		j = texte.indexOf(b);
		return texte.substring(0, j);
	}

	public String detecteContenu(String texte, String nomFait) {
		String a = ">";
		String b = "</";
		int i, j;
		i = texte.indexOf(a);
		texte = texte.substring(i + a.length());
		j = texte.indexOf(b);
		return texte.substring(0, j);
	}

	private void detecteNode() {
		String jsTest = "function detecteNode() {\n"
				+ " var range, node1, node2, node3;\n"
				+ " range = window.getSelection().getRangeAt(0);\n"
				+ " node = range.endContainer.parentNode;\n"



				+ "return node;"
				+ "}detecteNode()";
		try {
			Object o = engine.executeScript(jsTest);
			if (o instanceof JSObject) {
				JSObject jso = (JSObject) o;
				analyse(jso);
			}
		} catch (JSException e) {
			// A JavaScript Exception Occured
			e.printStackTrace();
		}
	}

	public void fermeChoix() {
		if (fenChoix != null) {
			fenChoix.dispose();
		}
	}

	public void fermeSaisie() {
		if (fenSaisie != null) {
			fenSaisie.dispose();
		}
	}

	public void reinitFenChoix() {
		fenChoix = null;
	}

	public void reinitFenSaisie() {
		fenSaisie = null;
	}
}
