/*
  EdiNoS - Editeur de NoS
  Copyright (C) 2018-2019 - Luc Poittevin — Tous droits réservés.
  Contact : edinos@free.fr  -  https://www.edinos.org
  
  Ce programme est un logiciel libre ; vous pouvez le redistribuer ou le
  modifier suivant les termes de la “GNU General Public License” telle que
  publiée par la Free Software Foundation : soit la version 3 de cette
  licence, soit (à votre gré) toute version ultérieure.
  
  Ce programme est distribué dans l’espoir qu’il vous sera utile, mais SANS
  AUCUNE GARANTIE : sans même la garantie implicite de COMMERCIALISABILITÉ
  ni d’ADÉQUATION À UN OBJECTIF PARTICULIER. Consultez la Licence Générale
  Publique GNU pour plus de détails.
  
  Vous devriez avoir reçu une copie de la Licence Générale Publique GNU avec
  ce programme ; si ce n’est pas le cas, consultez :
  <http://www.gnu.org/licenses/>. 
 */

package nos.edit.html;

import java.awt.Component;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.List;

import javax.swing.AbstractCellEditor;
import javax.swing.DefaultCellEditor;
import javax.swing.Icon;
import javax.swing.ImageIcon;
import javax.swing.JButton;
import javax.swing.JComboBox;
import javax.swing.JTable;
import javax.swing.table.TableCellEditor;
import javax.swing.table.TableCellRenderer;

import nos.edit.FenDescriptionFait;
import nos.edit.FenEdiNoS;
import nos.obj.Action;
import nos.obj.Edi;
import nos.obj.FaitAttendu;

@SuppressWarnings("serial")
public class PanelFaitsAction extends javax.swing.JPanel {

	private JComboBox<String> listeTypes;

	private Action action;

	/**
	 * Creates new form PanelFaitsAction
	 */
	public PanelFaitsAction(Action a) {
		action = a;
		initComponents();
		initComponents2();
	}

	private void initComponents2() {

		jTable1.setRowSelectionAllowed(false);
		jTable1.setModel(new TableFaitsEdit(action.getFaitsAttendus()));

		listeTypes = new JComboBox<String>(
				((TableFaitsEdit) jTable1.getModel()).getListeType());

		java.net.URL imgURL = getClass().getResource("/icones/add.png");
		jButton1.setIcon(new ImageIcon(imgURL));
		jButton1.setMargin(new Insets(0, 0, 0, 0));
		jButton1.setToolTipText("Ajouter un fait");

		jTable1.getColumnModel().getColumn(0).setPreferredWidth(20);
		jTable1.getColumnModel().getColumn(0).setMaxWidth(20);
		jTable1.getColumnModel().getColumn(3).setPreferredWidth(20);
		jTable1.getColumnModel().getColumn(3).setMaxWidth(20);
		jTable1.getColumnModel().getColumn(0)
				.setCellRenderer(new ButtonRenderer());
		jTable1.getColumnModel().getColumn(0)
				.setCellEditor(new ButtonEditor(jTable1));
		jTable1.getColumnModel().getColumn(2)
				.setCellEditor(new DefaultCellEditor(listeTypes));
		jTable1.getColumnModel().getColumn(2)
				.setCellRenderer(new ComboTableCellRenderer());
		jTable1.getColumnModel().getColumn(3)
				.setCellRenderer(new ButtonRenderer());
		jTable1.getColumnModel().getColumn(3)
				.setCellEditor(new ButtonEditor(jTable1));
	}

	public List<FaitAttendu> listeFaitsAttendus() {
		return ((TableFaitsEdit) jTable1.getModel())
				.construitListeFaitsAttendus();
	}

	public void updateModel() {
		jTable1.setModel(new TableFaitsEdit(action.getFaitsAttendus()));
		jTable1.repaint();
	}

	private void jButton1ActionPerformed(java.awt.event.ActionEvent evt) {
		// clic sur Bouton "Ajouter un fait":

		FaitAttendu f = new FaitAttendu("");
		f.setType("Proposition"); // Type Proposition par défaut
		((TableFaitsEdit) jTable1.getModel()).ajouteData(f);
		((TableFaitsEdit) jTable1.getModel()).fireTableDataChanged();
	}

	public class ButtonRenderer extends JButton implements TableCellRenderer {

		public ButtonRenderer() {
			setOpaque(true);
		}

		public Component getTableCellRendererComponent(JTable table,
				Object value, boolean isSelected, boolean hasFocus, int row,
				int column) {
			setIcon((Icon) value);

			switch (column) {
			case 0: // colonne Bouton Supprimer
				setToolTipText("Supprimer le fait");
				return this;
			case 3: // colonne Bouton Voir
				setToolTipText("Voir la description du fait");
				return this;
			}

			return this;
		}
	}

	public void ouvreDescription(int row) {
		Object[] tab = ((TableFaitsEdit) jTable1.getModel()).listeFaits
				.get(row);
		FenDescriptionFait f = new FenDescriptionFait((String) tab[4], this);
		action.getBase().ajouteFen(f);
		((FenEdiNoS) Edi.nos().fen()).ouvreInternalFrame(f);
		f.init();
	}

	public void rafraichitLigneCourante(String desc) {
		Object[] tab = ((TableFaitsEdit) jTable1.getModel()).listeFaits
				.get(jTable1.getSelectedRow());
		tab[4] = desc;

		if (Edi.nos().getBaseCourante().contientQuestion(action.getNom())) {
			((FenEdiNoS) Edi.nos().fen()).activeFen("Question <"
					+ action.getNom() + ">");
		} else { // c'est une procédure
			((FenEdiNoS) Edi.nos().fen()).activeFen("Procédure <"
					+ action.getNom() + ">");
		}
	}

	class ButtonEditor extends AbstractCellEditor implements TableCellEditor {
		protected JButton button;
		protected Object editorValue;
		protected JTable table;

		public ButtonEditor(JTable t) {
			super();
			table = t;
			button = new JButton();
			button.setOpaque(true);
			button.addActionListener(new ActionListener() {
				public void actionPerformed(ActionEvent arg0) {
					int row = table.convertRowIndexToModel(table
							.getEditingRow());
					int col = table.convertColumnIndexToModel(table
							.getEditingColumn());
					fireEditingStopped();
					if (col == 0) {
						((TableFaitsEdit) table.getModel()).supprime(row);
					} else if (col == 3) {
						ouvreDescription(row);
					}
				}
			});
		}

		public Object getCellEditorValue() {
			return editorValue;
		}

		public Component getTableCellEditorComponent(JTable table,
				Object value, boolean isSelected, int row, int column) {
			if (value == null) {
				button.setText("");
				button.setIcon(null);
			} else if (value instanceof Icon) {
				button.setText("");
				button.setIcon((Icon) value);
			} else {
				button.setText(value.toString());
				button.setIcon(null);
			}
			this.editorValue = value;
			return button;
		}
	}

	class ComboTableCellRenderer extends JComboBox<Object> implements
			TableCellRenderer {

		public Component getTableCellRendererComponent(JTable table,
				Object value, boolean isSelected, boolean hasFocus, int row,
				int column) {

			removeAllItems();
			if (value != null) {
				addItem(value);
			}
			setSelectedItem(value);
			return this;
		}
	}

	// <editor-fold defaultstate="collapsed" desc="Generated Code">
	private void initComponents() {

		jScrollPane1 = new javax.swing.JScrollPane();
		jTable1 = new javax.swing.JTable();
		jButton1 = new javax.swing.JButton();
		jLabel1 = new javax.swing.JLabel();

		jTable1.setModel(new javax.swing.table.DefaultTableModel(
				new Object[][] { { null, null, null, null },
						{ null, null, null, null }, { null, null, null, null },
						{ null, null, null, null } }, new String[] { "", "Nom",
						"Type", "" }) {
			Class[] types = new Class[] { java.lang.Object.class,
					java.lang.String.class, java.lang.String.class,
					java.lang.Object.class };

			public Class getColumnClass(int columnIndex) {
				return types[columnIndex];
			}
		});
		jScrollPane1.setViewportView(jTable1);
		if (jTable1.getColumnModel().getColumnCount() > 0) {
			jTable1.getColumnModel().getColumn(0).setResizable(false);
			jTable1.getColumnModel().getColumn(0).setPreferredWidth(20);
			jTable1.getColumnModel().getColumn(3).setResizable(false);
			jTable1.getColumnModel().getColumn(3).setPreferredWidth(20);
		}

		jButton1.setMargin(new java.awt.Insets(0, 0, 0, 0));
		jButton1.setMaximumSize(new java.awt.Dimension(20, 20));
		jButton1.setMinimumSize(new java.awt.Dimension(20, 20));
		jButton1.addActionListener(new java.awt.event.ActionListener() {
			public void actionPerformed(java.awt.event.ActionEvent evt) {
				jButton1ActionPerformed(evt);
			}
		});

		jLabel1.setForeground(new java.awt.Color(51, 51, 255));
		jLabel1.setText("Faits renvoyés");

		javax.swing.GroupLayout layout = new javax.swing.GroupLayout(this);
		this.setLayout(layout);
		layout.setHorizontalGroup(layout
				.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
				.addGroup(
						javax.swing.GroupLayout.Alignment.TRAILING,
						layout.createSequentialGroup()
								.addContainerGap()
								.addGroup(
										layout.createParallelGroup(
												javax.swing.GroupLayout.Alignment.TRAILING)
												.addComponent(
														jScrollPane1,
														javax.swing.GroupLayout.PREFERRED_SIZE,
														0, Short.MAX_VALUE)
												.addGroup(
														layout.createSequentialGroup()
																.addComponent(
																		jLabel1)
																.addPreferredGap(
																		javax.swing.LayoutStyle.ComponentPlacement.RELATED,
																		130,
																		Short.MAX_VALUE)
																.addComponent(
																		jButton1,
																		javax.swing.GroupLayout.PREFERRED_SIZE,
																		20,
																		javax.swing.GroupLayout.PREFERRED_SIZE)))
								.addContainerGap()));
		layout.setVerticalGroup(layout
				.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
				.addGroup(
						javax.swing.GroupLayout.Alignment.TRAILING,
						layout.createSequentialGroup()
								.addGroup(
										layout.createParallelGroup(
												javax.swing.GroupLayout.Alignment.LEADING)
												.addGroup(
														layout.createSequentialGroup()
																.addContainerGap()
																.addComponent(
																		jLabel1))
												.addGroup(
														layout.createSequentialGroup()
																.addGap(14, 14,
																		14)
																.addComponent(
																		jButton1,
																		javax.swing.GroupLayout.PREFERRED_SIZE,
																		15,
																		javax.swing.GroupLayout.PREFERRED_SIZE)))
								.addPreferredGap(
										javax.swing.LayoutStyle.ComponentPlacement.RELATED,
										javax.swing.GroupLayout.DEFAULT_SIZE,
										Short.MAX_VALUE)
								.addComponent(jScrollPane1,
										javax.swing.GroupLayout.PREFERRED_SIZE,
										136,
										javax.swing.GroupLayout.PREFERRED_SIZE)));
	}// </editor-fold>

	// Variables declaration - do not modify
	private javax.swing.JButton jButton1;
	private javax.swing.JLabel jLabel1;
	private javax.swing.JScrollPane jScrollPane1;
	private javax.swing.JTable jTable1;
	// End of variables declaration
}
