/*
  EdiNoS - Editeur de NoS
  Copyright (C) 2014-2021 - Luc Poittevin — Tous droits réservés.
  Contact : edinos@free.fr - https://www.edinos.org
  
  Ce programme est un logiciel libre ; vous pouvez le redistribuer ou le
  modifier suivant les termes de la “GNU General Public License” telle que
  publiée par la Free Software Foundation : soit la version 3 de cette
  licence, soit (à votre gré) toute version ultérieure.
  
  Ce programme est distribué dans l’espoir qu’il vous sera utile, mais SANS
  AUCUNE GARANTIE : sans même la garantie implicite de COMMERCIALISABILITÉ
  ni d’ADÉQUATION À UN OBJECTIF PARTICULIER. Consultez la Licence Générale
  Publique GNU pour plus de détails.
  
  Vous devriez avoir reçu une copie de la Licence Générale Publique GNU avec
  ce programme ; si ce n’est pas le cas, consultez :
  <http://www.gnu.org/licenses/>. 
 */

package nos.obj;

import java.awt.Dimension;
import java.io.File;
import java.net.URISyntaxException;
import java.util.ArrayList;
import java.util.List;

import javafx.application.Platform;

import javax.swing.JFrame;
import javax.swing.JOptionPane;
import javax.swing.UIManager;
import javax.swing.UIManager.LookAndFeelInfo;

import nos.edit.FenEdiNoS;
import nos.run.Interface;
import nos.run.InterfaceEdiNoS;
import nos.run.Memoire;
import nos.run.Moteur;
import nos.run.Moteur.Contexte;
import nos.run.Moteur.ModeExecutionProc;
import nos.run.SimuleProc;
import nos.run.edit.FenAffiche;
import nos.run.edit.FenAfficheJFrame;
import nos.run.edit.FenDialogueQuest;
import nos.run.edit.FenDialogueQuestJFrame;
import nos.run.edit.FenFinLancement;
import nos.run.edit.FenFinLancementJFrame;
import nos.run.edit.FenLancement;
import nos.utils.PrefsManager;
import nos.utils.ToolNos;

import org.kohsuke.args4j.Argument;
import org.kohsuke.args4j.CmdLineException;
import org.kohsuke.args4j.CmdLineParser;
import org.kohsuke.args4j.Option;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

public class EdiNoS extends Edi {

	static private final String version = "3.5.2";
	
	static private final String copyright = "Copyright (C) 2013-2021 - Luc Poittevin";

	private static final long serialVersionUID = 1L;
	
	public static class MesOptions {


		@Option(name = "-base", usage = "chemin de la base à charger")
		// private File ficBase;
		private String cheminBase;

		@Option(name = "-graphe", usage = "nom du graphe à lancer")
		private String nomGraphe;

		// receives other command line parameters than options
		@Argument
		private List<String> arguments = new ArrayList<String>();
	}

	private static void init() {
		Edi.nos().montreInfo("EdiNoS version " + version);
		Edi.nos().montreInfo(copyright);
		Edi.nos().montreInfo(
				"Ce programme vient SANS ABSOLUMENT AUCUNE GARANTIE.");
		Edi.nos().montreInfo(
				"Ceci est un logiciel libre et vous êtes invité à le redistribuer"
						+ " suivant certaines conditions.");
		Edi.nos().montreInfo("Runtime Java : " + System.getProperty("java.version") + " (" +
				System.getProperty("java.vendor")+")");
		Edi.nos().montreInfo("Installé dans : " + System.getProperty("java.home"));

		Edi.nos().initPrefs();
		
	}

	public static void main(String argv[]) {

		MesOptions bean = new MesOptions();
		CmdLineParser parser = new CmdLineParser(bean);
		try {
			parser.parseArgument(argv);
		} catch (CmdLineException e) {
			System.err.println(e.getMessage());
			System.err.println("java -jar edinos.jar [options...]");
			parser.printUsage(System.err);
			return;
		}

		
		EdiNoS.setAppleMenus();

		EdiNoS ed = new EdiNoS();
		ed.initFX();
		Edi.forceNos(ed);


		if (bean.nomGraphe == null) {
			// appel Edinos standard, sans argument
			ed.afficheFen(); // ouverture de la fenêtre principale
			init();
		} else {
			// appel Edinos avec arguments
			ed.resetFen(); // nécessaire pour FEST (force fenEdinos à
							// null)
			init();
			ed.montreInfo("Graphe à lancer : " + bean.nomGraphe);

			 ed.run(bean.cheminBase, bean.nomGraphe, false);
		}
	}

	public static void setAppleMenus() {
		// Menus Mac...
		// voir :
		// http://www.macdevcenter.com/pub/a/mac/2002/09/06/osx_java.html?page=2
		// ou pour aller plus loin :
		// http://www.developer.com/java/other/article.php/1577161


		System.setProperty("apple.laf.useScreenMenuBar", "true");
		System.setProperty("com.apple.mrj.application.apple.menu.about.name",
				"EE");
	}

	private boolean utiliseJavaFX = false;

	private FenEdiNoS fenEdinos;

	private final Logger log = LoggerFactory.getLogger(Edi.class);

	private PrefsManager preferences;

	public EdiNoS() {
		super();
		setLookAndFeel();
	}

	@Override
	public void affiche(Message m, Moteur mot) {
		if (Edi.nos().fen() != null) {
			FenAffiche f = new FenAffiche(m, mot);
			getBaseCourante().ajouteFen(f);
			fen().ouvreInternalFrameSansPositionner(f);
			f.setVisible(true);
		} else {
			FenAfficheJFrame f = new FenAfficheJFrame(m, mot);
			f.setVisible(true);
		}
	}
	
	@Override
	public void affiche(String texte, Moteur moteur) {
		if (Edi.nos().fen() != null) {
			FenAffiche f = new FenAffiche(texte, moteur);
			getBaseCourante().ajouteFen(f);
			fen().ouvreInternalFrameSansPositionner(f);
			f.setVisible(true);
		} else {
			FenAfficheJFrame f = new FenAfficheJFrame(texte, moteur);
			f.setVisible(true);
		}
	}

	@Override
	public void afficheBase() {
		fenEdinos.afficheBase();
	}

	public void afficheFen() {
		fenEdinos = new FenEdiNoS();
		fenEdinos.setPreferredSize(new Dimension(1360, 820)); 
		
		fenEdinos.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
		fenEdinos.setVisible(true);
	}

	@Override
	public void afficheFinLancement(Moteur moteur) {
		if (Edi.nos().fen() != null) {
			FenFinLancement f = new FenFinLancement(moteur);
			getBaseCourante().ajouteFen(f);
			fen().ouvreInternalFrame(f);
			f.setVisible(true);
		} else {
			FenFinLancementJFrame f = new FenFinLancementJFrame(moteur);
			f.setVisible(true);
		}
	}

	@Override
	public void ajouteBase(String nom) {
		if (fenEdinos != null) { // possible si appel pour run
			fenEdinos.ajouteBase(nom);
		}
	}

	public FenEdiNoS fen() {
		return fenEdinos;
	}


	public void forceJavaFX(boolean b) {
		utiliseJavaFX = b;
	}
	
	public PrefsManager getPreferences() {
		return preferences;
	}

	@Override
	public String getRepertoireTravail() {
		return getPreferences().getRepertoireTravail();
	}

	public String getVersion() {
		return version;
	}

	public  void initFX() {
		// nécessaire pour cohabitation Java FX / Swing
		// https://blog.idrsolutions.com/2012/12/integrating-javafx-with-swing-the-jfxpanel/
		// :
			this.utiliseJavaFX = true;
			Platform.setImplicitExit(false);
	}

	@Override
	public void initPrefs() {
		preferences = new PrefsManager(fenEdinos);
	}

	public String initRepertoireBC() {
		String rep = System.getProperty("user.dir") + ToolNos.renvoieSeparateurRepertoires() +"NOS.BC";
		Edi.nos().montreInfo("User.dir : " + System.getProperty("user.dir"));

		File file = new File(rep).getAbsoluteFile();
		if (!file.exists()) {
			fenEdinos.montre(rep + " non trouvé");
			// Deuxième chance : si le "user.dir" ne contient
			// pas de sous répertoire "NOS.BC", on regarde si le
			// répertoire courant (celui où est le .jar), contient "NOS.BC".
			// Si ce n'est pas le cas, ce n'est pas bien grave :
			// edinos.properties
			// sera créé à vide et le répertoire de travail n'étant pas connu,
			// on ne pourra juste pas charger les bases EDINOS et COMMUN

			String s = "";
			try { // https://stackoverflow.com/questions/320542/how-to-get-the-path-of-a-running-jar-file
				s =  new File(EdiNoS.class.getProtectionDomain().getCodeSource().getLocation().toURI()).getPath();
			} catch (URISyntaxException e) {
				fenEdinos.montre("Erreur lors de la recherche de localisation du jar courant...");
			}
			
			rep = s.substring(0, s.lastIndexOf(ToolNos.renvoieSeparateurRepertoires()));
			rep = rep+ ToolNos.renvoieSeparateurRepertoires() +"NOS.BC";
			file = new File(rep).getAbsoluteFile();
			if (!file.exists()) {
				fenEdinos.montre(rep + " non trouvé");
				return "";
			} else {
				fenEdinos.montre("Le répertoire de travail est initialisé à : "
						+ rep);
				return rep;
			}
		} else {
			fenEdinos.montre("Le répertoire de travail est initialisé à : "
					+ rep);
			return rep;
		}
	}

	public void logue(String string) {
		log.info(string);
	}

	@Override
	public void message(String s) {
		JOptionPane.showMessageDialog(fenEdinos, s);
	}

	@Override
	public void montre(String s) {
		fenEdinos.montre(s);
	}

	@Override
	public void montreAvertissement(String s) {
		log.warn(s);
		if (fenEdinos != null) {
			fenEdinos.montre(s);
		}
	}

	@Override
	public void montreInfo(String s) {
		log.info(s);
		if (fenEdinos != null) {
			fenEdinos.montre(s);
		}
	}

	@Override
	public void poseQuestion(Question question, Moteur moteur) {
		if (Edi.nos().fen() != null) {
			FenDialogueQuest f = new FenDialogueQuest(question, moteur);
			getBaseCourante().ajouteFen(f);
			fen().ouvreInternalFrameSansPositionner(f);
			f.setVisible(true);
		} else {
			FenDialogueQuestJFrame f = new FenDialogueQuestJFrame(question,
					moteur);
			f.setVisible(true);
		}
	}

	public void prepareFest(FenEdiNoS f) {
		// Configure le singleton pour les tests FEST (non Run)

		fenEdinos = f;
		init();
	}

	@Override
	public void refreshListeNoS() {
		fenEdinos.refreshListeNoS();
	}

	private void resetFen() {
		// utile pour FEST : pour enchainer TesteDialoguesRun derrière
		// TesteDialoguesRevinos
		fenEdinos = null;
	}

	@Override
	public void retireBase(String nom) {
		fenEdinos.retireBase(nom);
	}

	public void run(String cheminBase, String nomGraphe,
			boolean avecFenLancement) {
		File fic = new File(cheminBase).getAbsoluteFile();
		if (fic.exists()) {
			creeBase(fic);
		} else {
			Edi.nos().montreInfo("fichier non trouvé : " + cheminBase);
			return;
		}

		NoS n = (NoS) Edi.nos().getBaseCourante().getAssocNos()
				.get(nomGraphe);
		if (n == null) {
			Edi.nos().montreInfo("nos <" + nomGraphe + "> introuvable ...");
		} else {
			if (avecFenLancement) {
				FenLancement f = new FenLancement(n);
				f.setSize(550, 350);
				Edi.nos().getBaseCourante().ajouteFen(f);
				fenEdinos.ouvreInternalFrameSansPositionner(f);
				f.setVisible(true);
			} else {
				Moteur.afficheMT = false;
				Moteur mot = new Moteur(n, Contexte.Lancement,
						ModeExecutionProc.Mixte);
				Memoire mem = new Memoire();
				mot.declencheAvec(mem);

			}
		}
	}

	public void setLookAndFeel() {
		logue("Look & Feel name : " + UIManager.getLookAndFeel().getName());
		logue("Look & Feel : "
				+ UIManager.getLookAndFeel().getClass().getCanonicalName());


		if (!UIManager.getLookAndFeel().getName().equals("Mac OS X")) {
			try {
				for (LookAndFeelInfo info : UIManager
						.getInstalledLookAndFeels()) {
					if ("Nimbus".equals(info.getName())) {
						UIManager.setLookAndFeel(info.getClassName());
						break;
					}
				}
			} catch (Exception e) {
				// If Nimbus is not available, you can set the GUI to another
				// look
				// and feel.
				try {
					UIManager.setLookAndFeel(UIManager
							.getSystemLookAndFeelClassName());
					// Metal :
					// UIManager.setLookAndFeel(UIManager.getCrossPlatformLookAndFeelClassName());
					// Motif :
					// UIManager.setLookAndFeel("com.sun.java.swing.plaf.motif.MotifLookAndFeel");
				} catch (Exception e1) {
					// TODO Auto-generated catch block
					e1.printStackTrace();
				}
				
			}
		}
	}

	@Override
	public void simuleProc(Procedure p, Moteur m) {
		new SimuleProc(p, m);
	}
	
	public boolean utiliseJavaFX() {
		return utiliseJavaFX;
	}
}
