/*
  EdiNoS - Editeur de NoS
  Copyright (C) 2014-2020 - Luc Poittevin — Tous droits réservés.
  Contact : edinos@free.fr  -  https://www.edinos.org
  
  Ce programme est un logiciel libre ; vous pouvez le redistribuer ou le
  modifier suivant les termes de la “GNU General Public License” telle que
  publiée par la Free Software Foundation : soit la version 3 de cette
  licence, soit (à votre gré) toute version ultérieure.
  
  Ce programme est distribué dans l’espoir qu’il vous sera utile, mais SANS
  AUCUNE GARANTIE : sans même la garantie implicite de COMMERCIALISABILITÉ
  ni d’ADÉQUATION À UN OBJECTIF PARTICULIER. Consultez la Licence Générale
  Publique GNU pour plus de détails.
  
  Vous devriez avoir reçu une copie de la Licence Générale Publique GNU avec
  ce programme ; si ce n’est pas le cas, consultez :
  <http://www.gnu.org/licenses/>. 
 */

package nos.revinos;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.DefaultListModel;
import javax.swing.JInternalFrame;
import javax.swing.JMenu;
import javax.swing.JMenuBar;
import javax.swing.JMenuItem;
import javax.swing.JOptionPane;
import javax.swing.KeyStroke;
import javax.swing.event.InternalFrameEvent;
import javax.swing.event.InternalFrameListener;

import nos.obj.Edi;
import nos.edit.FenEdiNoS;
import nos.run.Moteur;
import nos.utils.ToolNos;

@SuppressWarnings("serial")
public class FenChoixAction2 extends JInternalFrame implements
		InternalFrameListener {

	private Moteur moteur;

	private JMenuBar menuBar;

	public FenChoixAction2(Moteur m) {
		super("", true, // resizable
				true, // closable
				true, // maximizable
				true);// iconifiable
		// Pour retirer la bordure basse sous MacOS :
		// this.putClientProperty("JInternalFrame.frameType", "normal");
		addInternalFrameListener(this);
		moteur = m;
		initComponents();
		initComponents2();
		installeMenus();
	}

	private void initComponents2() {
		Edi.nos().montreInfo("REVINOS - Choix action 2");

		String s = moteur.getBaseFaits().valeur("actionErronee.nom");
		if (moteur.getBaseFaits().valeur("actionErronee.type")
				.equals("afficher:")) {
			if (Edi.nos().getBaseCourante().contientMessage(s)) {
				s = "afficher: " + s;
			} else {
				s = "afficher: \"" + s+"\"";
			}
		}

		jLabel4.setText(s);
		jLabel4.setName("action"); // pour FEST

		DefaultListModel<String> listModel = new DefaultListModel<String>();
		for (String nomAction : moteur.getCible().getNoS().getBase()
				.getListeActionsPlusAfficher()) {
			listModel.addElement(nomAction);
		}

		jList1.setModel(listModel);
		jScrollPane1.setViewportView(jList1);

		jRadioButton1.setActionCommand("actionExisteDeja");
		jRadioButton2.setActionCommand("corrigerAction");
		jRadioButton3.setActionCommand("nouvelleAction");
		jRadioButton1.setName("actionExisteDeja"); // pour FEST
		jRadioButton2.setName("corrigerAction"); // pour FEST
		jRadioButton3.setName("nouvelleAction"); // pour FEST

		buttonGroup1.add(jRadioButton1);
		buttonGroup1.add(jRadioButton2);
		buttonGroup1.add(jRadioButton3);

		jLabel1.setName("jLabel1"); // pour FEST
		jLabel2.setName("jLabel2"); // pour FEST
		jLabel6.setName("jLabel6"); // pour FEST
		jButton1.setName("OK"); // pour FEST
		jList1.setName("liste"); // pour FEST
	}

	private void jButton1ActionPerformed(java.awt.event.ActionEvent evt) {
		// Bouton Ok
		String choix = "";
		if (buttonGroup1.getSelection() != null) {
			choix = buttonGroup1.getSelection().getActionCommand();
		} else {
			JOptionPane.showMessageDialog(this,
					"Vous n'avez pas choisi de réponse...");
			return;
		}

		if ((choix.equals("actionExisteDeja"))
				|| (choix.equals("corrigerAction")))
			if (jList1.getSelectedValue() == null) {
				JOptionPane
						.showMessageDialog(this,
								"Vous devez sélectionner cette action dans la liste...");
				return;
			}

		if (choix.equals("nouvelleAction")) {
			moteur.retourne(choix);
			dispose();
			return;
		}

		dispose();

		String nomAction, typeAction;
		if (jList1.getSelectedValue().toString().contains("(afficher)")) {
			nomAction = jList1
					.getSelectedValue()
					.toString()
					.substring(
							jList1.getSelectedValue().toString().indexOf(")") + 1);
			typeAction = "afficher:";
		} else {
			nomAction = jList1.getSelectedValue().toString();
			typeAction = Edi.nos().getBaseCourante()
					.getTypeDeProc(jList1.getSelectedValue());
		}

		if (choix.equals("actionExisteDeja")) {
			moteur.getBaseFaits()
					.ajouteFait("actionAutiliser.nom " + nomAction);
			moteur.getBaseFaits().ajouteFait(
					"actionAutiliser.type " + typeAction);
		}

		if (choix.equals("corrigerAction")) {
			moteur.getBaseFaits().ajouteFait("actionCorriger.nom " + nomAction);
			moteur.getBaseFaits().ajouteFait(
					"actionCorriger.type " + typeAction);
		}
		
		moteur.retourne(choix);
	}

	private void jButton2ActionPerformed(java.awt.event.ActionEvent evt) {
		// Bouton Annuler
		dispose();
		moteur.retourne("annuler");
	}

	private void jButton3ActionPerformed(java.awt.event.ActionEvent evt) {
		// Bouton Fermer
		dispose();
	}

	private void installeMenus() {
		menuBar = ((FenEdiNoS) Edi.nos().fen()).menuBarEdinos();
		menuBar.add(menuRevinos());
		Edi.nos().fen().setJMenuBar(menuBar);
		Edi.nos().fen().setVisible(true);
	}

	private JMenu menuRevinos() {

		JMenuItem menuItem;
		JMenu menu = new JMenu("Dialogue ReviNoS");

		menuItem = new JMenuItem("Fermer la fenêtre");
		menuItem.setAccelerator(KeyStroke.getKeyStroke(
				java.awt.event.KeyEvent.VK_W, ToolNos.renvoieModifiers()));
		menuItem.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				dispose();
			}
		});
		menu.add(menuItem);
		return (menu);
	}

	public void internalFrameActivated(InternalFrameEvent e) {
		Edi.nos().fen().setJMenuBar(menuBar);
		Edi.nos().fen().setVisible(true);
	}

	public void internalFrameClosed(InternalFrameEvent e) {
	}

	public void internalFrameClosing(InternalFrameEvent e) {
		ToolNos.fermeFen((JInternalFrame) this);
	}

	public void internalFrameDeactivated(InternalFrameEvent e) {
		((FenEdiNoS) Edi.nos().fen()).setMenuBar();
	}

	public void internalFrameDeiconified(InternalFrameEvent e) {
	}

	public void internalFrameIconified(InternalFrameEvent e) {
	}

	public void internalFrameOpened(InternalFrameEvent e) {
	}

	/**
	 * This method is called from within the constructor to initialize the form.
	 * WARNING: Do NOT modify this code. The content of this method is always
	 * regenerated by the Form Editor.
	 */

	// <editor-fold defaultstate="collapsed" desc="Generated Code">
	private void initComponents() {

		buttonGroup1 = new javax.swing.ButtonGroup();
		jLabel6 = new javax.swing.JLabel();
		jButton1 = new javax.swing.JButton();
		jButton2 = new javax.swing.JButton();
		jPanel1 = new javax.swing.JPanel();
		jButton3 = new javax.swing.JButton();
		jScrollPane1 = new javax.swing.JScrollPane();
		jList1 = new javax.swing.JList<String>();
		jLabel1 = new javax.swing.JLabel();
		jLabel2 = new javax.swing.JLabel();
		jLabel3 = new javax.swing.JLabel();
		jLabel4 = new javax.swing.JLabel();
		jRadioButton1 = new javax.swing.JRadioButton();
		jRadioButton2 = new javax.swing.JRadioButton();
		jRadioButton3 = new javax.swing.JRadioButton();

		// setDefaultCloseOperation(javax.swing.WindowConstants.EXIT_ON_CLOSE);

		jLabel6.setText("il faut :");

		jButton1.setText("OK");
		jButton1.setToolTipText("");
		jButton1.addActionListener(new java.awt.event.ActionListener() {
			public void actionPerformed(java.awt.event.ActionEvent evt) {
				jButton1ActionPerformed(evt);
			}
		});

		jButton2.setText("Annuler");
		jButton2.setToolTipText("");
		jButton2.addActionListener(new java.awt.event.ActionListener() {
			public void actionPerformed(java.awt.event.ActionEvent evt) {
				jButton2ActionPerformed(evt);
			}
		});

		jPanel1.setBorder(javax.swing.BorderFactory.createEtchedBorder());

		jButton3.setText("Fermer");
		jButton3.setFocusable(false);
		jButton3.setIgnoreRepaint(true);
		jButton3.setMaximumSize(new java.awt.Dimension(60, 30));
		jButton3.addActionListener(new java.awt.event.ActionListener() {
			public void actionPerformed(java.awt.event.ActionEvent evt) {
				jButton3ActionPerformed(evt);
			}
		});

		org.jdesktop.layout.GroupLayout jPanel1Layout = new org.jdesktop.layout.GroupLayout(
				jPanel1);
		jPanel1.setLayout(jPanel1Layout);
		jPanel1Layout.setHorizontalGroup(jPanel1Layout.createParallelGroup(
				org.jdesktop.layout.GroupLayout.LEADING).add(
				jPanel1Layout
						.createSequentialGroup()
						.add(jButton3,
								org.jdesktop.layout.GroupLayout.PREFERRED_SIZE,
								org.jdesktop.layout.GroupLayout.DEFAULT_SIZE,
								org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
						.addContainerGap(437, Short.MAX_VALUE)));
		jPanel1Layout.setVerticalGroup(jPanel1Layout.createParallelGroup(
				org.jdesktop.layout.GroupLayout.LEADING).add(
				jPanel1Layout
						.createSequentialGroup()
						.add(jButton3,
								org.jdesktop.layout.GroupLayout.DEFAULT_SIZE,
								org.jdesktop.layout.GroupLayout.DEFAULT_SIZE,
								Short.MAX_VALUE).addContainerGap()));

		jList1.setModel(new javax.swing.AbstractListModel() {
			String[] strings = { "Item 1", "Item 2", "Item 3", "Item 4",
					"Item 5" };

			public int getSize() {
				return strings.length;
			}

			public Object getElementAt(int i) {
				return strings[i];
			}
		});
		jScrollPane1.setViewportView(jList1);

		jLabel1.setText("Action effectuée pour l'instant : ");

		jLabel2.setText("Pour aller chercher l'information manquante,");

		jLabel3.setText("Liste des actions définies :");

		jLabel4.setBackground(java.awt.Color.lightGray);
		jLabel4.setText("jLabel4");
		jLabel4.setBorder(javax.swing.BorderFactory.createEtchedBorder());
		jLabel4.setFocusable(false);
		jLabel4.setOpaque(true);

		jRadioButton1.setText("exécuter une action déjà existante");

		jRadioButton2.setText("corriger une action déjà existante");

		jRadioButton3.setText("exécuter une nouvelle action");

		org.jdesktop.layout.GroupLayout layout = new org.jdesktop.layout.GroupLayout(
				getContentPane());
		getContentPane().setLayout(layout);
		layout.setHorizontalGroup(layout
				.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
				.add(org.jdesktop.layout.GroupLayout.TRAILING, jPanel1,
						org.jdesktop.layout.GroupLayout.DEFAULT_SIZE,
						org.jdesktop.layout.GroupLayout.DEFAULT_SIZE,
						Short.MAX_VALUE)
				.add(layout
						.createSequentialGroup()
						.addContainerGap()
						.add(layout
								.createParallelGroup(
										org.jdesktop.layout.GroupLayout.LEADING,
										false)
								.add(jScrollPane1)
								.add(jLabel3,
										org.jdesktop.layout.GroupLayout.DEFAULT_SIZE,
										org.jdesktop.layout.GroupLayout.DEFAULT_SIZE,
										Short.MAX_VALUE))
						.add(30, 30, 30)
						.add(layout
								.createParallelGroup(
										org.jdesktop.layout.GroupLayout.LEADING,
										false)
								.add(jLabel6)
								.add(jLabel1)
								.add(layout
										.createSequentialGroup()
										.add(26, 26, 26)
										.add(layout
												.createParallelGroup(
														org.jdesktop.layout.GroupLayout.LEADING)
												.add(jRadioButton3)
												.add(jRadioButton2)
												.add(layout
														.createParallelGroup(
																org.jdesktop.layout.GroupLayout.TRAILING,
																false)
														.add(org.jdesktop.layout.GroupLayout.LEADING,
																layout.createSequentialGroup()
																		.add(jButton2,
																				org.jdesktop.layout.GroupLayout.PREFERRED_SIZE,
																				93,
																				org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
																		.addPreferredGap(
																				org.jdesktop.layout.LayoutStyle.RELATED,
																				org.jdesktop.layout.GroupLayout.DEFAULT_SIZE,
																				Short.MAX_VALUE)
																		.add(jButton1,
																				org.jdesktop.layout.GroupLayout.PREFERRED_SIZE,
																				93,
																				org.jdesktop.layout.GroupLayout.PREFERRED_SIZE))
														.add(org.jdesktop.layout.GroupLayout.LEADING,
																jRadioButton1))))
								.add(jLabel4,
										org.jdesktop.layout.GroupLayout.DEFAULT_SIZE,
										org.jdesktop.layout.GroupLayout.DEFAULT_SIZE,
										Short.MAX_VALUE).add(jLabel2))
						.addContainerGap()));
		layout.setVerticalGroup(layout
				.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
				.add(layout
						.createSequentialGroup()
						.add(jPanel1,
								org.jdesktop.layout.GroupLayout.PREFERRED_SIZE,
								32,
								org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
						.addPreferredGap(
								org.jdesktop.layout.LayoutStyle.UNRELATED)
						.add(jLabel3)
						.addPreferredGap(
								org.jdesktop.layout.LayoutStyle.RELATED)
						.add(layout
								.createParallelGroup(
										org.jdesktop.layout.GroupLayout.LEADING)
								.add(layout
										.createSequentialGroup()
										.add(jLabel1)
										.addPreferredGap(
												org.jdesktop.layout.LayoutStyle.RELATED)
										.add(jLabel4)
										.addPreferredGap(
												org.jdesktop.layout.LayoutStyle.RELATED)
										.add(jLabel2,
												org.jdesktop.layout.GroupLayout.PREFERRED_SIZE,
												22,
												org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
										.addPreferredGap(
												org.jdesktop.layout.LayoutStyle.RELATED)
										.add(jLabel6,
												org.jdesktop.layout.GroupLayout.PREFERRED_SIZE,
												16,
												org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
										.addPreferredGap(
												org.jdesktop.layout.LayoutStyle.UNRELATED)
										.add(jRadioButton1)
										.add(7, 7, 7)
										.add(jRadioButton2)
										.addPreferredGap(
												org.jdesktop.layout.LayoutStyle.UNRELATED)
										.add(jRadioButton3)
										.add(18, 18, 18)
										.add(layout
												.createParallelGroup(
														org.jdesktop.layout.GroupLayout.BASELINE)
												.add(jButton2).add(jButton1)))
								.add(jScrollPane1,
										org.jdesktop.layout.GroupLayout.DEFAULT_SIZE,
										238, Short.MAX_VALUE))
						.addContainerGap()));

		pack();
	}// </editor-fold>

	// Variables declaration - do not modify
	private javax.swing.ButtonGroup buttonGroup1;
	private javax.swing.JButton jButton1;
	private javax.swing.JButton jButton2;
	private javax.swing.JButton jButton3;
	private javax.swing.JLabel jLabel1;
	private javax.swing.JLabel jLabel2;
	private javax.swing.JLabel jLabel3;
	private javax.swing.JLabel jLabel4;
	private javax.swing.JLabel jLabel6;
	private javax.swing.JList<String> jList1;
	private javax.swing.JPanel jPanel1;
	private javax.swing.JRadioButton jRadioButton1;
	private javax.swing.JRadioButton jRadioButton2;
	private javax.swing.JRadioButton jRadioButton3;
	private javax.swing.JScrollPane jScrollPane1;
	// End of variables declaration

}
