/*
  EdiNoS - Editeur de NoS
  Copyright (C) 2013-2019 - Luc Poittevin — Tous droits réservés.
  Contact : edinos@free.fr  -  https://www.edinos.org
  
  Ce programme est un logiciel libre ; vous pouvez le redistribuer ou le
  modifier suivant les termes de la “GNU General Public License” telle que
  publiée par la Free Software Foundation : soit la version 3 de cette
  licence, soit (à votre gré) toute version ultérieure.
  
  Ce programme est distribué dans l’espoir qu’il vous sera utile, mais SANS
  AUCUNE GARANTIE : sans même la garantie implicite de COMMERCIALISABILITÉ
  ni d’ADÉQUATION À UN OBJECTIF PARTICULIER. Consultez la Licence Générale
  Publique GNU pour plus de détails.
  
  Vous devriez avoir reçu une copie de la Licence Générale Publique GNU avec
  ce programme ; si ce n’est pas le cas, consultez :
  <http://www.gnu.org/licenses/>. 
 */

package nos.revinos;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;
import java.util.SortedSet;

import javax.swing.JInternalFrame;

import nos.obj.Edi;
import nos.obj.NoS;
import nos.edit.FenEdiNoS;
import nos.revinos.Sessions.Statut;
import nos.run.MoteurBilan;
import nos.run.Bilan;
import nos.run.InterfaceRevi;
import nos.run.Memoire;
import nos.run.Moteur;
import nos.run.NoSException;
import nos.run.edit.FenTexte;
import nos.utils.ToolNos;

public class Revi {
	// La classe Revi est un singleton
	// son instance est accessible via Revi.nos()

	/** L'instance statique : */
	private static Revi instance;

	/**
	 * Récupère l'instance unique de la class Revi
	 */
	public static Revi nos() {
		if (null == instance) { // Premier appel
			instance = new Revi();
		}
		return instance;
	}

	private String historiqueSession;
	private List<MoteurBilan> exAsoumettre;
	private List<MoteurBilan> exValides;
	private List<MoteurBilan> exAvalider;
	private List<MoteurBilan> exDejaSoumis;
	private MoteurBilan exSoumis;
	private MoteurBilan exCourant;
	private Moteur filRevision;
	private int verrou;
	private boolean generalise;
	private FenSessions fenSessions;

	// <historiqueSession> est le texte qui résume
	// tout ce qui s'est passé durant la session
	// de révision.
	// <exCourant> est le <MoteurBilan> de la situation en cours de
	// révision.
	// <exValides> est une collection de <MoteurBilan> déjà validés.
	// <filRevision> est un <Moteur> sur le graphe 'REVINOS'
	// (nécessaire pour la continuité de la session de révision…)
	// <exAvalider> est un dictionaire où :
	// clé = un 'NomNodRepEnCours'
	// valeur = collection de <MoteurBilan> qu'il faut présenter
	// pour valider la modif du nos.
	// <exAsoumettre> est une collection d'exemples à soumettre
	// 'uniquement', càd qu'il ne faut pas déclencher
	// de révision en cas de refus de la part de l'utilisateur.
	// Si <ExAsoumettre> vaut null, celà signifie que l'exemple
	// courant a été soumis pour révision (voir 'retourne:').
	// <exSoumis> est l'exemple soumis (extrait de <ExAsoumettre>).
	// <verrou> est provisoirement utilisé pour détecter des cas
	// particuliers de révision. Il vaut 0 au départ, et
	// est incrémenté à chaque exemple soumis rejeté
	// (si <Verrou> vaut 2 => stop).
	// <exDejaSoumis> est rempli au fur et à messure de la soumission d'exemples
	// partiels (pour l'instant uniquement en phase de généralisation).
	// C'est un dictionnaire où :
	// clé = 'KO' ou 'OK' selon le resultat de la soumission.
	// valeur = collection d'exemples soumis.
	// <generalise>
	// Quand <Generalise> est une collection,
	// on est en phase de généralisation et on y trouve:
	// - en pos 1: nodCourant
	// - en pos 2: nodSuivant
	// - en pos 3: nodCible
	// - en pos 4: règle utilisée
	// sinon <Generalise> vaut nil.
	// <fenSessions> est la fenêtre des cas en cours de révision
	// (par ex lorsque "statuter plus tard" a été cliqué


	private Revi() {
		/**
		 * Constructeur redéfini comme étant privé pour interdire son appel et
		 * forcer à passer par la méthode nos() :
		 */
		init();
		fenSessions = null;
	}

	public void ajouteHistorique(String s) {
		historiqueSession = historiqueSession
				+ "------------------------------------------------------------------\n"
				+ s;
	}

	private void annuleSes() {
		System.out.println("interruption annuleSes");
		InterfaceRevi.revisionInterrompue(filRevision);
	}

	private void casAVoir() {
		System.out.println("interruption casAVoir");
		InterfaceRevi.revisionInterrompue(filRevision);
	}

	private void clotSes() {
		fenSessions.termineSessionEnCours();
		transcritExemples();
		FenTexte f = new FenTexte(Revi.nos().getHistoriqueSession(),
				"Révision validée", "Révision", Edi.nos().getBaseCourante()
						.getNom(), "rev");
		f.setSize(500, 350);
		Edi.nos().getBaseCourante().ajouteFen(f);
		((FenEdiNoS) Edi.nos().fen()).ouvreInternalFrame(f);
	}

	private void continueRevision() {
		String ch = "Poursuit avec l'exemple :" + "\n";
		ch = ch + exCourant.chaineResultat() + "\n";
		ajouteHistorique(ch);

		// "*** <nomAct> blanc si l'action du nos ***"
		// "*** courant est une action non terminale ***"
		String nomAct = "";
		if (exCourant.getNoS().actionTerminale()) {
			nomAct = exCourant.getNoS().getActions();
		}

		JInternalFrame f;
		if (!nomAct.isEmpty()) {
			 f = new FenSoumissionExemple3(exCourant,
					nomAct, "Situation à valider");
		} else {
			 f = new FenSoumissionExemple2(exCourant,
					nomAct, "Situation à éclaircir");
		}

		filRevision.getNoS().getBase().ajouteFen(f);
		((FenEdiNoS) Edi.nos().fen()).ouvreInternalFrame(f);
		f.setVisible(true);
	}

	private void continueSoumission() {
		String ch = "";
		if (exAsoumettre != null) {
			exSoumis = exAsoumettre.remove(0);
		}
		
		if (exCourant != null) {
			System.out.println(exCourant.chaineResultat());
			System.out.println(exSoumis.chaineResultatVirgule());
			if (exCourant.chaineResultat().contains(
					exSoumis.chaineResultatVirgule())) {
				ch = "Exemple non soumis :" + "\n";
				ch = ch + exSoumis.chaineResultatPlusAction() + "\n";
				ajouteHistorique(ch);
				retourne("rejetteExemple");
				return;
			}
		}

		for (MoteurBilan ex : exValides) {
			if (ex != null) {
				if (ex.chaineResultat().contains(exSoumis.chaineResultat())) {
					ch = "Exemple non soumis :" + "\n";
					ch = ch + exSoumis.chaineResultatPlusAction() + "\n";
					ajouteHistorique(ch);
					retourne("accepteExemple");
					return;
				}
			}
		}

		// "*** sinon: ***"

		String nomAct = exSoumis.getNoS().getActions();
		ch = "Exemple soumis :" + "\n";
		ch = ch + exSoumis.chaineResultatPlusAction() + "\n";
		ajouteHistorique(ch);
		FenSoumissionExemple f = new FenSoumissionExemple(exSoumis, nomAct,
				"Une question...");
		filRevision.getNoS().getBase().ajouteFen(f);
		((FenEdiNoS) Edi.nos().fen()).ouvreInternalFrame(f);
		f.setVisible(true);
	}

	private void echecRev() {
		FenTexte f = new FenTexte(Revi.nos().getHistoriqueSession(),
				"La révision a échoué", "Révision", "échec", "rev");
		f.setSize(500, 350);
		Edi.nos().getBaseCourante().ajouteFen(f);
		((FenEdiNoS) Edi.nos().fen()).ouvreInternalFrame(f);
	}

	private void etapeSuivante(List<MoteurBilan> col) {
		// "*** <col> est une collection de <MoteurBilan> qu'il faut valider    ***"
		// "*** Vraiment intéressant ? ***" // LPO

		exAvalider.addAll(col);

		MoteurBilan nouv = nouvelExCourant();
		if (nouv == null) {
			Edi.nos().message("Anomalie - etapeSuivante");
		}
		
		exCourant = nouv;
		continueRevision();
	}

	private Moteur getExempleEnCours() {
		if (exAsoumettre == null) {
			return exCourant;
		} else {
			return exSoumis;
		}
	}

	public String getHistoriqueSession() {
		return historiqueSession;
	}

	public void init() {
		// * Force à réinitialiser l'instance : le prochain appel à Revi.nos()
		// * appellera le constructeur

		historiqueSession = "";
		exAvalider = new ArrayList<MoteurBilan>();
		exValides = new ArrayList<MoteurBilan>();
		exCourant = null;
		filRevision = null;
		exDejaSoumis = new ArrayList<MoteurBilan>();
		verrou = 0;
		generalise = false;
	}

	private void majExAvalider(List<MoteurBilan> col) {
		// "*** <col> est une collection de <MoteurBilan> qu'il faut  ***"
		// "*** retirer de <ExAvalider> s'ils y figurent ...    ***"
		// "*** (les éléments de <col> ont des 'chaineResultat' ***"
		// "*** éventuellement plus longs ...)                  ***"

		for (MoteurBilan c : col) {
			String l = c.chaineResultat();
			for (MoteurBilan val : exAvalider) {
			}
		}

	}

	private MoteurBilan nouvelExCourant() {
		// "*** Renvoie le nouvel exemple courant à valider ***"

		if (exAvalider.isEmpty()) {
			return null;
		} else {
			return exAvalider.remove(0);
		}
	}

	public void operateurA2(Moteur fil) {

		filRevision = fil;

		String nomNosCourant = filRevision.getBaseFaits().valeur(
				"nodCourant.nom");
		NoS nosCourant = Edi.nos().getBaseCourante().getNoS(nomNosCourant);
		String nomNosSuivant = filRevision.getBaseFaits().valeur(
				"nodSuivant.nom");
		String nomNosCible = filRevision.getBaseFaits().valeur("nodCible.nom");
		NoS nosCible = Edi.nos().getBaseCourante().getNoS(nomNosCible);

		// "*** On commence par identifier le fait discriminant : ***"

		List<String> faitD1 = new ArrayList<String>();
		for (String fait : exSoumis.getBaseFaits().getFaits()) {
			if (!exCourant.getBaseFaits().getFaits().contains(fait)) {
				faitD1.add(fait);
			}
		}
		;
		List<String> faitD2 = new ArrayList<String>();
		for (String fait : exCourant.getBaseFaits().getFaits()) {
			if (!exSoumis.getBaseFaits().getFaits().contains(fait)) {
				faitD2.add(fait);
			}
		}

		String faitD = null;
		String nomNosVise = null;
		String nomAlterNos = null;
		if (!faitD1.isEmpty()) {
			if (!faitD1.get(0).isEmpty()) {
				faitD = faitD1.get(0);
				nomNosVise = nomNosSuivant;
				nomAlterNos = nomNosCible;
			}
		}
		;
		if (!faitD2.isEmpty()) {
			if (!faitD2.get(0).isEmpty()) {
				faitD = faitD2.get(0);
				nomNosVise = nomNosCible;
				nomAlterNos = nomNosSuivant;
			}
		}
		;
		if (faitD == null) {
			Edi.nos().message("Anomalie 1 - opérateurA2");
		}
		
		// "*** On repère <regleActivable> : ***"

		List<String> col = Revi.nos().regleCourante(nosCourant,
				filRevision.getCible().getBaseFaits());
		String regleActivable = col.get(0);
		String premisse = col.get(1);
		int numRegle = Integer.valueOf(col.get(2));
		int prioriteDepart = Integer.valueOf(col.get(3));

		// "*** On repère la proc qui renvoie <faitD> : ***"
		String nomAction = Edi.nos().getBaseCourante()
				.getActionQuiRenvoieFait(faitD);
		if (nomAction == null) {
			Edi.nos().message("Anomalie 2 - opérateurA2");
		}

		// "*** <regleActivable> est remplacée par deux nouvelles règles : ***"

		String chaine = "si: " + premisse + " ET " + faitD + " alorsAller: "
				+ nomNosVise;
		if (prioriteDepart == 10) {
			chaine = chaine + ";\n";
		} else {
			chaine = chaine + " priorite: " + prioriteDepart + ";\n";
		}

		String alterFait = Edi.nos().getBaseCourante().getAction(nomAction)
				.alterFait(faitD);
		if (alterFait == null) {
			int newPriorite = prioriteDepart + 1;
			chaine = chaine + "si: " + premisse + " alorsAller: " + nomAlterNos
					+ " priorite: " + newPriorite + ";\n";
		} else {
			// "*** la procédure peut renvoyer ***"
			// "*** un autre fait que <faitD>  ***"
			chaine = chaine + "si: " + premisse + " ET " + alterFait
					+ " alorsAller: " + nomAlterNos;
			if (prioriteDepart != 10) {
				chaine = chaine + " priorite: " + prioriteDepart + ";\n";
			}
			;
		}

		if (!nosCourant.getRegles().contains(regleActivable)) {
			Edi.nos().message("Anomalie 3 - opérateurA2");
		}

		String chaine2 = nosCourant.getRegles().replace(regleActivable, chaine);
		Revi.nos().ajouteHistorique(
				"PARCOURS : " + filRevision.getBaseFaits().ligneParcours()
						+ "\n");
		Revi.nos().ajouteHistorique(
				"corriger_règles (" + nomNosCourant + ", " + nomNosCible
						+ ") : remplaceRègle.\n" + "ANCIENNES : \n"
						+ nosCourant.getRegles() + "\nNOUVELLES : \n" + chaine2
						+ "\n");
		Revi.nos().majReglesRevues(nomNosCourant, null);

		// "*** On met à jour le graphe : ***"
		nosCible.setPosition((String.valueOf(nosCourant.getPosX() + 70)) + "@"
				+ String.valueOf(nosCourant.getPosY() + 50));
		nosCourant.addSuivant(nosCible.getNom());

		// "*** Poursuite de la révision: calcul des répercussions: ***"
		Revi.nos().reviseNoduleAvecRegle(nosCourant, chaine2);
	}

	public List<String> regleCourante(NoS nosCourant, Memoire baseFaits) {
		// "*** renvoie une collection contenant:              ***"
		// "*** - en pos 1 : la règle utilisée pour choisir le ***"
		// "***   le nos successeur de <nosCourant> avec la MT <m>,***"
		// "*** - en pos 2 : la prémissse de cette règle,      ***"
		// "*** - en pos 3 : le numéro de cette règle,         ***"
		// "*** - en pos 4 : la priorité de cette règle.       ***"

		String regleCourante = "", premisse = "";
		int numRegle = 0, priorite = 11;

		List<List<String>> st = ToolNos.asStructureSyntaxe(nosCourant.getRegles());
		for (List<String> elem : st) {
			if (elem.get(0).equals("si:")) {
				try {
					if (baseFaits.evalueFormule(ToolNos.analyseFormule(elem))) {
						int p = elem.lastIndexOf("priorite:");
						int prioriteLocale = 10;
						if (p != -1) {
							prioriteLocale = new Integer(elem.get(p + 1))
									.intValue();
						}
						
						if (prioriteLocale < priorite) {
							regleCourante = ToolNos
									.construitRegleAvecCollection(elem);
							premisse = ToolNos.analyseFormule(elem);
							numRegle = st.lastIndexOf(elem);
							priorite = prioriteLocale;
						}
					}
				} catch (NoSException e) {
					e.printStackTrace();
				}
			}
			if (elem.get(0).equals("aller:")) {
				int p = elem.lastIndexOf("priorite:");
				int prioriteLocale = 10;
				if (p != -1) {
					prioriteLocale = new Integer(elem.get(p + 1)).intValue();
				}
				
				if (prioriteLocale < priorite) {
					regleCourante = ToolNos.construitRegleAvecCollection(elem);
					premisse = "";
					numRegle = st.lastIndexOf(elem);
					priorite = prioriteLocale;
				}
			}
		}
		
		if (regleCourante.isEmpty()) {
			Edi.nos().message("Anomalie - regleCourante");
		}
		
		return Arrays.asList(regleCourante, premisse, String.valueOf(numRegle),
				String.valueOf(priorite));
	}

	public void retourne(String fait) {
		// "*** aiguillage en fonction de <exAsoumettre> : ***"
		// "*** on remplit un peu l'historique : ***"

		String ch;
		System.out.println("Revi : retourne(" + fait + ")");

		if (fait == null) {
			ch = "Annulation...";
		} else {
			ch = fait;
		}
		
		historiqueSession = historiqueSession + ch + "\n";

		if (fait == null) {
			annuleSes();
			return;
		}
		;
		if (exAsoumettre == null) {
			retourneRevision(fait);
		} else {
			if (generalise) {
				retourneSoumGen(fait);
			} else {
				retourneSoumission(fait);
			}
		}
	}

	private void retourneRevision(String fait) {
		// "*** (à la suite de la présentation de <ExCourant> à l'utilisateur) ***"

		if (fait.equals("rejetteExemple")) {
			Edi.nos()
					.message(
							"Ce cas de révision a été enregistré dans la fenêtre des sessions de révision. Vous pourrez le réviser plus tard...");
		}

		exValides.add(exCourant);
		MoteurBilan nouv = nouvelExCourant();
		if (nouv == null) {
			clotSes();
		} else {
			exCourant = nouv;
			continueRevision();
		}
	}

	private void retourneSoumGen(String fait) {
		System.out.println("interruption retourneSoumGen");
		InterfaceRevi.revisionInterrompue(filRevision);
	}

	private void retourneSoumission(String fait) {
		// "*** à la suite de la soumission de <ExSoumis> ***"
		// "*** S'il y a eu acceptation, c'est terminé.   ***"
		// "*** Sinon, on continue ...                    ***"

		if (fait.equals("rejetteExemple")) {
			// "*** tout d'abord, un contrôle : ***"
			for (MoteurBilan ex : exAvalider) {
				if (ex != null) {
					if (ex.chaineResultat().contains(exSoumis.chaineResultat())) {
						Edi.nos().message("Anomalie - retourneSoumission");
					}
				}
			}
			verrou = verrou + 1;
			if (exAsoumettre.isEmpty()) {
				exAsoumettre = null;
				filRevision.retourne("nodSuivantAoublier");
			} else {
				continueSoumission();
			}
		}

		if (fait.equals("accepteExemple")) {
			// "*** Retrait éventuel de <ExAvalider> : ***"
			majExAvalider(Arrays.asList(exSoumis));
			if (verrou > 1) {
				casAVoir();
				return;
			}
			;
			exAsoumettre = null;
			filRevision.retourne("");
		}
	}

	public void reviseNoduleAvecRegle(NoS nosCourant, String regle) {

		NoS nosInit = Edi.nos().getBaseCourante()
				.getGrapheQuiContientNos(nosCourant.getNom()).getPremierNoS();
		if (nosInit == null) {
			// TODO : lancer une exception
			Edi.nos().message("Anomalie - reviseNoduleAvecRegle");
		}

		Bilan departAvant = new Bilan();
		departAvant.lance(nosInit, nosCourant);
		Bilan departApres = departAvant.duplique();

		// * "*** lancement de la seconde phase ***"
		NoS n = new NoS(nosCourant.getNom());
		n.setActions(nosCourant.getActions());
		n.setRegles(nosCourant.getRegles());
		n.setBase(nosCourant.getBase());
		Bilan explorAvant = departAvant.poursuitAvec(n);
		SortedSet<String> avant = explorAvant.crdu();

		n.setRegles(regle);
		Bilan explorApres = departApres.poursuitAvec(n);
		SortedSet<String> apres = explorApres.crdu();

		if (apres.equals(avant)) {
			echecRev();
			return;
		}

		String string = "EN MOINS :" + "\n";
		List<MoteurBilan> explorMoins = new ArrayList<MoteurBilan>();
		for (MoteurBilan c : explorAvant.getMoteurBilans()) {
			if (!apres.contains(c.chaineResultat())) {
				explorMoins.add(c);
				string = string + c.chaineResultat() + "\n";
			}
		}
		
		string = string + "\n" + "EN PLUS :" + "\n";
		List<MoteurBilan> explorPlus = new ArrayList<MoteurBilan>();
		for (MoteurBilan c : explorApres.getMoteurBilans()) {
			if (!avant.contains(c.chaineResultat())) {
				explorPlus.add(c);
				string = string + c.chaineResultat() + "\n";
			}
		}
		;
		Revi.nos().ajouteHistorique(string);

		// * "*** La révision est acquittée: ***"
		nosCourant.setRegles(regle);

		// /* "*** <explorMoins> est utile pour retirer ***"
		// * "*** des exemples de <ExAvalider>  :      ***"
		Revi.nos().majExAvalider(explorMoins);

		// * "*** <explorPlus> sert pour remplir <ExAvalider> : ***"
		Revi.nos().etapeSuivante(explorPlus);
	}

	public void soumet(List<MoteurBilan> exs, Moteur fil) {
		String string = "A SOUMETTRE : " + "\n";
		for (MoteurBilan c : exs) {
			string = string + c.chaineResultatPlusAction() + "\n";
		}
		
		Revi.nos().ajouteHistorique(string);
		exAsoumettre = exs;
		filRevision = fil;
		continueSoumission();
	}

	private void transcritExemples() {
		ajouteHistorique("EXEMPLES VALIDÉS : \n");
		for (MoteurBilan ex : exValides) {
			historiqueSession = historiqueSession + ex.chaineResultat() + "\n";
		}
	}

	public void lance(Moteur moteurCible, Moteur moteurRev) {
		// nouvelle session de révision.
		// On réinit <ExCourant>, <Verrou> et <FilRevision>. 

		filRevision = moteurRev;
		exCourant = new MoteurBilan(moteurCible.getNoS(), moteurCible.contexte);
		exCourant.setBaseFaits(moteurCible.getBaseFaits().duplique());
		verrou = 0;
	}

	public void operateurA1(Moteur fil) {
		// "*** Remplace dans la règle activable, <nomNodSuivant> par <nomNodCible> :***"
		filRevision = fil;

		String nomNosCourant = filRevision.getBaseFaits().valeur(
				"nodCourant.nom");
		NoS nosCourant = Edi.nos().getBaseCourante().getNoS(nomNosCourant);
		String nomNosSuivant = filRevision.getBaseFaits().valeur(
				"nodSuivant.nom");
		String nomNosCible = filRevision.getBaseFaits().valeur("nodCible.nom");
		NoS nosCible = Edi.nos().getBaseCourante().getNoS(nomNosCible);

		// "*** On repère <regleActivable> : ***"
		List<String> col = Revi.nos().regleCourante(nosCourant,
				filRevision.getCible().getBaseFaits());
		String regleActivable = col.get(0);
		int numRegle = Integer.valueOf(col.get(2));

		String chaine2 = regleActivable.replace(nomNosSuivant, nomNosCible);
		if (!nosCourant.getRegles().contains(regleActivable)) {
			// TODO : lancer une exception
			Edi.nos().message("Anomalie - operateurA1");
		}

		String chaine3 = nosCourant.getRegles()
				.replace(regleActivable, chaine2);
		Revi.nos().ajouteHistorique(
				"PARCOURS : " + filRevision.getBaseFaits().ligneParcours()
						+ "\n");
		Revi.nos().ajouteHistorique(
				"corriger_règles (" + nomNosCourant + ", " + nomNosCible
						+ ") : modifieRègle.\n" + "ANCIENNES : \n"
						+ nosCourant.getRegles() + "\nNOUVELLES : \n" + chaine3
						+ "\n");
		Revi.nos().majReglesRevues(nomNosCourant, null);

		// "*** On met à jour le graphe : ***"
		nosCible.setPosition((String.valueOf(nosCourant.getPosX() + 70)) + "@"
				+ String.valueOf(nosCourant.getPosY() + 50));
		nosCourant.addSuivant(nosCible.getNom());

		// "*** Poursuite de la révision: calcul des répercussions: ***"
		Revi.nos().reviseNoduleAvecRegle(nosCourant, chaine3);
	}

	private void majReglesRevues(String nomNosCourant, List<String> assoc) {

	}

	public void creeRegle(Moteur fil) {
		// "*** Crée une règle pour aller dans <nomNodCible> :***"
		
		filRevision = fil;
		String chaine = "";

		String nomNosCourant = filRevision.getBaseFaits().valeur(
				"nodCourant.nom");
		NoS nosCourant = Edi.nos().getBaseCourante().getNoS(nomNosCourant);
		String nomNosCible = filRevision.getBaseFaits().valeur("nodCible.nom");
		NoS nosCible = Edi.nos().getBaseCourante().getNoS(nomNosCible);

		System.out.println("nosCourant = " + nomNosCourant + " - nosCible = "
				+ nomNosCible);

		List<String> histo = filRevision.getCible().getBaseFaits()
				.getListeHistorique();
		String ligneHisto = histo.get(histo.size() - 1);

		
		
		String[] st = ligneHisto.split(" -> ");
		
		String fait = "";
		if (st.length > 1) {
			fait = st[1];
		}
		

		if (fait.isEmpty()) {
			List<String> colFaits = nosCourant.tousLesFaitsAttendus();
			if (colFaits.size() > 2) {
				chaine = "aller: " + nomNosCible + ";\n";
			} else {
				for (String f : colFaits) {
					if (!f.isEmpty()) {
						chaine = "si: NON(" + f + ") alorsAller: "
								+ nomNosCible + ";\n";
					}
				}
			}
		} else { // fait n'est pas vide...
			chaine = "si: " + fait + " alorsAller: " + nomNosCible + ";\n";
		}

		String choix = nosCourant.getRegles();
		if (!choix.isEmpty()) {
			if (choix.endsWith(";")) {
				chaine = "\n" + chaine;
			} else {
				if (choix.charAt(choix.length() - 1) != '\n') {
					chaine = ";\n" + chaine;
				}
			}
		}

		String newRegle = choix + chaine;
		Revi.nos().ajouteHistorique(
				"PARCOURS : " + filRevision.getBaseFaits().ligneParcours()
						+ "\n");
		Revi.nos().ajouteHistorique(
				"corriger_règles (" + nomNosCourant + ", " + nomNosCible
						+ ") : créeRègle.\nANCIENNES:\n"
						+ nosCourant.getRegles() + "\nNOUVELLES:\n" + newRegle
						+ "\n");

		// "*** On met à jour le graphe : ***"
		nosCible.setPosition((String.valueOf(nosCourant.getPosX() + 70)) + "@"
				+ String.valueOf(nosCourant.getPosY() + 50));
		nosCourant.addSuivant(nosCible.getNom());

		// "*** Poursuite de la révision: calcul des répercussions: ***"
		Revi.nos().reviseNoduleAvecRegle(nosCourant, newRegle);
	}

	public void ouvreFenSessions(Moteur m, Statut s) {
		if (fenSessions == null) {
			fenSessions = new FenSessions();
			fenSessions.setSize(400, 120); // 400, 160);
			fenSessions.setLocation(13, 464); // (5, 440);
			Edi.nos().getBaseCourante().ajouteFen(fenSessions);
			((FenEdiNoS) Edi.nos().fen()).ouvreInternalFrameSansPositionner(fenSessions);
			fenSessions.setVisible(true);
		}
		fenSessions.majSession(m, s);
	}

	public void fermeFenSessions() {
		fenSessions = null;
	}
}
