/*
  EdiNoS - Editeur de NoS
  Copyright (C) 2014 - Luc Poittevin — Tous droits réservés.
  Contact : edinos@free.fr  -  https://www.edinos.org
  
  Ce programme est un logiciel libre ; vous pouvez le redistribuer ou le
  modifier suivant les termes de la “GNU General Public License” telle que
  publiée par la Free Software Foundation : soit la version 3 de cette
  licence, soit (à votre gré) toute version ultérieure.
  
  Ce programme est distribué dans l’espoir qu’il vous sera utile, mais SANS
  AUCUNE GARANTIE : sans même la garantie implicite de COMMERCIALISABILITÉ
  ni d’ADÉQUATION À UN OBJECTIF PARTICULIER. Consultez la Licence Générale
  Publique GNU pour plus de détails.
  
  Vous devriez avoir reçu une copie de la Licence Générale Publique GNU avec
  ce programme ; si ce n’est pas le cas, consultez :
  <http://www.gnu.org/licenses/>. 
 */

package nos.revinos;

import java.util.AbstractMap;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;

import javax.swing.ImageIcon;
import javax.swing.table.AbstractTableModel;

import nos.obj.Edi;
import nos.run.FenVoirSituation;
import nos.edit.FenEdiNoS;
import nos.run.Moteur;

@SuppressWarnings("serial")
public class Sessions extends AbstractTableModel {

	public static enum Statut {
		Terminée, // la session de révision est terminée
		NonDémarrée, // la session de révision n'a pas encore démarré
		EnCours, // / une session de révision est en cours pour ce cas
		AStatuer, // cas à statuer (examen reporté)
	}

	private String[] colonnes = new String[] { "", "N°", "Nom du cas", "Statut",
			"", "" };

	private List<Map.Entry<String, Moteur>> listeMoteurs;

	private List<Statut> listeStatuts;

	private int prochainNum;

	public Sessions() {
		super();
		listeMoteurs = new ArrayList<Map.Entry<String, Moteur>>();
		listeStatuts = new ArrayList<Statut>();
		prochainNum = 1;
	}

	public int getColumnCount() {
		return colonnes.length;
	}

	public int getRowCount() {
		return listeMoteurs.size();
	}

	public Object getValueAt(int i, int j) {
		java.net.URL imgURL = null;
		switch (j) {
		case 0: // colonne Bouton Supprimer
			imgURL = getClass().getResource("/icones/delete.png");
			return new ImageIcon(imgURL);
		case 1: // colonne Num
			return String.valueOf(i + 1);
		case 2: // colonne Nom
			return listeMoteurs.get(i).getKey();
		case 3: // colonne Statut
			switch (listeStatuts.get(i)) {
			case EnCours:
				return "Révision en cours";
			case NonDémarrée:
				return "Révision non démarrée";
			case Terminée:
				return "Révision terminée";
			case AStatuer:
				return "Cas à statuer";
			default:
				return "";
			}
		case 4: // colonne Bouton Voir
			imgURL = getClass().getResource("/icones/application_form_magnify.png");
			return new ImageIcon(imgURL);
		case 5: // colonne Bouton Sauvegarder
			imgURL = getClass().getResource("/icones/disk.png");
			return new ImageIcon(imgURL);
		default:
			return null;
		}
	}

	public boolean isCellEditable(int row, int col) {
		switch (col) {
		case 0: // colonne Bouton Supprimer
			return true;
		case 1: // colonne Num
			return false;
		case 2: // colonne Nom
			return true;
		case 3: // colonne Statut
			return false;
		case 4: // colonne Bouton Voir
			return true;
		case 5: // colonne Bouton Sauvegarder
			return true;
		default:
			return false;
		}
	}

	public void setValueAt(Object value, int row, int col) {
		if (col == 2) {
			Map.Entry<String, Moteur> tuple = new AbstractMap.SimpleEntry<String, Moteur>(
					value.toString(), listeMoteurs.get(row).getValue());
			listeMoteurs.set(row, tuple);
		}
	}

	public String getColumnName(int c) {
		return (colonnes[c]);
	}

	public void ouvre(int i) {
		Edi.nos().montreAvertissement(
				"Ouverture de " + listeMoteurs.get(i).getKey());
		FenVoirSituation f = new FenVoirSituation(listeMoteurs.get(i)
				.getValue());
		f.setLocation(2, 2);
		listeMoteurs.get(i).getValue().getNoS().getBase().ajouteFen(f);
		((FenEdiNoS) Edi.nos().fen()).ouvreInternalFrameSansPositionner(f);
		f.setVisible(true);
	}

	public void sauve(int i) {
		listeMoteurs.get(i).getValue().sauve();
	}

	public void supprime(int i) {
		listeMoteurs.remove(i);
		listeStatuts.remove(i);
		this.fireTableDataChanged();
	}

	public void ajouteSession(Moteur m, Statut s) {
		listeMoteurs.add(new AbstractMap.SimpleEntry<String, Moteur>(
				"Cas numéro " + String.valueOf(prochainNum), m));
		listeStatuts.add(s);
		prochainNum = prochainNum + 1;
		this.fireTableDataChanged();
	}

	public void termineSessionEnCours() {
		int i = listeStatuts.indexOf(Statut.EnCours);
		listeStatuts.set(i, Statut.Terminée);
		this.fireTableDataChanged();
	}

	public void majSession(Moteur m, Statut s) {
		// on ajoute la session que si elle n'est pas déjà dans la liste...
		int i = 0;
		for (Map.Entry<String, Moteur> entry : listeMoteurs) {
			if (entry.getValue().equals(m)) {
				listeStatuts.set(i, s);
				this.fireTableDataChanged();
				return;
			}
			i++;
		}
		ajouteSession(m, s);
	}
}
