/*
  EdiNoS - Editeur de NoS
  Copyright (C) 2013-2021 - Luc Poittevin — Tous droits réservés.
  Contact : edinos@free.fr  -  https://www.edinos.org
  
  Ce programme est un logiciel libre ; vous pouvez le redistribuer ou le
  modifier suivant les termes de la “GNU General Public License” telle que
  publiée par la Free Software Foundation : soit la version 3 de cette
  licence, soit (à votre gré) toute version ultérieure.
  
  Ce programme est distribué dans l’espoir qu’il vous sera utile, mais SANS
  AUCUNE GARANTIE : sans même la garantie implicite de COMMERCIALISABILITÉ
  ni d’ADÉQUATION À UN OBJECTIF PARTICULIER. Consultez la Licence Générale
  Publique GNU pour plus de détails.
  
  Vous devriez avoir reçu une copie de la Licence Générale Publique GNU avec
  ce programme ; si ce n’est pas le cas, consultez :
  <http://www.gnu.org/licenses/>. 
*/

package nos.run;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.KeyEvent;
import java.util.List;

import javax.swing.DefaultListModel;
import javax.swing.JComponent;
import javax.swing.JInternalFrame;
import javax.swing.JMenu;
import javax.swing.JMenuBar;
import javax.swing.JMenuItem;
import javax.swing.KeyStroke;
import javax.swing.event.InternalFrameEvent;
import javax.swing.event.InternalFrameListener;

import nos.edit.FenNos;
import nos.edit.FenEdiNoS;
import nos.obj.Base;
import nos.obj.Edi;
import nos.obj.NoS;
import nos.revinos.Revi;
import nos.revinos.Sessions;
import nos.run.Moteur.Contexte;
import nos.run.Moteur.ModeExecutionProc;
import nos.run.edit.FenLancementFaitsConnus;
import nos.utils.ToolNos;

@SuppressWarnings("serial")
public class FenVoirSituation extends JInternalFrame implements
		InternalFrameListener {

	private Moteur moteur;

	private JMenuBar menuBar;

	private boolean situationInedite;

	public FenVoirSituation(Moteur m) {
		super("Voir la situation", true, // resizable
				true, // closable
				true, // maximizable
				true);// iconifiable
		addInternalFrameListener(this);
		moteur = m;
		initComponents();
		initComponents2();
		installeMenus();
	}

	private void initComponents2() {
		// "*** Situation inédite si l'action du nos courant ***"
		// "*** est une proc qui a déjà été exécutée...         ***"
		

		
		situationInedite = moteur.getBaseFaits().actionDejaExecutee(
				moteur.getNomActionCourante());

		if (situationInedite) {
			this.setTitle("Situation inédite");
			jButton3.setEnabled(false);
			jLabel6.setText("A définir");
			jTextArea1.setText("A définir");
			jTextArea2.setText("A définir");
			// Nom de la situation précédente :
			jLabel7.setText(moteur.nos.getNom());
		} else {
			jLabel6.setText(moteur.nos.getNom());
			jTextArea1.setText(moteur.nos.getCommentaires());
			jTextArea2.setText(moteur.nos.getActions());
			// Nom de la situation précédente :
			List<String> p = moteur.getBaseFaits().getParcours();
			if (p.size() > 1) {
				jLabel7.setText(p.get(p.size() - 2));
			}
		}
		
		
		/* Onglet "Etat de la MT" : */
		DefaultListModel<String> listModel = new DefaultListModel<String>();
		for (String nomFait : moteur.baseFaits.getFaits()) {
			listModel.addElement(nomFait);
		}
		jList3.setModel(listModel);
		jScrollPane5.setViewportView(jList3);

		/* Onglet "Historique" : */
		DefaultListModel<String> listModel2 = new DefaultListModel<String>();
		for (String nomFait : moteur.baseFaits.getListeHistorique()) {
			listModel2.addElement(nomFait);
		}
		jList2.setModel(listModel2);
		jScrollPane4.setViewportView(jList2);
		
		jButton1.setName("Réviser"); // pour FEST
		jButton2.setName("Annuler"); // pour FEST
 		jLabel6.setName("NomSituation"); // pour FEST
    	jList3.setName("bF"); // pour FEST
    	jList2.setName("historique"); // pour FEST
    	jLabel7.setName("SituationPrec"); // pour FEST
    	jTextArea2.setName("ActionCourante"); // pour FEST
    	
		ActionListener actionListenerEsc = new ActionListener() {
			public void actionPerformed(ActionEvent actionEvent) {
				// Modif 10/03/2021 : // LPO
				fermeFenetre();
			}
		};

		KeyStroke stroke = KeyStroke.getKeyStroke(KeyEvent.VK_ESCAPE, 0);
		getRootPane().registerKeyboardAction(actionListenerEsc, stroke,
					JComponent.WHEN_IN_FOCUSED_WINDOW);      
	}

	private void jButton2ActionPerformed(java.awt.event.ActionEvent evt) {
		// Bouton Annuler
		// Modif 10/03/2021 : // LPO
		// dispose(); // LPO
		fermeFenetre();
	}

	private void jButton1ActionPerformed(java.awt.event.ActionEvent evt) {

		Base b = Edi.nos().getBase("REVINOS");
		if (b == null) {
			Edi.nos().message("Le graphe REVINOS n'a pas été trouvé...");
		} else {
			// TODO : mettre les coordonnées dans la MT // Pourquoi ? // LPO
			// Point p = getLocation(); // LPO
			// Dimension size = getContentPane().getSize(); // LPO
			// p.setLocation(p.x + size.width + 5, p.y); // LPO
			// System.out.println(p.toString()); // LPO

			Revi.nos().init();
	    	Revi.nos().ouvreFenSessions(moteur, Sessions.Statut.EnCours);
	    	
	    	// On ferme les fenêtres qui ont amené à la révision :
	    	FenEdiNoS ed = ((FenEdiNoS) Edi.nos().fen());
	    	ed.fermeFen("Lancement du graphe terminé");
	    	ed.fermeFen("Simulation de procédure");
	    	ed.fermeFen("Question");
	    	ed.fermeFen("Affichage");

			Revi.nos().ajouteHistorique(
					"ORIGINE DE LA REVISION :" + "\n"
							+ moteur.getBaseFaits().ligneHistorique() + ".\n"
							+ "BASE : " + Edi.nos().getBaseCourante().getNom()
							+ "\n");
			
			NoS reviNoS = b.getNoS("REVINOS");
			Moteur m = new Moteur(reviNoS, Contexte.Lancement, ModeExecutionProc.Mixte);
			m.setCible(moteur);
			Memoire mem = new Memoire();
			if (situationInedite) {
				mem.ajouteFait("situationInédite");
			}
			m.declencheAvec(mem);
		}
	}
	
	private void jButton3ActionPerformed(java.awt.event.ActionEvent evt) {
		// Bouton Voir le nos courant
		FenNos f = new FenNos(moteur.getNoS());
		Edi.nos().getBaseCourante().ajouteFen(f);
		((FenEdiNoS) Edi.nos().fen()).ouvreInternalFrame(f);
	}
	
	private void jButton4ActionPerformed(java.awt.event.ActionEvent evt) {
		// Bouton Voir le nos précédent
		if (!jLabel7.getText().isEmpty()) {
		FenNos f = new FenNos((NoS) Edi.nos().getBaseCourante()
				.getAssocNos().get(jLabel7.getText()));
		Edi.nos().getBaseCourante().ajouteFen(f);
		((FenEdiNoS) Edi.nos().fen()).ouvreInternalFrame(f);
		}
	}

	private void installeMenus() {
		menuBar = ((FenEdiNoS) Edi.nos().fen()).menuBarEdinos();
		menuBar.add(menuSituation());
		Edi.nos().fen().setJMenuBar(menuBar);
		Edi.nos().fen().setVisible(true);
	}

	private JMenu menuSituation() {
		JMenuItem menuItem;
		JMenu menu = new JMenu("Situation");

		menuItem = new JMenuItem("MonTrer les faits connus");
		menuItem.setAccelerator(KeyStroke.getKeyStroke(
				java.awt.event.KeyEvent.VK_T, ToolNos.renvoieModifiers()));
		menuItem.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				montreFaitsConnus();
			}
		});
		menu.add(menuItem);
		menu.addSeparator();
		
		menuItem = new JMenuItem("Sauvegarder la trace");
		menuItem.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				moteur.sauve();
			}
		});
		menu.add(menuItem);
		menu.addSeparator();

		menuItem = new JMenuItem("Fermer la fenêtre");
		menuItem.setAccelerator(KeyStroke.getKeyStroke(
				java.awt.event.KeyEvent.VK_W, ToolNos.renvoieModifiers()));
		menuItem.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				// Modif 10/03/2021 : // LPO
				fermeFenetre();
			}
		});
		menu.add(menuItem);
		return (menu);
	}

	protected void fermeFenetre() {		
		// Ajout 10/03/2021 : // LPO
		if (ToolNos.nomSysteme().equals("Mac")) {
			ToolNos.fermeFen((JInternalFrame) this);
		} else {
			dispose();
		}
		((FenEdiNoS) Edi.nos().fen()).checkDerniereFen(); 
	}
	
	protected void montreFaitsConnus() {
		FenLancementFaitsConnus f = new FenLancementFaitsConnus(null, moteur.getBaseFaits().getHistoriqueFaits());
		f.setSize(550, 500);
		Edi.nos().getBaseCourante().ajouteFen(f);
		((FenEdiNoS) Edi.nos().fen()).ouvreInternalFrameSansPositionner(f);
		f.setVisible(true);
	}

	public void internalFrameActivated(InternalFrameEvent e) {
		Edi.nos().fen().setJMenuBar(menuBar);
		Edi.nos().fen().setVisible(true);
	}

	public void internalFrameClosed(InternalFrameEvent e) {
	}

	public void internalFrameClosing(InternalFrameEvent e) {
		fermeFenetre();
	}

	public void internalFrameDeactivated(InternalFrameEvent e) {
		((FenEdiNoS) Edi.nos().fen()).setMenuBar();
	}

	public void internalFrameDeiconified(InternalFrameEvent e) {
	}

	public void internalFrameIconified(InternalFrameEvent e) {
	}

	public void internalFrameOpened(InternalFrameEvent e) {
	}

    /** This method is called from within the constructor to
     * initialize the form.
     * WARNING: Do NOT modify this code. The content of this method is
     * always regenerated by the Form Editor.
     */
    
    // <editor-fold defaultstate="collapsed" desc="Generated Code">
    private void initComponents() {

        jLabel2 = new javax.swing.JLabel();
        jLabel1 = new javax.swing.JLabel();
        jLabel3 = new javax.swing.JLabel();
        jLabel4 = new javax.swing.JLabel();
        jLabel5 = new javax.swing.JLabel();
        jButton1 = new javax.swing.JButton();
        jButton2 = new javax.swing.JButton();
        jScrollPane2 = new javax.swing.JScrollPane();
        jTextArea1 = new javax.swing.JTextArea();
        jScrollPane3 = new javax.swing.JScrollPane();
        jTextArea2 = new javax.swing.JTextArea();
        jLabel6 = new javax.swing.JLabel();
        jLabel7 = new javax.swing.JLabel();
        jTabbedPane1 = new javax.swing.JTabbedPane();
        jScrollPane4 = new javax.swing.JScrollPane();
        jList2 = new javax.swing.JList<String>();
        jScrollPane5 = new javax.swing.JScrollPane();
        jList3 = new javax.swing.JList<String>();
        jLabel8 = new javax.swing.JLabel();
        jButton3 = new javax.swing.JButton();
        jButton4 = new javax.swing.JButton();

        setMinimumSize(new java.awt.Dimension(720, 460));
        setPreferredSize(new java.awt.Dimension(720, 460));

//        setMinimumSize(new java.awt.Dimension(710, 440));
//        setPreferredSize(new java.awt.Dimension(710, 440));
        
        
        jLabel2.setText("Situation :");

        jLabel1.setText("Informations connues");

        jLabel3.setText("Description :");

        jLabel4.setText("Actions effectuées dans cette situation :");

        jLabel5.setText("Situation précédente :");

        jButton1.setText("Réviser");
        jButton1.setToolTipText("Lancer la révision avec ReviNoS");
        jButton1.setMaximumSize(new java.awt.Dimension(90, 30));
        jButton1.setMinimumSize(new java.awt.Dimension(90, 30));
        jButton1.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jButton1ActionPerformed(evt);
            }
        });

        jButton2.setText("Annuler");
        jButton2.setToolTipText("La révision est inutile");
        jButton2.setMaximumSize(new java.awt.Dimension(90, 30));
        jButton2.setMinimumSize(new java.awt.Dimension(90, 30));
        jButton2.setPreferredSize(new java.awt.Dimension(89, 29));
        jButton2.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jButton2ActionPerformed(evt);
            }
        });

        jTextArea1.setBackground(java.awt.Color.lightGray);
        jTextArea1.setColumns(20);
        jTextArea1.setEditable(false);
        jTextArea1.setRows(5);
        jTextArea1.setCursor(new java.awt.Cursor(java.awt.Cursor.DEFAULT_CURSOR));
        jTextArea1.setFocusable(false);
        jScrollPane2.setViewportView(jTextArea1);

        jTextArea2.setBackground(java.awt.Color.lightGray);
        jTextArea2.setColumns(20);
        jTextArea2.setEditable(false);
        jTextArea2.setRows(3);
        jTextArea2.setFocusable(false);
        jScrollPane3.setViewportView(jTextArea2);

        jLabel6.setBackground(java.awt.Color.lightGray);
        jLabel6.setAlignmentX(0.5F);
        jLabel6.setBorder(javax.swing.BorderFactory.createEtchedBorder());
        jLabel6.setFocusable(false);
        jLabel6.setMinimumSize(new java.awt.Dimension(45, 20));
        jLabel6.setOpaque(true);

        jLabel7.setBackground(java.awt.Color.lightGray);
        jLabel7.setHorizontalAlignment(javax.swing.SwingConstants.LEFT);
        jLabel7.setVerticalAlignment(javax.swing.SwingConstants.TOP);
        jLabel7.setAlignmentX(0.5F);
        jLabel7.setBorder(javax.swing.BorderFactory.createEtchedBorder());
        jLabel7.setFocusable(false);
        jLabel7.setMinimumSize(new java.awt.Dimension(10, 4));
        jLabel7.setOpaque(true);

        jList2.setModel(new javax.swing.AbstractListModel() {
            String[] strings = { "Item 1", "Item 2", "Item 3", "Item 4", "Item 5" };
            public int getSize() { return strings.length; }
            public Object getElementAt(int i) { return strings[i]; }
        });
        jScrollPane4.setViewportView(jList2);

        jTabbedPane1.addTab("Historique  ", jScrollPane4);

        jList3.setModel(new javax.swing.AbstractListModel() {
            String[] strings = { "Element 1", "Element 2", "Element 3", "Element 4", "Element 5" };
            public int getSize() { return strings.length; }
            public Object getElementAt(int i) { return strings[i]; }
        });
        jScrollPane5.setViewportView(jList3);

        jTabbedPane1.addTab("Etat de la MT", jScrollPane5);

        jLabel8.setText("(Mémoire de Travail) :");

        jButton3.setText("...");
        jButton3.setToolTipText("Voir le nos");
        jButton3.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jButton3ActionPerformed(evt);
            }
        });

        jButton4.setText("..");
        jButton4.setToolTipText("Voir le nos");
        jButton4.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jButton4ActionPerformed(evt);
            }
        });

        org.jdesktop.layout.GroupLayout layout = new org.jdesktop.layout.GroupLayout(getContentPane());
        getContentPane().setLayout(layout);
        layout.setHorizontalGroup(
            layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
            .add(layout.createSequentialGroup()
                .addContainerGap()
                .add(layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
                    .add(layout.createSequentialGroup()
                        .add(layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
                            .add(jLabel4)
                            .add(layout.createSequentialGroup()
                                .add(layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
                                    .add(layout.createSequentialGroup()
                                        .add(jLabel3)
                                        .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED, 87, Short.MAX_VALUE))
                                    .add(layout.createSequentialGroup()
                                        .add(jLabel5)
                                        .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED, 11, Short.MAX_VALUE)
                                        .add(jButton4, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, 20, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE))
                                    .add(layout.createSequentialGroup()
                                        .add(jLabel2)
                                        .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED, 84, Short.MAX_VALUE)
                                        .add(jButton3, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, 20, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)))
                                .add(1, 1, 1)
                                .add(layout.createParallelGroup(org.jdesktop.layout.GroupLayout.TRAILING)
                                    .add(jLabel7, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, 245, Short.MAX_VALUE)
                                    .add(org.jdesktop.layout.GroupLayout.LEADING, jScrollPane2, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, 245, Short.MAX_VALUE)
                                    .add(org.jdesktop.layout.GroupLayout.LEADING, jLabel6, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, 245, Short.MAX_VALUE)
                                    .add(org.jdesktop.layout.GroupLayout.LEADING, jScrollPane3, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, 245, Short.MAX_VALUE))))
                        .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                        .add(layout.createParallelGroup(org.jdesktop.layout.GroupLayout.TRAILING, false)
                            .add(layout.createSequentialGroup()
                                .add(jButton2, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                                .add(18, 18, 18)
                                .add(jButton1, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, 97, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE))
                            .add(jTabbedPane1, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, 222, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE))
                        .add(38, 38, 38))
                    .add(org.jdesktop.layout.GroupLayout.TRAILING, layout.createSequentialGroup()
                        .add(layout.createParallelGroup(org.jdesktop.layout.GroupLayout.TRAILING)
                            .add(jLabel8)
                            .add(jLabel1))
                        .add(69, 69, 69))))
        );
        layout.setVerticalGroup(
            layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
            .add(layout.createSequentialGroup()
                .add(27, 27, 27)
                .add(jLabel1)
                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                .add(jLabel8)
                .add(18, 18, 18)
                .add(layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
                    .add(jTabbedPane1, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, 290, Short.MAX_VALUE)
                    .add(layout.createSequentialGroup()
                        .add(4, 4, 4)
                        .add(layout.createParallelGroup(org.jdesktop.layout.GroupLayout.BASELINE)
                            .add(jLabel2)
                            .add(jButton3, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, 20, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE))
                        .add(266, 266, 266))
                    .add(org.jdesktop.layout.GroupLayout.TRAILING, layout.createSequentialGroup()
                        .add(4, 4, 4)
                        .add(jLabel6, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, 20, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                        .addPreferredGap(org.jdesktop.layout.LayoutStyle.UNRELATED)
                        .add(layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
                            .add(jLabel3)
                            .add(jScrollPane2, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, 95, Short.MAX_VALUE))
                        .add(18, 18, 18)
                        .add(jLabel4)
                        .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                        .add(jScrollPane3, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, 67, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                        .add(15, 15, 15)
                        .add(layout.createParallelGroup(org.jdesktop.layout.GroupLayout.TRAILING)
                            .add(org.jdesktop.layout.GroupLayout.LEADING, jLabel7, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, 20, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                            .add(org.jdesktop.layout.GroupLayout.LEADING, layout.createParallelGroup(org.jdesktop.layout.GroupLayout.BASELINE)
                                .add(jLabel5)
                                .add(jButton4, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, 20, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)))
                        .add(13, 13, 13)))
                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                .add(layout.createParallelGroup(org.jdesktop.layout.GroupLayout.BASELINE)
                    .add(jButton1, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                    .add(jButton2, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE))
                .addContainerGap())
        );

        pack();
    }// </editor-fold>        

    // Variables declaration - do not modify
    private javax.swing.JButton jButton1;
    private javax.swing.JButton jButton2;
    private javax.swing.JButton jButton3;
    private javax.swing.JButton jButton4;
    private javax.swing.JLabel jLabel1;
    private javax.swing.JLabel jLabel2;
    private javax.swing.JLabel jLabel3;
    private javax.swing.JLabel jLabel4;
    private javax.swing.JLabel jLabel5;
    private javax.swing.JLabel jLabel6;
    private javax.swing.JLabel jLabel7;
    private javax.swing.JLabel jLabel8;
    private javax.swing.JList<String> jList2;
    private javax.swing.JList<String> jList3;
    private javax.swing.JScrollPane jScrollPane2;
    private javax.swing.JScrollPane jScrollPane3;
    private javax.swing.JScrollPane jScrollPane4;
    private javax.swing.JScrollPane jScrollPane5;
    private javax.swing.JTabbedPane jTabbedPane1;
    private javax.swing.JTextArea jTextArea1;
    private javax.swing.JTextArea jTextArea2;
    // End of variables declaration

}
