/*
  EdiNoS - Editeur de NoS
  Copyright (C) 2017-2021 - Luc Poittevin — Tous droits réservés.
  Contact : edinos@free.fr  -  https://www.edinos.org
  
  Ce programme est un logiciel libre ; vous pouvez le redistribuer ou le
  modifier suivant les termes de la “GNU General Public License” telle que
  publiée par la Free Software Foundation : soit la version 3 de cette
  licence, soit (à votre gré) toute version ultérieure.
  
  Ce programme est distribué dans l’espoir qu’il vous sera utile, mais SANS
  AUCUNE GARANTIE : sans même la garantie implicite de COMMERCIALISABILITÉ
  ni d’ADÉQUATION À UN OBJECTIF PARTICULIER. Consultez la Licence Générale
  Publique GNU pour plus de détails.
  
  Vous devriez avoir reçu une copie de la Licence Générale Publique GNU avec
  ce programme ; si ce n’est pas le cas, consultez :
  <http://www.gnu.org/licenses/>. 
 */

package nos.run.edit;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Container;
import java.awt.Cursor;
import java.awt.Dimension;

import javafx.application.Platform;
import javafx.beans.value.ChangeListener;
import javafx.beans.value.ObservableValue;
import javafx.concurrent.Worker.State;
import javafx.embed.swing.JFXPanel;
import javafx.scene.Group;
import javafx.scene.Scene;
import javafx.scene.web.WebEngine;
import javafx.scene.web.WebView;

import javax.swing.BorderFactory;
import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.JFrame;
import javax.swing.JInternalFrame;
import javax.swing.JLabel;
import javax.swing.JPanel;

import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;
import org.w3c.dom.events.Event;
import org.w3c.dom.events.EventListener;
import org.w3c.dom.events.EventTarget;

import nos.obj.Edi;
import nos.obj.Message;
import nos.run.Moteur;
import nos.utils.PrefsManager;

public class FenAfficheDelegFX extends FenDialogueDeleg implements IFenDialogue {

	protected JFXPanel fxPanel = new JFXPanel();

	protected WebEngine webEngine;
	
	private Message message;

	public FenAfficheDelegFX(Message m, Container fen) {
		super();
		message = m;
		frame = fen;
		init("Message <" + m.getNom() + ">");

		if (getMoteur() != null) {
			getMoteur().traceAffichageMessage(m.getNom());
		}
	}
	
	public void init(String titre) {
		if (frame instanceof JInternalFrame) {
			((JInternalFrame) frame).setTitle(titre);
			((JInternalFrame) frame).getContentPane().setLayout(
					new BorderLayout());
		} else {
			((JFrame) frame).setTitle(titre);
			((JFrame) frame).getContentPane().setLayout(new BorderLayout());
		}

		JPanel panel = new JPanel();
		panel.setLayout(new BoxLayout(panel, BoxLayout.LINE_AXIS));
		getBoutonRetour().setName("Retour"); // pour FEST
		panel.add(getBoutonRetour());
		getBoutonAide().setName("?"); // pour FEST
		getBoutonAide().setToolTipText("Pourquoi m'affiche-t-on ce message ?");
		panel.add(getBoutonAide());
		if (getMoteur() != null) {
			getBoutonFermer().setText("Interrompre");
			getBoutonFermer().setToolTipText("Interrompre le lancement");
		} else {
			getBoutonFermer().setText("Fermer");
		}
		getBoutonFermer().setName("Interrompre"); // pour FEST
		panel.add(getBoutonFermer());

		panel.add(Box.createHorizontalGlue());
		panel.setBorder(BorderFactory.createLineBorder(Color.white));
		panel.setPreferredSize(new Dimension(350, 30));

		addToContentPane(panel, BorderLayout.NORTH);

		JPanel panel2 = new JPanel();
		panel2.setLayout(new BoxLayout(panel2, BoxLayout.Y_AXIS));
		JPanel panel2b = new JPanel();
		JLabel l2 = new JLabel(libelle(), JLabel.CENTER);
		l2.setForeground(Color.BLUE);
		l2.setName("libelle"); // pour FEST
		l2.setAlignmentY(JFrame.CENTER_ALIGNMENT);
		panel2b.add(l2, BorderLayout.CENTER);
		panel2.add(panel2b);
		
		Edi.nos().fen().setCursor(Cursor.getPredefinedCursor(Cursor.WAIT_CURSOR));

		Platform.runLater(new Runnable() {
			public void run() {
				// This method is invoked on the JavaFX thread
				Scene scene = createScene();
				fxPanel.setScene(scene);
			}
		});

		int l = ((PrefsManager) Edi.nos().getPreferences())
				.getFenDialogueLargeur();
		int h = ((PrefsManager) Edi.nos().getPreferences())
				.getFenDialogueHauteur();
		panel2.setPreferredSize(new Dimension(l, h));

		panel2.add(fxPanel, BorderLayout.CENTER);
		addToContentPane(panel2, BorderLayout.CENTER);

		setPanelBoutons(new JPanel());
		construitBoutons();
		addToContentPane(getPanelBoutons(), BorderLayout.SOUTH);

		registerEventListeners();

		installeMenus();

		// Resize the container to the preferred size
		if (frame instanceof JInternalFrame) {
			((JInternalFrame) frame).pack();
		} else {
			((JFrame) frame).pack();
		}
		Edi.nos().fen().setCursor(Cursor.getDefaultCursor());
	}

	
	public String libelle() {
		return "Message";
	}

	public void construitBoutons() {
	}

	@Override
	void escape() {
		fermeFenetre();
	}

	private String prepareText() {
		String texte = message.getTexte();
		
		if (getMoteur() != null) {
			texte = getMoteur().valoriseFaits(texte);
		}

		String avant = "<html dir=\"ltr\"><head></head><body contenteditable=\"true\">";
		String debutForm = "<form action=\".\" method=\"post\">";
		String apres = "</body></html>";
		String finForm = "";

		finForm = finForm + "<input type=\"submit\" value=\"Vu\"></form>";
			
		int i = texte.indexOf(avant);
		if (texte.startsWith(avant)) {
			texte = avant + debutForm + texte.substring(i + avant.length());
		} else {
			Edi.nos().montreAvertissement(
					"Erreur de formatage du message <" + message.getNom()
							+ "> (début)");
		}
		if (texte.endsWith(apres)) {
			texte = texte.substring(0, texte.indexOf(apres)) + finForm + apres;
		} else {
			Edi.nos().montreAvertissement(
					"Erreur de formatage du message<" + message.getNom()
							+ "> (fin)");
		}
		return texte;
	}
	
	protected Scene createScene() {

		Group root = new Group();

		WebView browser = new WebView();
		webEngine = browser.getEngine();

		webEngine.getLoadWorker().stateProperty()
				.addListener(new ChangeListener<State>() {
					@Override
					public void changed(ObservableValue ov, State oldState,
							State newState) {
						if (newState == State.SUCCEEDED) {
							Document doc = webEngine.getDocument();

							EventListener clickListener = new EventListener() {
								public void handleEvent(Event ev) {
									Node n = (Node) ev.getTarget();
									Node nodeId = n.getAttributes().getNamedItem("id");

									if (nodeId != null) {
										String id = nodeId.getNodeValue().toString();
										dispose();
										if (getMoteur() != null) {
											getMoteur().retourne(id);
										}
									} else {
										String value = n.getAttributes()
												.getNamedItem("value")
												.getNodeValue().toString();
										dispose();
										if (getMoteur() != null) {
											getMoteur().retourne(null);
										}
									}
								}
							};

							NodeList n = doc.getElementsByTagName("input");
							for (int i = 0, len = n.getLength(); i < len; i++) {
								Element el = (Element) n.item(i);
								((EventTarget) el).addEventListener("click",
										clickListener, false);
							}
						}
					}
				});

		String st = prepareText();
		webEngine.loadContent(st);
		root.getChildren().add(browser);
		Scene scene = new Scene(root, javafx.scene.paint.Color.ALICEBLUE);
		return scene;
	}

	public void remplitZones() {
	}

	protected void cliqueVu() {
		dispose();
		if (getMoteur() != null) {
			getMoteur().retourne(null);
		} 
	}
	
	@Override
	public void retour() {
		dispose();
		Moteur m = getMoteur();
		if (m != null) {
			getMoteur().back();
		} 
	}
}
