/*
  EdiNoS - Editeur de NoS
  Copyright (C) 2013-2021 - Luc Poittevin — Tous droits réservés.
  Contact : edinos@free.fr  -  https://www.edinos.org
  
  Ce programme est un logiciel libre ; vous pouvez le redistribuer ou le
  modifier suivant les termes de la “GNU General Public License” telle que
  publiée par la Free Software Foundation : soit la version 3 de cette
  licence, soit (à votre gré) toute version ultérieure.
  
  Ce programme est distribué dans l’espoir qu’il vous sera utile, mais SANS
  AUCUNE GARANTIE : sans même la garantie implicite de COMMERCIALISABILITÉ
  ni d’ADÉQUATION À UN OBJECTIF PARTICULIER. Consultez la Licence Générale
  Publique GNU pour plus de détails.
  
  Vous devriez avoir reçu une copie de la Licence Générale Publique GNU avec
  ce programme ; si ce n’est pas le cas, consultez :
  <http://www.gnu.org/licenses/>. 
 */

package nos.run.edit;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.KeyEvent;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.InputStream;

import javax.swing.JComponent;
import javax.swing.JEditorPane;
import javax.swing.JInternalFrame;
import javax.swing.JMenu;
import javax.swing.JMenuBar;
import javax.swing.JMenuItem;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.KeyStroke;
import javax.swing.event.InternalFrameEvent;
import javax.swing.event.InternalFrameListener;
import javax.swing.text.BadLocationException;
import javax.swing.text.rtf.RTFEditorKit;

import nos.edit.FenEdiNoS;
import nos.obj.Edi;
import nos.utils.ToolNos;

@SuppressWarnings("serial")
public class FenAideLancement extends JInternalFrame implements
		InternalFrameListener {

	private JMenuBar menuBar;

	public FenAideLancement() {

		super("Aide au paramétrage du lancement", true, // resizable
				true, // closable
				true, // maximizable
				true);// iconifiable
		addInternalFrameListener(this);

		getContentPane().setLayout(new BorderLayout());

		setBackground(Color.gray);

		JPanel topPanel = new JPanel();
		topPanel.setLayout(new BorderLayout());

		// Create an RTF editor window
		RTFEditorKit rtf = new RTFEditorKit();
		JEditorPane editor = new JEditorPane();
		editor.setEditorKit(rtf);
		editor.setBackground(Color.white);		
		editor.setMargin(new Insets(5, 5, 5, 5));
		editor.setEditable(false);

//		// This text could be big so add a scroll pane
		JScrollPane scroller = new JScrollPane();
		scroller.getViewport().add(editor);
		topPanel.add(scroller, BorderLayout.CENTER);
		
		getContentPane().add(topPanel, BorderLayout.CENTER);

		// Load an RTF file into the editor
		try {
			InputStream fi = FenAideLancement.class
					.getResourceAsStream("/doc/ParamLancementEdiNoS.rtf");

			rtf.read(fi, editor.getDocument(), 0);
		} catch (FileNotFoundException e) {
			System.out.println("File not found");
		} catch (IOException e) {
			System.out.println("I/O error");
		} catch (BadLocationException e) {
		}
		
		// Ajout 17/03/2021 : // LPO
		ActionListener actionListenerEsc = new ActionListener() {
			public void actionPerformed(ActionEvent actionEvent) {
				fermeFen();
			}
		};
		KeyStroke stroke = KeyStroke.getKeyStroke(KeyEvent.VK_ESCAPE, 0);
		getRootPane().registerKeyboardAction(actionListenerEsc, stroke, JComponent.WHEN_IN_FOCUSED_WINDOW);

		// installation des menus
		 installeMenus();

		pack();
	}

	private void installeMenus() {
		FenEdiNoS ed = (FenEdiNoS) Edi.nos().fen();
		menuBar = ed.menuBarEdinos();
		menuBar.add(ed.menuBase());
		menuBar.add(menuAide());
		Edi.nos().fen().setJMenuBar(menuBar);
		Edi.nos().fen().setVisible(true);
	}

	private JMenu menuAide() {
		JMenuItem menuItem;
		JMenu menu = new JMenu("Aide");

		menuItem = new JMenuItem("Fermer la fenêtre");
		menuItem.setAccelerator(KeyStroke.getKeyStroke(
				java.awt.event.KeyEvent.VK_W, ToolNos.renvoieModifiers()));
		menuItem.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				fermeFen();
			}
		});
		menu.add(menuItem);
		return (menu);
	}
	
	
	protected void fermeFen() {
		if (ToolNos.nomSysteme().equals("Mac")) {
			ToolNos.fermeFen(this);
		} else {
			dispose();
		} 
		((FenEdiNoS) Edi.nos().fen()).checkDerniereFen();
	}

	public void internalFrameActivated(InternalFrameEvent e) {
		Edi.nos().fen().setJMenuBar(menuBar);
		Edi.nos().fen().setVisible(true);
	}

	public void internalFrameClosed(InternalFrameEvent e) {
	}

	public void internalFrameClosing(InternalFrameEvent e) {
		ToolNos.fermeFen((JInternalFrame) this);
	}

	public void internalFrameDeactivated(InternalFrameEvent e) {
		((FenEdiNoS) Edi.nos().fen()).setMenuBar();
	}

	public void internalFrameDeiconified(InternalFrameEvent e) {
	}

	public void internalFrameIconified(InternalFrameEvent e) {
	}

	public void internalFrameOpened(InternalFrameEvent e) {
	}
}
