/*
  EdiNoS - Editeur de NoS
  Copyright (C) 2013-2019 - Luc Poittevin — Tous droits réservés.
  Contact : edinos@free.fr  -  https://www.edinos.org
  
  Ce programme est un logiciel libre ; vous pouvez le redistribuer ou le
  modifier suivant les termes de la “GNU General Public License” telle que
  publiée par la Free Software Foundation : soit la version 3 de cette
  licence, soit (à votre gré) toute version ultérieure.
  
  Ce programme est distribué dans l’espoir qu’il vous sera utile, mais SANS
  AUCUNE GARANTIE : sans même la garantie implicite de COMMERCIALISABILITÉ
  ni d’ADÉQUATION À UN OBJECTIF PARTICULIER. Consultez la Licence Générale
  Publique GNU pour plus de détails.
  
  Vous devriez avoir reçu une copie de la Licence Générale Publique GNU avec
  ce programme ; si ce n’est pas le cas, consultez :
  <http://www.gnu.org/licenses/>. 
 */

package nos.run.edit;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Container;
import java.awt.Dimension;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.KeyEvent;

import javax.swing.BorderFactory;
import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.JButton;
import javax.swing.JComponent;
import javax.swing.JInternalFrame;
import javax.swing.JLabel;
import javax.swing.JMenu;
import javax.swing.JMenuBar;
import javax.swing.JMenuItem;
import javax.swing.JPanel;
import javax.swing.JTextArea;
import javax.swing.JFrame;
import javax.swing.KeyStroke;

import nos.obj.Edi;
import nos.obj.EdiNoS;
import nos.edit.FenEdiNoS;
import nos.run.FenVoirSituation;
import nos.run.Moteur;
import nos.utils.ToolNos;

public abstract class FenDialogueDeleg {

	protected Container frame;

	protected JMenuBar menuBar;

	FenDialogueDeleg() {
		super();
	}

	abstract String libelle();

	abstract void construitBoutons();

	abstract void remplitZones();

	abstract void escape();

	public void init(String titre) {
		if (frame instanceof JInternalFrame) {
			((JInternalFrame) frame).setTitle(titre);
			((JInternalFrame) frame).getContentPane().setLayout(
					new BorderLayout());
		} else {
			((JFrame) frame).setTitle(titre);
			((JFrame) frame).getContentPane().setLayout(new BorderLayout());
		}

		JPanel panel = new JPanel();
		panel.setLayout(new BoxLayout(panel, BoxLayout.LINE_AXIS));
		getBoutonRetour().setName("Retour"); // pour FEST
		panel.add(getBoutonRetour());
		getBoutonAide().setName("?"); // pour FEST
		getBoutonAide()
				.setToolTipText("Pourquoi me pose-t-on cette question ?");
		panel.add(getBoutonAide());
		getBoutonFermer().setText("Interrompre");
		getBoutonFermer().setToolTipText("Interrompre le lancement");
		getBoutonFermer().setName("Interrompre"); // pour FEST
		panel.add(getBoutonFermer());

		panel.add(Box.createHorizontalGlue());
		panel.setBorder(BorderFactory.createLineBorder(Color.white));
		addToContentPane(panel, BorderLayout.NORTH);

		JPanel panel2 = new JPanel();
		panel2.setLayout(new BoxLayout(panel2, BoxLayout.Y_AXIS));
		JPanel panel2b = new JPanel();
		JLabel l2 = new JLabel(libelle(), JLabel.CENTER);
		l2.setForeground(Color.BLUE);
		l2.setName("libelle"); // pour FEST
		l2.setAlignmentY(JFrame.CENTER_ALIGNMENT);

		panel2b.add(l2, BorderLayout.CENTER);
		panel2.add(panel2b);
		getZoneTexte().setLineWrap(true);
		getZoneTexte().setMargin(new Insets(0, 3, 0, 3));
		getZoneTexte().setName("texte"); // pour FEST
		panel2.add(getZoneTexte());
		panel2.setPreferredSize(new Dimension(350, 200));
		addToContentPane(panel2, BorderLayout.CENTER);

		setPanelBoutons(new JPanel());
		construitBoutons();
		addToContentPane(getPanelBoutons(), BorderLayout.SOUTH);

		// Register the Listeners
		registerEventListeners();

		remplitZones();
		if (getZoneTexte().getLineCount() > 11) {
			panel2.setPreferredSize(new Dimension(350, getZoneTexte()
					.getLineCount() * 18));
		}

		installeMenus();

		// Resize the container to the preferred size
		if (frame instanceof JInternalFrame) {
			((JInternalFrame) frame).pack();
		} else {
			((JFrame) frame).pack();
		}
	}

	protected void addToContentPane(JPanel p, String pos) {
		if (frame instanceof JInternalFrame) {
			((JInternalFrame) frame).getContentPane().add(p, pos);
		} else {
			((JFrame) frame).getContentPane().add(p, pos);
		}
	}

	void dispose() {
		if (frame instanceof JInternalFrame) {
			((JInternalFrame) frame).dispose();
		} else {
			((JFrame) frame).setVisible(false);
			// Attention la ligne suivante fait planter Java 8 / Mac OS 10.12 :
			// ((JFrame) frame).dispose();
		}
	}

	protected void fermeFenetre() {
		if (frame instanceof JFrame) {
			System.exit(0);
		}
		// sinon (JInternalJFrame) on continue :
		
		// Modif 10/03/2021 : // LPO 
		if (ToolNos.nomSysteme().equals("Mac")) {
			ToolNos.fermeFen((JInternalFrame) frame);
		} else {
			dispose();
		}
		((FenEdiNoS) Edi.nos().fen()).checkDerniereFen(); 
	}

	void sauveTrace() {
		getMoteur().sauve();
	}

	public void installeMenus() {
		if (Edi.nos().fen() != null) {
			menuBar = ((FenEdiNoS) Edi.nos().fen()).menuBarEdinos();
			menuBar.add(menuDialogue());
			Edi.nos().fen().setJMenuBar(menuBar);
			Edi.nos().fen().setVisible(true);
		} else {
			EdiNoS.setAppleMenus();
			menuBar = new JMenuBar();
			menuBar.add(menuDialogue());
			((JFrame) frame).setJMenuBar(menuBar);
		}
	}

	public JMenu menuDialogue() {
		JMenu menu = new JMenu("Dialogue");

		menu.add(menuItemVoirSituation());
		menu.add(menuItemEnregistreTrace());

		menu.addSeparator();
		menu.add(menuItemFermeFenetre());
		return (menu);
	}

	protected JMenuItem menuItemFermeFenetre() {
		JMenuItem menuItem;
		menuItem = new JMenuItem("Fermer la fenêtre");
		menuItem.setAccelerator(KeyStroke.getKeyStroke(
				java.awt.event.KeyEvent.VK_W, ToolNos.renvoieModifiers()));
		menuItem.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				fermeFenetre();
			}
		});
		return menuItem;
	}

	protected JMenuItem menuItemEnregistreTrace() {
		JMenuItem menuItem;
		menuItem = new JMenuItem("Sauvegarder la trace");
		menuItem.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				sauveTrace();
			}
		});
		return menuItem;
	}

	protected JMenuItem menuItemVoirSituation() {
		JMenuItem menuItem;
		menuItem = new JMenuItem("Voir la Situation");
		menuItem.setAccelerator(KeyStroke.getKeyStroke(
				java.awt.event.KeyEvent.VK_T, ToolNos.renvoieModifiers()));
		menuItem.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				voirSituation();
			}
		});
		if ((getMoteur() == null) || (Edi.nos().fen() == null)) {
			// fonctionnalité inactive en exécution par ligne de commande {
			menuItem.setEnabled(false);
		}
		return menuItem;
	}

	public void voirSituation() {
		if ((Edi.nos().fen() != null) && (getMoteur() != null)) {
			// fonctionnalité inactive en exécution par ligne de commande

			FenVoirSituation f = new FenVoirSituation(getMoteur());
			f.setLocation(2, 2);
			getMoteur().getNoS().getBase().ajouteFen(f);
			((FenEdiNoS) Edi.nos().fen()).ouvreInternalFrameSansPositionner(f);
			f.setVisible(true);
		}
	}

	public void retour() {
		dispose();

		Moteur m = getMoteur();
		if (m != null) {
			getMoteur().back();
		}
	}

	public void registerEventListeners() {
		getBoutonRetour().addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent evt) {
				retour();
			}
		});

		getBoutonFermer().addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent evt) {
				fermeFenetre();
			}
		});

		getBoutonAide().addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent evt) {
			}
		});

		ActionListener actionListenerEsc = new ActionListener() {
			public void actionPerformed(ActionEvent actionEvent) {
				escape();
			}
		};

		KeyStroke stroke = KeyStroke.getKeyStroke(KeyEvent.VK_ESCAPE, 0);
		if (frame instanceof JInternalFrame) {
			((FenDialogue) frame).getRootPane().registerKeyboardAction(
					actionListenerEsc, stroke,
					JComponent.WHEN_IN_FOCUSED_WINDOW);
		} else {
			((FenDialogueJFrame) frame).getRootPane().registerKeyboardAction(
					actionListenerEsc, stroke,
					JComponent.WHEN_IN_FOCUSED_WINDOW);
		}
	}

	public JMenuBar getMenuBar() {
		return menuBar;
	}

	public Moteur getMoteur() {
		if (frame instanceof JInternalFrame) {
			return ((FenDialogue) frame).getMoteur();
		} else {
			return ((FenDialogueJFrame) frame).getMoteur();
		}
	}

	public JButton getBoutonAide() {
		if (frame instanceof JInternalFrame) {
			return ((FenDialogue) frame).getBoutonAide();
		} else {
			return ((FenDialogueJFrame) frame).getBoutonAide();
		}
	}

	public JButton getBoutonFermer() {
		if (frame instanceof JInternalFrame) {
			return ((FenDialogue) frame).getBoutonFermer();
		} else {
			return ((FenDialogueJFrame) frame).getBoutonFermer();
		}
	}

	public JButton getBoutonRetour() {
		if (frame instanceof JInternalFrame) {
			return ((FenDialogue) frame).getBoutonRetour();
		} else {
			return ((FenDialogueJFrame) frame).getBoutonRetour();
		}
	}

	public JTextArea getZoneTexte() {
		if (frame instanceof JInternalFrame) {
			return ((FenDialogue) frame).getZoneTexte();
		} else {
			return ((FenDialogueJFrame) frame).getZoneTexte();
		}
	}

	public JPanel getPanelBoutons() {
		if (frame instanceof JInternalFrame) {
			return ((FenDialogue) frame).getPanelBoutons();
		} else {
			return ((FenDialogueJFrame) frame).getPanelBoutons();
		}
	}

	public void setPanelBoutons(JPanel p) {
		if (frame instanceof JInternalFrame) {
			((FenDialogue) frame).setPanelBoutons(new JPanel());
		} else {
			((FenDialogueJFrame) frame).setPanelBoutons(new JPanel());
		}
	}
}
