/*
  EdiNoS - Editeur de NoS
  Copyright (C) 2015-2020 - Luc Poittevin — Tous droits réservés.
  Contact : edinos@free.fr  -  https://www.edinos.org
  
  Ce programme est un logiciel libre ; vous pouvez le redistribuer ou le
  modifier suivant les termes de la “GNU General Public License” telle que
  publiée par la Free Software Foundation : soit la version 3 de cette
  licence, soit (à votre gré) toute version ultérieure.
  
  Ce programme est distribué dans l’espoir qu’il vous sera utile, mais SANS
  AUCUNE GARANTIE : sans même la garantie implicite de COMMERCIALISABILITÉ
  ni d’ADÉQUATION À UN OBJECTIF PARTICULIER. Consultez la Licence Générale
  Publique GNU pour plus de détails.
  
  Vous devriez avoir reçu une copie de la Licence Générale Publique GNU avec
  ce programme ; si ce n’est pas le cas, consultez :
  <http://www.gnu.org/licenses/>. 
 */

package nos.run.edit;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Dimension;
import java.awt.Insets;

import javax.swing.BorderFactory;
import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.JEditorPane;
import javax.swing.JInternalFrame;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JFrame;
import javax.swing.event.HyperlinkEvent;
import javax.swing.event.HyperlinkListener;
import javax.swing.text.html.FormSubmitEvent;
import javax.swing.text.html.FormView;
import javax.swing.text.html.HTML;
import javax.swing.text.html.HTMLEditorKit;
import javax.swing.text.html.StyleSheet;

import nos.obj.Edi;
import nos.utils.PrefsManager;

public abstract class FenDialogueHTMLDeleg extends FenDialogueDeleg {

	protected JEditorPane editorPane = new JEditorPane();

	FenDialogueHTMLDeleg() {
		super();
	}

	public void init(String titre) {
		if (frame instanceof JInternalFrame) {
			((JInternalFrame) frame).setTitle(titre);
			((JInternalFrame) frame).getContentPane().setLayout(
					new BorderLayout());
		} else {
			((JFrame) frame).setTitle(titre);
			((JFrame) frame).getContentPane().setLayout(new BorderLayout());
		}

		JPanel panel = new JPanel();
		panel.setLayout(new BoxLayout(panel, BoxLayout.LINE_AXIS));
		getBoutonRetour().setName("Retour"); // pour FEST
		panel.add(getBoutonRetour());
		getBoutonAide().setName("?"); // pour FEST
		getBoutonAide()
				.setToolTipText("Pourquoi me pose-t-on cette question ?");
		panel.add(getBoutonAide());
		getBoutonFermer().setText("Interrompre");
		getBoutonFermer().setToolTipText("Interrompre le lancement");
		getBoutonFermer().setName("Interrompre"); // pour FEST
		panel.add(getBoutonFermer());

		panel.add(Box.createHorizontalGlue());
		panel.setBorder(BorderFactory.createLineBorder(Color.white));
		panel.setPreferredSize(new Dimension(350, 30));
		
		addToContentPane(panel, BorderLayout.NORTH);

		JPanel panel2 = new JPanel();
		panel2.setLayout(new BoxLayout(panel2, BoxLayout.Y_AXIS));
		JPanel panel2b = new JPanel();
		JLabel l2 = new JLabel(libelle(), JLabel.CENTER);
		l2.setForeground(Color.BLUE);
		l2.setName("libelle"); // pour FEST
		l2.setAlignmentY(JFrame.CENTER_ALIGNMENT);

		panel2b.add(l2, BorderLayout.CENTER);
		panel2.add(panel2b);

		editorPane.setMargin(new Insets(5, 5, 5, 5));
		// getZoneTexte().setName("texte"); // pour FEST
		editorPane.setName("texte"); // pour FEST

		editorPane.setContentType("text/html");
		HTMLEditorKit kit = (HTMLEditorKit) editorPane.getEditorKit();
		kit.setAutoFormSubmission(false);
		StyleSheet s = kit.getStyleSheet();
		s.addRule("input.choix {background-color: silver; }");
		s.addRule("input.choix {color: blue; }");
		s.addRule("input.choix {width: 25px; }");

		editorPane.setEditable(false);
		panel2.add(editorPane);
		int l = ((PrefsManager) Edi.nos().getPreferences()).getFenDialogueLargeur();
		int h = ((PrefsManager) Edi.nos().getPreferences()).getFenDialogueHauteur();
		if ( h < 350) {h = 350;}; 
		panel2.setPreferredSize(new Dimension(l, h));
		
		addToContentPane(panel2, BorderLayout.CENTER);

		setPanelBoutons(new JPanel());
		construitBoutons();
		addToContentPane(getPanelBoutons(), BorderLayout.SOUTH);

		registerEventListeners();

		registerEventListeners2();

		remplitZones();

		installeMenus();

		// Resize the container to the preferred size
		if (frame instanceof JInternalFrame) {
			((JInternalFrame) frame).pack();
		} else {
			((JFrame) frame).pack();
		}
	}

	private void registerEventListeners2() {
		editorPane.addHyperlinkListener(new HyperlinkListener() {
			public void hyperlinkUpdate(HyperlinkEvent e) {
				FormView f = (FormView) e.getSource();
				String nomBouton = f.getElement().getAttributes()
						.getAttribute(HTML.Attribute.VALUE).toString();
				String data = ((FormSubmitEvent) e).getData();
				if (nomBouton.equals("OK")) {
					if (data.isEmpty()) {
						Edi.nos().message(
								"Vous n'avez pas choisi de réponse...");
						return;
					} else {
						dispose();
						String fait = data.substring(data.indexOf("=")+1);
						if (getMoteur() != null) {
							getMoteur().retourne(fait);
						}
					}
				} else if (nomBouton.equals("⇒")) {
					dispose();
					String id = f.getElement().getAttributes()
							.getAttribute(HTML.Attribute.ID).toString();
					if (getMoteur() != null) {
						getMoteur().retourne(id);
					}
				} else if (nomBouton.equals("Vu")) {
					dispose();
					if (getMoteur() != null) {
						getMoteur().retourne(null);
					}
				} else if (nomBouton.equals("Annuler")) {
					dispose();
				}
			}
		});
	}
}
