/*
  EdiNoS - Editeur de NoS
  Copyright (C) 2017-2021 - Luc Poittevin — Tous droits réservés.
 Contact : edinos@free.fr  -  https://www.edinos.org
  
  Ce programme est un logiciel libre ; vous pouvez le redistribuer ou le
  modifier suivant les termes de la “GNU General Public License” telle que
  publiée par la Free Software Foundation : soit la version 3 de cette
  licence, soit (à votre gré) toute version ultérieure.
  
  Ce programme est distribué dans l’espoir qu’il vous sera utile, mais SANS
  AUCUNE GARANTIE : sans même la garantie implicite de COMMERCIALISABILITÉ
  ni d’ADÉQUATION À UN OBJECTIF PARTICULIER. Consultez la Licence Générale
  Publique GNU pour plus de détails.
  
  Vous devriez avoir reçu une copie de la Licence Générale Publique GNU avec
  ce programme ; si ce n’est pas le cas, consultez :
  <http://www.gnu.org/licenses/>. 
 */

package nos.run.edit;

import java.awt.Container;

import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;
import org.w3c.dom.events.Event;
import org.w3c.dom.events.EventListener;
import org.w3c.dom.events.EventTarget;

import javafx.beans.value.ChangeListener;
import javafx.beans.value.ObservableValue;
import javafx.concurrent.Worker.State;
import javafx.scene.Group;
import javafx.scene.Scene;
import javafx.scene.web.WebView;
import nos.obj.Edi;
import nos.obj.Question;

public class FenDialogueQuestHTMLDelegFX extends FenDialogueHTMLDelegFX
		implements IFenDialogue {

	private Question question;

	public FenDialogueQuestHTMLDelegFX(Question q, Container f) {
		super();
		question = q;
		frame = f;

		init("Question <" + question.getNom() + ">");
	}

	public String libelle() {
		if (question.formateFaitsAttendus().isEmpty()) {
			return "Message";
		} else {
			return "Question";
		}
	}

	public void remplitZones() {
	}

	private String prepareText() {
		String texte = question.getTexte();

		if (getMoteur() != null) {
			texte = getMoteur().valoriseFaits(texte);
		}

		String avant = "<html dir=\"ltr\"><head></head><body contenteditable=\"true\">";

		String debutForm = "<form action=\".\" method=\"post\">";
		String apres = "</body></html>";
		String finForm = "";

		if (question.formateFaitsAttendus().isEmpty()) {
			// Message
			finForm = finForm + "<input type=\"submit\" value=\"Vu\"></form>";
		}

		texte = transformeEnBoutons(texte);

		int i = texte.indexOf(avant);
		if (texte.startsWith(avant)) {
			texte = avant + debutForm + texte.substring(i + avant.length());
		} else {
			Edi.nos().montreAvertissement(
					"Erreur de formatage de la question <" + question.getNom()
							+ "> (début)");
		}
		if (texte.endsWith(apres)) {
			texte = texte.substring(0, texte.indexOf(apres)) + finForm + apres;
		} else {
			Edi.nos().montreAvertissement(
					"Erreur de formatage de la question <" + question.getNom()
							+ "> (fin)");
		}
		return texte;
	}

	private String transformeEnBoutons(String texte) {
		texte = texte.replaceAll("type=\"radio\"", "type=\"submit\"");

		String a = "<input";
		String value = "";
		String t = texte;
		while (t.indexOf(a) > 0) {
			value = detecteFait(t);
			texte = texte.replaceAll("value=\"" + value + "\" id=\"" + value
					+ "\">", "value=\"&#8658\" class=\"choix\" id=\"" + value
					+ "\">&nbsp;");



			t = t.substring(t.indexOf("</p>") + 1);
		}
		return texte;
	}

	public String detecteFait(String texte) {
		String a = "value=\"";
		String b = "\"";
		int i, j;
		i = texte.indexOf(a);
		texte = texte.substring(i + a.length());
		j = texte.indexOf(b);
		return texte.substring(0, j);
	}

	public void construitBoutons() {
	}

	@Override
	void escape() {
		fermeFenetre();
	}

	protected Scene createScene() {

		Group root = new Group();

		WebView browser = new WebView();
		webEngine = browser.getEngine();

		webEngine.getLoadWorker().stateProperty()
				.addListener(new ChangeListener<State>() {
					@Override
					public void changed(ObservableValue ov, State oldState,
							State newState) {
						if (newState == State.SUCCEEDED) {
							Document doc = webEngine.getDocument();

							EventListener clickListener = new EventListener() {
								public void handleEvent(Event ev) {
									Node n = (Node) ev.getTarget();
									Node nodeId = n.getAttributes()
											.getNamedItem("id");

									if (nodeId != null) {
										String id = nodeId.getNodeValue()
												.toString();
										dispose();
										if (getMoteur() != null) {
											getMoteur().retourne(id);
										}
									} else {
										String value = n.getAttributes()
												.getNamedItem("value")
												.getNodeValue().toString();
										dispose();
										if (getMoteur() != null) {
											getMoteur().retourne("");
										}
									}
								}
							};

							NodeList n = doc.getElementsByTagName("input");
							for (int i = 0, len = n.getLength(); i < len; i++) {
								Element el = (Element) n.item(i);
								((EventTarget) el).addEventListener("click",
										clickListener, false);
							}
						}
					}
				});

		String st = prepareText();
		webEngine.loadContent(st);
		root.getChildren().add(browser);
		Scene scene = new Scene(root, javafx.scene.paint.Color.ALICEBLUE);
		return scene;
	}
}
