/*
  EdiNoS - Editeur de NoS
  Copyright (C) 2017-2021 - Luc Poittevin — Tous droits réservés.
  Contact : edinos@free.fr  -  https://www.edinos.org
  
  Ce programme est un logiciel libre ; vous pouvez le redistribuer ou le
  modifier suivant les termes de la “GNU General Public License” telle que
  publiée par la Free Software Foundation : soit la version 3 de cette
  licence, soit (à votre gré) toute version ultérieure.
  
  Ce programme est distribué dans l’espoir qu’il vous sera utile, mais SANS
  AUCUNE GARANTIE : sans même la garantie implicite de COMMERCIALISABILITÉ
  ni d’ADÉQUATION À UN OBJECTIF PARTICULIER. Consultez la Licence Générale
  Publique GNU pour plus de détails.
  
  Vous devriez avoir reçu une copie de la Licence Générale Publique GNU avec
  ce programme ; si ce n’est pas le cas, consultez :
  <http://www.gnu.org/licenses/>. 
 */

package nos.run.edit;

import java.awt.Container;

import javax.swing.SwingUtilities;

import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;
import org.w3c.dom.events.Event;
import org.w3c.dom.events.EventListener;
import org.w3c.dom.events.EventTarget;
import org.w3c.dom.html.HTMLFormElement;
import org.w3c.dom.html.HTMLInputElement;

import javafx.beans.value.ChangeListener;
import javafx.beans.value.ObservableValue;
import javafx.concurrent.Worker.State;
import javafx.scene.Group;
import javafx.scene.Scene;
import javafx.scene.web.WebView;
import nos.obj.Edi;
import nos.obj.Question;

public class FenDialogueQuestHTMLRadioDelegFX extends FenDialogueHTMLDelegFX
		implements IFenDialogue {

	private Question question;

	public FenDialogueQuestHTMLRadioDelegFX(Question q, Container f) {
		super();
		question = q;
		frame = f;
		init("Question <" + question.getNom() + ">");
	}

	public String libelle() {
		if (question.formateFaitsAttendus().isEmpty()) {
			return "Message";
		} else {
			return "Question";
		}
	}

	@Override
	public void remplitZones() {
	}

	private String prepareText() {
		String texte = question.getTexte();
		
		if (getMoteur() != null) {
			texte = getMoteur().valoriseFaits(texte);
		}

		String avant = "<html dir=\"ltr\"><head></head><body contenteditable=\"true\">";
		String debutForm = "<form action=\".\" method=\"post\">";
		String apres = "</body></html>";
		String finForm = "";

		if (question.formateFaitsAttendus().isEmpty()) {
			// Message
			finForm = finForm + "<input type=\"submit\" value=\"Vu\"></form>";
		} else {
			// Question
			finForm = finForm + "<input type=\"submit\" value=\"Annuler\">"
					+ "&nbsp;&nbsp;&nbsp;&nbsp;"
					+ "<input type=\"submit\" value=\"OK\"></form>";
		}
		int i = texte.indexOf(avant);
		if (texte.startsWith(avant)) {
			texte = avant + debutForm + texte.substring(i + avant.length());
		} else {
			Edi.nos().montreAvertissement(
					"Erreur de formatage de la question <" + question.getNom()
							+ "> (début)");
		}
		if (texte.endsWith(apres)) {
			texte = texte.substring(0, texte.indexOf(apres)) + finForm + apres;
		} else {
			Edi.nos().montreAvertissement(
					"Erreur de formatage de la question <" + question.getNom()
							+ "> (fin)");
		}
		return texte;
	}

	public void construitBoutons() {
	}

	@Override
	void escape() {
		fermeFenetre();
	}

	protected Scene createScene() {

		Group root = new Group();

		WebView browser = new WebView();
		webEngine = browser.getEngine();

		webEngine.getLoadWorker().stateProperty()
				.addListener(new ChangeListener<State>() {
					@Override
					public void changed(ObservableValue ov, State oldState,
							State newState) {
						if (newState == State.SUCCEEDED) {
							Document doc = webEngine.getDocument();

							EventListener clickListener = new EventListener() {
								public void handleEvent(Event ev) {
									Node n = (Node) ev.getTarget();
									Node nodeId = n.getAttributes()
											.getNamedItem("id");
									String value = n.getAttributes()
											.getNamedItem("value")
											.getNodeValue().toString();
									if (value.equals("OK")) {
										HTMLFormElement form = (HTMLFormElement) webEngine
												.getDocument()
												.getElementsByTagName("form")
												.item(0);
										NodeList nl = form
												.getElementsByTagName("input");
										String fait = null;
										for (int i = 0; i < nl.getLength(); i++) {
											HTMLInputElement input = (HTMLInputElement) nl
													.item(i);
											if ((input.getName() != null)
													&& (input.getName()
															.equals("choix"))
													&& input.getChecked()) {
												fait = input.getValue();
											}
										}
										if (fait == null) {
											SwingUtilities
													.invokeLater(new Runnable() {
														@Override
														public void run() {
															Edi.nos()
																	.message(
																			"Vous n'avez pas choisi de réponse...");
														}
													});
											return;
										} else {
											dispose();
											if (getMoteur() != null) {
												getMoteur().retourne(fait);
											}

										}
									} else if (value.equals("Annuler")) {
										dispose();
									} else if (value.equals("Vu")) {
										dispose();
										if (getMoteur() != null) {
											getMoteur().retourne(null);
										}
									}
								}
							};

							NodeList n = doc.getElementsByTagName("input");
							for (int i = 0, len = n.getLength(); i < len; i++) {
								Element el = (Element) n.item(i);
								((EventTarget) el).addEventListener("click",
										clickListener, false);
							}
						}
					}
				});

		String st = prepareText();
		webEngine.loadContent(st);
		root.getChildren().add(browser);
		Scene scene = new Scene(root);
		return scene;
	}
}
