/*
  EdiNoS - Editeur de NoS
  Copyright (C) 2013-2021 - Luc Poittevin — Tous droits réservés.
  Contact : edinos@free.fr  -  https://www.edinos.org
  
  Ce programme est un logiciel libre ; vous pouvez le redistribuer ou le
  modifier suivant les termes de la “GNU General Public License” telle que
  publiée par la Free Software Foundation : soit la version 3 de cette
  licence, soit (à votre gré) toute version ultérieure.
  
  Ce programme est distribué dans l’espoir qu’il vous sera utile, mais SANS
  AUCUNE GARANTIE : sans même la garantie implicite de COMMERCIALISABILITÉ
  ni d’ADÉQUATION À UN OBJECTIF PARTICULIER. Consultez la Licence Générale
  Publique GNU pour plus de détails.
  
  Vous devriez avoir reçu une copie de la Licence Générale Publique GNU avec
  ce programme ; si ce n’est pas le cas, consultez :
  <http://www.gnu.org/licenses/>. 
 */

package nos.run.edit;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Container;
import java.awt.Dimension;
import java.awt.Point;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.KeyEvent;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.util.ArrayList;
import java.util.Collections;
import java.util.List;
import java.util.Map;

import javax.swing.BorderFactory;
import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.ButtonGroup;
import javax.swing.DefaultListModel;
import javax.swing.JButton;
import javax.swing.JComponent;
import javax.swing.JInternalFrame;
import javax.swing.JList;
import javax.swing.JMenu;
import javax.swing.JMenuItem;
import javax.swing.JPanel;
import javax.swing.JPopupMenu;
import javax.swing.JRadioButton;
import javax.swing.JScrollPane;
import javax.swing.JTextArea;
import javax.swing.KeyStroke;

import nos.edit.FenEdiNoS;
import nos.obj.Action;
import nos.obj.Edi;
import nos.obj.NoS;
import nos.run.FaitRenvoye;
import nos.run.Histo;
import nos.run.Memoire;
import nos.run.Moteur;
import nos.run.Moteur.Contexte;
import nos.run.Moteur.ModeExecutionProc;
import nos.utils.ToolNos;

public class FenLancementDeleg extends FenDialogueDeleg implements IFenDialogue {

	private JButton boutonAide = new JButton("Aide");
	
	private JButton boutonLancer = new JButton("Lancer");

	private JButton boutonFermer = new JButton("Fermer");

	private ButtonGroup groupeBoutons;

	private JRadioButton boutonSimul = new JRadioButton("Procédures simulées",
			true);

	private JRadioButton boutonExec = new JRadioButton("Procédures exécutées",
			false);

	private JRadioButton boutonIgnore = new JRadioButton("Procédures ignorées",
			false);

	private JList<String> listeActions = new JList<String>();

	private JList<String> listeFaits = new JList<String>();

	private Map<String, Action> collectionActions;

	private NoS nos;

	private JPopupMenu popup;

	private Histo historiqueFaits;


	public FenLancementDeleg(NoS n, Container fen) {
		nos = n;
		frame = fen;
		collectionActions = nos.getBase().getAssocActions();

		historiqueFaits = new Histo();
		historiqueFaits.setFaitsconnus(new ArrayList<FaitRenvoye>());

		init("Paramètres du lancement");
	}

	public void installeMenus() {
		if (Edi.nos().fen() != null) {
			menuBar = ((FenEdiNoS) Edi.nos().fen()).menuBarEdinos();
			menuBar.add(menuLancement());
			Edi.nos().fen().setJMenuBar(menuBar);
			Edi.nos().fen().setVisible(true);
		}
	}

	public JMenu menuLancement() {

		JMenuItem menuItem;
		JMenu menu = new JMenu("Lancement");
		
		menuItem = new JMenuItem("MonTrer les faits connus");
		menuItem.setAccelerator(KeyStroke.getKeyStroke(
				java.awt.event.KeyEvent.VK_T, ToolNos.renvoieModifiers()));
		menuItem.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				ouvreFenLancementFaitsConnus();
			}
		});
		menu.add(menuItem);
		menu.addSeparator();

		menuItem = new JMenuItem("Charger une liste de faits");
		menuItem.setEnabled(false);

		menu.add(menuItem);
		menuItem = new JMenuItem("Lancer");
		menuItem.setAccelerator(KeyStroke.getKeyStroke(
				java.awt.event.KeyEvent.VK_L, ToolNos.renvoieModifiers()));
		menuItem.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				lance();
			}
		});
		menu.add(menuItem);

		menu.addSeparator();
		menuItem = new JMenuItem("Fermer la fenêtre");
		menuItem.setAccelerator(KeyStroke.getKeyStroke(
				java.awt.event.KeyEvent.VK_W, ToolNos.renvoieModifiers()));
		menuItem.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				fermeFen();
			}
		});
		menu.add(menuItem);
		return (menu);
	}

	public void registerEventListeners() {
		boutonAide.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent evt) {
				ouvreAide();
			}
		});
		
		boutonLancer.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent evt) {
				lance();
			}
		});

		boutonFermer.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent evt) {
				fermeFen();
			}
		});

		listeActions.addMouseListener(new MouseAdapter() {
			public void mousePressed(MouseEvent e) {
				if ((e.getClickCount() == 1)
						&& (e.getButton() == MouseEvent.BUTTON3)) {
					int index = listeActions.locationToIndex(e.getPoint());
					listeActions.setSelectedIndex(index);
					Action p = nos.getBase().getAction(
							listeActions.getSelectedValue().toString());

					popup = new JPopupMenu();
					// définition de l'action à exécuter lors du clic sur un
					// fait de la popUp menu :
					ActionListener menuListener = new ActionListener() {
						public void actionPerformed(ActionEvent e) {
							// System.out.println(e.getActionCommand());
							DefaultListModel<String> l = (DefaultListModel<String>) listeFaits
									.getModel();
							l.addElement(e.getActionCommand() + " <- "
									+ p.getNom());

							FaitRenvoye f = new FaitRenvoye();
							f.setAction(p.getNom());
							f.setFait(e.getActionCommand());
							historiqueFaits.ajoute(f);
						}
					};

					if (listeActions.getSelectedValue() != null) {
						JMenuItem menuItem = null;
						for (List<String> l : p.formateFaitsAttendus()) {
							for (String fait : (List<String>) l) {
								menuItem = new JMenuItem(fait);
								menuItem.addActionListener(menuListener);
								popup.add(menuItem);
							}
						}

						popup.show(e.getComponent(), e.getX(), e.getY());
					}
				}
			}
		});

		listeFaits.addMouseListener(new MouseAdapter() {
			public void mouseClicked(MouseEvent e) {
				if (e.getClickCount() == 2) {
					ouvreFenLancementFaitsConnus();
				}
			}
		});
		
		// Ajout 17/03/2021 : // LPO
		ActionListener actionListenerEsc = new ActionListener() {
			public void actionPerformed(ActionEvent actionEvent) {
				escape();
			}
		};
		KeyStroke stroke = KeyStroke.getKeyStroke(KeyEvent.VK_ESCAPE, 0);
		((JInternalFrame) frame).getRootPane().registerKeyboardAction(actionListenerEsc, stroke, JComponent.WHEN_IN_FOCUSED_WINDOW);

	}
	
	protected void fermeFen() {
		if (ToolNos.nomSysteme().equals("Mac")) {
			ToolNos.fermeFen((JInternalFrame) frame);
		} else {
			dispose();
		} 
		((FenEdiNoS) Edi.nos().fen()).checkDerniereFen();
	}

	protected void ouvreAide() {
		FenAideLancement f = new FenAideLancement();
		f.setSize(650, 580);
		Edi.nos().getBaseCourante().ajouteFen(f);
		((FenEdiNoS) Edi.nos().fen()).ouvreInternalFrame(f);
	}

	protected void ouvreFenLancementFaitsConnus() {
		FenLancementFaitsConnus fen = new FenLancementFaitsConnus(this, historiqueFaits);
		Point p = frame.getLocation();
		Dimension size = frame.getSize();
		p.setLocation(p.x + size.width + 15, p.y); // +5
		fen.setLocation(p);
		
		fen.setSize(550, 500);
		Edi.nos().getBaseCourante().ajouteFen(fen);
		((FenEdiNoS) Edi.nos().fen()).ouvreInternalFrameSansPositionner(fen);
		fen.setVisible(true);
	}

	private void construitListeActions() {
		DefaultListModel<String> listModel = new DefaultListModel<String>();
		List<String> l = new ArrayList<String>();

		for (Map.Entry<String, Action> e : collectionActions.entrySet()) {
			String classe = e.getValue().getClass().getName();
			if ((classe == "nos.obj.Procedure")
					|| (classe == "nos.obj.Question")) {
				l.add(e.getKey());
			}
		}

		Collections.sort(l);
		for (String nomAction : l) {
			listModel.addElement(nomAction);
		}

		listeActions.setModel(listModel);
	}

	private void construitListeFaits() {
		DefaultListModel<String> listModel = new DefaultListModel<String>();
		listeFaits.setModel(listModel);
	}

	private void lance() {
//		dispose();
		fermeFen();

		Memoire mem = new Memoire();

		if (historiqueFaits != null) {
			for (FaitRenvoye f : historiqueFaits.getFaitsconnus()) {
				mem.ajoute(f.getAction(), f.getFait());
			}
		}

		String choix = groupeBoutons.getSelection().getActionCommand();
		if (choix.equals("Procédures simulées")) {
			Moteur mot = new Moteur(nos, Contexte.Lancement,
					ModeExecutionProc.Simule);
			mot.declencheAvec(mem);
		} else if (choix.equals("Procédures exécutées")) {
			Moteur mot = new Moteur(nos, Contexte.Lancement,
					ModeExecutionProc.Execute);

			mot.declencheAvec(mem);
		} else if (choix.equals("Procédures ignorées")) {
			Moteur mot = new Moteur(nos, Contexte.Lancement,
					ModeExecutionProc.Ignore);
			mot.declencheAvec(mem);
		}
	}

	@Override
	public String libelle() {
		return null;
	}

	@Override
	public void construitBoutons() {
		JPanel panel = new JPanel();
		panel.setLayout(new BoxLayout(panel, BoxLayout.LINE_AXIS));
		panel.add(boutonAide);
		panel.add(boutonLancer);
		panel.add(boutonFermer);
		panel.add(Box.createHorizontalGlue());

		addToContentPane(panel, BorderLayout.NORTH);

		JPanel panelBoutons = new JPanel();
		panelBoutons.setLayout(new BoxLayout(panelBoutons, BoxLayout.Y_AXIS));

		JPanel panel21 = new JPanel();
		groupeBoutons = new ButtonGroup();
		boutonSimul.setActionCommand("Procédures simulées");
		groupeBoutons.add(boutonSimul);
		panel21.add(boutonSimul);
		boutonExec.setActionCommand("Procédures exécutées");
		groupeBoutons.add(boutonExec);
		panel21.add(boutonExec);
		boutonIgnore.setActionCommand("Procédures ignorées");
		groupeBoutons.add(boutonIgnore);
		panel21.add(boutonIgnore);
		panelBoutons.add(panel21);

		JPanel panelCentre = new JPanel();
		JPanel panelProc = new JPanel();
		JPanel panelFait = new JPanel();
		JPanel panelListes = new JPanel();

		JScrollPane listPane1 = new JScrollPane(listeActions);
		JScrollPane listPane2 = new JScrollPane(listeFaits);

		listPane1.setPreferredSize(new Dimension(200, 150));
		listPane2.setPreferredSize(new Dimension(200, 150));

		JTextArea text1 = new JTextArea("Actions :");
		text1.setOpaque(false);
		panelProc.setPreferredSize(new Dimension(200, 175));
		panelProc.add(text1, BorderLayout.NORTH);
		panelProc.add(listPane1, BorderLayout.SOUTH);
		panelListes.add(panelProc, BorderLayout.EAST);

		JTextArea text2 = new JTextArea("Faits connus :");
		text2.setOpaque(false);
		panelFait.setPreferredSize(new Dimension(200, 175));
		panelFait.add(text2, BorderLayout.NORTH);
		panelFait.add(listPane2, BorderLayout.SOUTH);
		panelListes.add(panelFait, BorderLayout.WEST);

		panelListes.setBorder(BorderFactory.createLineBorder(Color.black));
		panelCentre.setBorder(BorderFactory.createLineBorder(Color.white));

		panelCentre.add(panelBoutons, BorderLayout.NORTH);
		panelCentre.add(panelListes, BorderLayout.CENTER);

		addToContentPane(panelCentre, BorderLayout.CENTER);

	}

	@Override
	public void remplitZones() {
		construitListeActions();
		construitListeFaits();
	}

	@Override
	void escape() {
		fermeFen();
	}

	public void setHistoriqueFaits(Histo h) {
		historiqueFaits = h;

		DefaultListModel<String> listModel = new DefaultListModel<String>();

		for (FaitRenvoye f : historiqueFaits.getFaitsconnus()) {
			listModel.addElement(f.getFait() + " <- " + f.getAction());
		}
		listeFaits.setModel(listModel);
	}
}
