/*
  EdiNoS - Editeur de NoS
  Copyright (C) 2019 - Luc Poittevin — Tous droits réservés.
  Contact : edinos@free.fr  -  https://www.edinos.org
  
  Ce programme est un logiciel libre ; vous pouvez le redistribuer ou le
  modifier suivant les termes de la “GNU General Public License” telle que
  publiée par la Free Software Foundation : soit la version 3 de cette
  licence, soit (à votre gré) toute version ultérieure.
  
  Ce programme est distribué dans l’espoir qu’il vous sera utile, mais SANS
  AUCUNE GARANTIE : sans même la garantie implicite de COMMERCIALISABILITÉ
  ni d’ADÉQUATION À UN OBJECTIF PARTICULIER. Consultez la Licence Générale
  Publique GNU pour plus de détails.
  
  Vous devriez avoir reçu une copie de la Licence Générale Publique GNU avec
  ce programme ; si ce n’est pas le cas, consultez :
  <http://www.gnu.org/licenses/>. 
 */

package nos.run.edit;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Container;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.IOException;

import javax.swing.BorderFactory;
import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.JButton;
import javax.swing.JFrame;
import javax.swing.JInternalFrame;
import javax.swing.JMenu;
import javax.swing.JMenuItem;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTextArea;
import javax.swing.KeyStroke;

import com.fasterxml.jackson.core.JsonParseException;
import com.fasterxml.jackson.core.JsonProcessingException;
import com.fasterxml.jackson.databind.JsonMappingException;
import com.fasterxml.jackson.databind.ObjectMapper;

import nos.edit.FenEdiNoS;
import nos.obj.Edi;
import nos.run.FaitRenvoye;
import nos.run.Histo;
import nos.run.Memoire;
import nos.utils.ToolNos;

public class FenLancementFaitsConnusDeleg extends FenDialogueDeleg implements
		IFenDialogue {

	private JButton boutonAnnuler = new JButton("Annuler");

	private JButton boutonFormater = new JButton("Formater");

	private JButton boutonMettreAJour = new JButton("Mettre à jour");

	private JTextArea zoneTexteJSON = new JTextArea();

	private FenLancementDeleg fenLancement;

	private Histo histo;

	// create ObjectMapper instance
	private ObjectMapper objectMapper = new ObjectMapper();

	public FenLancementFaitsConnusDeleg(FenLancementDeleg fenLancementDeleg, Histo h,
			Container fen) {
		fenLancement = fenLancementDeleg;
		frame = fen;
		histo = h;
		if (fenLancement == null) {
			init("Faits connus");
			boutonMettreAJour.setEnabled(false);
		} else {
			init("Saisie des faits connus");
		}
	}

	@Override
	public void init(String titre) {
		if (frame instanceof JInternalFrame) {
			((JInternalFrame) frame).setTitle(titre);
			((JInternalFrame) frame).getContentPane().setLayout(
					new BorderLayout());
		} else {
			((JFrame) frame).setTitle(titre);
			((JFrame) frame).getContentPane().setLayout(new BorderLayout());
		}

		JPanel panel = new JPanel();
		panel.setLayout(new BoxLayout(panel, BoxLayout.LINE_AXIS));
		panel.add(boutonAnnuler);
		panel.add(boutonFormater);
		panel.add(boutonMettreAJour);
		panel.add(Box.createHorizontalGlue());
		panel.setBorder(BorderFactory.createLineBorder(Color.white));

		addToContentPane(panel, BorderLayout.NORTH);

		JPanel panelCentre = new JPanel();
		panelCentre.setLayout(new BorderLayout());
		zoneTexteJSON.setLineWrap(true);
		zoneTexteJSON.setMargin(new Insets(0, 3, 0, 3));

		JScrollPane scrollPane = new JScrollPane(zoneTexteJSON);
		scrollPane
				.setVerticalScrollBarPolicy(JScrollPane.VERTICAL_SCROLLBAR_ALWAYS);

		panelCentre.add(scrollPane, BorderLayout.CENTER);

		addToContentPane(panelCentre, BorderLayout.CENTER);

		// Register the Listeners
		registerEventListeners();

		remplitZones();

		installeMenus();

		// Resize the container to the preferred size
		if (frame instanceof JInternalFrame) {
			((JInternalFrame) frame).pack();
		} else {
			((JFrame) frame).pack();
		}
	}

	public void installeMenus() {
		if (Edi.nos().fen() != null) {
			menuBar = ((FenEdiNoS) Edi.nos().fen()).menuBarEdinos();
			menuBar.add(menuLancement());
			Edi.nos().fen().setJMenuBar(menuBar);
			Edi.nos().fen().setVisible(true);
		}
	}

	public JMenu menuLancement() {

		JMenuItem menuItem;
		JMenu menu = new JMenu("Lancement");

		menuItem = new JMenuItem("Charger une liste de faits");
		menuItem.setEnabled(false);

		menu.add(menuItem);

		menu.addSeparator();
		menuItem = new JMenuItem("Fermer la fenêtre");
		menuItem.setAccelerator(KeyStroke.getKeyStroke(
				java.awt.event.KeyEvent.VK_W, ToolNos.renvoieModifiers()));
		menuItem.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				dispose();
			}
		});
		menu.add(menuItem);
		return (menu);
	}

	public void registerEventListeners() {
		boutonAnnuler.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent evt) {
				dispose();
			}
		});

		boutonFormater.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent evt) {

				String texte = zoneTexteJSON.getText();
				Histo histo = mappeHisto(texte);
				if (histo != null) {
					try {
						texte = objectMapper.writerWithDefaultPrettyPrinter()
								.writeValueAsString(histo);
					} catch (JsonProcessingException e) {
						e.printStackTrace();
						Edi.nos().message(e.getMessage());
						texte = zoneTexteJSON.getText();
					}
				} else {
					texte = zoneTexteJSON.getText();
				}
				zoneTexteJSON.setText(texte);
			}
		});

		boutonMettreAJour.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent evt) {
				metAjourFaitsConnus();
			}
		});
	}

	private void metAjourFaitsConnus() {
		Histo h = mappeHisto(zoneTexteJSON.getText());
		fenLancement.setHistoriqueFaits(h);
		dispose();
	}

	@Override
	public String libelle() {
		return null;
	}

	@Override
	public void construitBoutons() {

	}

	@Override
	public void remplitZones() {
		String texte = "";

		try {
			texte = objectMapper.writerWithDefaultPrettyPrinter()
					.writeValueAsString(histo);
		} catch (JsonProcessingException e) {
			e.printStackTrace();
			Edi.nos().message(e.getMessage());
		}
		zoneTexteJSON.append(texte);
	}

	@Override
	void escape() {
		dispose();
	}

	private Histo mappeHisto(String chaineHisto) {
		Memoire mem = new Memoire();

		Histo histo = null;
		// convert json string to object
		try {
			histo = objectMapper.readValue(chaineHisto, Histo.class);
		} catch (JsonParseException e) {
			e.printStackTrace();
			Edi.nos().message(e.getMessage());
		} catch (JsonMappingException e) {
			e.printStackTrace();
			Edi.nos().message(e.getMessage());
		} catch (IOException e) {
			e.printStackTrace();
			Edi.nos().message(e.getMessage());
		}

		if (histo != null) {
			histo.affiche();
			for (FaitRenvoye f : histo.getFaitsconnus()) {
				mem.ajoute(f.getAction(), f.getFait());
			}
		}
		return histo;
	}
}
