/*
  EdiNoS - Editeur de NoS
  Copyright (C) 2013-2021 - Luc Poittevin — Tous droits réservés.
  Contact : edinos@free.fr  -  https://www.edinos.org
  
  Ce programme est un logiciel libre ; vous pouvez le redistribuer ou le
  modifier suivant les termes de la “GNU General Public License” telle que
  publiée par la Free Software Foundation : soit la version 3 de cette
  licence, soit (à votre gré) toute version ultérieure.
  
  Ce programme est distribué dans l’espoir qu’il vous sera utile, mais SANS
  AUCUNE GARANTIE : sans même la garantie implicite de COMMERCIALISABILITÉ
  ni d’ADÉQUATION À UN OBJECTIF PARTICULIER. Consultez la Licence Générale
  Publique GNU pour plus de détails.
  
  Vous devriez avoir reçu une copie de la Licence Générale Publique GNU avec
  ce programme ; si ce n’est pas le cas, consultez :
  <http://www.gnu.org/licenses/>. 
*/

package nos.run.edit;

import java.awt.BorderLayout;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.BufferedWriter;
import java.io.File;
import java.io.FileWriter;
import java.io.IOException;

import javax.swing.JFileChooser;
import javax.swing.JInternalFrame;
import javax.swing.JMenu;
import javax.swing.JMenuBar;
import javax.swing.JMenuItem;
import javax.swing.JScrollPane;
import javax.swing.JTextArea;
import javax.swing.KeyStroke;
import javax.swing.event.InternalFrameEvent;
import javax.swing.event.InternalFrameListener;

import nos.edit.FenEdiNoS;
import nos.obj.Edi;
import nos.utils.ToolNos;

@SuppressWarnings("serial")
public class FenTexte extends JInternalFrame implements InternalFrameListener {

	private JTextArea textArea;

	private String nomFic;

	private String extFic;
	
	private String titreMenu;

	private JMenuBar menuBar;

	public FenTexte(String texte, String titreFen, String titreMen, String nom, String extension) {

		super(titreFen, true, // resizable
				true, // closable
				true, // maximizable
				true);// iconifiable
		addInternalFrameListener(this);

		nomFic = nom;
		extFic = extension;
		titreMenu = titreMen;
		getContentPane().setLayout(new BorderLayout());

		textArea = new JTextArea(texte);
		textArea.setMargin(new Insets(0, 3, 0, 3));
		textArea.setEditable(false);
		textArea.setName("texte"); // pour FEST
		JScrollPane scrollPane = new JScrollPane(textArea);
		scrollPane
				.setVerticalScrollBarPolicy(JScrollPane.VERTICAL_SCROLLBAR_ALWAYS);

		add(scrollPane);

		// installation des menus
		installeMenus();

		pack();
	}

	private void installeMenus() {
		FenEdiNoS ed = (FenEdiNoS) Edi.nos().fen();
		menuBar = ed.menuBarEdinos();
		menuBar.add(ed.menuBase());
		menuBar.add(menuBilan());
		Edi.nos().fen().setJMenuBar(menuBar);
		Edi.nos().fen().setVisible(true);
	}

	private JMenu menuBilan() {
		JMenu menu = new JMenu(titreMenu);
		JMenuItem menuItem = new JMenuItem("Enregistrer sous...");
		menuItem.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				enregistre();
			}
		});
		menuItem.setName("EnregistrerSous"); // pour FEST
		menu.add(menuItem);

		menu.addSeparator();
		menuItem = new JMenuItem("Fermer la fenêtre");
		menuItem.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				fermeFen();
			}
		});
		menuItem.setAccelerator(KeyStroke.getKeyStroke(
				java.awt.event.KeyEvent.VK_W, ToolNos.renvoieModifiers()));
		menuItem.setName("FermerFenêtre"); // pour FEST
		menu.add(menuItem);

		return menu;
	}
	
	protected void fermeFen() {
		if (ToolNos.nomSysteme().equals("Mac")) {
			ToolNos.fermeFen((JInternalFrame) this);
		} else {
			dispose();
		} 
		((FenEdiNoS) Edi.nos().fen()).checkDerniereFen();
	}

	private void enregistre() {
		JFileChooser filechoose = new JFileChooser();
		// Créer un JFileChooser
		// filechoose.setCurrentDirectory(new File("."));
		// Le répertoire source du JFileChooser est le répertoire d’où est
		// lancé notre programme
		filechoose.setSelectedFile(new File(nomFic + "." + extFic));
		// Le bouton pour valider l’enregistrement portera la mention
		// Enregistrer
		int resultatEnregistrer = filechoose.showSaveDialog(filechoose);
		if (resultatEnregistrer == JFileChooser.APPROVE_OPTION)
		// Si l’utilisateur clique sur le bouton Enregistrer
		{
			String monFichier = new String(filechoose.getSelectedFile()
					.toString());
			// Récupérer le nom du fichier qu’il a spécifié
			if (monFichier.endsWith("." + extFic)) {
				;
			}
			// Si ce nom de fichier finit par .cou, ne rien faire et
			// passer à la suite
			else {
				monFichier = monFichier + ".cou";
			}
			;
			// Sinon renommer le fichier pour qu’il porte l’extension .cou
			{
				try {
					FileWriter lu = new FileWriter(monFichier);
					// Créer un objet java.io.FileWriter avec comme argument
					// le mon du fichier dans lequel enregsitrer
					BufferedWriter out = new BufferedWriter(lu);
					// Mettre le flux en tampon (en cache)
					out.write(textArea.getText()); //
					// Balancer dans le flux le contenu de la zone de texte
					out.close(); // Fermer le flux (c’est
					// toujours mieux de le fermer explicitement)
				} catch (IOException er) {
					er.printStackTrace();
				}
			}
		}
	}

	public void internalFrameActivated(InternalFrameEvent e) {
		Edi.nos().fen().setJMenuBar(menuBar);
		Edi.nos().fen().setVisible(true);
	}

	public void internalFrameClosed(InternalFrameEvent e) {
	}

	public void internalFrameClosing(InternalFrameEvent e) {
		fermeFen();
	}

	public void internalFrameDeactivated(InternalFrameEvent e) {
		((FenEdiNoS) Edi.nos().fen()).setMenuBar();
	}

	public void internalFrameDeiconified(InternalFrameEvent e) {
	}

	public void internalFrameIconified(InternalFrameEvent e) {
	}

	public void internalFrameOpened(InternalFrameEvent e) {
	}
}
