/*
  EdiNoS - Editeur de NoS
  Copyright (C) 2013-2020 - Luc Poittevin — Tous droits réservés.
  Contact : edinos@free.fr  -  https://www.edinos.org
  
  Ce programme est un logiciel libre ; vous pouvez le redistribuer ou le
  modifier suivant les termes de la “GNU General Public License” telle que
  publiée par la Free Software Foundation : soit la version 3 de cette
  licence, soit (à votre gré) toute version ultérieure.
  
  Ce programme est distribué dans l’espoir qu’il vous sera utile, mais SANS
  AUCUNE GARANTIE : sans même la garantie implicite de COMMERCIALISABILITÉ
  ni d’ADÉQUATION À UN OBJECTIF PARTICULIER. Consultez la Licence Générale
  Publique GNU pour plus de détails.
  
  Vous devriez avoir reçu une copie de la Licence Générale Publique GNU avec
  ce programme ; si ce n’est pas le cas, consultez :
  <http://www.gnu.org/licenses/>. 
 */

package nos.utils;

import java.io.File;
import java.net.URISyntaxException;

import nos.edit.FenCGU;
import nos.edit.FenEdiNoS;
import nos.obj.Edi;
import nos.obj.EdiNoS;
import nos.run.Moteur;

import org.apache.commons.configuration.ConfigurationException;
import org.apache.commons.configuration.PropertiesConfiguration;

public class PrefsManager {
	PropertiesConfiguration config;
	FenEdiNoS edinos;

	private int fenDialogueLargeurDefaut = 600;
	private int fenDialogueHauteurDefaut = 300;

	public PrefsManager(FenEdiNoS ed) {
		edinos = ed;

		File f = new File("edinos.properties").getAbsoluteFile();
		if (!f.exists()) {
			// On recherche "edinos.properties" dans user.home/Documents/EdiNoS :
			String fichier = System.getProperty("user.home") + ToolNos.renvoieSeparateurRepertoires() + "Documents"
					+ ToolNos.renvoieSeparateurRepertoires() + "EdiNoS";
			fichier = fichier + ToolNos.renvoieSeparateurRepertoires() + "edinos.properties";
			f = new File(fichier).getAbsoluteFile();
		}

		try {
			config = new PropertiesConfiguration(f);
		} catch (ConfigurationException e) {
			edinos.montre("Erreur lors de l'ouverture de 'edinos.properties'");
			e.printStackTrace();
		}
		if (!f.exists()) {
			// Il faut faire accepter les CGU
			// Désactivation de la fen EdinoS :
			Edi.nos().fen().setEnabled(false);
			if (ToolNos.nomSysteme().equals("Mac")) {
				// inutile sous Windows et Linux et de plus fait planter :
				Edi.nos().fen().getMenuBar().getMenu(0).setEnabled(false);
			}

			FenCGU fen = new FenCGU(this);
			fen.setSize(570, 400);
			fen.setLocation(400, 150);
			fen.setVisible(true);
			// si les CGU sont acceptées, on reviendra par la méthode
			// accepteCGU()
		} else {
			if (edinos != null) { // cas du lancement avec arguments (run)
				edinos.montre("Répertoire de travail : " + getRepertoireTravail());
				suiteInit();
			}
		}
	}

	private void configurationRepertoireDonnees() {
		// Copie des fichiers utiles dans le répertoire des données :

		String fichier = System.getProperty("user.home") + ToolNos.renvoieSeparateurRepertoires() + "Documents";

		edinos.montre("Répertoire des données : " + fichier + ToolNos.renvoieSeparateurRepertoires() + "EdiNoS");
		edinos.montre("Configuration du répertoire des données");
		edinos.montre("User.dir : " + System.getProperty("user.dir"));

		File f = new File(fichier);
		if (!f.exists()) {
			boolean bool = f.mkdir();
			if (!bool) {
				edinos.montre("Création du répertoire " + fichier + " impossible");
				return;
			}
		}

		fichier = fichier + ToolNos.renvoieSeparateurRepertoires() + "EdiNoS";
		f = new File(fichier);
		if (!f.exists()) {
			boolean bool = f.mkdir();
			if (!bool) {
				edinos.montre("Création du répertoire " + fichier + " impossible");
				return;
			}
		}
				
		ToolNos.copieFichier(fichier, "LicenceEdinos3.5.rtf");
		ToolNos.copieRepertoire(fichier, "NOS.BC"+ ToolNos.renvoieSeparateurRepertoires() + "COMMUN");
		ToolNos.copieRepertoire(fichier, "NOS.BC"+ ToolNos.renvoieSeparateurRepertoires() + "REVINOS");
		ToolNos.copieRepertoire(fichier, "NOS.BC"+ ToolNos.renvoieSeparateurRepertoires() + "animaux");
		ToolNos.copieRepertoire(fichier, "NOS.BC"+ ToolNos.renvoieSeparateurRepertoires() + "metal2");
		ToolNos.copieRepertoire(fichier, "NOS.BC"+ ToolNos.renvoieSeparateurRepertoires() + "Participe8");
		ToolNos.copieFichier(fichier, "NOS.BC"+ ToolNos.renvoieSeparateurRepertoires() + "edinos-interface-2.0.dtd");
		ToolNos.copieFichier(fichier, "NOS.BC"+ ToolNos.renvoieSeparateurRepertoires() + "edinos.nod.dtd");
		ToolNos.copieRepertoire(fichier, "Doc");
	}

	private void suiteInit() {
		Moteur.afficheMT = true;
		edinos.chargeBases(getRepertoireTravail());
	}

	public String getSimuleProcBoutons() {
		// renvoie "faits" ou "numéros"
		return config.getString("simuleProcBoutons", "faits");
	}

	public String getRepertoireTravail() {
		return config.getString("repertoireTravail", "");
	}

	public String getQuestionsBoutonRadio() {
		// renvoie "sans" ou "avec"
		return config.getString("questionsBoutonRadio", "sans");
	}

	public void sauve() {
		try {
			config.save();
		} catch (ConfigurationException e) {
			edinos.montre("Erreur lors de la sauvegarde de 'edinos.properties'");
			edinos.montre(e.getMessage());
			e.printStackTrace();
		}
	}

	public void setSimuleProcBoutons(String s) {
		config.setProperty("simuleProcBoutons", s);
	}

	public void setRepertoireTravail(String s) {
		config.setProperty("repertoireTravail", s);
	}

	public void setQuestionsBoutonRadio(String s) {
		config.setProperty("questionsBoutonRadio", s);
	}

	public void setFenDialogueHauteur(int i) {
		config.setProperty("dialogueHauteur", i);
	}

	public void setFenDialogueLargeur(int i) {
		config.setProperty("dialogueLargeur", i);
	}

	public int getFenDialogueHauteur() {
		return config.getInt("dialogueHauteur", fenDialogueHauteurDefaut);
	}

	public int getFenDialogueLargeur() {
		return config.getInt("dialogueLargeur", fenDialogueLargeurDefaut);
	}

	public int getFenDialogueLargeurDefaut() {
		return fenDialogueLargeurDefaut;
	}

	public int getFenDialogueHauteurDefaut() {
		return fenDialogueHauteurDefaut;
	}

	public void accepteCGU() {

		// Réactivation de la fen EdinoS
		Edi.nos().fen().setEnabled(true);
		if (ToolNos.nomSysteme().equals("Mac")) {
			// inutile sous Windows et Linux et de plus fait planter :
			Edi.nos().fen().getMenuBar().getMenu(0).setEnabled(true);
		}

		String s = "";
		try { // https://stackoverflow.com/questions/320542/how-to-get-the-path-of-a-running-jar-file
			s = new File(EdiNoS.class.getProtectionDomain().getCodeSource().getLocation().toURI()).getPath();
		} catch (URISyntaxException e) {
			edinos.montre("Erreur lors de la recherche de localisation du jar courant...");
		}

		String rep = "";
		if ((s.contains("EdiNoS.app"))||(s.contains("/opt/"))||(s.contains("Program Files"))) { // application MacOS, Linux ou Windows
			configurationRepertoireDonnees();

			String fichier = System.getProperty("user.home") + ToolNos.renvoieSeparateurRepertoires() + "Documents"
					+ ToolNos.renvoieSeparateurRepertoires() + "EdiNoS";
			rep = fichier + ToolNos.renvoieSeparateurRepertoires() + "NOS.BC";
			edinos.montre("Création du fichier 'edinos.properties'");
			edinos.montre("Le répertoire de travail est initialisé à : " + rep);

		} else {
			edinos.montre("Création du fichier 'edinos.properties'");
			rep = ((EdiNoS) Edi.nos()).initRepertoireBC();
		}

		if (!rep.equals("")) {
			setRepertoireTravail(rep);
			// pour initialisation sous Linux
			String nomFic = rep.substring(0, rep.lastIndexOf(ToolNos.renvoieSeparateurRepertoires()));
			nomFic = nomFic + ToolNos.renvoieSeparateurRepertoires() + "edinos.properties";
			edinos.montre("Emplacement du fichier 'edinos.properties' :" + nomFic);
			File f = new File(nomFic).getAbsoluteFile();
			config.setFile(f);
		}

		sauve();
		suiteInit();
	}
}
