/*
  EdiNoS - Editeur de NoS
  Copyright (C) 2013-2021 - Luc Poittevin — Tous droits réservés.
  Contact : edinos@free.fr  -  https://www.edinos.org
  
  Ce programme est un logiciel libre ; vous pouvez le redistribuer ou le
  modifier suivant les termes de la “GNU General Public License” telle que
  publiée par la Free Software Foundation : soit la version 3 de cette
  licence, soit (à votre gré) toute version ultérieure.
  
  Ce programme est distribué dans l’espoir qu’il vous sera utile, mais SANS
  AUCUNE GARANTIE : sans même la garantie implicite de COMMERCIALISABILITÉ
  ni d’ADÉQUATION À UN OBJECTIF PARTICULIER. Consultez la Licence Générale
  Publique GNU pour plus de détails.
  
  Vous devriez avoir reçu une copie de la Licence Générale Publique GNU avec
  ce programme ; si ce n’est pas le cas, consultez :
  <http://www.gnu.org/licenses/>. 
 */

package nos.obj;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.List;

import nos.utils.ToolNos;

public class Action implements Serializable {

	private static final long serialVersionUID = 1L;

	String nom;
	String description;
	Base base;
	List<FaitAttendu> listeFaitsAttendus;

	public Base getBase() {
		return base;
	}

	public void setBase(Base b) {
		this.base = b;
	}

	public Action(String s) {
		nom = s;
		description = "";
		listeFaitsAttendus = new ArrayList<FaitAttendu>();
	}

	public String getDescription() {
		return description;
	}

	public void setDescription(String d) {
		this.description = d;
	}

	public String getNom() {
		return nom;
	}

	public void setFaitsAttendus(List<FaitAttendu> l) {
		this.listeFaitsAttendus = l;
	}

	public List<FaitAttendu> getFaitsAttendus() {
		return listeFaitsAttendus;
	}

	public List<List<String>> structureFaitsAttendus() {
		/* OBSOLETE depuis EdiNoS v.3.4.0 (edinosrun 1.0.12)
		 * encore appelé par convertitEn20() 
		 * utiliser plutôt formateFaitsAttendus()
		 * 
		 * "*** Renvoie l'ensemble des combinaisons de faits que      ***"
		 * "*** peut renvoyer l'objet : une collection de collection  ***"
		 * "*** de proprietes. Le premier niveau est celui des lignes ***"
		 * "*** de declaration. Le second donne la liste des faits.   ***"
		 */
		List<List<String>> res = new ArrayList<List<String>>();
		List<List<String>> col;
		List<String> res2;

		String[] ligne;

		if (description.length() == 0) {
			return res;
		}

		col = ToolNos.asStructureFait(ToolNos.rejectComments(description));
		if (col.isEmpty()) {
			return res;
		}

		for (List<String> l : col) {
			res2 = new ArrayList<String>();
			if (l.size() == 1) {// pas de liste de valeurs
				String ch = ((String) l.get(0)).trim();
				ligne = ch.split(" ");
				// System.out.println(ligne[0]);
				if (ligne.length == 1) {// propriété booléenne
					res2.add(ligne[0]);
					res2.add("");
				}

				if (ligne.length == 2) {// propriété non booléenne
					// res2.add(ch);
					// spécial liste des faits sans cycleAction :
					res2.add(ligne[0] + " " + ligne[1]);
				}
			}

			if (l.size() == 2) {// liste de valeurs
				// System.out.println(l);
				String s = ((String) l.get(1)).trim();
				List<String> col2 = ToolNos.asArrayOfFormules(s);
				for (String val : col2) {
					if (val.equals("#")) {
						val = "";
					}
					res2.add(val);
				}
			}
			res.add(res2);
			// System.out.println(l.toString());
		}
		return res;
	}

	public void setNom(String nouveauNom) {
		nom = nouveauNom;
	}

	public List<List<String>> formateFaitsAttendus() {
		/*
		 * Renvoie l'ensemble des combinaisons de faits que peut renvoyer l'action
		 * Le premier niveau est celui des faits 
		 * Le second est celui des valeurs possibles (pour le type Proposition)
		 */
		
		List<List<String>> res = new ArrayList<List<String>>();
		List<String> elem, elemProposition;
		
		elemProposition = new ArrayList<String>();
		
		for (FaitAttendu fa : listeFaitsAttendus) {
			if (fa.type.equals("Proposition")) {
				elemProposition.add(fa.nom);
			} else {
				elem = new ArrayList<String>();
				elem.add(fa.nom +" "+fa.getType()); 
				res.add(elem);
			}
		}
		if (!elemProposition.isEmpty()) {
			if (elemProposition.size() == 1) {
				// il faut ajouter une valeur vide pour l'alternative
				elemProposition.add("");
			}
			res.add(elemProposition);
		}

		return res;		
	}

	public String alterFait(String nomFait) {
		// "*** Renvoie null si <this> peut renvoyer alternativement   ***"
		// "*** plusieurs autres faits que <nomFait>. Renvoie un nom  ***"
		// "*** de fait s'il n'y a qu'une seule alternative possible. ***"
		// "*** Renvoie 'NON(nomFait)' si cette alternative est null.  ***"

		List<String> col = new ArrayList<String>();
		for (List<String> el : formateFaitsAttendus()) {
			if (el.contains(nomFait)) {
				col = el;
			}
		}
		if (!col.isEmpty()) {
			col.remove(nomFait);
		}
		if (col.size() > 1) {
			return null;
		}
		if (col.get(0).isEmpty()) {
			return "NON(" + nomFait + ")";
		}
		return col.get(0);
	}

	public void convertitEn20() {
		String[] ligne;
		String type = "";

		for (List<String> el : structureFaitsAttendus()) {

			for (String ch : el) {
				if (!ch.isEmpty()) {
					ligne = ch.split(" ");
					// System.out.println(ligne[0]);
					if (ligne.length == 1) {// propriété booléenne
						FaitAttendu f = new FaitAttendu(ch);
						f.setType("Proposition");
						f.setDescription("");
						listeFaitsAttendus.add(f);
					}
					if (ligne.length == 2) {// propriété non booléenne
						FaitAttendu f = new FaitAttendu(ligne[0]);
						if (ligne[1].equals("ENTIER")) {
							type = "Entier";
						} else if (ligne[1].equals("CHAINE")) {
							type = "Chaine";
						} else if (ligne[1].equals("REEL")) {
							type = "Réel";
						} else {
							type = ligne[1];
						}
						f.setType(type);
						f.setDescription("");
						listeFaitsAttendus.add(f);
					}
				}
			}
		}
		setDescription(ToolNos.detecteComments(description));
	}
}
