/*
  EdiNoS - Editeur de NoS
  Copyright (C) 2013-2021 - Luc Poittevin — Tous droits réservés.
  Contact : edinos@free.fr  -  https://www.edinos.org
  
  Ce programme est un logiciel libre ; vous pouvez le redistribuer ou le
  modifier suivant les termes de la “GNU General Public License” telle que
  publiée par la Free Software Foundation : soit la version 3 de cette
  licence, soit (à votre gré) toute version ultérieure.
  
  Ce programme est distribué dans l’espoir qu’il vous sera utile, mais SANS
  AUCUNE GARANTIE : sans même la garantie implicite de COMMERCIALISABILITÉ
  ni d’ADÉQUATION À UN OBJECTIF PARTICULIER. Consultez la Licence Générale
  Publique GNU pour plus de détails.
  
  Vous devriez avoir reçu une copie de la Licence Générale Publique GNU avec
  ce programme ; si ce n’est pas le cas, consultez :
  <http://www.gnu.org/licenses/>. 
 */

package nos.obj;

import java.awt.Container;
import java.io.File;
import java.io.Serializable;
import java.util.ArrayList;
import java.util.Collections;
import java.util.HashMap;
import java.util.HashSet;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;
import java.util.Set;

import javax.swing.JInternalFrame;

import nos.utils.*;

import org.apache.commons.lang.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

public class Base implements Serializable {

	private static final long serialVersionUID = 1L;

	private Map<String, Action> assocActions;

	private Map<String, Graphe> assocGraphes;

	private Map<String, NoS> assocNoS;

	private String nom = "";

	private String uriDoc = "";

	private List<Container> fenetres;

	private final Logger log;

	public Base(File f) {

		log = LoggerFactory.getLogger(this.getClass());
		init();
		log.debug(f.toString());

		uriDoc = f.toString();
		ToolNosXML.chargeInterface(f, this);
		ToolNosXML.chargeGraphes(f, this);

		setNom(f.getParentFile().getName());
	}

	public Base(String nom) {
		// Création d'une nouvelle base

		log = LoggerFactory.getLogger(this.getClass());
		init();
		setNom(nom);

		if (Edi.nos().getPreferences() != null) { // possible si appel pour
			// run
			uriDoc = Edi.nos().getRepertoireTravail()
					+ ToolNos.renvoieSeparateurRepertoires() + nom
					+ ToolNos.renvoieSeparateurRepertoires() + nom + ".int.xml";
		}
	}

	public void ajouteFen(Container f) {
		fenetres.add(f);
	}

	public boolean contientGraphe(String n) {
		return assocGraphes.containsKey(n);
	}

	public boolean contientNoS(String n) {
		return assocNoS.containsKey(n);
	}

	public boolean contientProc(String n) {
		if (assocActions.containsKey(n)) {
			Action a = assocActions.get(n);
			return a instanceof Procedure;
		} else {
			return false;
		}
	}

	public boolean contientProprieteTypee(String ch) {
		// "*** Renvoie <true> si la propriété <ch> est typée ***"
		// "*** (CHAINE, ENTIER ou REEL), renvoie <false> sinon... ***"

		String type = null;

		for (String f : getListeFaitsTotale()) {
			if (!StringUtils.isEmpty(f)) {
				String[] tab = f.split(" ");
				if (tab[0].equals(ch)) {
					type = tab[1];
				}
			}
		}

		if (type == null) {
			return false;
		} else {
			return (type.equals("Chaine") || type.equals("Entier") || type
					.equals("Réel"));
		}
	}

	public boolean contientQuestion(String n) {
		if (assocActions.containsKey(n)) {
			Action a = assocActions.get(n);
			return a instanceof Question;
		} else {
			return false;
		}
	}

	public void creeGraphe(String text) {
		NoS nos = creeNoS(text);
		Graphe g = new Graphe(text);
		g.setPremierNoS(nos);
		assocGraphes.put(g.getNom(), g);

		ConstruitDom cd = new ConstruitDom(text);
		g.setDoc(cd.document());
		g.setDocURI(ToolNos.cheminRep(uriDoc) + text + ".nod.xml");

		
	}

	public NoS creeNoS(String nom) {
		NoS nos = new NoS(nom);
		nos.setBase(this);
		assocNoS.put(nos.getNom(), nos);
		return nos;
	}

	public Procedure creeProcedure(String nom) {
		Procedure proc = new Procedure(nom);
		FaitAttendu fa = new FaitAttendu(nom);
		fa.setType("Proposition");
		List<FaitAttendu> listeFaitsAttendus = new ArrayList<FaitAttendu>();
		listeFaitsAttendus.add(fa);
		proc.setFaitsAttendus(listeFaitsAttendus);
		proc.setBase(this);
		assocActions.put(proc.getNom(), proc);
		return proc;
	}

	public Question creeQuestion(String nom) {
		Question quest = new Question(nom);
		FaitAttendu fa = new FaitAttendu(nom);
		fa.setType("Proposition");
		List<FaitAttendu> listeFaitsAttendus = new ArrayList<FaitAttendu>();
		listeFaitsAttendus.add(fa);
		quest.setFaitsAttendus(listeFaitsAttendus);
		quest.setBase(this);
		assocActions.put(quest.getNom(), quest);
		return quest;
	}

	public void fermeFens() {
		for (Container f : fenetres) {
			((JInternalFrame) f).dispose();
		}
	}

	public Action getAction(String n) {
		return assocActions.get(n);
	}

	public String getActionQuiRenvoieFait(String f) {
		for (NoS n : assocNoS.values()) {
			if (n.tousLesFaitsAttendus().contains(f)) {
				return n.getNomPremiereAction();
			}
		}
		;
		return "";
	}

	public Map<String, Action> getAssocActions() {
		return assocActions;
	}

	public Map<String, Graphe> getAssocGraphes() {
		return assocGraphes;
	}

	public Map<String, NoS> getAssocNos() {
		return assocNoS;
	}

	public Graphe getGraphe(String nom) {
		return assocGraphes.get(nom);
	}

	public Graphe getGrapheQuiContientNos(String nomNos) {
		// "*** renvoie le graphe qui contient 		***"
		// "*** le nos <chaine>, renvoie <null> si  ***"
		// "*** le nos n'appartient pas à la base   ***"

		for (Graphe g : assocGraphes.values()) {
			if (g.contientNos(nomNos)) {
				return g;
			}
		}
		
		return null;
	}

	public List<String> getListeActions() {
		List<String> listeActions = new LinkedList<String>();
		listeActions.addAll(assocActions.keySet());
		Collections.sort(listeActions);
		return listeActions;
	}

	public List<String> getListeActionsPlusAfficher() {
		// "*** renvoie tous les noms d'actions déclarées ***"
		// "*** plus les procs 'afficher' ..."

		List<String> listeActions = getListeActions();
		for (NoS n : assocNoS.values()) {
			if (n.contientUneActionAfficher()) {
				listeActions.add("(afficher)" + n.getNomPremiereAction());
			}
		}
		Collections.sort(listeActions);
		return listeActions;
	}

	public List<String> getListeFaitsAppeles() {
		
		List<String> listeFaits = new LinkedList<String>();
		for (String nomNoS : getListeNoS()) {
			NoS n = assocNoS.get(nomNoS);
			listeFaits.addAll(n.tousLesFaitsAttendus());
		}
		
		Set<String> listeFaitsUniques = new HashSet<String>(listeFaits);
		List<String> listeFinale = new LinkedList<String>(listeFaitsUniques);
		Collections.sort(listeFinale);
		return listeFinale;
	}

	public List<String> getListeFaitsTotale() {
		
		List<String> listeFaits = new LinkedList<String>();
		for (Action a : assocActions.values()) {
			for (List<String> l : a.formateFaitsAttendus()) {
				listeFaits.addAll(l);
			}
		}
		return listeFaits;
	}

	public List<String> getListeGraphes() {
		List<String> listeGraphes = new LinkedList<String>();
		listeGraphes.addAll(assocGraphes.keySet());
		Collections.sort(listeGraphes);
		return listeGraphes;
	}

	public List<String> getListeNoS() {
		List<String> listeNoS = new LinkedList<String>();
		listeNoS.addAll(assocNoS.keySet());
		Collections.sort(listeNoS);
		return listeNoS;
	}

	public List<String> getListeNoSActions(String a) {
		// "*** renvoie la liste de tous les nodules ***"
		// "*** qui appellent l'action <a> ...        ***"

		List<List<String>> listeActions = new ArrayList<List<String>>();
		List<String> liste = new LinkedList<String>();

		for (NoS n : assocNoS.values()) {
			listeActions = ToolNos.asStructureSyntaxe(n.getActions());
			for (List<String> ligne : listeActions) {
				if (ligne.size() > 1) { // ajout LP du 16/10/2013
					if (ligne.get(1).equals(a)) {
						liste.add(n.getNom());
					}
				}
			}
		}
		return liste;
	}

	public List<String> getListeProcs() {
		List<String> listeActions = new LinkedList<String>();
		for (Action a : assocActions.values()) {
			if (a.getClass().getName().equals("nos.obj.Procedure")) {
				listeActions.add(a.getNom());
			}
		}

		Collections.sort(listeActions);
		return listeActions;
	}

	public List<String> getListeQuestions() {
		List<String> listeActions = new LinkedList<String>();
		for (Action a : assocActions.values()) {
			if (a.getClass().getName().equals("nos.obj.Question")) {
				listeActions.add(a.getNom());
			}
		}

		Collections.sort(listeActions);
		return listeActions;
	}

	public List<String> getListeMessages() {
		List<String> listeActions = new LinkedList<String>();
		for (Action a : assocActions.values()) {
			if (a.getClass().getName().equals("nos.obj.Message")) {
				listeActions.add(a.getNom());
			}
		}

		Collections.sort(listeActions);
		return listeActions;
	}

	public String getNom() {
		return nom;
	}

	public NoS getNoS(String n) {
		return assocNoS.get(n);
	}

	public Procedure getProc(String n) {
		return (Procedure) assocActions.get(n);
	}

	public Question getQuestion(String n) {
		return (Question) assocActions.get(n);
	}

	public String getTypeDeProc(String nom) {
		if (contientProc(nom)) {
			return ("evaluer:");
		}

		if (contientQuestion(nom)) {
			return ("demander:");
		}

		return ("afficher:");
	}

	private void init() {
		assocActions = new HashMap<String, Action>();
		assocGraphes = new HashMap<String, Graphe>();
		assocNoS = new HashMap<String, NoS>();
		fenetres = new ArrayList<Container>();
	}

	public void majSuivants(String ancienNomNos, String nouveauNomNos) {
		// remplace toutes les réfs à <ancienNomNos> dans les suivants du graphe
		// par <nouveauNomNos>
		

		for (NoS n : assocNoS.values()) {
			if (n.getSuivants().contains(ancienNomNos)) {
				n.removeSuivant(ancienNomNos);
				n.addSuivant(nouveauNomNos);
			}
		}
	}

	public void renommeGraphe(NoS n, String nouveauNom) {
		String ancienNom = n.getNom();
		n.setNom(nouveauNom);
		assocNoS.remove(ancienNom);
		assocNoS.put(nouveauNom, n);
		majSuivants(ancienNom, nouveauNom);
		Edi.nos().refreshListeNoS();

		Graphe g = assocGraphes.get(ancienNom);
		g.supprimeFichier();
		g.setNom(nouveauNom);
		g.setPremierNoS(n);
		g.setDocURI(ToolNos.cheminRep(uriDoc) + nouveauNom + ".nod.xml");
		assocGraphes.remove(ancienNom);
		assocGraphes.put(nouveauNom, g);
		Edi.nos().afficheBase();
	}

	public void retireFen(Container f) {
		fenetres.remove(f);
	}

	public void sauve() {

		Graphe gr;
		SauveDom s = null;
		org.jdom.Document doc = null;
		File f = new File(Edi.nos().getRepertoireTravail()
				+ ToolNos.renvoieSeparateurRepertoires() + nom)
				.getAbsoluteFile();

		boolean repExiste = f.exists();
		if (!repExiste) {
			boolean repCree = f.mkdir();
			if (!repCree) {
				Edi.nos().message("Echec dans la création du répertoire <"
						+ Edi.nos().getRepertoireTravail()
						+ ToolNos.renvoieSeparateurRepertoires() + nom
						+ ">...");
				return;
			}
		}

		for (String nom : assocGraphes.keySet()) {
			gr = assocGraphes.get(nom);

			doc = gr.reconstruitDoc(gr.getDocURI());


			s = new SauveDom(doc);

			try {
				s.sauveJ(".." + ToolNos.renvoieSeparateurRepertoires()
						+ "edinos.nod.dtd", "graphe");
			} catch (Exception t) {
				Edi.nos().montreInfo(t.toString());
				Edi.nos().message("Erreur pendant la sauvegarde du graphe <"
						+ gr.getNom() + ">...");
				return;
			}
		}

		doc = ToolNosXML.reconstruitDoc(uriDoc, this);


		s = new SauveDom(doc);

		try {
			s.sauveJ(".." + ToolNos.renvoieSeparateurRepertoires()
					+ "edinos-interface-2.0.dtd", "interface");
		} catch (Exception t) {
			Edi.nos().montreInfo(t.toString());
			Edi.nos().message("Erreur pendant la sauvegarde de l'interface...");
			return;
		}
	}

	public void sauveSous(String nouveauNom) {
		Edi.nos().retireBaseCourante();
		setNom(nouveauNom);
		uriDoc = Edi.nos().getRepertoireTravail()
				+ ToolNos.renvoieSeparateurRepertoires() + getNom()
				+ ToolNos.renvoieSeparateurRepertoires() + getNom()
				+ ".int.xml";
		for (Graphe g : assocGraphes.values()) {
			g.setDocURI(ToolNos.cheminRep(uriDoc) + g.getNom() + ".nod.xml");
		}
		sauve();
		Edi.nos().setBaseCourante(this);
		Edi.nos().ajouteBaseCourante();
		Edi.nos().montreInfo("Base <" + nouveauNom + "> sauvegardée");
	}

	public void setAssocActions(Map<String, Action> a) {
		assocActions = a;
	}

	public void setAssocGraphes(Map<String, Graphe> g) {
		assocGraphes = g;
	}

	public void setAssocNos(Map<String, NoS> n) {
		assocNoS = n;
	}

	public void setNom(String n) {
		nom = n;
	}

	public void supprimeAction(String nomAction) {
		assocActions.remove(nomAction);
	}

	public void supprimeSuivants(String nomNos) {
		// remplace toutes les réfs à <nomNos> dans les suivants du graphe
		

		for (NoS n : assocNoS.values()) {
			if (n.getSuivants().contains(nomNos)) {
				n.removeSuivant(nomNos);
			}
		}
	}

	public void supprimeGraphe(String nomGraphe) {
		assocNoS.remove(nomGraphe);
		Edi.nos().refreshListeNoS();

		Graphe g = assocGraphes.get(nomGraphe);
		g.supprimeFichier();
		assocGraphes.remove(nomGraphe);
		Edi.nos().afficheBase();
	}

	public Message creeMessage(String nom) {
		Message m = new Message(nom);
		m.setBase(this);
		assocActions.put(m.getNom(), m);
		return m;
	}

	public boolean contientMessage(String n) {
		if (assocActions.containsKey(n)) {
			Action a = assocActions.get(n);
			return a instanceof Message;
		} else {
			return false;
		}
	}

	public Message getMessage(String n) {
		return (Message) assocActions.get(n);
	}

	public void convertitActionsEn20() {
		for (Action act : assocActions.values()) {
			act.convertitEn20();
		}
	}
}
