/*
  EdiNoS - Editeur de NoS
  Copyright (C) 2013-2017 - Luc Poittevin — Tous droits réservés.
  Contact : edinos@free.fr  -  https://www.edinos.org
  
  Ce programme est un logiciel libre ; vous pouvez le redistribuer ou le
  modifier suivant les termes de la “GNU General Public License” telle que
  publiée par la Free Software Foundation : soit la version 3 de cette
  licence, soit (à votre gré) toute version ultérieure.
  
  Ce programme est distribué dans l’espoir qu’il vous sera utile, mais SANS
  AUCUNE GARANTIE : sans même la garantie implicite de COMMERCIALISABILITÉ
  ni d’ADÉQUATION À UN OBJECTIF PARTICULIER. Consultez la Licence Générale
  Publique GNU pour plus de détails.
  
  Vous devriez avoir reçu une copie de la Licence Générale Publique GNU avec
  ce programme ; si ce n’est pas le cas, consultez :
  <http://www.gnu.org/licenses/>. 
 */

package nos.obj;

import java.io.File;
import java.io.Serializable;
import java.util.Collections;
import java.util.HashMap;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;

import javax.swing.JFrame;

import nos.run.Moteur;

public abstract class Edi implements Serializable {

	// La classe Edi est un singleton
	// son instance est accessible via Edi.nos()

	/**
	 * Récupère l'instance unique de la class Edi
	 */
	public static Edi nos() {
		return instance;
	}

	private static final long serialVersionUID = 1L;

	protected Map<String, Base> assocBases;
	
	protected Base baseCourante; 
	
	/** L'instance statique : */
	protected static Edi instance;
	
	/**
	 * Constructeur redéfini comme étant privé pour interdire son appel et
	 * forcer à passer par la méthode nos() :
	 */
	protected Edi() {
		assocBases = new HashMap<String, Base>();
		// La construction de fenEdinos est faite
		// plus loin pour cause de références circulaires notamment...
	}
	
	public static void forceNos(Edi i) {
		instance = i;
	}
	
	public abstract void affiche(String s, Moteur m);
	
	public abstract void afficheFinLancement(Moteur moteur);
	
	public void ajouteBase(String n, Base b) {
		assocBases.put(n, b);
	}
	
	void ajouteBaseCourante() {
		if (this.getBaseCourante().getNom() == "") {
			Edi.nos().montreAvertissement("Erreur dans ouverture de la base");
			setBaseCourante(null);
			return;
		}

		String nom = getBaseCourante().getNom();
		ajouteBase(nom, getBaseCourante());
		Edi.nos().ajouteBase(nom);
	}
	
	public boolean contientBase(String text) {
		return assocBases.containsKey(text);
	}
	
	public boolean contientMessage(String nom) {
		for (String nomBase : getListeBases()) {
			if (getBase(nomBase).contientMessage(nom)) {
				return true;
			}
		}
		return false;
	}
	
	public boolean contientNoS(String chaine) {
		for (String nomBase : getListeBases()) {
			if (getBase(nomBase).contientNoS(chaine)) {
				return true;
			}
		}
		return false;
	}

	public boolean contientProc(String nom) {
		for (String nomBase : getListeBases()) {
			if (getBase(nomBase).contientProc(nom)) {
				return true;
			}
		}
		return false;
	}
	
	public boolean contientProprieteTypee(String n) {
		// "*** Renvoie <true> si la propriété <ch> est typée ***"
		// "*** (CHAINE, ENTIER ou REEL), renvoie <false> sinon... ***"
		// "*** Recherche la déclaration de la propriété dans   ***"
		// "*** la base courante,puis dans 'COMMUN', enfin dans ***"
		// "*** 'REVINOS' ('pour 'REVI')...                     ***"



		
		for (String nomBase : getListeBases()) {
			if (getBase(nomBase).contientProprieteTypee(n)) {
				return true;
			}
		}
		return false;
	}
	
	public boolean contientQuestion(String nom) {
		for (String nomBase : getListeBases()) {
			if (getBase(nomBase).contientQuestion(nom)) {
				return true;
			}
		}
		return false;
	}
	
	public void creeBase(File file) {
		Base b = new Base(file);
		if (assocBases.containsKey(b.getNom())) {
			message("Une base nommée <" + b.getNom() + "> est déjà ouverte...");
			return;
		}
		setBaseCourante(b);
		ajouteBaseCourante();
		Edi.nos().montreInfo("Base <" + b.getNom() + "> chargée");
	}

	public void creeBase(String text) {
		this.setBaseCourante(new Base(text));
		ajouteBaseCourante();
	}

	public abstract JFrame fen();

	public Map<String, Base> getAssocBases() {
		return assocBases;
	}

	public Base getBase(String n) {
		return assocBases.get(n);
	}

	public Base getBaseCourante() {
		return baseCourante;
	}

	public List<String> getListeBases() {
		List<String> listeBases = new LinkedList<String>();
		listeBases.addAll(assocBases.keySet());
		Collections.sort(listeBases);
		return listeBases;
	}

	public NoS getNoS(String chaine) {
		NoS nos = null;
		Base b = null;
		for (String nomBase : getListeBases()) {
			b = getBase(nomBase);
			if (b.contientNoS(chaine)) {
				nos = b.getNoS(chaine);
			}
		}
		return nos;
	}

	public Procedure getProc(String chaine) {
		Procedure p = null;
		Base b = null;
		for (String nomBase : getListeBases()) {
			b = getBase(nomBase);
			if (b.contientProc(chaine)) {
				p = b.getProc(chaine);
			}
		}
		return p;
	}

	public Question getQuestion(String chaine) {
		Question q = null;
		Base b = null;
		for (String nomBase : getListeBases()) {
			b = getBase(nomBase);
			if (b.contientQuestion(chaine)) {
				q = b.getQuestion(chaine);
			}
		}
		return q;
	}

	public abstract void initPrefs();

	public abstract void message (String s);
	
	public abstract void montreAvertissement(String s);

	public abstract void montreInfo(String s);
	
	public abstract Object getPreferences();

	public abstract void poseQuestion(Question question, Moteur moteur);

	public void retireBaseCourante() {
		assocBases.remove(baseCourante.getNom());
		Edi.nos().retireBase(baseCourante.getNom());
	}

	public void setBaseCourante(Base b) {
		baseCourante = b;
	}
	
	public Message getMessage(String chaine) {
		Message m = null;
		Base b = null;
		for (String nomBase : getListeBases()) {
			b = getBase(nomBase);
			if (b.contientMessage(chaine)) {
				m = b.getMessage(chaine);
			}
		}
		return m;
	}

	public abstract void simuleProc(Procedure p, Moteur m);
	
	public abstract String getRepertoireTravail();
	
	public abstract void refreshListeNoS();
	
	public abstract void afficheBase();
	
	public abstract void montre(String s);
	
	public abstract void ajouteBase(String nom);
	
	public abstract void retireBase(String nom);

	public abstract void affiche(Message m, Moteur mot);
}
