/*
  EdiNoS - Editeur de NoS
  Copyright (C) 2013-2021 - Luc Poittevin — Tous droits réservés.
  Contact : edinos@free.fr  -  https://www.edinos.org
  
  Ce programme est un logiciel libre ; vous pouvez le redistribuer ou le
  modifier suivant les termes de la “GNU General Public License” telle que
  publiée par la Free Software Foundation : soit la version 3 de cette
  licence, soit (à votre gré) toute version ultérieure.
  
  Ce programme est distribué dans l’espoir qu’il vous sera utile, mais SANS
  AUCUNE GARANTIE : sans même la garantie implicite de COMMERCIALISABILITÉ
  ni d’ADÉQUATION À UN OBJECTIF PARTICULIER. Consultez la Licence Générale
  Publique GNU pour plus de détails.
  
  Vous devriez avoir reçu une copie de la Licence Générale Publique GNU avec
  ce programme ; si ce n’est pas le cas, consultez :
  <http://www.gnu.org/licenses/>. 
*/

package nos.obj;

import java.awt.Dimension;
import java.io.File;
import java.io.Serializable;

import nos.run.Bilan;

import org.jdom.Element;
import org.w3c.dom.Document;

public class Graphe implements Serializable {

	private static final long serialVersionUID = 1L;

	private String nom;

	private Document doc;

	private String docURI;

	private NoS premierNoS;

	private Dimension taille;

	public Graphe(String s) {
		nom = s;
		taille = new Dimension(600, 440);
	}

	public Document getDoc() {
		return doc;
	}

	public String getDocURI() {
		return docURI;
	}

	public String getNom() {
		return nom;
	}

	public NoS getPremierNoS() {
		return premierNoS;
	}

	public org.jdom.Document reconstruitDoc(String uri) {
		Element nosElem = null, actElem = null, choixElem = null, comElem = null, suivElem = null, destElem = null;

		
		

		Element racine = new Element("graphe");
		racine.setAttribute("nom", this.getNom());

		
		

		for (NoS nos : this.getPremierNoS().tousLesNoSSuivants()) {

			nosElem = new Element("nos");
			nosElem.setAttribute("nom", nos.getNom());
			nosElem.setAttribute("position", nos.getPosition());

			if (nos.getActions().length() > 0) {
				actElem = new Element("actions");
				actElem.setText(nos.getActions());
				nosElem.addContent(actElem);
			}

			if (nos.getRegles().length() > 0) {
				choixElem = new Element("choix");
				choixElem.setText(nos.getRegles());
				nosElem.addContent(choixElem);
			}

			if (nos.getCommentaires().length() > 0) {
				comElem = new Element("commentaires");
				comElem.setText(nos.getCommentaires());
				nosElem.addContent(comElem);
			}

			racine.addContent(nosElem);
		}

		for (NoS nos : this.getPremierNoS().tousLesNoSSuivants()) {

			if (!nos.getSuivants().isEmpty()) {
				suivElem = new Element("suivantsVue");
				suivElem.setAttribute("origine", nos.getNom());
				for (String s : nos.getSuivants()) {
					destElem = new Element("dest");
					destElem.setText(s);
					suivElem.addContent(destElem);
				}
				
				racine.addContent(suivElem);
			}
		}

		org.jdom.Document jdoc = new org.jdom.Document(racine);
		jdoc.setBaseURI(uri);
		return jdoc;
	}

	public void setDoc(Document d) {
		this.doc = d;
	}

	public void setDocURI(String str) {
		docURI = str;
	}

	public void setNom(String s) {
		this.nom = s;
	}

	public void setPremierNoS(NoS noS) {
		premierNoS = noS;
	}

	public Bilan bilan() {

		Bilan bilan = new Bilan();
		try {
			bilan.lance(premierNoS);
		} catch (StackOverflowError e) {
			e.printStackTrace();
			Edi.nos().message("Erreur lors du calcul du bilan. ...");
			Edi.nos().montre("Erreur lors du calcul du bilan. ...");
		}

		return bilan;
	}

	public void setTaille(Dimension size) {
		taille = size;
	}

	public Dimension getTaille() {
		return taille;
	}

	public boolean contientNos(String nomNos) {
		// "*** renvoie true si le nos <nomNos> appartient à <this> ***"

		for (NoS n : premierNoS.tousLesNoSSuivants()) {
			if (n.getNom().equals(nomNos)) {
				return true;
			}
		}

		return false;
	}

	public void supprimeFichier() {
		File f = new File(docURI);
		boolean res = f.delete();
		if (!res) {
			String s = "Problème lors de la suppression du fichier <" + docURI + ">...";
			Edi.nos().message(s);
			Edi.nos().montreAvertissement(s);
		}
	}
}
