/*
  EdiNoS - Editeur de NoS
  Copyright (C) 2013-2021 - Luc Poittevin — Tous droits réservés.
  Contact : edinos@free.fr  -  https://www.edinos.org
  
  Ce programme est un logiciel libre ; vous pouvez le redistribuer ou le
  modifier suivant les termes de la “GNU General Public License” telle que
  publiée par la Free Software Foundation : soit la version 3 de cette
  licence, soit (à votre gré) toute version ultérieure.
  
  Ce programme est distribué dans l’espoir qu’il vous sera utile, mais SANS
  AUCUNE GARANTIE : sans même la garantie implicite de COMMERCIALISABILITÉ
  ni d’ADÉQUATION À UN OBJECTIF PARTICULIER. Consultez la Licence Générale
  Publique GNU pour plus de détails.
  
  Vous devriez avoir reçu une copie de la Licence Générale Publique GNU avec
  ce programme ; si ce n’est pas le cas, consultez :
  <http://www.gnu.org/licenses/>. 
*/

package nos.obj;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.Collection;
import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.Set;

import javax.xml.bind.annotation.XmlTransient;

import org.w3c.dom.Node;

import nos.run.*;
import nos.run.Moteur.Contexte;
import nos.run.Moteur.ModeExecutionProc;
import nos.utils.ToolNos;

public class NoS implements Serializable {
	// anciennement Nodule

	private static final long serialVersionUID = 1L;

	private String nom = "";

	private String actions = "";

	private String regles = "";

	private String commentaires = "";

	private Node nodeDom;

	private Base base;

	private String pos;

	private List<String> listeSuivantsVue;

	private Boolean raccourci = false;

	public NoS(String s) {
		nom = s;
		listeSuivantsVue = new ArrayList<String>();
		setPosition("200@20");
	}

	public NoS() { // Pour JAXB
	}

	public void addSuivant(String nomNos) {
		listeSuivantsVue.add(nomNos);
	}

	public void removeSuivant(String nomNos) {
		listeSuivantsVue.remove(nomNos);
	}

	public void declencheGraphe() {
		Moteur mot = new Moteur(this, Contexte.Lancement, ModeExecutionProc.Execute);
		Memoire mem = new Memoire();
		mot.declencheAvec(mem);
	}

	@XmlTransient
	public String getActions() {
		return actions;
	}

	@XmlTransient
	public Base getBase() {
		return base;
	}

	@XmlTransient
	public String getCommentaires() {
		return commentaires;
	}

	public String getNom() {
		return nom;
	}

	@XmlTransient
	public String getPosition() {
		return pos;
	}

	@XmlTransient
	public int getPosX() {
		return new Integer(pos.split("@")[0]).intValue();
	}

	@XmlTransient
	public int getPosY() {
		return new Integer(pos.split("@")[1]).intValue();
	}

	@XmlTransient
	public Boolean getRaccourci() {
		return raccourci;
	}

	@XmlTransient
	public String getRegles() {
		return regles;
	}

	@XmlTransient
	public List<String> getSuivants() {
		return listeSuivantsVue;
	}

	public Set<NoS> parentsAffichesDans(Set<NoS> autres) {
		// "*** renvoie la liste des parents NON raccourcis du receveur ***"

		Set<NoS> parents = new HashSet<NoS>();
		for (NoS n : autres) {
			if ((n.getSuivants().contains(nom)) && (n.getRaccourci() == false)) {
				parents.add(n);
			}
		}
		return parents;
	}

	public void raccourcitTousLesSuivants() {
		/*
		 * "***  raccourcit tous les descendants suivantsVues ***"
		 * "***  du receveur (suivantsVue des suivantsVue des ***"
		 */

		List<NoS> liste = tousLesNoSSuivants();
		// "*** <tousLesNoSSuivants> renvoie <this> : il faut l'enlever ***"
		liste.remove(this);

		for (NoS n : liste) {
			if (n.getSuivants().isEmpty() == false) {
				n.setRaccourci(true);
			}
		}
	}

	public void rallongeTousLesSuivants() {
		for (NoS n : tousLesNoSSuivants()) {
			n.setRaccourci(false);
		}
	}

	public void setActions(String str) {
		this.actions = str;
	}

	public void setBase(Base b) {
		this.base = b;
	}

	public void setCommentaires(String str) {
		this.commentaires = str;
	}

	public void setNode(Node node) {
		nodeDom = node;
	}

	public void setNom(String nouveauNom) {
		nom = nouveauNom;
	}

	public void setPos(String p) {
		this.pos = p;
	}

	public void setPosition(String string) {
		pos = string;
	}

	public void setRaccourci(Boolean b) {
		raccourci = b;
	}

	public void setRegles(String r) {
		this.regles = r;
	}

	public List<String> tousLesFaitsAttendus() {

		Collection<List<String>> listeFaits;
		List<String> liste;

		liste = new ArrayList<String>();
		listeFaits = new ArrayList<List<String>>();

		MoteurAction ca = new MoteurAction(this);
		ca.executeActions();
		listeFaits = ca.descriptionFaitsAttendus().values();
		for (List<String> l : listeFaits) {
			liste.addAll(l);
		}
		return liste;
	}

	public Set<NoS> tousLesNoSSuivants(Set<NoS> baluchon) {

		Set<NoS> ens;
		ens = new HashSet<NoS>();
		ens.add(this);
		ens.addAll(baluchon);

		for (String nom : listeSuivantsVue) {
			NoS courant = (NoS) base.getAssocNos().get(nom);
			if (ens.add(courant) == true) {
				// on ne va chercher les descendants
				// que si <courant> n'est pas déjà dans le baluchon
				// (gestion des cycles)
				ens.addAll(courant.tousLesNoSSuivants(ens));
			}
		}
		return ens;
	}

	public List<NoS> tousLesNoSSuivants() {
		/*
		 * renvoie la liste de tous les descendants suivantsVues du receveur
		 * (suivantsVue des suivantsVue des ... <this> y compris ... en premiere position
		 */

		Set<NoS> ens = new HashSet<NoS>();
		ens = tousLesNoSSuivants(new HashSet<NoS>());

		List<NoS> l = new ArrayList<NoS>(ens);
		// On met <this> en première position :
		l.remove(this);
		l.add(0, this);

		return l;
	}

	public Set<NoS> tousLesNoSSuivantsAafficher(Set<NoS> baluchon) {
		/*
		 * "***  renvoie la liste de tous les descendants suivantsVues ***"
		 * "***  du receveur (suivantsVue des suivantsVue des ... )    ***" 
		 * <this> y compris ET GERE LES NOS RACCOURCIS (s'arrete) ***"
		 */

		Set<NoS> ens;
		ens = new HashSet<NoS>();
		ens.add(this);
		ens.addAll(baluchon);

		if (raccourci == false) {
			for (String nom : listeSuivantsVue) {
				NoS courant = (NoS) base.getAssocNos().get(nom);
				if (ens.add(courant) == true) {
					// on ne va chercher les descendants
					// que si <courant> n'est pas déjà dans le baluchon
					// (gestion des cycles) 
					ens.addAll(courant.tousLesNoSSuivantsAafficher(ens));
				}
			}
		}
		return ens;
	}

	public Map<String, List<String>> tousLesProcsEtFaitsAttendus(
			String nomAction) {
		// "*** Renvoie une collection d'associations : ***"
		// "*** nomProc => collection de faits. ***"

		MoteurAction ca = new MoteurAction(this);
		ca.executeActions();
		return ca.descriptionFaitsAttendus();

		
	}

	public void updateNos() {
		nodeDom.getAttributes().getNamedItem("position").setNodeValue(pos);
	}

	public void declencheAvec(Memoire faits, Moteur ancetre, Moteur cible, Contexte c, ModeExecutionProc mode, String l,
			ArchiveMemoire a) {
		Moteur m = new Moteur(this, c, mode);
		m.setAncetre(ancetre);
		m.setCible(cible);
		m.setLog(l);
		m.declencheAvec(faits, a);
	}

	public String getTypeAction() {
		// "*** renvoie <afficher:>, <declencher:>, ***"
		// "*** <evaluer:>, <demander:> ou """... ***"

		List<List<String>> st = ToolNos.asStructureSyntaxe(actions);
		if (st.isEmpty()) {
			return "";
		} else {
			return st.get(0).get(0);
		}
	}

	public boolean contientUneActionAfficher() {
		// "*** renvoie <true> si <this> comprend une ***"
		// "*** action 'afficher', renvoie <false> sinon ***"

		List<List<String>> st = ToolNos.asStructureSyntaxe(actions);
		if (st.size() > 1) {
			return false;
		} else {
			return getTypeAction().equals("afficher:");
		}
	}

	public String getNomPremiereAction() {
		List<List<String>> st = ToolNos.asStructureSyntaxe(actions);
		if (st.isEmpty()) {
			return "";
		} else {
			if (st.get(0).isEmpty() || (st.get(0).size() == 1)) {
				return "";
			} else {
				return st.get(0).get(1);
			}
		}
	}

	public String getTypePremiereAction() {
		List<List<String>> st = ToolNos.asStructureSyntaxe(actions);
		if (st.isEmpty()) {
			return "";
		} else {
			return st.get(0).get(0);
		}
	}

	public boolean actionTerminale() {
//		"*** renvoie <true> si <self> comprend une seule action,     ***"
//		"*** et si elle est terminale (elle ne renvoie pas de faits) ***"
		return (tousLesProcsEtFaitsAttendus(getNomPremiereAction()).size() == 0);
	}

	public String getLibelleFlecheVers(NoS n) {
		// Pour affichage d'un libellé dans les liens
		// dans <FenGraphe>

		String libelle = "";
		List<List<String>> liste = ToolNos.asStructureSyntaxe(ToolNos.retireCommentaires(regles));
		List<String> l;

		if (!liste.isEmpty()) {
			for (int i = 0; i < liste.size(); i++) {
				l = (List<String>) liste.get(i);
				if (l.get(0).equals("si:")) {
					String nomNos = "";
					int ind = l.indexOf("alorsAller:");
					if (ind > 0) {
						nomNos = (String) l.get(ind + 1);
						if (nomNos.equals(n.getNom())) {
							if (!libelle.isEmpty()) {
								// il y a déjà une règle qui cible nomNos :
								// le libellé sera <...> :
								return "<...>";
							} else {
								for (String s : l.subList(1, ind)) {
									if (libelle.isEmpty()) {
										libelle = s;
									} else {
										libelle = libelle + " " + s;
									}
								}
								// on tronque éventuellement pour des raisons de lisibilité :
								if (libelle.length() >= 50) {
									libelle = libelle.substring(0, 50) + "...";
								}
							}
						}
					}

				} else if (l.get(0).equals("aller:")) {
					String nomNos = "";
					nomNos = (String) l.get(1);
					if (nomNos.equals(n.getNom())) {
						libelle = "<par défaut>";
					}
				}
			}
		}

		return libelle;
	}
}
