/*
  EdiNoS - Editeur de NoS
  Copyright (C) 2013-2021 - Luc Poittevin — Tous droits réservés.
  Contact : edinos@free.fr  -  https://www.edinos.org
  
  Ce programme est un logiciel libre ; vous pouvez le redistribuer ou le
  modifier suivant les termes de la “GNU General Public License” telle que
  publiée par la Free Software Foundation : soit la version 3 de cette
  licence, soit (à votre gré) toute version ultérieure.
  
  Ce programme est distribué dans l’espoir qu’il vous sera utile, mais SANS
  AUCUNE GARANTIE : sans même la garantie implicite de COMMERCIALISABILITÉ
  ni d’ADÉQUATION À UN OBJECTIF PARTICULIER. Consultez la Licence Générale
  Publique GNU pour plus de détails.
  
  Vous devriez avoir reçu une copie de la Licence Générale Publique GNU avec
  ce programme ; si ce n’est pas le cas, consultez :
  <http://www.gnu.org/licenses/>. 
 */

package nos.run;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Map.Entry;
import java.util.SortedSet;
import java.util.TreeSet;

import nos.obj.Edi;
import nos.obj.NoS;
import nos.run.Moteur.Contexte;
import nos.utils.ToolNos;

public class Bilan {
	// anciennement Exploration

	// <nomNosInitial> est le nom du nos initial du graphe,
	// <cheminsAExplorer> est une collection de <MoteurBilan> à explorer,
	// <cheminsExplores> est une collection de <MoteurBilan> explorés,
	// <nomNosFinal> est le nom du nos analysé
	// (répercussions première phase)

	public Bilan() {
		super();
		cheminsExplores = new ArrayList<MoteurBilan>();
		cheminsAExplorer = new ArrayList<MoteurBilan>();
		nomNosFinal = null;
	}

	private List<MoteurBilan> cheminsExplores; // anciennement panier;

	private List<MoteurBilan> cheminsAExplorer; // anciennement courant;

	private String nomNosInitial;

	private String nomNosFinal;

	private List<Map<String, List<List<String>>>> listeCibles;

	private void ajouteExplorationValeur(MoteurBilan c, Map<String, List<List<String>>> v) {


		MoteurBilan doublon = new MoteurBilan(Edi.nos().getNoS(nomNosInitial), c.contexte);
		doublon.setBilan(this);

		Memoire mem = new Memoire();
		mem.setMoteurBilan(c);
		doublon.setBaseFaits(mem);
		doublon.setAncetre(c.getAncetre());

		doublon.setValeurs(v);

		HashMap<String, List<List<String>>> dupliqueValeursReperees = new HashMap<String, List<List<String>>>();
		dupliqueValeursReperees.putAll(c.getValeursReperees());
		doublon.setValeursReperees(dupliqueValeursReperees);

		doublon.getBaseFaits().setMoteurBilan(doublon);

		cheminsAExplorer.add(doublon);
	}

	public void ajouteBilan(MoteurBilan c, String nomProc, List<String> collecFaits) {
		// "*** <collecFaits> est une collection de faits à ajouter à la ***"
		// "*** M.T. de <c> avant de dupliquer <c> dans <cheminsAExplorer> ***"

		MoteurBilan doublon = new MoteurBilan(c.getNoS(), c.contexte);
		doublon.setBilan(this);
		doublon.setActionsAFaire(c.getActionsAFaire());
		doublon.setBaseFaits(c.getBaseFaits().duplique());
		doublon.setNomActionCourante(c.getNomActionCourante());
		doublon.setAncetre(c.getAncetre());

		HashMap<String, List<List<String>>> dupliqueValeurs = new HashMap<String, List<List<String>>>();
		dupliqueValeurs.putAll(c.getValeurs());
		doublon.setValeurs(dupliqueValeurs);

		HashMap<String, List<List<String>>> dupliqueValeursReperees = new HashMap<String, List<List<String>>>();
		dupliqueValeursReperees.putAll(c.getValeursReperees());
		doublon.setValeursReperees(dupliqueValeursReperees);

		for (String nomFait : collecFaits) {
			doublon.getBaseFaits().ajoute(nomProc, nomFait);
		}

		doublon.getBaseFaits().setMoteurBilan(doublon);
		cheminsAExplorer.add(doublon);
	}

	private void continueExploration() {

		if (cheminsAExplorer.size() > 0) {
			cheminsAExplorer.remove(0).declenche(Edi.nos().getNoS(nomNosInitial));
		}
	}

	public SortedSet<String> crdu() {

		SortedSet<String> crdu = new TreeSet<String>();
		for (MoteurBilan c : cheminsExplores) {
			crdu.add(c.chaineResultat());


		}
		return crdu;
	}

	public void lance(NoS n) {
		// "*** <n> est le nos à partir ***"
		// "*** duquel on veut fabriquer un bilan ***"

		nomNosInitial = n.getNom();
		MoteurBilan c = new MoteurBilan(n, Contexte.Bilan);
		c.setBilan(this);
		Memoire mem = new Memoire();
		mem.setMoteurBilan(c);
		c.declencheAvec(mem);
	}

	public void lance(NoS n, NoS nosButoir) {
		// "*** <n> est le nos à partir ***"
		// "*** duquel on veut fabriquer un bilan ***"
		// "*** <nosButoir> est le nos butoir ***"

		nomNosInitial = n.getNom();
		nomNosFinal = nosButoir.getNom();
		Memoire mem = new Memoire();
		MoteurBilan c = new MoteurBilan(n, Contexte.Bilan);
		c.setBilan(this);
		if (n.equals(nosButoir)) {
			c.setBaseFaits(mem);
			this.ajouteCheminsExplores(c);
		} else {
			c.declencheAvec(mem);
		}
	}

	private void ajouteCheminsExplores(MoteurBilan c) {
		cheminsExplores.add(c);
	}

	public String resultat() {
		String res = "";
		for (String s : crdu()) {
			res = res + s + "\n";
		}
		return res;
	}

	public void termine(MoteurBilan c) {
		// "*** on stocke dans <cheminsExplores> uniquement si ***"
		// "*** on n'est pas en calcul de répercussion (phase 1) ***"

		if (nomNosFinal == null) { // nous ne sommes pas en phase 1
			cheminsExplores.add(c);

			if (!c.getValeursReperees().isEmpty()) {
				enrichitValeursReperees(c); 
				
				detecteExplorations(c);
			}
		}
		continueExploration();
	}

	private void enrichitValeursReperees(MoteurBilan c) {


		// prétraitement : si une seule valeur, pour une propriété alors envisager
		// l'antivaleur...
		// ex [{bb.ville=[[Genève, =]], aa.age=[[45, =]]}] => <>

		Map<String, List<List<String>>> valeursCible;
		listeCibles = new ArrayList<Map<String, List<List<String>>>>();
		
		for (Entry<String, List<List<String>>> e : c.getValeursReperees().entrySet()) {
			if (e.getValue().size() == 1) {
				// une seule valeur : on recherche "l'antivaleur"
				String antivaleur = "";
				List<String> l = e.getValue().get(0);
//				}
				antivaleur = operateurInverse(l.get(1));
				
				List<List<String>> valeurs = new ArrayList<List<String>>();
				valeurs.add(l);

				List<String> l2 = new ArrayList<String>();
				l2.add(l.get(0));
				l2.add(antivaleur);
				valeurs.add(l2);
				e.setValue(valeurs); 
				valeursCible = new HashMap<String, List<List<String>>>();
				valeursCible.put(e.getKey(), valeurs);
				listeCibles.add(valeursCible);
			}
		}
	}

	private void detecteExplorations(MoteurBilan c) {
		// Examine valeursReperees de <c> pour détecter
		// éventuellement une nouvelle exploration




		Map<String, List<List<String>>> valeursCible;
		listeCibles = new ArrayList<Map<String, List<List<String>>>>();

		for (Entry<String, List<List<String>>> e : c.getValeursReperees().entrySet()) {

			for (List<String> l : e.getValue()) {
				List<List<String>> col = new ArrayList<List<String>>();
				col.add(l);
				if (listeCibles.isEmpty()) {
					valeursCible = new HashMap<String, List<List<String>>>();
					valeursCible.put(e.getKey(), col);
					listeCibles.add(valeursCible);
				} else if (!listeCiblesContientProp(e.getKey())) {
					// enrichir ligne
					for (Map<String, List<List<String>>> v : listeCibles) {
						v.put(e.getKey(), col);
					}
				} else {
					// ajouter lignes
					ArrayList<Map<String, List<List<String>>>> liste2 = new ArrayList<Map<String, List<List<String>>>>();
					for (Map<String, List<List<String>>> v : listeCibles) {
						if (v.size() == 1) {
							// faute de mieux...
							liste2.clear();
						}
						valeursCible = new HashMap<String, List<List<String>>>();
						valeursCible.putAll(v);
						valeursCible.put(e.getKey(), col);
						liste2.add(valeursCible);
					}
					listeCibles.addAll(liste2);
				}
			}
		}


		for (Map<String, List<List<String>>> v : listeCibles) {
			if (valeurAExplorer(v)) {
				ajouteExplorationValeur(c, v);
			}
		}
	}

	private boolean valeurDejaExploree(Map<String, List<List<String>>> v) {
		for (MoteurBilan c : cheminsExplores) {
			if (c.getValeurs().equals(v)) {
				return true;
			}
		}
		return false;
	}

	private boolean valeurDejaAExplorer(Map<String, List<List<String>>> v) {
		for (MoteurBilan c : cheminsAExplorer) {
			if (c.getValeurs().equals(v)) {
				return true;
			}
		}
		return false;
	}

	private boolean valeurAExplorer(Map<String, List<List<String>>> v) {
		return ((!valeurDejaExploree(v)) && (!valeurDejaAExplorer(v)));
	}

	private boolean listeCiblesContientProp(String key) {
		for (Map<String, List<List<String>>> valeursCible : listeCibles) {
			if (valeursCible.containsKey(key)) {
				return true;
			}
		}
		return false;
	}

	public String getNomNosInitial() {
		return nomNosInitial;
	}

	public String getNomNosFinal() {
		return nomNosFinal;
	}

	public List<MoteurBilan> getMoteurBilans() {
		return cheminsExplores;
	}

	public void suspend(MoteurBilan c) {
		// "*** <nomNosFinal> est rempli ***"
		cheminsExplores.add(c);
		continueExploration();
	}

	public Bilan duplique() {
		Bilan clone = new Bilan();

		for (MoteurBilan elem : cheminsAExplorer) {
			MoteurBilan doublon = new MoteurBilan(null, null);
			doublon.setBilan(clone);
			doublon.setBaseFaits(elem.getBaseFaits().duplique());
			doublon.setNomActionCourante(elem.getNomActionCourante());
			clone.ajouteCheminsAExplorer(doublon);
		}

		for (MoteurBilan elem : cheminsExplores) {
			MoteurBilan doublon = new MoteurBilan(null, null);
			doublon.setBilan(clone);
			doublon.setBaseFaits(elem.getBaseFaits().duplique());
			doublon.setNomActionCourante(elem.getNomActionCourante());
			clone.ajouteCheminsExplores(doublon);
		}

		clone.setNosInitial(nomNosInitial);
		return clone;
	}

	private void setNosInitial(String n) {
		nomNosInitial = n;
	}

	private void ajouteCheminsAExplorer(MoteurBilan c) {
		cheminsAExplorer.add(c);
	}
	
	private String operateurInverse(String op) {

		if (op.equals(">")) {
			return "<=";
		} else if (op.equals(">=")) {
			return "<";
		} else if (op.equals("<")) {
			return ">=";
		} else if (op.equals("<=")) {
			return ">";
		} else if (op.equals("=")) {
			return "<>";
		} else if (op.equals("<>")) {
			return "=";
		}
		return "";
	}

	public Bilan poursuitAvec(NoS n) {

		nomNosFinal = null;
		cheminsAExplorer.addAll(cheminsExplores);
		cheminsExplores = new ArrayList<MoteurBilan>();
		for (MoteurBilan elem : cheminsAExplorer) {
			elem.init(n);
			if (!n.getActions().isEmpty()) {
				elem.setActionsAFaire(ToolNos.asStructureSyntaxe(n.getActions()));
			}
		}

		// On modifie temporairement <n> :
		if (!Edi.nos().getBaseCourante().contientNoS(n.getNom())) {
			Edi.nos().message("Erreur lors du calcul de réperussion : nos modifié hors de la base courante");
		} else {
			NoS nos = Edi.nos().getBaseCourante().getNoS(n.getNom());
			String actions = nos.getActions();
			String regles = nos.getRegles();
			nos.setActions(n.getActions());
			nos.setRegles(n.getRegles());

			continueExploration();

			nos.setActions(actions);
			nos.setRegles(regles);
		}

		return this;
	}
}
