/*
  EdiNoS - Editeur de NoS
  Copyright (C) 2013-2021 - Luc Poittevin — Tous droits réservés.
  Contact : edinos@free.fr  -  https://www.edinos.org
  
  Ce programme est un logiciel libre ; vous pouvez le redistribuer ou le
  modifier suivant les termes de la “GNU General Public License” telle que
  publiée par la Free Software Foundation : soit la version 3 de cette
  licence, soit (à votre gré) toute version ultérieure.
  
  Ce programme est distribué dans l’espoir qu’il vous sera utile, mais SANS
  AUCUNE GARANTIE : sans même la garantie implicite de COMMERCIALISABILITÉ
  ni d’ADÉQUATION À UN OBJECTIF PARTICULIER. Consultez la Licence Générale
  Publique GNU pour plus de détails.
  
  Vous devriez avoir reçu une copie de la Licence Générale Publique GNU avec
  ce programme ; si ce n’est pas le cas, consultez :
  <http://www.gnu.org/licenses/>. 
 */

package nos.run;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collections;
import java.util.List;
import java.util.StringTokenizer;

import javax.xml.bind.annotation.XmlElement;
import javax.xml.bind.annotation.XmlRootElement;
import javax.xml.bind.annotation.XmlTransient;

import nos.obj.Edi;
import nos.obj.NoS;
import nos.utils.*;

@XmlRootElement
public class Memoire implements Serializable {

	private static final long serialVersionUID = 1L;

	/**
	 * "*** <listeObjets> est la collections des objets connus ***" 
	 * "*** <parcours> est la liste des nos parcourus ***" 
	 * "*** <historique> est une collection de collections : ***" 
	 * "*** <nomProc> => (fait1 fait2 fait3) ***" 
	 * "*** <moteurBilan> est nécessaire pour le calcul de bilan ***"
	 **/

	private List<NodMem> listeObjets;

	private List<String> parcours;

	private List<List<String>> historique;

	private Histo historiqueFaits; // doit remplacer historique

	private String procCourante;

	private MoteurBilan moteurBilan;

	public Memoire() {
		listeObjets = new ArrayList<NodMem>();
		parcours = new ArrayList<String>();
		historique = new ArrayList<List<String>>();
		historiqueFaits = new Histo();
		historiqueFaits.setFaitsconnus(new ArrayList<FaitRenvoye>());
	}

	public void ajoute(String elem) {
		/* <elem> a le format : o[.p]* [valeur] */

		String[] tab = null, tabProps = null, tabFaits = null;
		String propriete;
		NodMem n = null, dernier = null;


		// On remplit <listeObjets> :
		tab = elem.split(" ");
		if (tab.length == 0) {
			return;
		}

		tabProps = tab[0].split("\\.");
		if (contientObjet(tabProps[0])) {
			// "maj de <nod> :"
			n = getObjet(tabProps[0]);
		} else {
			// "création de l'objet…"
			n = new NodMem(tabProps[0]);
			ajouteObjet(n);
		}


		int i = 1;
		while (tabProps.length > i) {
			// Balayage et création de la chaine de propriétés :


			n = n.creeNodMemSiNecessaire("si: " + tabProps[i] + " alorsAller: " + tabProps[i], tabProps[i]);

			i = i + 1;
		}

		if (tabProps.length == 1) {
			propriete = tabProps[0];
		} else {
			propriete = tabProps[i - 1];
		}

		if (tab.length > 1) {
			// il y a une valeur...
			if (Edi.nos().contientProprieteTypee(tab[0])) {
				// "*** La propriété était typée : ***"
				// "*** création d'un <NodMem> pour la valeur ***"
				String str = tab[1];
				dernier = new NodMem(str);
			} else {
				// "recherche du <nodMem> de la valeur pour branchement"
				tabFaits = tab[1].split("\\.");
				NodMem nodVal = getObjet(tabFaits[0]);
				if (nodVal != null) {
					MoteurTrouve moteur = new MoteurTrouve(nodVal);
					Memoire m = new Memoire();
					for (String s : tabFaits) {
						m.ajoute("", s);
					}
					moteur.declencheAvec(m);
					dernier = (NodMem) moteur.getNoS();
				}
			}
		} else {
			if (tabProps.length > 1) {
				// "cas : 'o.p' :"
				dernier = new NodMem(propriete);
			}
		}
		if (dernier != null) {
			if (tabProps.length > 1) {
				if (n.choixExisteDeja(propriete)) {
					n.remplaceChoix(propriete, "si: " + propriete + " alorsAller: " + dernier.getNom(), dernier);
				} else {
					n.ajouteChoixSiNecessaire("si: " + propriete + " alorsAller: " + dernier.getNom(), dernier);
				}
			} else {
				// " cas 'objet val' :"
				n.ajouteChoixSiNecessaire("aller: " + dernier.getNom(), dernier);
			}
		}
	}

	public void ajoute(String nomAction, String nomFait) {
		nomFait = ToolNos.traiteValeur(nomFait);

		if ((nomAction != null) && (nomAction.equals("afficher"))) {
			// on n'enlève pas les ""...
		} else {
			nomFait = ToolNos.retireDoublesQuotes(nomFait);
		}
		ajouteHisto(nomAction, nomFait);
		ajoute(nomFait);
	}

	public void ajouteFait(String nomFait) {
		ajoute(procCourante, nomFait);
	}

	// modif 17/03/2019 :
	private void ajouteHisto(String nomAction, String nomFait) {
		List<String> ent = new ArrayList<String>();

		ent.add(nomAction);
		ent.add(nomFait);
		historique.add(ent);

		FaitRenvoye f = new FaitRenvoye();
		f.setAction(nomAction);
		f.setFait(nomFait);
		historiqueFaits.ajoute(f);
	}

	private void ajouteObjet(NodMem n) {
		listeObjets.add(n);
	}

	public void ajouteParcours(String nom) {
		if (!parcours.contains(nom)) { 
			parcours.add(nom);
		}
	}

	public void ajouteProc(String nomActionCourante) {
		procCourante = nomActionCourante;
	}

	private boolean ajouteResultat(boolean nouveauRes, boolean ancienRes, String contexte) {

		if (contexte.length() == 0) {
			return nouveauRes;
		}
		if (contexte.equals("ET")) {
			return (ancienRes && nouveauRes);
		}
		if (contexte.equals("OU")) {
			return (ancienRes || nouveauRes);
		}
		return false;
	}

	private boolean compareOperateur(String res, String op, String res2) {
		Integer i = 0, i2 = 0;

		try {
			i = Integer.parseInt(res);
			i2 = Integer.parseInt(res2);
		} catch (NumberFormatException e) {
			// res et res2 ne peuvent être convertis en entier
			if (op.equals("=")) {
				return (res.equals(res2));
			}
			if (op.equals("<>")) {
				return (!res.equals(res2));
			}
			if (op.equals(">")) {
				return (res.compareTo(res2) > 0);
			}
			if (op.equals(">=")) {
				return (res.compareTo(res2) >= 0);
			}
			if (op.equals("<")) {
				return (res.compareTo(res2) < 0);
			}
			if (op.equals("<=")) {
				return (res.compareTo(res2) <= 0);
			}
		}
		if (op.equals("=")) {
			return (i == i2);
		}
		if (op.equals("<>")) {
			return (i != i2);
		}
		if (op.equals(">")) {
			return (i > i2);
		}
		if (op.equals(">=")) {
			return (i >= i2);
		}
		if (op.equals("<")) {
			return (i < i2);
		}
		if (op.equals("<=")) {
			return (i <= i2);
		}
		// TODO : Traiter le cas des réels
		return false;
	}

	public boolean contientDansParcours(String s) {
		return parcours.contains(s);
	}

	public boolean contientObjet(String s) {

		for (NodMem n : listeObjets) {
			if (n.getNom().equals(s)) {
				return true;
			}
		}
		return false;
	}

	public Memoire duplique() {
		Memoire copie = new Memoire();
		for (NodMem n : listeObjets) {
			copie.ajouteObjet(n);
		}
		for (String s : parcours) {
			copie.ajouteParcours(s);
		}
		for (List<String> e : historique) {
			copie.ajouteHisto(e.get(0), e.get(1));
		}
		return copie;
	}

	private int elemDans(List<String> l, int i) {
		// "******************************************************"
		// "*** renvoie la position de la parenthèse fermante. ***"
		// "*** <l> est une collection et <i> la ***"
		// "*** position de la parenthèse ouvrante. ***"
		// "*** renvoie <0> en cas d'échec. ***"
		// "******************************************************"
		
		int index = i + 1;
		int nbPar = 1;
		if ((String) l.get(index) == ")") {
			// element vide
			return 0;
		}
		while (index < l.size()) {
			if (((String) l.get(index)).equals(")")) {
				if (nbPar == 1) {
					return index;
				} else {
					nbPar = nbPar + 1;
				}
			}
			if (((String) l.get(index)).equals(")")) {
				nbPar = nbPar + 1;
			}
			index = index + 1;
		} // end while
		return 0;
	}

	private boolean evalueElem(List<String> l) throws NoSException {
		// "******************************************************"
		// "*** La methode renvoie les messages suivants : ***"
		// "*** null: si une erreur a eu lieu lors de ***"
		// "*** l'evaluation de <l>. ***"
		// "*** true: si <l> est VRAIE. ***"
		// "*** false: si <l> est FAUSSE. ***"
		// "******************************************************"

		String element;
		boolean resultat = false;
		String boucle = "";
		ArrayList<Object> listeMotsCles = Collections.list(new StringTokenizer("OU ET NON ) ("));

		int compteur = 0;
		element = (String) l.get(compteur);

		if (listeMotsCles.contains(element)) {
			if (element.equals("(")) {
				int fin = elemDans(l, compteur);
				resultat = ajouteResultat(evalueElem(l.subList(compteur + 1, fin)), resultat, boucle);
				compteur = fin;
			}
			if (element.equals("NON")) {
				compteur = compteur + 1;
				if (((String) l.get(compteur)).equals("(")) {
					int fin = elemDans(l, compteur);
					resultat = ajouteResultat(!evalueElem(l.subList(compteur + 1, fin)), resultat, boucle);
					compteur = fin;
				} else {
					throw new NoSException();
				}
			}
		} else {
			boolean resTemp = evalueProposition(element);
			resultat = ajouteResultat(resTemp, resultat, boucle);
		}

		while (compteur < l.size() - 1) {
			compteur = compteur + 1;
			element = (String) l.get(compteur);
			if ((!element.equals("OU")) && (!element.equals("ET"))) {
				throw new NoSException();
			}
			if ((element.equals("OU")) && (boucle.length() == 0)) {
				boucle = "OU";
			}
			if ((element.equals("ET")) && (boucle.length() == 0)) {
				boucle = "ET";
			}
			if ((element.equals("OU")) && (boucle.equals("ET"))) {
				throw new NoSException();
			}
			if ((element.equals("ET")) && (boucle.equals("OU"))) {
				throw new NoSException();
			}

			compteur = compteur + 1;
			element = (String) l.get(compteur);

			if (listeMotsCles.contains(element)) {
				if (element.equals("(")) {
					int fin = elemDans(l, compteur);
					resultat = ajouteResultat(evalueElem(l.subList(compteur + 1, fin)), resultat, boucle);
					compteur = fin;
				}
				if (element.equals("NON")) {
					compteur = compteur + 1;
					if (((String) l.get(compteur)).equals("(")) {
						int fin = elemDans(l, compteur);
						resultat = ajouteResultat(!evalueElem(l.subList(compteur + 1, fin)), resultat, boucle);
						compteur = fin;
					} else {
						throw new NoSException();
					}
				}
			} else {
				boolean resTemp;
				try {
					resTemp = evalueProposition(element);
				} catch (NoSException e) {
					throw e;
				}
				resultat = ajouteResultat(resTemp, resultat, boucle);
			}
		}
		return resultat;
	}

	public boolean evalueFormule(String f) throws NoSException {

		List<String> l = ToolNos.asArrayOfFormules(f);

		boolean resultat = evalueElem(ToolNos.regroupeProposition(l));
		return resultat;
	}

	private boolean evalueProposition(String s) throws NoSException {
		// "*****************************"
		// "*** Renvoie VRAI ou FAUX ***"
		// "*****************************"

		String[] tabChaine, tabFaits;
		String res, res2, op = "";

		tabChaine = s.split(" ");

		if (tabChaine.length == 2) {
			if (!tabChaine[1].equals("inconnu")) {
				throw new NoSException();
			} else {
				tabFaits = tabChaine[0].split("\\.");
				if (!contientObjet(tabFaits[0])) {
					return true;
				}
			}
		}

		if (tabChaine.length == 3) {
			// Opérateur :
			op = tabChaine[1];
			ArrayList<Object> listeOp = Collections.list(new StringTokenizer("= > < >= <= <>"));
			if (!listeOp.contains(op)) {
				throw new NoSException();
			}
		}

		if (tabChaine.length > 3) {
			throw new NoSException();
		}

		tabFaits = tabChaine[0].split("\\.");

		if (!contientObjet(tabFaits[0])) {
			if ((op.equals("=")) && (tabChaine[2].equals("\"\""))) {
				return true;
			} else {
				return false;
			}
		}

		res = evalueFait(tabFaits);

		if (tabChaine.length == 2) { // cas opérateur "inconnu"
			if (Arrays.asList(tabFaits).contains(res)) {
				// si la valeur de retour correspond à un des nos
				// c'est que la valeur est vide (inconnue)
				return true;
			} else {
				return false;
			}
		}

		if (tabChaine.length == 1) {
			return (res.equals(tabFaits[tabFaits.length - 1]));
		} else {
			tabFaits = tabChaine[2].split("\\.");
			if (tabFaits.length == 1) {
				res2 = tabFaits[0];
			} else {
				if (!contientObjet(tabFaits[0])) {
					return false;
				}

				res2 = evalueFait(tabFaits);
			}

			if ((op.equals("=")) && (res2.equals("\"\""))) {
				tabFaits = tabChaine[0].split("\\.");
				if (Arrays.asList(tabFaits).contains(res)) {
					// si la valeur de retour correspond à un des nos
					// c'est que la valeur est vide
					// (pour une propriété typée uniquement)
					if (Edi.nos().contientProprieteTypee(tabChaine[0])) {
						return true;
					}
				}
			}

			// Si exploration, il faut prévenir <moteurBilan>
			if (moteurBilan != null) {
				return moteurBilan.detetectePropriete(tabChaine[0], res2, op);
			}

			return (compareOperateur(res, op, res2));
		}
	}

	public String evalueFait(String[] tabFaits) {


		MoteurTrouve mt = new MoteurTrouve(getObjet(tabFaits[0]));
		Memoire m = new Memoire();
		for (String st : tabFaits) {
			m.ajoute("", st);
		}
		mt.declencheAvec(m);

		String retour = mt.retour();
		return retour;
	}

	public boolean actionDejaExecutee(String nomAction) {

		if (nomAction == null) {
			return false;
		}
		for (FaitRenvoye f : historiqueFaits.getFaitsconnus()) {
			if (f != null) {
				if (nomAction.equals(f.getAction())) {
					return true;
				}
			}
		}
		return false;
	}

	public boolean faitDejaRenvoye(String nomAction, String nomFait) {

		if ((nomAction == null) || (nomFait == null)) {
			return false;
		}
		for (FaitRenvoye f : historiqueFaits.getFaitsconnus()) {
			if (f != null) {
				if ((nomAction.equals(f.getAction()) && (nomFait.equals(f.getFait())))) {
					return true;
				}
			}
		}
		return false;
	}

	public List<String> getFaits() {
		// "*** renvoie la liste des faits de la MT ***"
		// "*** (on ignore les procs 'afficher:'...) ***"


		List<String> l = new ArrayList<String>();

		for (FaitRenvoye f : historiqueFaits.getFaitsconnus()) {
			if (f.getAction() != null) {
				if (!f.getAction().equals("afficher")) {
					String fait = f.getFait();
					fait = fait.replaceAll("_", " ");
					l.add(fait);
				}
			}
		}

		return l;
	}

	public List<String> getListeHistorique() {
		// "*** renvoie une représentation de l'historique ***"
		// "*** pour l'affichage de la MT ***"


		List<String> l = new ArrayList<String>();

		for (FaitRenvoye f : historiqueFaits.getFaitsconnus()) {
			if (f.getAction() != null) {
				if (!f.getAction().equals("afficher")) {
					String fait = f.getFait();
					fait = fait.replaceAll("_", " ");
					l.add(f.getAction() + " -> " + fait);
				}
			}
		}
		return l;
	}

	private NodMem getObjet(String ch) {
		for (NodMem n : listeObjets) {
			if (n.getNom().equals(ch)) {
				return n;
			}
		}
		return null;
	}

	@XmlElement
	public List<String> getParcours() {
		return parcours;
	}

	@XmlElement
	public String getHistorique() { // pour JAXB
		return ligneHistorique();
	}

	@XmlElement
	public List<NodMem> getListeObjets() { // pour JAXB
		return listeObjets;
	}

	public Histo getHistoriqueFaits() {
		return historiqueFaits;
	}

	public void setHistorique(String histo) {
		// pour import
		// appelé par JAXB

		String[] tabChaine;
		String fait, action;

		tabChaine = histo.split(",");
		for (String e : tabChaine) {
			action = e.substring(0, e.indexOf(" -> ")).trim();
			fait = e.substring(e.indexOf(" -> ") + 4, e.length());
			if (!action.equals("null")) {
				ajouteHisto(action, fait);
			}
		}
	}
	
	public String ligneHistorique() {
		// "*** renvoie une chaine représentant l'historique du ***"
		// "*** remplissage de la MT. Utilisé notamment pour ***"
		// "*** écrire dans l'historique de la révision. ***"

		// 04/12/19 : refonte pour remplacement de () par ->
		// et bascule sur historiqueFaits

		// 06/03/21 : ajouter: monFait devient -> monFait
		// afficher: monMessage devient monMessage
		// afficher: "monTexte" devient "monTexte"
		
		String string = "";

		for (FaitRenvoye f : historiqueFaits.getFaitsconnus()) {
			if (f.getAction() != null) {
				String fait = f.getFait();
				String action = f.getAction();
				fait = fait.replaceAll("_", " ");
				if (!string.isEmpty()) {
					string = string + ", ";
				}
				if (action.equals("afficher")) {

				} else if ((Edi.nos().contientMessage(action))) {
					// action est le nom d'un message
					string = string + action;
				} else if (action.equals("ajouter")) {
					string = string + " -> ";
				} else {
					string = string + action + " -> ";
				}			
				string = string + fait;
			}
		}

		return string;
	}

	public String ligneHistorique(MoteurBilan cycle) {
		// "*** renvoie une chaine représentant l'historique dit ***"
		// "*** 'abstrait' de la MT (càd qu'on utilise la v.i. ***"
		// "*** <valeurs> de <unCycle>). Utilisé pour les bilans ***"
		// "*** et les répercussions. ***"

		// 04/12/19 : refonte pour remplacement de () par ->
		// et bascule sur historiqueFaits

		// 06/03/21 : ajouter: monFait devient -> monFait
		// afficher: monMessage devient monMessage
		// afficher: "monTexte" devient "monTexte"

		String string = "";

		for (FaitRenvoye f : historiqueFaits.getFaitsconnus()) {
			if (f.getAction() != null) {
				String fait = f.getFait();
				String action = f.getAction();
				fait = fait.replaceAll("_", " ");
				if (!string.isEmpty()) {
					string = string + ", ";
				}
				if (action.equals("afficher")) {
					
				} else if ((Edi.nos().contientMessage(action))) {
					// action est le nom d'un message
					string = string + action;
				} else if (action.equals("ajouter")) {
					string = string + " -> ";
				} else {
					string = string + action + " -> ";
				}
				
				String[] tab = fait.split(" ");

				if (tab.length > 0) {
					if (cycle.getValeurs().containsKey(tab[0])) {
						for (List<String> l : cycle.getValeurs().get(tab[0])) {
							// <l> est une collection (valeur, op)"

							string = string + tab[0] + " " + l.get(1) + " " + l.get(0);
						}
					} else {
						// // propriété non typée
						string = string + fait;
					}
				}
			}
		}

		return string;
	}

	public String ligneParcours() {
		// "*** renvoie une chaine représentant le parcours ***"
		String s = "";
		for (String nomNos : parcours) {
			s = s + nomNos + ", ";
		}
		if (!s.isEmpty()) {
			s = s.substring(0, s.length() - 2) + ".";
		}
		return s;
	}

	
	public String valeur(String ch) {
		String[] tabFaits;

		tabFaits = ch.split("\\.");
		NodMem n = getObjet(tabFaits[0]);
		if (n == null) {
			return "";
		}

		MoteurTrouve moteur = new MoteurTrouve(n);
		Memoire m = new Memoire();
		for (String s : tabFaits) {
			m.ajoute("", s);
		}

		moteur.declencheAvec(m);
		return moteur.retour();
	}


	public void affiche() {
		for (NodMem n : listeObjets) {
			System.out.println("Objet: " + n.getNom());
			for (NoS n2 : n.tousLesNodMemSuivants()) {
				System.out.println("NodMem: " + n2.getNom() + " règles: " + n2.getRegles());
			}
		}
	}

	@XmlTransient
	public MoteurBilan getMoteurBilan() {
		return moteurBilan;
	}

	public void setMoteurBilan(MoteurBilan m) {
		this.moteurBilan = m;
	}
}
