/*
  EdiNoS - Editeur de NoS
  Copyright (C) 2013-2021 - Luc Poittevin — Tous droits réservés.
  Contact : edinos@free.fr  -  https://www.edinos.org
  
  Ce programme est un logiciel libre ; vous pouvez le redistribuer ou le
  modifier suivant les termes de la “GNU General Public License” telle que
  publiée par la Free Software Foundation : soit la version 3 de cette
  licence, soit (à votre gré) toute version ultérieure.
  
  Ce programme est distribué dans l’espoir qu’il vous sera utile, mais SANS
  AUCUNE GARANTIE : sans même la garantie implicite de COMMERCIALISABILITÉ
  ni d’ADÉQUATION À UN OBJECTIF PARTICULIER. Consultez la Licence Générale
  Publique GNU pour plus de détails.
  
  Vous devriez avoir reçu une copie de la Licence Générale Publique GNU avec
  ce programme ; si ce n’est pas le cas, consultez :
  <http://www.gnu.org/licenses/>. 
 */

package nos.run;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import nos.obj.*;

public class MoteurBilan extends Moteur {
	// anciennement Cycle

	private static final long serialVersionUID = 1L;

	// "*** Un <MoteurBilan> est un <Moteur> en mode 'exploration' : ***"
	// "*** utilisé par Bilan>>lance, pour calculer des ***"
	// "*** bilans ou des répercussions. ***"
	//
	// "*** v.i. utilisées pour la gestion des propriétés typées : ***"

	// "<valeurs> est le dictionnaire des valeurs courantes :
	// clé = nom de la proposition instanciée.
	// valeur = collection de collections (uneValeur, unOpérateur)"

	// "<valeursReperees> est le dictionnaire des valeurs 'repérées':
	// clé = nom de la proposition instanciée.
	// valeur = collection de collections (uneValeur, unOpérateur)"

	// <proprieteReperee> est un String

	// <valeurReperee> est une collection (uneValeur, unOpérateur)

	
	private Bilan bilan;

	private Map<String, List<List<String>>> valeurs;

	private Map<String, List<List<String>>> valeursReperees; 

	private String proprieteReperee; 

	private List<String> valeurReperee; 

	public MoteurBilan(NoS n, Contexte c) {
		super(n, c, null);
		valeurs = new HashMap<String, List<List<String>>>();
		valeursReperees = new HashMap<String, List<List<String>>>();
		proprieteReperee = null;
	}

	public Map<String, List<List<String>>> getValeurs() {
		return valeurs;
	}

	public void setValeurs(Map<String, List<List<String>>> valeurs) {
		this.valeurs = valeurs;
	}

	public Map<String, List<List<String>>> getValeursReperees() {
		return valeursReperees;
	}

	public void setValeursReperees(Map<String, List<List<String>>> v) {
		this.valeursReperees = v;
	}

	protected void afficher(String nomMess) {
		Message m = null;
		m = Edi.nos().getMessage(nomMess);
		if (nomMess != null) {
			nomActionCourante = nomMess;
			if (!baseFaits.actionDejaExecutee(nomMess)) {
				enrichitBilan(m, m.formateFaitsAttendus());
			}
			continueActions();
		}
	}

	protected void afficherTexte(String ch) {
		baseFaits.ajoute("afficher", ch);
		continueActions();
	}

	protected void loguer(String ch) {
		baseFaits.ajoute("loguer", ch);
		continueActions();
	}

	protected void ajouter(String chaine) {

		if (chaine != null) {
			if (!baseFaits.faitDejaRenvoye("ajouter", chaine)) {
				baseFaits.ajoute("ajouter", chaine);
			}

			continueActions();
		}
	}

	public String chaineResultat() {
		// "*** renvoie une chaine présentant l'historique ***"
		// "*** la chaine est cloturée par un point ***"

		String s = baseFaits.ligneHistorique(this);
		// On ajoute un point en fin de chaine :
		if (s.length() > 0) {
			s = s + ".";
		}
		return s;
	}


	protected void declenche(NoS n) {

		// * "*** appelé par <suiteDeclenche> quand BILAN, CONTROLE ou TEST ***"
		// * "*** (sinon c'est <declencheAvec> qui est appelé) ***"

		if (proprieteReperee != null) {
			// on reste dans le même nos...
			ajouteValeurReperee(proprieteReperee, valeurReperee);
			relancheChoix();
		} else {
			// si <valeursConnue> rempli mais <valeurs> n'est pas à niveau,
			// il faut remplir <valeurs> et reboucler...

			if ((!valeursReperees.isEmpty()) && (valeurs.size() != valeursReperees.size())) {
				// on reste dans le même nos...
				detecteValeur();
			} else {
				declencheVraiment(n);
			}
		}
	}

	private void declencheVraiment(NoS n) {
		// "*** cas habituel ... ***"


		baseFaits.ajouteParcours(n.getNom());

		nos = n;
		assocChoix = new HashMap<NoS, Integer>();
		actionsAFaire = new ArrayList<List<String>>();

		if ((bilan != null) && (bilan.getNomNosFinal() != null)) {
			if (bilan.getNomNosFinal().equals(n.getNom())) {
				bilan.suspend(this);
				return;
			}
		}
		executeActions();
	}

	protected void declencher(String nomNos) {
		NoS n = null;
		n = Edi.nos().getNoS(nomNos);
		if (n != null) {
			Moteur m = new MoteurBilan(nos, contexte);
			m.setAncetre(getAncetre());
			setAncetre(m);
			declenche(n);
		}
	}

	protected void demander(String nomQuest) {
		Question ques = null;
		ques = Edi.nos().getQuestion(nomQuest);
		if (ques != null) {
			nomActionCourante = nomQuest;
			if (!baseFaits.actionDejaExecutee(nomQuest)) {
				enrichitBilan(ques, ques.formateFaitsAttendus());
			}
			continueActions();
		}
	}

	private void enrichitBilan(Action act, List<List<String>> structureFaits) {
		// "*** enrichit <bilan> avec la proc <p>. ***"
		// "*** <structureFaits> est la collection ***"
		// "*** de collection des faits de <p> ***"


		List<List<String>> collecFaits = new ArrayList<List<String>>();
		List<String> faitsCourant = new ArrayList<String>();

		for (List<String> faits : structureFaits) {
			collecFaits = new ArrayList<List<String>>();
			faitsCourant = new ArrayList<String>();

			for (String s : faits) {
				List<String> newCol = new ArrayList<String>();
				newCol.add(s);
				collecFaits.add(newCol);
			}

			if (collecFaits.size() > 0) {
				faitsCourant = collecFaits.remove(0);
				for (List<String> colFaits : collecFaits) {
					bilan.ajouteBilan(this, act.getNom(), colFaits);
				}
			} else { // cas où l'action ne renvoie aucun fait
				faitsCourant.add("");
			}
			for (String nomFait : faitsCourant) {
				baseFaits.ajoute(act.getNom(), nomFait);
			}
		}

		if (structureFaits.size() == 0) {
			baseFaits.ajoute(act.getNom(), "");
		}
	}


	protected void evaluer(String nomProc) {
		Procedure proc = null;
		proc = Edi.nos().getProc(nomProc);
		if (proc != null) {
			nomActionCourante = nomProc;
			if (!baseFaits.actionDejaExecutee(nomProc)) {
				enrichitBilan(proc, proc.formateFaitsAttendus());
			}
			continueActions();
		}
	}

	protected void evaluer(String nomProc, String par) {
		this.evaluer(nomProc);
	}

	protected void finit() {

		if (proprieteReperee != null) {
			// on reste dans le même nos...
			ajouteValeurReperee(proprieteReperee, valeurReperee);
			relancheChoix();
		} else {
			// si <valeursReperees> rempli mais <valeurs> n'est pas à niveau,
			// il faut remplir <valeurs> et reboucler...

			if ((!valeursReperees.isEmpty()) && (valeurs.size() != valeursReperees.size())) {
				// on reste dans le même nos...
				detecteValeur();
			} else {
				// cas normal
				bilan.termine(this);
			}
		}
	}

	private void detecteValeur() {
		for (String prop : valeursReperees.keySet()) {
			if (!valeurs.containsKey(prop)) {
				List<List<String>> coll = valeursReperees.get(prop);

				ajouteValeur(prop, coll.get(0));
				relancheChoix();
				return;
			}
		}
	}


	public boolean detetectePropriete(String p, String val, String op) {
		// S'il y a déjà une propriété repérée, renvoie false
		// Sinon, repère une propriété et sa valeur


		// ajout 21/06/2019 :
		if (proprieteReperee != null) {
			return false;
		}

		List<String> coll = new ArrayList<String>();
		coll.add(val);
		coll.add(op);

		if (!valeursReperees.containsKey(p)) {
			proprieteReperee = p;
			valeurReperee = coll;
			return false;
		}

		if (!valeursReperees.get(p).contains(coll)) {
			proprieteReperee = p;
			valeurReperee = coll;
			return false;
		}

		if (!valeurs.containsKey(p)) {
			return false;
		}



		return (valeurs.get(p).contains(coll));

	}

	public Bilan getBilan() {
		return bilan;
	}

	public void setBilan(Bilan bilan) {
		this.bilan = bilan;
	}


	public void relancheChoix() {

		proprieteReperee = null;
		assocChoix = new HashMap<NoS, Integer>();
		suiteDeclenche();
	}

	protected void retourDeclenche() {

		// 06/01/2021 : // LPO
		nos = getAncetre().getNoS();
		ancetre = getAncetre().getAncetre();
		assocChoix = new HashMap<NoS, Integer>();
		actionsAFaire = new ArrayList<List<String>>();

		// TODO : il faudra reprendre les actionsAFaire de l'ancetre...
		continueActions();
	}

	public void ajouteValeurReperee(String v, List<String> col) {

		// <v> est le nom d'une propriété,
		// <col> est une collection (valeur, opérateur)) ***"

		if (valeursReperees.containsKey(v)) {
			List<List<String>> c = valeursReperees.get(v);
			if (!c.contains(col)) {
				c.add(col);
			}
			valeursReperees.replace(v, c);
		} else {
			List<List<String>> c = new ArrayList<List<String>>();
			c.add(col);
			valeursReperees.put(v, c);
		}
	}

	public void ajouteValeur(String v, List<String> col) {

		// <v> est le nom d'une propriété,
		// <col> est une collection (valeur, opérateur)) ***"

		List<List<String>> c = new ArrayList<List<String>>();
		c.add(col);

		if (col.get(1).equals("=")) {
			// "*** Si l'opérateur dans <col> est '=', on écrase dans <valeurs> ***"
			// "*** la collection pour la propriété correspondante : ***"
			valeurs.put(v, c);
		} else {
			if (valeurs.containsKey(v)) {
				Boolean remplacer = false;
				Boolean oublier = false;
				for (List<String> l : valeurs.get(v)) {
					if (Integer.valueOf(l.get(0)) < Integer.valueOf(col.get(0))) {
						if (l.get(1).equals(col.get(1))) {
							// même opérateur
							if ((l.get(1).equals("<")) || (l.get(1).equals("<="))) {
								oublier = true;
							}
							if (l.get(1).equals(">")) {
								remplacer = true;
							}
						}
						if ((l.get(1).equals("<>")) && ((col.get(1).equals(">")) || (col.get(1).equals(">=")))) {
							remplacer = true;
						}
					}
					if (Integer.valueOf(l.get(0)) > Integer.valueOf(col.get(0))) {
						if (l.get(1).equals(col.get(1))) {
							// même opérateur
							if ((col.get(1).equals("<")) || (col.get(1).equals("<="))) {
								remplacer = true;
							}
							if (col.get(1).equals(">")) {
								oublier = true;
							}
						}
						if ((col.get(1).equals("<>")) && ((l.get(1).equals(">")) || (l.get(1).equals(">=")))) {
							oublier = true;
						}
					}
				}
				if (remplacer) {
				}
				if ((!oublier) && (!remplacer)) {
					// "*** cas d'un intervalle : '>' puis '<' ***"
					// "*** ou inversement ***"
				}

				valeurs.replace(v, c);
			} else {
				// " <valeurs> est vide pour cette propriété : sans se ***"
				// "*** soucier de l'opérateur, on l'ajoute dans <valeurs ***"
				valeurs.put(v, c);
			}
		}
	}
}
