/*
  EdiNoS - Editeur de NoS
  Copyright (C) 2013-2020 - Luc Poittevin — Tous droits réservés.
  Contact : edinos@free.fr  -  https://www.edinos.org
  
  Ce programme est un logiciel libre ; vous pouvez le redistribuer ou le
  modifier suivant les termes de la “GNU General Public License” telle que
  publiée par la Free Software Foundation : soit la version 3 de cette
  licence, soit (à votre gré) toute version ultérieure.
  
  Ce programme est distribué dans l’espoir qu’il vous sera utile, mais SANS
  AUCUNE GARANTIE : sans même la garantie implicite de COMMERCIALISABILITÉ
  ni d’ADÉQUATION À UN OBJECTIF PARTICULIER. Consultez la Licence Générale
  Publique GNU pour plus de détails.
  
  Vous devriez avoir reçu une copie de la Licence Générale Publique GNU avec
  ce programme ; si ce n’est pas le cas, consultez :
  <http://www.gnu.org/licenses/>. 
*/

package nos.run;

import nos.obj.Edi;
import nos.obj.NoS;

public class MoteurTrouve extends Moteur {
	
	private static final long serialVersionUID = 1L;

	/**
	// Moteur mode 'trouver' : utilisé par Memoire>>evalueProposition et
	// Memoire>>valeur.
	// Fonctionnement : la BF est spécifiée au départ : pas d'action exécutée,
	// renvoie le nom du nos final (méthode <retour>).
	**/

	public MoteurTrouve(NodMem n) {
		super(n, null, null);
	}

	protected void aller(String nomNos, String p) {

		if (p.length() == 0) {
			p = "0";
		}

		NodMem n = ((NodMem) nos).nodSuivantDeNom(nomNos);
		Integer i = Integer.valueOf(p);
		assocChoix.put(n, i);
	}

	public void declencheAvec(Memoire mem) {
		baseFaits = mem;
		baseFaits.ajouteParcours(nos.getNom());
		this.continueActions();
	}

	public String retour() {
		return getNoS().getNom();
	}

	protected void siAlorsAller(String f, String nomNos, String p) {

		if (p.length() == 0) {
			p = "0";
		}
		
		boolean b = false;
		try {
			b = baseFaits.evalueFormule(f);
		} catch (NoSException e) {
			e.printStackTrace();
			Edi.nos().message("Anomalie dans l'évaluation de la formule.");
		}
		if (b) {
			NodMem n = ((NodMem) nos).nodSuivantDeNom(nomNos);
			Integer i = Integer.valueOf(p); 
			assocChoix.put(n, i);
		}
	}

	protected void suiteDeclenche() {

		if (nos.getRegles().length() > 0) {
			this.executeChoix(nos.getRegles());
		}
		
		NoS suivant = choixSuivant();

		// décision finale :
		if (suivant != null) {
			this.init(suivant);
			this.declencheAvec(baseFaits);
		} else { // pas de suivant
			this.finit();
		}
	}

	protected void finit() {
		// on ne fait rien...
	}
}
