/*
  EdiNoS - Editeur de NoS
  Copyright (C) 2013-2020 - Luc Poittevin — Tous droits réservés.
  Contact : edinos@free.fr  -  https://www.edinos.org
  
  Ce programme est un logiciel libre ; vous pouvez le redistribuer ou le
  modifier suivant les termes de la “GNU General Public License” telle que
  publiée par la Free Software Foundation : soit la version 3 de cette
  licence, soit (à votre gré) toute version ultérieure.
  
  Ce programme est distribué dans l’espoir qu’il vous sera utile, mais SANS
  AUCUNE GARANTIE : sans même la garantie implicite de COMMERCIALISABILITÉ
  ni d’ADÉQUATION À UN OBJECTIF PARTICULIER. Consultez la Licence Générale
  Publique GNU pour plus de détails.
  
  Vous devriez avoir reçu une copie de la Licence Générale Publique GNU avec
  ce programme ; si ce n’est pas le cas, consultez :
  <http://www.gnu.org/licenses/>. 
 */

package nos.run;

import nos.obj.Edi;
import nos.obj.Message;
import nos.obj.NoS;
import nos.obj.Procedure;
import nos.obj.Question;
import nos.utils.ToolNos;

public class MoteurVerifie extends Moteur {

	private static final long serialVersionUID = 1L;

	private ErreursVerif erreursVerif = new ErreursVerif();

	public MoteurVerifie(NoS n) {
		super(n, null, null);
	}

	public MoteurVerifie(NoS n, Contexte c) {
		super(n, c, null);
	}

	protected void afficher(String s) {

		Message m = null;
		m = Edi.nos().getMessage(s);
		if (m == null) {
			erreursVerif.ajouteData(nos.getNom(), "le message <" + s + "> n'est pas répertoriée...");
		}
	}

	protected void afficherTexte(String s) {
		// Surtout ne rien faire...
	}

	protected void ajouteErreur(String s) {
		erreursVerif.ajouteData(nos.getNom(), s);
	}

	protected void aller(String nomNos, String p) {

		if ((nomNos == null) || (nomNos.isEmpty())) {
			objetIntrouvable("nos");
			return;
		}

		if (p.length() == 0) {
			p = "0";
		}

		NoS n = null;
		n = Edi.nos().getNoS(nomNos);
		if (n == null) {
			erreursVerif.ajouteData(nos.getNom(), "le nos <" + nomNos + "> est inconnu...");
		} else {
			Integer i = Integer.valueOf(p); 
			if (assocChoix.containsKey(n)) {
				if (assocChoix.get(n).compareTo(i) > 0) {
					assocChoix.put(n, i);
				}
			} else {
				assocChoix.put(n, i);
			}
		}
	}

	protected void declencher(String s) {
		NoS n = null;
		n = Edi.nos().getNoS(s);
		if (n == null) {
			erreursVerif.ajouteData(nos.getNom(), "le nos <" + s + "> est inconnu...");
		}
	}

	protected void demander(String s) {
		Question q = null;
		q = Edi.nos().getQuestion(s);
		if (q == null) {
			erreursVerif.ajouteData(nos.getNom(), "la question <" + s + "> n'est pas répertoriée...");
		}
	}

	protected void evaluer(String chaine) {
		Procedure p = null;
		p = Edi.nos().getProc(chaine);
		if (p == null) {
			erreursVerif.ajouteData(nos.getNom(), "la procédure <" + chaine + "> n'est pas répertoriée...");
		}
	}

	protected void evaluer(String chaine, String par) {
		if ((chaine == null) || (chaine.isEmpty())) {
			objetIntrouvable("procédure");
			return;
		}
		Procedure p = null;
		p = Edi.nos().getProc(chaine);
		if (p == null) {
			erreursVerif.ajouteData(nos.getNom(), "la procédure <" + chaine + "> n'est pas répertoriée...");
		}
	}

	protected void objetIntrouvable(String chaine) {
		erreursVerif.ajouteData(nos.getNom(), chaine + " introuvable, vérifiez la syntaxe...");
	}

	protected void executeActionSuivante() {
		super.executeActionSuivante();
		continueActions();
	}

	protected void siAlorsAller(String f, String nomNos, String p) {

		if ((nomNos == null) || (nomNos.isEmpty())) {
			objetIntrouvable("nos");
			return;
		}

		if (p.length() == 0) {
			p = "0";
		}
		NoS n = null;
		n = Edi.nos().getNoS(nomNos);
		if (n == null) {
			erreursVerif.ajouteData(nos.getNom(), "le nos <" + nomNos + "> est inconnu...");
		} else {
			Integer i = Integer.valueOf(p); 
			if (assocChoix.containsKey(n)) {
				if (assocChoix.get(n).compareTo(i) > 0) {
					assocChoix.put(n, i);
				}
			} else {
				assocChoix.put(n, i);
			}
		}
	}

	protected void suiteDeclenche() {

		String regles = ToolNos.retireCommentaires(nos.getRegles());
		if (regles.length() > 0) {
			this.executeChoix(regles);
		}

		if (contexte == Contexte.ControleNoS) {
			// on s'arrête là (controle du nos courant uniquement
		} else { // on contrôle tous les nos du graphe
			for (NoS n : assocChoix.keySet()) {
				MoteurVerifie m = new MoteurVerifie(n);
				m.setErreursVerif(erreursVerif); 
				m.declencheAvec(baseFaits);
			}
		}
	}

	public ErreursVerif getErreursVerif() {
		return erreursVerif;
	}

	public void setErreursVerif(ErreursVerif e) {
		erreursVerif = e;
	}
}
