/*
  EdiNoS - Editeur de NoS
  Copyright (C) 2013-2019 - Luc Poittevin — Tous droits réservés.
  Contact : edinos@free.fr  -  https://www.edinos.org
  
  Ce programme est un logiciel libre ; vous pouvez le redistribuer ou le
  modifier suivant les termes de la “GNU General Public License” telle que
  publiée par la Free Software Foundation : soit la version 3 de cette
  licence, soit (à votre gré) toute version ultérieure.
  
  Ce programme est distribué dans l’espoir qu’il vous sera utile, mais SANS
  AUCUNE GARANTIE : sans même la garantie implicite de COMMERCIALISABILITÉ
  ni d’ADÉQUATION À UN OBJECTIF PARTICULIER. Consultez la Licence Générale
  Publique GNU pour plus de détails.
  
  Vous devriez avoir reçu une copie de la Licence Générale Publique GNU avec
  ce programme ; si ce n’est pas le cas, consultez :
  <http://www.gnu.org/licenses/>. 
 */

package nos.run;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.Set;

import org.apache.commons.lang.StringUtils;

import nos.obj.NoS;

public class NodMem extends NoS {

	private static final long serialVersionUID = 1L;

	// Les objets <NodMem> sont créés et utilisés par <Memoire> :
	// ce sont des nodules qui 'connaissent' leurs successeurs éventuels,
	// grâce à <assocNosSuivants>...

	private Map<String, NodMem> assocNosSuivants;

	public NodMem(String s) {
		super(s);
		assocNosSuivants = new HashMap<String, NodMem>();
	}

	public NodMem() {// Pour JAXB
		assocNosSuivants = new HashMap<String, NodMem>();
	}

	private void ajouteBaseNodsSuivants(NodMem nod) {
		assocNosSuivants.put(nod.getNom(), nod);
	}

	public boolean choixExisteDeja(String propriete) {
		String chaine = "si: " + propriete;
		if (getRegles().indexOf(chaine) == -1) {
			return false;
		} else {
			return true;
		}
	}

	public void ajouteChoixSiNecessaire(String chaine, NodMem nod) {
		// "*** on n'enrichit <regles> que s'il ***"
		// "*** ne contient pas déjà <chaine>  ***"

		if (StringUtils.isEmpty(getRegles())) {
			setRegles(chaine);
			ajouteBaseNodsSuivants(nod);
		} else {
			if (getRegles().indexOf(chaine) == -1) {
				this.setRegles(this.getRegles() + "; " + chaine);
				ajouteBaseNodsSuivants(nod);
			}
		}
	}

	public void remplaceChoix(String propriete, String chaine, NodMem nod) {
		String s = "si: " + propriete;
		String regles = "";
		String[] l = getRegles().split(";");
		for (String elem : l) {
			if (elem.contains(s))  {
				// on remplace...
				regles = regles + chaine + ";";
				ajouteBaseNodsSuivants(nod);

				String[] l2 = elem.split(" ");
				String ancienNod = l2[l2.length-1];
				assocNosSuivants.remove(ancienNod);
			} else {
				regles = regles + elem + ";";
			}
		}
		this.setRegles(regles);

	}

	public NodMem nodSuivantDeNom(String nomNos) {
		return assocNosSuivants.get(nomNos);
	}

	public NodMem creeNodMemSiNecessaire(String chaine, String s) {
		// On n'enrichit <regles> que si ce champ
		// ne contient pas déjà <chaine>.
		// On renvoie le NodMem qui correspond à <s>,
		// après l'avoir créé si nécessaire

		NodMem nod = new NodMem(s);

		if (StringUtils.isEmpty(getRegles())) {
			setRegles(chaine);
			ajouteBaseNodsSuivants(nod);
		} else {
			if (getRegles().indexOf(chaine) == -1) {
				this.setRegles(this.getRegles() + "; " + chaine);
				ajouteBaseNodsSuivants(nod);
			} else {
				// la règle existait déjà :
				// on renvoie le nos préexistant...
				NodMem n = nodSuivantDeNom(s);
				return n;
			}
		}
		return nod;
	}

	public List<NodMem> tousLesNodMemSuivants() {
		/*
		 * renvoie la liste de tous les descendants du receveur (suivantsVue des
		 * suivantsVue des ... <this> y compris ...en premiere position !!
		 */

		Set<NodMem> ens = new HashSet<NodMem>();
		ens = tousLesNodMemSuivants(new HashSet<NodMem>());

		List<NodMem> l = new ArrayList<NodMem>(ens);
		// On met <this> en première position :
		l.remove(this);
		l.add(0, this);

		return l;
	}

	private Set<NodMem> tousLesNodMemSuivants(Set<NodMem> baluchon) {

		Set<NodMem> ens;
		ens = new HashSet<NodMem>();
		ens.add(this);
		ens.addAll(baluchon);

		for (NodMem courant : assocNosSuivants.values()) {
			if (ens.add(courant) == true) {
				// on ne va chercher les descendants
				// que si <courant> n'est pas déjà dans le baluchon
				// (gestion des cycles) 
				ens.addAll(courant.tousLesNodMemSuivants(ens));
			}
		}

		return ens;
	}
}
