/*
  EdiNoS - Editeur de NoS
  Copyright (C) 2013-2020 - Luc Poittevin — Tous droits réservés.
  Contact : edinos@free.fr  -  https://www.edinos.org
  
  Ce programme est un logiciel libre ; vous pouvez le redistribuer ou le
  modifier suivant les termes de la “GNU General Public License” telle que
  publiée par la Free Software Foundation : soit la version 3 de cette
  licence, soit (à votre gré) toute version ultérieure.
  
  Ce programme est distribué dans l’espoir qu’il vous sera utile, mais SANS
  AUCUNE GARANTIE : sans même la garantie implicite de COMMERCIALISABILITÉ
  ni d’ADÉQUATION À UN OBJECTIF PARTICULIER. Consultez la Licence Générale
  Publique GNU pour plus de détails.
  
  Vous devriez avoir reçu une copie de la Licence Générale Publique GNU avec
  ce programme ; si ce n’est pas le cas, consultez :
  <http://www.gnu.org/licenses/>. 
 */

package nos.utils;

import java.beans.PropertyVetoException;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.nio.file.Files;
import java.nio.file.StandardCopyOption;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collections;
import java.util.List;
import java.util.StringTokenizer;

import javax.swing.JDesktopPane;
import javax.swing.JInternalFrame;
import javax.xml.bind.JAXBContext;
import javax.xml.bind.JAXBElement;
import javax.xml.bind.JAXBException;
import javax.xml.bind.Unmarshaller;
import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.transform.stream.StreamSource;
import javax.xml.xpath.XPath;
import javax.xml.xpath.XPathConstants;
import javax.xml.xpath.XPathFactory;

import nos.obj.Edi;
import nos.run.Moteur;

import org.apache.commons.io.FileUtils;
import org.jdom.DocType;
import org.jdom.Document;
import org.jdom.JDOMException;
import org.jdom.input.DOMBuilder;
import org.jdom.input.SAXBuilder;
import org.jdom.output.DOMOutputter;
import org.jdom.output.Format;
import org.jdom.output.XMLOutputter;
import org.w3c.dom.Node;

public class ToolNos {

	public static String analyseFormule(List<String> l) {
		String s = "";

		for (int i = 0; i < l.size(); i++) {
			if (!l.get(i).equals("si:")) {
				if (l.get(i).equals("alorsAller:")) {
					return s;
				} else {
					if (s.length() > 0) {
						s = s + " " + (l.get(i));
					} else {
						s = (String) l.get(i);
					}
				}
			}
		}
		return s;
	}

	public static List<List<Object>> analyseRegle(String regles) {
		// renvoie une liste de liste de 2 éléménts :
		// - le premier élément contient la liste des choix permis
		// - le second, le fait renvoyé

		List<Object> ligne;
		String nomFait;
		int ind;
		List<String> listeChoix;
		List<List<String>> liste = null;
		List<List<Object>> res = null;
		res = new ArrayList<List<Object>>();
		liste = ToolNos.asStructureSyntaxe(regles);
		if (!liste.isEmpty()) {
			for (List<String> l : liste) {
				if (l.get(0).equals("choix:")) {
					ligne = null;
					nomFait = "";
					ind = l.indexOf("signifie:");
					if (ind > 0) {
						listeChoix = l.subList(1, ind);
						nomFait = l.get(ind + 1);
						ligne = new ArrayList<Object>();
						ligne.add(listeChoix);
						ligne.add(nomFait);
						res.add(ligne);
					}
				} else if (l.get(0).equals("ignore:")) {
					ligne = null;
					nomFait = "";
					ind = l.indexOf("ignore:");
					listeChoix = l.subList(1, l.size());
					ligne = new ArrayList<Object>();
					ligne.add(listeChoix);
					ligne.add(nomFait);
					res.add(ligne);
				}
			}
		}
		return res;
	}

	public static List<String> asArrayOfFormules(String chaine) {
		chaine = chaine.replaceAll("\\(", " \\( ");
		chaine = chaine.replaceAll("\\)", " \\) ");
		chaine = remplaceEspacesDansDoubleQuote(chaine);
		chaine = retireDoublesQuotes(chaine);
		List<String> liste = Arrays.asList(chaine.split(" "));
		List<String> liste2 = new ArrayList<String>();
		liste2.addAll(liste);
		List<String> l = new ArrayList<String>(1);
		l.add("");
		liste2.removeAll(l);

		return liste2;
	}

	public static List<List<String>> asStructureFait(String chaine) {

		String parentheseOuverte = "\\(";
		String parentheseFermee = "\\)";
		String[] tab, el2;
		ArrayList<String> el;
		List<List<String>> liste;

		liste = new ArrayList<List<String>>();
		if (chaine.length() == 0) {
			return liste;
		}

		tab = chaine.split(";");

		for (int i = 0; i < tab.length; i++) {
			el = new ArrayList<String>();
			for (String s : tab[i].split(parentheseOuverte)) {
				el.add(s);
			}
			if (el.size() > 1) {
				el2 = el.get(1).toString().split(parentheseFermee);
				el.remove(1);
				el.add(el2[0]);
			}
			liste.add(el);
		}
		return liste;
	}

	public static List<List<String>> asStructureSyntaxe(String chaine) {
		// décompose <chaine> sur le car. ';' puis sur le
		// car. ' ', et renvoie une liste
		// avec dans chaque élément de la liste,
		// une liste de chaines de car.

		String[] tab;
		List<List<String>> liste = new ArrayList<List<String>>();

		chaine = chaine.replaceAll("\\n", "");
		chaine = chaine.trim();
		tab = chaine.split(";");

		for (int i = 0; i < tab.length; i++) {
			liste.add(Arrays.asList(tab[i].trim().split(" ")));
		}
		return liste;
	}

	public static Document ChargeJDom(String nomFichier) {
		org.jdom.Document doc = null;
		try {
			SAXBuilder builder = new SAXBuilder();
			builder.setValidation(true);
			try {
				doc = builder.build(nomFichier);
				doc.setBaseURI(nomFichier);
			} catch (JDOMException e) {
				throw e;
			} catch (IOException e) {
				throw e;
			}

		} catch (Exception t) {
			t.printStackTrace();
			Edi.nos().montreInfo(t.getMessage());
		}
		return doc;
	}

	public static String cheminRep(String chaine) {
		char c = renvoieSeparateurRepertoires();
		int i = chaine.lastIndexOf(c);
		return chaine.substring(0, i + 1);
	}

	public static org.jdom.Document DOMtoJDOM(org.w3c.dom.Document documentDOM)
			throws Exception {
		// On utilise la classe DOMBuilder pour cette transformation
		DOMBuilder builder = new DOMBuilder();
		org.jdom.Document documentJDOM = builder.build(documentDOM);
		return documentJDOM;
	}

	public static org.w3c.dom.Document JDOMtoDOM(org.jdom.Document documentJDOM)
			throws Exception {
		// On utilise la classe DOMOutputter pour cette transformation
		DOMOutputter domOutputter = new DOMOutputter();
		org.w3c.dom.Document documentDOM = domOutputter.output(documentJDOM);
		return documentDOM;
	}

	public static String nomSysteme() {
		// renvoie "Mac", "Linux" ou "Windows"
		return System.getProperty("os.name").split(" ")[0];
	}

	public static List<String> oldAsArrayOfFormules(String chaine) {
		List<String> liste = Arrays.asList(chaine.split(" "));

		return liste;
	}

	public static List<String> regroupeProposition(List<String> tab) {

		// "******************************************************"
		// "*** regroupe dans une seule case tous les éléments ***"
		// "*** qui correspondent à la même proposition        ***"
		// "*** (une proposition a pour valeur VRAI ou FAUX).  ***"
		// "*** Pour ce faire, repère les opérateurs '=','<',  ***"
		// "*** '>','<=','>=' et '<>' et regroupe les éléments ***"
		// "*** antérieurs et postérieurs à l'opérateur.       ***"
		// "******************************************************"

		List<String> tab2 = new ArrayList<String>();
		int i = 0;
		ArrayList<Object> listeOp = Collections.list(new StringTokenizer(
				"= > < >= <= <> inconnu"));

		while (i < tab.size()) {
			if (listeOp.contains(tab.get(i))) {
				int j = tab2.size() - 1;
				String s = tab2.remove(j);
				// Modif du 07/04/2019 :
				if (tab.get(i).equals("inconnu")) {
					tab2.add(j, s + " " + tab.get(i));
				} else {
					tab2.add(j, s + " " + tab.get(i) + " " + tab.get(i + 1));
					i = i + 1;
				}
			} else {
				tab2.add(tab.get(i));
			}
			i = i + 1;
		}

		return tab2;
	}

	public static String retireCommentaires(String chaine) {
		String[] ligne = chaine.split("\n");
		String res = "";
		for (String s : ligne) {
			if (s.startsWith("//")) {
			} else {
				if (!res.isEmpty()) {
					res = res + "\n";
				}
				res = res + s;
			}
		}
		return res;
	}

	public static String rejectComments(String chaine) {
		/* OBSOLETE depuis EdiNoS v.3.4.0 (edinosrun 1.0.12)
		 * encore appelé par Action>>structureFaitsAttendus() 
		/**
		 * "Answer the receiver with all of the embedded comments deleted." |
		 * input output delimeter inComment character | input := ReadStream on:
		 * self. output := WriteStream on: (self subcollectionSpecies new: self
		 * size). delimeter := $". inComment := false. [input atEnd] whileFalse:
		 * [(character := input next) = delimeter ifTrue: [inComment :=
		 * inComment not] ifFalse: [inComment ifFalse: [output nextPut:
		 * character]]]. ^output contents
		 **/

		char guillemet = '"';
		String retour = "";
		boolean inComment = false;

		for (char c : chaine.toCharArray()) {
			if (c == guillemet) {
				inComment = !inComment;
			} else if (!inComment) {
				retour = retour + c;
			}
		}

		return retour.replaceAll("\\n", "");
	}

	public static String detecteComments(String chaine) {
		/**
		 * "Answer the embedded comments inside the receiver
		 **/

		char guillemet = '"';
		String retour = "";
		boolean inComment = false;

		for (char c : chaine.toCharArray()) {
			if (c == guillemet) {
				inComment = !inComment;
			} else if (inComment) {
				retour = retour + c;
			}
		}
		return retour;
	}

	public static String remplaceEspacesDansDoubleQuote(String chaine) {
		/**
		 * Détecte les espaces entre les doubles quotes et les
		 * remplace par _
		 **/

		char doubleQuote = '\"';
//		char espace = ' ';
		String retour = "";
		boolean in = false;

		for (char c : chaine.toCharArray()) {
			if (c == doubleQuote) {
				in = !in;
				retour = retour + c;
			} else if (in && (c == ' ')) {
				retour = retour + '_';
			} else {
				retour = retour + c;
			}
		}
		return retour;
	}

	public static String retireDoublesQuotes(String chaine) {
		/**
		 * Retire les doublequotes mais laisse en l'état "" ...
		 **/

		if (chaine.contains("\"\"")) {
			return chaine;
		} else {
			return chaine.replaceAll("\"", "");
		}
	}

	public static String construitRegleAvecCollection(List<String> elem) {
		String regle = "";
		for (String st : elem) {
			if (regle.isEmpty()) {
				regle = st;
			} else {
				regle = regle + " " + st;
			}
		}
		return regle;
	}

	public static int renvoieModifiers() {
		// Renvoie la valeur du "modifier" à utiliser
		// pour les touches de raccourcis (dépend de l'OS)

		if (nomSysteme().equals("Windows")) {
			return java.awt.Event.CTRL_MASK;
		} else if (nomSysteme().equals("Mac")) {
			return java.awt.Event.META_MASK;
		} else if (nomSysteme().equals("Linux")) {
			return java.awt.Event.ALT_MASK;
		}

		return java.awt.Event.CTRL_MASK;
	}
	
	public static char renvoieSeparateurRepertoires() {
		// Renvoie le séparateur de répertoire à utiliser
		// pour le path du fichier (dépend de l'OS)

		if (nomSysteme().equals("Windows")) {
			return '\\';
		} else if (nomSysteme().equals("Mac")) {
			return '/';
		} else if (nomSysteme().equals("Linux")) {
			return '/';
		}

		return '/';
	}

	public static void SauveJDom(org.jdom.Document doc, String dtd,
			String doctype) throws FileNotFoundException, IOException {
		DocType dt = new DocType(doctype, dtd);
		doc.setDocType(dt);
		Format f = Format.getPrettyFormat();
		f.setEncoding("iso-8859-1");
		XMLOutputter sortie = new XMLOutputter(f);
		sortie.output(doc, new FileOutputStream(doc.getBaseURI()));
	}

	public static String getTextContent(Node n) {
		return n.getTextContent().trim(); // 1.5 et +
	}

	public static void setTextContent(Node n, String s) {
		n.setTextContent(s); // 1.5 et +
	}

	public static Moteur chargeMoteur(File file) {
		Moteur m = null;

		if (!file.exists()) {
			System.out.println("fichier non trouvé...");
		}

		try {
			JAXBContext context = JAXBContext.newInstance(Moteur.class);

			Unmarshaller unmarshaller = context.createUnmarshaller();

			JAXBElement<Moteur> root = unmarshaller.unmarshal(new StreamSource(
					file), Moteur.class);
			m = root.getValue();
		} catch (JAXBException e) {
			e.printStackTrace();
			Edi.nos().message(
					"Erreur dans le fichier <" + file.toString() + ">...");
			return null;
		}

		String actual = "";
		String xpathExpression = "moteur/noS/nom";

		try {
			DocumentBuilderFactory documentBuilderFactory = DocumentBuilderFactory
					.newInstance();
			documentBuilderFactory.setNamespaceAware(true);
			documentBuilderFactory.setIgnoringElementContentWhitespace(true);
			DocumentBuilder docBuilder = documentBuilderFactory
					.newDocumentBuilder();
			org.w3c.dom.Document doc = docBuilder.parse(file);

			XPathFactory xPathFactory = XPathFactory.newInstance();
			XPath xpath = xPathFactory.newXPath();

			actual = xpath
					.evaluate(xpathExpression, doc, XPathConstants.STRING)
					.toString();

		} catch (Exception e) {
			throw new RuntimeException(e);
		}

		if (Edi.nos().getBaseCourante().contientNoS(actual)) {
			m.setNos(Edi.nos().getBaseCourante().getNoS(actual));
		} else {
			Edi.nos()
					.message(
							"Cette trace ne provient pas d'un graphe de la base courante.");
		}

		return m;
	}

	public static double getJavaVersion() {
		String version = System.getProperty("java.version");
		int pos = version.indexOf('.');
		if (pos < 0) {
			return Double.parseDouble(version);
		} else {
			pos = version.indexOf('.', pos + 1);
			return Double.parseDouble(version.substring(0, pos));
		}
	}

	public static String traiteValeur(String chaine) {
		/**
		 * Repère la valeur de la propriété et y détecte les espaces pour les
		 * remplacer par _
		 **/

		chaine = chaine.trim(); 
		char espace = ' ';
		String retour = "";
		boolean inValeur = false;

		for (char c : chaine.toCharArray()) {
			if ((!inValeur) && (c == espace)) {
				inValeur = !inValeur;
				retour = retour + c;
			} else if (inValeur && (c == espace)) {
				retour = retour + '_';
			} else {
				retour = retour + c;
			}
		}
		return retour;
	}
	
	public static void fermeFen(JInternalFrame f) {
		// sous Mac, il y a depuis longtemps un bogue avec 
		// la sélection automatique de la première frame
		// derrière la frame qui est fermée...
		// Après recherche de solution en utilisant MyDesktopManager (voir cette classe),
		// on opte finalement plutôt pour des appels à cette méthode depuis les méthodes internalFrameClosing()
		
		if (nomSysteme().equals("Mac")) {
			JDesktopPane d = f.getDesktopPane();

			JInternalFrame[] iframes = d.getAllFrames();
			f.setVisible(false);
			f.getDesktopPane().remove(f);
			f.setVisible(false);
			for (int i = 0; i < iframes.length; i++) {
				JInternalFrame iframe = iframes[i];
				if (!iframe.equals(f)) {
					try {
						iframe.setSelected(true);
					} catch (PropertyVetoException e1) {
					}
					return;
				}
			}
		}
	}
	
	public static void copieFichier(String dest, String fic) {
		
		String rep = System.getProperty("user.dir") + ToolNos.renvoieSeparateurRepertoires() + fic;

		File file = new File(rep).getAbsoluteFile();
		if (!file.exists()) {
			Edi.nos().montreInfo("Fichier " + fic + " non trouvé");
		} else {
			try {
				Files.copy(file.toPath(), (new File(dest + ToolNos.renvoieSeparateurRepertoires() + fic)).toPath(),
						StandardCopyOption.REPLACE_EXISTING);
			} catch (IOException e) {
				Edi.nos().montreInfo("Erreur lors de la copie de " + fic);
			}
		}
	}
	
	public static void copieRepertoire(String dest, String repo) {
		
		String rep = System.getProperty("user.dir") + ToolNos.renvoieSeparateurRepertoires() + repo;
		File source = new File(rep);


		if (!source.exists()) {
			Edi.nos().montreInfo("Fichier " + source.getName() + " non trouvé");
		} else {
			File cible = new File(dest + ToolNos.renvoieSeparateurRepertoires() + repo);

			try {
				FileUtils.copyDirectory(source, cible);
			} catch (IOException e) {
				Edi.nos().montreInfo("Erreur lors de la copie de " + source.getName());
			}
		}
	}
}
