/*
  EdiNoS - Editeur de NoS
  Copyright (C) 2019 - Luc Poittevin — Tous droits réservés.
  Contact : edinos@free.fr  -  https://www.edinos.org
  
  Ce programme est un logiciel libre ; vous pouvez le redistribuer ou le
  modifier suivant les termes de la “GNU General Public License” telle que
  publiée par la Free Software Foundation : soit la version 3 de cette
  licence, soit (à votre gré) toute version ultérieure.
  
  Ce programme est distribué dans l’espoir qu’il vous sera utile, mais SANS
  AUCUNE GARANTIE : sans même la garantie implicite de COMMERCIALISABILITÉ
  ni d’ADÉQUATION À UN OBJECTIF PARTICULIER. Consultez la Licence Générale
  Publique GNU pour plus de détails.
  
  Vous devriez avoir reçu une copie de la Licence Générale Publique GNU avec
  ce programme ; si ce n’est pas le cas, consultez :
  <http://www.gnu.org/licenses/>. 
 */

package nos.utils;

import java.io.File;
import java.io.FilenameFilter;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;

import org.jdom.Element;
import org.w3c.dom.Document;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;

import nos.obj.Action;
import nos.obj.Base;
import nos.obj.Edi;
import nos.obj.FaitAttendu;
import nos.obj.Graphe;
import nos.obj.Message;
import nos.obj.NoS;
import nos.obj.Procedure;
import nos.obj.Question;

public class ToolNosXML {

	public static void chargeInterface(File f, Base b) {
		ConstruitDom cd = new ConstruitDom(f.toString());

		// Verification du fichier .int.xml :
		if (cd.testeEtChargeXML() == false) {
			Edi.nos().message(
					"Erreur dans le fichier <" + f.toString() + ">...");
			return;
		}

		Document docInterface = cd.document();

		// Chargement du fichier .int.xml :
		String nomDTD = docInterface.getDoctype().getSystemId();

		if (nomDTD.equals("../edinos.int.dtd")) {
			chargeInterfacev1(docInterface, b);
			b.convertitActionsEn20();
			Edi.nos().message(
					"La base <" + f.toString()
							+ "> a été convertie au format 2.0.");
		} else if (nomDTD.equals("../edinos-interface-2.0.dtd")) {
			
			chargeInterfacev20(docInterface, b);
		} else {
			Edi.nos().message(
					"<" + f.toString()
							+ "> : la base n'a pas pu être chargée (" + nomDTD
							+ " introuvable...)");
		}
	}

	private static void chargeInterfacev1(Document d, Base b) {
		// chargement pour une DTD "edinos-interface.dtd"
		Procedure proc;
		Question question;
		Message message;
		NodeList nl;
		String nom;
		Document docInterface;
		Map<String, Action> assocActions;

		docInterface = d;
		assocActions = b.getAssocActions();

		// chargement des procedures :
		for (int i = 0; i < docInterface.getElementsByTagName("procedure")
				.getLength(); i++) {
			nom = docInterface.getElementsByTagName("procedure").item(i)
					.getAttributes().getNamedItem("nom").getNodeValue();
			if (Edi.nos().contientProc(nom) || assocActions.containsKey(nom)) {
				Edi.nos().montreAvertissement(
						"La procédure <" + nom
								+ "> existe déjà, elle n'a pas été chargée...");
			} else {
				proc = new Procedure(nom);
				nl = docInterface.getElementsByTagName("procedure").item(i)
						.getChildNodes();
				proc.setDescription(ToolNos.getTextContent(nl.item(1)));
				proc.setBase(b);
				assocActions.put(proc.getNom(), proc);
			}
		}

		// chargement des ecrans :
		for (int i = 0; i < docInterface.getElementsByTagName("ecran")
				.getLength(); i++) {
			nom = docInterface.getElementsByTagName("ecran").item(i)
					.getAttributes().getNamedItem("nom").getNodeValue();
			if (Edi.nos().contientQuestion(nom)
					|| assocActions.containsKey(nom)) {
				Edi.nos().montreAvertissement(
						"La question <" + nom
								+ "> existe déjà, elle n'a pas été chargée...");
			} else {
				question = new Question(nom);
				nl = docInterface.getElementsByTagName("ecran").item(i)
						.getChildNodes();


				for (int j = 1; j < nl.getLength(); j = j + 2) {
					if (nl.item(j).getNodeName().equals("description")) {
						question.setDescription(ToolNos.getTextContent(nl
								.item(j)));
					}

					if (nl.item(j).getNodeName().equals("texte")) {
						// pas de trim() pour le contenu des questions...
						question.setTexte(nl.item(j).getTextContent());
					}

					if (nl.item(j).getNodeName().equals("regles")) {
						question.setRegles(ToolNos.getTextContent(nl.item(j)));
					}
				}
				
				question.setBase(b);
				assocActions.put(question.getNom(), question);
			}
		}

		// chargement des messages :
		for (int i = 0; i < docInterface.getElementsByTagName("message")
				.getLength(); i++) {
			nom = docInterface.getElementsByTagName("message").item(i)
					.getAttributes().getNamedItem("nom").getNodeValue();
			if (Edi.nos().contientMessage(nom) || assocActions.containsKey(nom)) {
				Edi.nos().montreAvertissement(
						"Le message <" + nom
								+ "> existe déjà, il n'a pas été chargé...");
			} else {
				message = new Message(nom);
				nl = docInterface.getElementsByTagName("message").item(i)
						.getChildNodes();

				for (int j = 1; j < nl.getLength(); j = j + 2) {
					if (nl.item(j).getNodeName().equals("description")) {
						message.setDescription(ToolNos.getTextContent(nl
								.item(j)));
					}

					if (nl.item(j).getNodeName().equals("texte")) {
						// pas de trim() pour le contenu des questions...
						message.setTexte(nl.item(j).getTextContent());
					}
				}
				
				message.setBase(b);
				assocActions.put(message.getNom(), message);
			}
		}
	}

	private static void chargeInterfacev20(Document d, Base b) {
		// chargement pour une DTD "edinos-interface-2.0.dtd"
		Procedure proc;
		Question question;
		Message message;
		NodeList nl;
		String nom;
		Document docInterface;
		Map<String, Action> assocActions;

		docInterface = d;
		assocActions = b.getAssocActions();

		// chargement des procedures :
		for (int i = 0; i < docInterface.getElementsByTagName("procedure")
				.getLength(); i++) {
			nom = docInterface.getElementsByTagName("procedure").item(i)
					.getAttributes().getNamedItem("nom").getNodeValue();
			if (Edi.nos().contientProc(nom) || assocActions.containsKey(nom)) {
				Edi.nos().montreAvertissement(
						"La procédure <" + nom
								+ "> existe déjà, elle n'a pas été chargée...");
			} else {
				proc = new Procedure(nom);
				nl = docInterface.getElementsByTagName("procedure").item(i)
						.getChildNodes();
				
				
				for (int j = 1; j < nl.getLength(); j = j + 2) {
					if (nl.item(j).getNodeName().equals("description")) {
						proc.setDescription(ToolNos.getTextContent(nl
								.item(j)));
					}

					if (nl.item(j).getNodeName().equals("faits")) {
						chargeFaits(nl.item(j), (Action) proc);
					}
				}
				
				proc.setBase(b);
				assocActions.put(proc.getNom(), proc);
			}
		}

		// chargement des questions :
		for (int i = 0; i < docInterface.getElementsByTagName("question")
				.getLength(); i++) {
			nom = docInterface.getElementsByTagName("question").item(i)
					.getAttributes().getNamedItem("nom").getNodeValue();
			if (Edi.nos().contientQuestion(nom)
					|| assocActions.containsKey(nom)) {
				Edi.nos().montreAvertissement(
						"La question <" + nom
								+ "> existe déjà, elle n'a pas été chargée...");
			} else {
				question = new Question(nom);
				nl = docInterface.getElementsByTagName("question").item(i)
						.getChildNodes();

//				}

				for (int j = 1; j < nl.getLength(); j = j + 2) {
					if (nl.item(j).getNodeName().equals("description")) {
						question.setDescription(ToolNos.getTextContent(nl
								.item(j)));
					}

					if (nl.item(j).getNodeName().equals("texte")) {
						// pas de trim() pour le contenu des questions...
						question.setTexte(nl.item(j).getTextContent());
					}

					if (nl.item(j).getNodeName().equals("faits")) {
						chargeFaits(nl.item(j), (Action) question);
					}
				}
				
				question.setBase(b);
				assocActions.put(question.getNom(), question);
			}
		}

		// chargement des messages :
		for (int i = 0; i < docInterface.getElementsByTagName("message")
				.getLength(); i++) {
			nom = docInterface.getElementsByTagName("message").item(i)
					.getAttributes().getNamedItem("nom").getNodeValue();
			if (Edi.nos().contientMessage(nom) || assocActions.containsKey(nom)) {
				Edi.nos().montreAvertissement(
						"Le message <" + nom
								+ "> existe déjà, il n'a pas été chargé...");
			} else {
				message = new Message(nom);
				nl = docInterface.getElementsByTagName("message").item(i)
						.getChildNodes();

				for (int j = 1; j < nl.getLength(); j = j + 2) {
					if (nl.item(j).getNodeName().equals("description")) {
						message.setDescription(ToolNos.getTextContent(nl
								.item(j)));
					}

					if (nl.item(j).getNodeName().equals("texte")) {
						// pas de trim() pour le contenu des questions...
						message.setTexte(nl.item(j).getTextContent());
					}
				}
				
				message.setBase(b);
				assocActions.put(message.getNom(), message);
			}
		}
	}

	private static void chargeFaits(Node node, Action action) {
		String nom = "";
		NodeList nl, nl2;
		List<FaitAttendu> listeFaitsAttendus = new ArrayList<FaitAttendu>();
		
		nl = node.getChildNodes();

		for (Integer k = 1; k < nl.getLength(); k = k + 2) {
			nom = nl.item(k).getAttributes().getNamedItem("nom").getNodeValue();
			FaitAttendu f = new FaitAttendu(nom);
			nl2 = nl.item(k).getChildNodes();

			for (int j = 1; j < nl2.getLength(); j = j + 2) {
				if (nl2.item(j).getNodeName().equals("typeFait")) {
					f.setType(ToolNos.getTextContent(nl2.item(j)));
				}

				if (nl2.item(j).getNodeName().equals("descriptionFait")) {
					f.setDescription(ToolNos.getTextContent(nl2.item(j)));
				}
			}
			listeFaitsAttendus.add(f);
		}
		action.setFaitsAttendus(listeFaitsAttendus);
	}

	public static void chargeGraphes(File f, Base base) {
		File[] tabFicNod;
		ConstruitDom cd = null;
		// Recherche des fichiers .nod.xml :
		tabFicNod = f.getParentFile().listFiles(
				(FilenameFilter) new FiltreNodXml());

		// Vérification et chargement des fichiers .nod.xml :
		for (int i = 0; i < tabFicNod.length; i++) {
			cd = new ConstruitDom(tabFicNod[i].toString());
			if (cd.testeEtChargeXML() == false) {
				Edi.nos().message(
						"Erreur dans le fichier <" + tabFicNod[i].toString()
								+ ">...");
				return;
			}
			chargeNos(cd.document(), base, tabFicNod[i].toString());
		}
	}

	private static void chargeNos(Document doc, Base b, String nomFic) {
		NoS nos;
		NodeList nl;
		String s;
		Map<String, Graphe> assocGraphes;
		Map<String, NoS> assocNoS;
		
		assocGraphes = b.getAssocGraphes();
		assocNoS = b.getAssocNos();

		s = doc.getElementsByTagName("graphe").item(0).getAttributes()
				.getNamedItem("nom").getNodeValue();

		Graphe g = new Graphe(s);
		assocGraphes.put(g.getNom(), g);
		
		g.setDoc(doc);
		g.setDocURI(nomFic);

		for (int i = 0; i < doc.getElementsByTagName("nos").getLength(); i++) {
			String nom = doc.getElementsByTagName("nos").item(i)
					.getAttributes().getNamedItem("nom").getNodeValue();
			if (Edi.nos().contientNoS(nom) || assocNoS.containsKey(nom)) {
				Edi.nos().message(
						"Le nos <" + nom
								+ "> existe déjà : chargement du graphe <"
								+ g.getNom() + "> interrompu...");
				return;
			} else {
				nos = new NoS(nom);
				nos.setNode(doc.getElementsByTagName("nos").item(i));

				nl = doc.getElementsByTagName("nos").item(i).getChildNodes();
				for (int j = 1; j < nl.getLength(); j = j + 2) {
					if (nl.item(j).getNodeName().equals("actions")) {
						nos.setActions(ToolNos.getTextContent(nl.item(j)));
					}
					
					if (nl.item(j).getNodeName().equals("choix")) {
						nos.setRegles(ToolNos.getTextContent(nl.item(j)));
					}
					
					if (nl.item(j).getNodeName().equals("commentaires")) {
						nos.setCommentaires(ToolNos.getTextContent(nl.item(j)));
					}
				}

				nos.setPosition(doc.getElementsByTagName("nos").item(i)
						.getAttributes().getNamedItem("position")
						.getNodeValue());
				nos.setBase(b);
				assocNoS.put(nos.getNom(), nos);
			}
		}

		NoS nodInit = assocNoS.get(g.getNom());
		g.setPremierNoS(nodInit);

		for (int i = 0; i < doc.getElementsByTagName("suivantsVue").getLength(); i++) {
			String nomNos = doc.getElementsByTagName("suivantsVue").item(i)
					.getAttributes().getNamedItem("origine").getNodeValue();
			if (assocNoS.containsKey(nomNos)) {
				nl = doc.getElementsByTagName("suivantsVue").item(i)
						.getChildNodes();
				for (int j = 1; j < nl.getLength(); j = j + 2) {
					String nomNosDest = ToolNos.getTextContent(nl.item(j));
					if (assocNoS.containsKey(nomNosDest)) {
						NoS n = assocNoS.get(nomNos);
						n.addSuivant(nomNosDest);
					} else {
						Edi.nos().montreInfo(
								"[chargement suivantsVue] nodule <"
										+ nomNosDest + "> non trouvé");
					}
				}
			} else {
				Edi.nos().montreInfo(
						"[chargement suivantsVue] nodule <" + nomNos
								+ "> non trouvé");
			}
		}
		nodInit.raccourcitTousLesSuivants();
	}
	
	public static org.jdom.Document reconstruitDoc(String uri, Base b) {
		Element actElem = null;
		Action act = null;
		
		Element racine = new Element("interface");

		for (String nom : b.getListeProcs()) {
			act = b.getAssocActions().get(nom);
			actElem = remplitElemPour(act);
			racine.addContent(actElem);
		}

		for (String nom : b.getListeQuestions()) {
			act = b.getAssocActions().get(nom);
			actElem = remplitElemPour(act);
			racine.addContent(actElem);
		}

		for (String nom : b.getListeMessages()) {
			act = b.getAssocActions().get(nom);
			actElem = remplitElemPour(act);
			racine.addContent(actElem);
		}

		org.jdom.Document jdoc = new org.jdom.Document(racine);
		jdoc.setBaseURI(uri);
		return jdoc;
	}

	private static Element remplitElemPour(Action act ) {
		Element actElem = null, descElem = null, texteElem = null;
		Element faitsElem = null, faitElem = null, typeElem = null;
		List<FaitAttendu> listeFaits = act.getFaitsAttendus();

		if (act.getClass().getName() == "nos.obj.Procedure") {
			actElem = new Element("procedure");
			actElem.setAttribute("nom", act.getNom());

			if (act.getDescription().length() > 0) {
				descElem = new Element("description");
				descElem.setText(act.getDescription());
				actElem.addContent(descElem);
			}
			
			faitsElem = new Element("faits");
			if (listeFaits != null) {
				for (FaitAttendu fa : listeFaits) {
					faitElem = new Element("fait");
					faitElem.setAttribute("nom", fa.getNom());
					typeElem = new Element("typeFait");
					typeElem.setText(fa.getType());
					faitElem.addContent(typeElem);
					descElem = new Element("descriptionFait");
					descElem.setText(fa.getDescription());
					faitElem.addContent(descElem);
					faitsElem.addContent(faitElem);
				}
			}
			actElem.addContent(faitsElem);
		} else if (act.getClass().getName() == "nos.obj.Question") {
			Question q = (Question) act;
			actElem = new Element("question");
			actElem.setAttribute("nom", act.getNom());

			if (act.getDescription().length() > 0) {
				descElem = new Element("description");
				descElem.setText(act.getDescription());
				actElem.addContent(descElem);
			}

			texteElem = new Element("texte");
			texteElem.setText(q.getTexte());
			actElem.addContent(texteElem);

			faitsElem = new Element("faits");
			if (listeFaits != null) {
				for (FaitAttendu fa : listeFaits) {
					faitElem = new Element("fait");
					faitElem.setAttribute("nom", fa.getNom());
					typeElem = new Element("typeFait");
					typeElem.setText(fa.getType());
					faitElem.addContent(typeElem);
					descElem = new Element("descriptionFait");
					descElem.setText(fa.getDescription());
					faitElem.addContent(descElem);
					faitsElem.addContent(faitElem);
				}
			}
			actElem.addContent(faitsElem);

		} else if (act.getClass().getName() == "nos.obj.Message") {
			Message q = (Message) act;
			actElem = new Element("message");
			actElem.setAttribute("nom", act.getNom());

			if (act.getDescription().length() > 0) {
				descElem = new Element("description");
				descElem.setText(act.getDescription());
				actElem.addContent(descElem);
			}

			texteElem = new Element("texte");
			texteElem.setText(q.getTexte());
			actElem.addContent(texteElem);
		}
		
		return actElem;
	}
}
